
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */




#ifdef MPICH_HAS_C2F
_EXTERN_C_ void *MPIR_ToPointer(int);
#endif // MPICH_HAS_C2F

#if defined(MPI_STATUS_SIZE) && MPI_STATUS_SIZE > 0
#define MPI_F_STATUS_SIZE MPI_STATUS_SIZE
#else
void get_mpi_f_status_size___(int*);
static inline int __get_f_status_size(){int size; get_mpi_f_status_size___(&size); return size;}
#define MPI_F_STATUS_SIZE __get_f_status_size()
#endif

void char_p_f2c(const char* fstr, int len, char** cstr)
{
  const char* end;
  int i;

  /* Leading and trailing blanks are discarded. */

  end = fstr + len - 1;

  for (i = 0; (i < len) && (' ' == *fstr); ++i, ++fstr) {
    continue;
  }

  if (i >= len) {
    len = 0;
  } else {
    for (; (end > fstr) && (' ' == *end); --end) {
      continue;
    }

    len = end - fstr + 1;
  }

  /* Allocate space for the C string, if necessary. */

  if (*cstr == NULL) {
    if ((*cstr = malloc(len + 1)) == NULL) {
      return;
    }
  }

  /* Copy F77 string into C string and NULL terminate it. */

  if (len > 0) {
    strncpy(*cstr, fstr, len);
  }
  (*cstr)[len] = '\0';
}

void char_p_c2f(const char* cstr, char* fstr, int len)
{
  int i;

  strncpy(fstr, cstr, len);
  for (i = strnlen(cstr, len); i < len; ++i) {
    fstr[i] = ' ';
  }
}

#if defined(__GNUC__) || defined(__INTEL_COMPILER) || defined(__PGI) || defined(_CRAYC)
#if defined(__GNUC__)
#define WEAK_POSTFIX __attribute__ ((weak))
#else
#define WEAK_POSTFIX
#define USE_WEAK_PRAGMA
#endif
/* OpenMPI */
_EXTERN_C_ MPI_Fint mpi_fortran_in_place WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_IN_PLACE WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpi_fortran_in_place_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_IN_PLACE_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpi_fortran_in_place__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_IN_PLACE__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpi_fortran_bottom WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_BOTTOM WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpi_fortran_bottom_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_BOTTOM_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpi_fortran_bottom__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPI_FORTRAN_BOTTOM__ WEAK_POSTFIX;
/* MPICH 2 */
_EXTERN_C_ MPI_Fint MPIFCMB3 WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb3 WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPIFCMB3_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb3_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPIFCMB3__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb3__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPIFCMB4 WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb4 WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPIFCMB4_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb4_ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint MPIFCMB4__ WEAK_POSTFIX;
_EXTERN_C_ MPI_Fint mpifcmb4__ WEAK_POSTFIX;
/* Argonne Fortran MPI wrappers */
_EXTERN_C_ void *MPIR_F_MPI_BOTTOM WEAK_POSTFIX;
_EXTERN_C_ void *MPIR_F_MPI_IN_PLACE WEAK_POSTFIX;
_EXTERN_C_ void *MPI_F_MPI_BOTTOM WEAK_POSTFIX;
_EXTERN_C_ void *MPI_F_MPI_IN_PLACE WEAK_POSTFIX;

/* MPICH 2 requires no special handling - MPI_BOTTOM may (must!) be passed through as-is. */
#define IsBottom(x) ((x) == (void *) &mpi_fortran_bottom ||   \
                     (x) == (void *) &MPI_FORTRAN_BOTTOM ||   \
                     (x) == (void *) &mpi_fortran_bottom_ ||  \
                     (x) == (void *) &MPI_FORTRAN_BOTTOM_ ||  \
                     (x) == (void *) &mpi_fortran_bottom__ || \
                     (x) == (void *) &MPI_FORTRAN_BOTTOM__)
#define IsInPlace(x) ((x) == (void *) &mpi_fortran_in_place ||   \
                      (x) == (void *) &MPI_FORTRAN_IN_PLACE ||   \
                      (x) == (void *) &mpi_fortran_in_place_ ||  \
                      (x) == (void *) &MPI_FORTRAN_IN_PLACE_ ||  \
                      (x) == (void *) &mpi_fortran_in_place__ || \
                      (x) == (void *) &MPI_FORTRAN_IN_PLACE__ || \
                      (x) == (void *) &MPIFCMB4 ||               \
                      (x) == (void *) &mpifcmb4 ||               \
                      (x) == (void *) &MPIFCMB4_ ||              \
                      (x) == (void *) &mpifcmb4_ ||              \
                      (x) == (void *) &MPIFCMB4__ ||             \
                      (x) == (void *) &mpifcmb4__ ||             \
                      (x) == MPIR_F_MPI_IN_PLACE ||              \
                      (x) == MPI_F_MPI_IN_PLACE)

#ifdef USE_WEAK_PRAGMA
#pragma weak mpi_fortran_in_place
#pragma weak MPI_FORTRAN_IN_PLACE
#pragma weak mpi_fortran_in_place_
#pragma weak MPI_FORTRAN_IN_PLACE_
#pragma weak mpi_fortran_in_place__
#pragma weak MPI_FORTRAN_IN_PLACE__
#pragma weak mpi_fortran_bottom
#pragma weak MPI_FORTRAN_BOTTOM
#pragma weak mpi_fortran_bottom_
#pragma weak MPI_FORTRAN_BOTTOM_
#pragma weak mpi_fortran_bottom__
#pragma weak MPI_FORTRAN_BOTTOM__
/* MPICH 2 */
#pragma weak MPIFCMB3
#pragma weak mpifcmb3
#pragma weak MPIFCMB3_
#pragma weak mpifcmb3_
#pragma weak MPIFCMB3__
#pragma weak mpifcmb3__
#pragma weak MPIFCMB4
#pragma weak mpifcmb4
#pragma weak MPIFCMB4_
#pragma weak mpifcmb4_
#pragma weak MPIFCMB4__
#pragma weak mpifcmb4__
/* Argonne Fortran MPI wrappers */
#pragma weak MPIR_F_MPI_BOTTOM
#pragma weak MPIR_F_MPI_IN_PLACE
#pragma weak MPI_F_MPI_BOTTOM
#pragma weak MPI_F_MPI_IN_PLACE
#endif

#if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH has no MPI_IN_PLACE */
#define BufferC2F(x) (IsBottom(x) ? MPI_BOTTOM : (x))
#else
#define BufferC2F(x) (IsBottom(x) ? MPI_BOTTOM : (IsInPlace(x) ? MPI_IN_PLACE : (x)))
#endif /* defined(MPICH_NAME) && (MPICH_NAME == 1) */

#else
#define BufferC2F(x) (x)
#endif /* defined(__GNUC__) || defined(__INTEL_COMPILER) || defined(__PGI) || defined(_CRAYC) */


#if (defined(PIC) || defined(__PIC__))
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#pragma weak pmpi_init_thread
#pragma weak PMPI_INIT_THREAD
#pragma weak pmpi_init_thread_
#pragma weak pmpi_init_thread__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT_THREAD(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread_(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread__(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#ifndef WRAP_MPI_CALL_PREFIX
#if defined(__clang__)
#define WRAP_MPI_CALL_PREFIX        \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#elif defined(__INTEL_COMPILER)
#define WRAP_MPI_CALL_PREFIX \
  _Pragma("warning push");   \
  _Pragma("warning(disable:1786)");
#define WRAP_MPI_CALL_POSTFIX _Pragma("warning pop")
#elif (defined(__GNUC__) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4))
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif
#endif

/* This file is part of P^nMPI.
 *
 * Copyright (c)
 *  2008-2018 Lawrence Livermore National Laboratories, United States of America
 *  2011-2016 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2013-2018 RWTH Aachen University, Federal Republic of Germany
 *
 *
 * P^nMPI is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation version 2.1 dated February 1999.
 *
 * P^nMPI is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with P^nMPI; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   51 Franklin St, Fifth Floor
 *   Boston, MA 02110, USA
 *
 *
 * Written by Martin Schulz, schulzm@llnl.gov.
 *
 * LLNL-CODE-402774
 */

#include "core.h"



int NQJ_Abort(MPI_Comm comm, int errorcode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Abort_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Abort[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Abort at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Abort));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Abort)[pnmpi_level](comm, errorcode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Abort at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Abort");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Abort(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Abort");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Accumulate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Accumulate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Accumulate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Accumulate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Accumulate)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Accumulate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Accumulate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Accumulate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Add_error_class(int *errorclass)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Add_error_class_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_class[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Add_error_class at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Add_error_class));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_class)[pnmpi_level](errorclass);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Add_error_class at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Add_error_class");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Add_error_class(errorclass);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Add_error_class");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Add_error_code(int errorclass, int *errorcode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Add_error_code_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_code[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Add_error_code at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Add_error_code));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_code)[pnmpi_level](errorclass, errorcode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Add_error_code at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Add_error_code");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Add_error_code(errorclass, errorcode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Add_error_code");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Add_error_string(int errorcode, const char *string)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Add_error_string_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_string[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Add_error_string at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Add_error_string));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Add_error_string)[pnmpi_level](errorcode, string);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Add_error_string at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Add_error_string");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Add_error_string(errorcode, string);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Add_error_string");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Address(void *location, MPI_Aint *address)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Address_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Address[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Address at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Address));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Address)[pnmpi_level](location, address);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Address at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Address");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Address(location, address);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Address");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Allgather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Allgather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Allgather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Allgather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Allgather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Allgather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Allgather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Allgather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Allgatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Allgatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Allgatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Allgatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Allgatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Allgatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Allgatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Allgatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Alloc_mem_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Alloc_mem[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Alloc_mem at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Alloc_mem));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Alloc_mem)[pnmpi_level](size, info, baseptr);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Alloc_mem at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Alloc_mem");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Alloc_mem(size, info, baseptr);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Alloc_mem");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Allreduce_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Allreduce[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Allreduce at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Allreduce));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Allreduce)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Allreduce at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Allreduce");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Allreduce");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Alltoall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Alltoall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Alltoall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Alltoall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Alltoall)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Alltoall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Alltoall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Alltoall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Alltoallv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Alltoallv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Alltoallv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Alltoallv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Alltoallv)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Alltoallv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Alltoallv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Alltoallv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Alltoallw_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Alltoallw[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Alltoallw at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Alltoallw));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Alltoallw)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Alltoallw at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Alltoallw");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Alltoallw");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Attr_delete(MPI_Comm comm, int keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Attr_delete_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Attr_delete[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Attr_delete at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Attr_delete));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Attr_delete)[pnmpi_level](comm, keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Attr_delete at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Attr_delete");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Attr_delete(comm, keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Attr_delete");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Attr_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Attr_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Attr_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Attr_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Attr_get)[pnmpi_level](comm, keyval, attribute_val, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Attr_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Attr_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Attr_get(comm, keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Attr_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Attr_put(MPI_Comm comm, int keyval, void *attribute_val)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Attr_put_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Attr_put[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Attr_put at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Attr_put));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Attr_put)[pnmpi_level](comm, keyval, attribute_val);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Attr_put at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Attr_put");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Attr_put(comm, keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Attr_put");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Barrier(MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Barrier_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Barrier[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Barrier at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Barrier));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Barrier)[pnmpi_level](comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Barrier at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Barrier");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Barrier(comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Barrier");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Bcast_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Bcast[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Bcast at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Bcast));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Bcast)[pnmpi_level](buffer, count, datatype, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Bcast at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Bcast");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Bcast(buffer, count, datatype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Bcast");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Bsend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Bsend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Bsend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Bsend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Bsend)[pnmpi_level](buf, count, datatype, dest, tag, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Bsend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Bsend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Bsend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Bsend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Bsend_init_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Bsend_init[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Bsend_init at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Bsend_init));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Bsend_init)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Bsend_init at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Bsend_init");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Bsend_init");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Buffer_attach(void *buffer, int size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Buffer_attach_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Buffer_attach[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Buffer_attach at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Buffer_attach));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Buffer_attach)[pnmpi_level](buffer, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Buffer_attach at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Buffer_attach");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Buffer_attach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Buffer_attach");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Buffer_detach(void *buffer, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Buffer_detach_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Buffer_detach[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Buffer_detach at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Buffer_detach));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Buffer_detach)[pnmpi_level](buffer, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Buffer_detach at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Buffer_detach");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Buffer_detach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Buffer_detach");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cancel(MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cancel_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cancel[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cancel at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cancel));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cancel)[pnmpi_level](request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cancel at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cancel");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cancel(request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cancel");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_coords_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_coords[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_coords at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_coords));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_coords)[pnmpi_level](comm, rank, maxdims, coords);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_coords at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_coords");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_coords(comm, rank, maxdims, coords);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_coords");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_create)[pnmpi_level](old_comm, ndims, dims, periods, reorder, comm_cart);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_get)[pnmpi_level](comm, maxdims, dims, periods, coords);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_get(comm, maxdims, dims, periods, coords);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_map_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_map[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_map at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_map));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_map)[pnmpi_level](comm, ndims, dims, periods, newrank);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_map at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_map");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_map(comm, ndims, dims, periods, newrank);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_map");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_rank(MPI_Comm comm, const int coords[], int *rank)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_rank_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_rank[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_rank at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_rank));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_rank)[pnmpi_level](comm, coords, rank);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_rank at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_rank");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_rank(comm, coords, rank);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_rank");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_shift_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_shift[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_shift at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_shift));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_shift)[pnmpi_level](comm, direction, disp, rank_source, rank_dest);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_shift at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_shift");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_shift");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cart_sub_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cart_sub[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cart_sub at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cart_sub));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cart_sub)[pnmpi_level](comm, remain_dims, new_comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cart_sub at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cart_sub");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cart_sub(comm, remain_dims, new_comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cart_sub");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Cartdim_get(MPI_Comm comm, int *ndims)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Cartdim_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Cartdim_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Cartdim_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Cartdim_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Cartdim_get)[pnmpi_level](comm, ndims);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Cartdim_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Cartdim_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Cartdim_get(comm, ndims);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Cartdim_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Close_port(const char *port_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Close_port_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Close_port[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Close_port at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Close_port));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Close_port)[pnmpi_level](port_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Close_port at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Close_port");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Close_port(port_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Close_port");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_accept_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_accept[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_accept at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_accept));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_accept)[pnmpi_level](port_name, info, root, comm, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_accept at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_accept");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_accept(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_accept");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_call_errhandler(MPI_Comm comm, int errorcode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_call_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_call_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_call_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_call_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_call_errhandler)[pnmpi_level](comm, errorcode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_call_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_call_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_call_errhandler(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_call_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_compare_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_compare[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_compare at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_compare));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_compare)[pnmpi_level](comm1, comm2, result);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_compare at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_compare");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_compare(comm1, comm2, result);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_compare");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_connect_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_connect[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_connect at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_connect));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_connect)[pnmpi_level](port_name, info, root, comm, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_connect at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_connect");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_connect(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_connect");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create)[pnmpi_level](comm, group, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_create(comm, group, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_create_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_create_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_create_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_errhandler)[pnmpi_level](function, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_create_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_create_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_create_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_create_group_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_group[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_create_group at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_create_group));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_group)[pnmpi_level](comm, group, tag, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_create_group at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_create_group");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_create_group(comm, group, tag, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_create_group");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_create_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_create_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_create_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_create_keyval)[pnmpi_level](comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_create_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_create_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_create_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_delete_attr(MPI_Comm comm, int comm_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_delete_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_delete_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_delete_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_delete_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_delete_attr)[pnmpi_level](comm, comm_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_delete_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_delete_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_delete_attr(comm, comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_delete_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_disconnect(MPI_Comm *comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_disconnect_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_disconnect[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_disconnect at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_disconnect));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_disconnect)[pnmpi_level](comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_disconnect at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_disconnect");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_disconnect(comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_disconnect");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_dup_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_dup[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_dup at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_dup));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_dup)[pnmpi_level](comm, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_dup at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_dup");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_dup(comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_dup");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_dup_with_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_dup_with_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_dup_with_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_dup_with_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_dup_with_info)[pnmpi_level](comm, info, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_dup_with_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_dup_with_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_dup_with_info(comm, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_dup_with_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_free(MPI_Comm *comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_free)[pnmpi_level](comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_free(comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_free_keyval(int *comm_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_free_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_free_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_free_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_free_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_free_keyval)[pnmpi_level](comm_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_free_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_free_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_free_keyval(comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_free_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_get_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_get_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_get_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_attr)[pnmpi_level](comm, comm_keyval, attribute_val, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_get_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_get_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_get_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_get_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_get_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_get_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_errhandler)[pnmpi_level](comm, erhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_get_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_get_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_get_errhandler(comm, erhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_get_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_get_info(MPI_Comm comm, MPI_Info *info_used)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_get_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_get_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_get_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_info)[pnmpi_level](comm, info_used);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_get_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_get_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_get_info(comm, info_used);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_get_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_get_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_get_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_get_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_name)[pnmpi_level](comm, comm_name, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_get_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_get_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_get_name(comm, comm_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_get_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_get_parent(MPI_Comm *parent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_get_parent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_parent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_get_parent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_get_parent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_get_parent)[pnmpi_level](parent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_get_parent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_get_parent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_get_parent(parent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_get_parent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_group(MPI_Comm comm, MPI_Group *group)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_group_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_group[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_group at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_group));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_group)[pnmpi_level](comm, group);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_group at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_group");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_group");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_idup_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_idup[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_idup at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_idup));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_idup)[pnmpi_level](comm, newcomm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_idup at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_idup");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_idup(comm, newcomm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_idup");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_join(int fd, MPI_Comm *intercomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_join_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_join[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_join at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_join));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_join)[pnmpi_level](fd, intercomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_join at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_join");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_join(fd, intercomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_join");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_rank(MPI_Comm comm, int *rank)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_rank_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_rank[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_rank at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_rank));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_rank)[pnmpi_level](comm, rank);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_rank at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_rank");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_rank(comm, rank);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_rank");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_remote_group(MPI_Comm comm, MPI_Group *group)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_remote_group_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_remote_group[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_remote_group at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_remote_group));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_remote_group)[pnmpi_level](comm, group);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_remote_group at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_remote_group");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_remote_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_remote_group");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_remote_size(MPI_Comm comm, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_remote_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_remote_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_remote_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_remote_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_remote_size)[pnmpi_level](comm, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_remote_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_remote_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_remote_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_remote_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_set_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_set_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_set_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_attr)[pnmpi_level](comm, comm_keyval, attribute_val);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_set_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_set_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_set_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_set_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_set_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_set_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_errhandler)[pnmpi_level](comm, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_set_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_set_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_set_errhandler(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_set_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_set_info(MPI_Comm comm, MPI_Info info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_set_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_set_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_set_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_info)[pnmpi_level](comm, info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_set_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_set_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_set_info(comm, info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_set_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_set_name(MPI_Comm comm, const char *comm_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_set_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_set_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_set_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_set_name)[pnmpi_level](comm, comm_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_set_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_set_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_set_name(comm, comm_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_set_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_size(MPI_Comm comm, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_size)[pnmpi_level](comm, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_split_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_split[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_split at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_split));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_split)[pnmpi_level](comm, color, key, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_split at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_split");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_split(comm, color, key, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_split");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_split_type_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_split_type[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_split_type at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_split_type));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_split_type)[pnmpi_level](comm, split_type, key, info, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_split_type at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_split_type");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_split_type(comm, split_type, key, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_split_type");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Comm_test_inter(MPI_Comm comm, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Comm_test_inter_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Comm_test_inter[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Comm_test_inter at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Comm_test_inter));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Comm_test_inter)[pnmpi_level](comm, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Comm_test_inter at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Comm_test_inter");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Comm_test_inter(comm, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Comm_test_inter");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Compare_and_swap_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Compare_and_swap[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Compare_and_swap at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Compare_and_swap));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Compare_and_swap)[pnmpi_level](origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Compare_and_swap at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Compare_and_swap");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Compare_and_swap");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Dims_create(int nnodes, int ndims, int dims[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Dims_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Dims_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Dims_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Dims_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Dims_create)[pnmpi_level](nnodes, ndims, dims);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Dims_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Dims_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Dims_create(nnodes, ndims, dims);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Dims_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Dist_graph_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Dist_graph_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Dist_graph_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_create)[pnmpi_level](comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Dist_graph_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Dist_graph_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Dist_graph_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Dist_graph_create_adjacent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_create_adjacent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Dist_graph_create_adjacent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Dist_graph_create_adjacent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_create_adjacent)[pnmpi_level](comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Dist_graph_create_adjacent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Dist_graph_create_adjacent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Dist_graph_create_adjacent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Dist_graph_neighbors_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_neighbors[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Dist_graph_neighbors at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Dist_graph_neighbors));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_neighbors)[pnmpi_level](comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Dist_graph_neighbors at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Dist_graph_neighbors");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Dist_graph_neighbors");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Dist_graph_neighbors_count_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_neighbors_count[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Dist_graph_neighbors_count at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Dist_graph_neighbors_count));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Dist_graph_neighbors_count)[pnmpi_level](comm, inneighbors, outneighbors, weighted);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Dist_graph_neighbors_count at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Dist_graph_neighbors_count");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Dist_graph_neighbors_count");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Errhandler_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Errhandler_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Errhandler_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_create)[pnmpi_level](function, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Errhandler_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Errhandler_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Errhandler_create(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Errhandler_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Errhandler_free(MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Errhandler_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Errhandler_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Errhandler_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_free)[pnmpi_level](errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Errhandler_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Errhandler_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Errhandler_free(errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Errhandler_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Errhandler_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Errhandler_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Errhandler_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_get)[pnmpi_level](comm, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Errhandler_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Errhandler_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Errhandler_get(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Errhandler_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Errhandler_set_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_set[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Errhandler_set at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Errhandler_set));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Errhandler_set)[pnmpi_level](comm, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Errhandler_set at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Errhandler_set");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Errhandler_set(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Errhandler_set");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Error_class(int errorcode, int *errorclass)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Error_class_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Error_class[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Error_class at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Error_class));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Error_class)[pnmpi_level](errorcode, errorclass);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Error_class at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Error_class");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Error_class(errorcode, errorclass);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Error_class");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Error_string(int errorcode, char *string, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Error_string_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Error_string[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Error_string at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Error_string));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Error_string)[pnmpi_level](errorcode, string, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Error_string at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Error_string");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Error_string(errorcode, string, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Error_string");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Exscan_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Exscan[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Exscan at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Exscan));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Exscan)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Exscan at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Exscan");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Exscan");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Fetch_and_op_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Fetch_and_op[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Fetch_and_op at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Fetch_and_op));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Fetch_and_op)[pnmpi_level](origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Fetch_and_op at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Fetch_and_op");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Fetch_and_op");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_call_errhandler(MPI_File fh, int errorcode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_call_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_call_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_call_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_call_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_call_errhandler)[pnmpi_level](fh, errorcode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_call_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_call_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_call_errhandler(fh, errorcode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_call_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_close(MPI_File *fh)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_close_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_close[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_close at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_close));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_close)[pnmpi_level](fh);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_close at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_close");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_close(fh);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_close");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_create_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_create_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_create_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_create_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_create_errhandler)[pnmpi_level](function, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_create_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_create_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_create_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_delete(const char *filename, MPI_Info info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_delete_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_delete[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_delete at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_delete));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_delete)[pnmpi_level](filename, info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_delete at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_delete");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_delete(filename, info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_delete");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_amode(MPI_File fh, int *amode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_amode_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_amode[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_amode at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_amode));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_amode)[pnmpi_level](fh, amode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_amode at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_amode");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_amode(fh, amode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_amode");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_atomicity(MPI_File fh, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_atomicity_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_atomicity[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_atomicity at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_atomicity));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_atomicity)[pnmpi_level](fh, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_atomicity at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_atomicity");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_atomicity");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_byte_offset_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_byte_offset[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_byte_offset at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_byte_offset));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_byte_offset)[pnmpi_level](fh, offset, disp);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_byte_offset at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_byte_offset");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_byte_offset(fh, offset, disp);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_byte_offset");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_errhandler)[pnmpi_level](file, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_group(MPI_File fh, MPI_Group *group)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_group_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_group[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_group at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_group));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_group)[pnmpi_level](fh, group);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_group at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_group");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_group(fh, group);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_group");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_info(MPI_File fh, MPI_Info *info_used)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_info)[pnmpi_level](fh, info_used);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_info(fh, info_used);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_position(MPI_File fh, MPI_Offset *offset)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_position_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_position[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_position at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_position));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_position)[pnmpi_level](fh, offset);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_position at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_position");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_position(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_position");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_position_shared(MPI_File fh, MPI_Offset *offset)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_position_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_position_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_position_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_position_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_position_shared)[pnmpi_level](fh, offset);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_position_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_position_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_position_shared(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_position_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_size(MPI_File fh, MPI_Offset *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_size)[pnmpi_level](fh, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_type_extent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_type_extent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_type_extent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_type_extent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_type_extent)[pnmpi_level](fh, datatype, extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_type_extent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_type_extent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_type_extent(fh, datatype, extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_type_extent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_get_view_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_get_view[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_get_view at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_get_view));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_get_view)[pnmpi_level](fh, disp, etype, filetype, datarep);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_get_view at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_get_view");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_get_view(fh, disp, etype, filetype, datarep);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_get_view");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iread_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iread[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iread at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iread));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iread)[pnmpi_level](fh, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iread at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iread");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iread(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iread");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iread_at_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iread_at[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iread_at at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iread_at));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iread_at)[pnmpi_level](fh, offset, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iread_at at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iread_at");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iread_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iread_at");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iread_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iread_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iread_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iread_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iread_shared)[pnmpi_level](fh, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iread_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iread_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iread_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iread_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iwrite_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iwrite at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iwrite));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite)[pnmpi_level](fh, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iwrite at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iwrite");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iwrite(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iwrite");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iwrite_at_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite_at[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iwrite_at at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iwrite_at));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite_at)[pnmpi_level](fh, offset, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iwrite_at at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iwrite_at");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iwrite_at");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_iwrite_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_iwrite_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_iwrite_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_iwrite_shared)[pnmpi_level](fh, buf, count, datatype, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_iwrite_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_iwrite_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_iwrite_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_iwrite_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_open_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_open[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_open at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_open));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_open)[pnmpi_level](comm, filename, amode, info, fh);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_open at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_open");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_open(comm, filename, amode, info, fh);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_open");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_preallocate(MPI_File fh, MPI_Offset size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_preallocate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_preallocate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_preallocate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_preallocate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_preallocate)[pnmpi_level](fh, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_preallocate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_preallocate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_preallocate(fh, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_preallocate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_all_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_all_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_all_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all_begin)[pnmpi_level](fh, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_all_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_all_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_all_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_all_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_all_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_all_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_all_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_all_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_all_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_all_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_at_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_at at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_at));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at)[pnmpi_level](fh, offset, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_at at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_at");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_at");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_at_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_at_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_at_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all)[pnmpi_level](fh, offset, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_at_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_at_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_at_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_at_all_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_at_all_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_at_all_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all_begin)[pnmpi_level](fh, offset, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_at_all_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_at_all_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_at_all_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_at_all_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_at_all_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_at_all_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_at_all_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_at_all_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_at_all_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_at_all_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_ordered_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_ordered at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_ordered));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_ordered at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_ordered");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_ordered");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_ordered_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_ordered_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_ordered_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered_begin)[pnmpi_level](fh, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_ordered_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_ordered_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_ordered_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_ordered_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_ordered_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_ordered_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_ordered_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_ordered_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_ordered_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_ordered_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_read_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_read_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_read_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_read_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_read_shared)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_read_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_read_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_read_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_read_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_seek(MPI_File fh, MPI_Offset offset, int whence)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_seek_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_seek[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_seek at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_seek));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_seek)[pnmpi_level](fh, offset, whence);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_seek at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_seek");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_seek(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_seek");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_seek_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_seek_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_seek_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_seek_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_seek_shared)[pnmpi_level](fh, offset, whence);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_seek_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_seek_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_seek_shared(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_seek_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_set_atomicity(MPI_File fh, int flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_set_atomicity_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_set_atomicity[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_set_atomicity at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_set_atomicity));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_set_atomicity)[pnmpi_level](fh, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_set_atomicity at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_set_atomicity");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_set_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_set_atomicity");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_set_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_set_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_set_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_set_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_set_errhandler)[pnmpi_level](file, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_set_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_set_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_set_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_set_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_set_info(MPI_File fh, MPI_Info info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_set_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_set_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_set_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_set_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_set_info)[pnmpi_level](fh, info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_set_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_set_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_set_info(fh, info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_set_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_set_size(MPI_File fh, MPI_Offset size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_set_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_set_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_set_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_set_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_set_size)[pnmpi_level](fh, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_set_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_set_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_set_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_set_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_set_view_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_set_view[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_set_view at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_set_view));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_set_view)[pnmpi_level](fh, disp, etype, filetype, datarep, info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_set_view at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_set_view");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_set_view");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_sync(MPI_File fh)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_sync_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_sync[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_sync at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_sync));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_sync)[pnmpi_level](fh);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_sync at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_sync");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_sync(fh);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_sync");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_all_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_all_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_all_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all_begin)[pnmpi_level](fh, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_all_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_all_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_all_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_all_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_all_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_all_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_all_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_all_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_all_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_all_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_at_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_at at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_at));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at)[pnmpi_level](fh, offset, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_at at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_at");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_at");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_at_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_at_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_at_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all)[pnmpi_level](fh, offset, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_at_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_at_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_at_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_at_all_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_at_all_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_at_all_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all_begin)[pnmpi_level](fh, offset, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_at_all_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_at_all_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_at_all_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_at_all_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_at_all_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_at_all_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_at_all_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_at_all_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_at_all_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_at_all_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_ordered_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_ordered at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_ordered));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_ordered at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_ordered");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_ordered");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_ordered_begin_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered_begin[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_ordered_begin at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_ordered_begin));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered_begin)[pnmpi_level](fh, buf, count, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_ordered_begin at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_ordered_begin");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_ordered_begin");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_ordered_end_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered_end[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_ordered_end at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_ordered_end));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_ordered_end)[pnmpi_level](fh, buf, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_ordered_end at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_ordered_end");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_ordered_end");
  pnmpi_level = start_level;
  return res;
}

int NQJ_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_File_write_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_File_write_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_File_write_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_File_write_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_File_write_shared)[pnmpi_level](fh, buf, count, datatype, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_File_write_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_File_write_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_File_write_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_File_write_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Finalized(int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Finalized_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Finalized[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Finalized at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Finalized));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Finalized)[pnmpi_level](flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Finalized at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Finalized");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Finalized(flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Finalized");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Free_mem(void *base)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Free_mem_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Free_mem[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Free_mem at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Free_mem));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Free_mem)[pnmpi_level](base);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Free_mem at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Free_mem");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Free_mem(base);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Free_mem");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Gather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Gather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Gather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Gather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Gather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Gather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Gather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Gather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Gatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Gatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Gatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Gatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Gatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Gatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Gatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Gatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_accumulate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_accumulate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_accumulate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_accumulate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_accumulate)[pnmpi_level](origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_accumulate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_accumulate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_accumulate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_address(const void *location, MPI_Aint *address)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_address_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_address[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_address at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_address));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_address)[pnmpi_level](location, address);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_address at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_address");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_address(location, address);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_address");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_count_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_count[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_count at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_count));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_count)[pnmpi_level](status, datatype, count);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_count at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_count");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_count(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_count");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_elements_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_elements[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_elements at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_elements));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_elements)[pnmpi_level](status, datatype, count);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_elements at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_elements");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_elements");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_elements_x_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_elements_x[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_elements_x at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_elements_x));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_elements_x)[pnmpi_level](status, datatype, count);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_elements_x at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_elements_x");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_elements_x");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_library_version(char *version, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_library_version_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_library_version[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_library_version at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_library_version));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_library_version)[pnmpi_level](version, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_library_version at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_library_version");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_library_version(version, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_library_version");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_processor_name(char *name, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_processor_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_processor_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_processor_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_processor_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_processor_name)[pnmpi_level](name, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_processor_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_processor_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_processor_name(name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_processor_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Get_version(int *version, int *subversion)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Get_version_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Get_version[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Get_version at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Get_version));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Get_version)[pnmpi_level](version, subversion);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Get_version at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Get_version");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Get_version(version, subversion);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Get_version");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graph_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graph_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graph_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graph_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graph_create)[pnmpi_level](comm_old, nnodes, index, edges, reorder, comm_graph);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graph_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graph_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graph_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graph_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graph_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graph_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graph_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graph_get)[pnmpi_level](comm, maxindex, maxedges, index, edges);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graph_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graph_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graph_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graph_map_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graph_map[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graph_map at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graph_map));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graph_map)[pnmpi_level](comm, nnodes, index, edges, newrank);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graph_map at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graph_map");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graph_map(comm, nnodes, index, edges, newrank);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graph_map");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graph_neighbors_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graph_neighbors[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graph_neighbors at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graph_neighbors));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graph_neighbors)[pnmpi_level](comm, rank, maxneighbors, neighbors);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graph_neighbors at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graph_neighbors");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graph_neighbors");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graph_neighbors_count_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graph_neighbors_count[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graph_neighbors_count at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graph_neighbors_count));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graph_neighbors_count)[pnmpi_level](comm, rank, nneighbors);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graph_neighbors_count at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graph_neighbors_count");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graph_neighbors_count(comm, rank, nneighbors);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graph_neighbors_count");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Graphdims_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Graphdims_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Graphdims_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Graphdims_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Graphdims_get)[pnmpi_level](comm, nnodes, nedges);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Graphdims_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Graphdims_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Graphdims_get(comm, nnodes, nedges);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Graphdims_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Grequest_complete(MPI_Request request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Grequest_complete_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Grequest_complete[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Grequest_complete at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Grequest_complete));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Grequest_complete)[pnmpi_level](request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Grequest_complete at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Grequest_complete");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Grequest_complete(request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Grequest_complete");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Grequest_start_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Grequest_start[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Grequest_start at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Grequest_start));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Grequest_start)[pnmpi_level](query_fn, free_fn, cancel_fn, extra_state, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Grequest_start at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Grequest_start");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Grequest_start");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_compare(MPI_Group group1, MPI_Group group2, int *result)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_compare_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_compare[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_compare at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_compare));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_compare)[pnmpi_level](group1, group2, result);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_compare at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_compare");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_compare(group1, group2, result);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_compare");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_difference_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_difference[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_difference at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_difference));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_difference)[pnmpi_level](group1, group2, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_difference at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_difference");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_difference(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_difference");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_excl_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_excl[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_excl at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_excl));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_excl)[pnmpi_level](group, n, ranks, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_excl at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_excl");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_excl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_excl");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_free(MPI_Group *group)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_free)[pnmpi_level](group);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_free(group);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_incl_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_incl[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_incl at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_incl));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_incl)[pnmpi_level](group, n, ranks, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_incl at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_incl");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_incl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_incl");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_intersection_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_intersection[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_intersection at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_intersection));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_intersection)[pnmpi_level](group1, group2, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_intersection at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_intersection");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_intersection(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_intersection");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_range_excl_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_range_excl[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_range_excl at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_range_excl));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_range_excl)[pnmpi_level](group, n, ranges, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_range_excl at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_range_excl");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_range_excl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_range_excl");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_range_incl_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_range_incl[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_range_incl at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_range_incl));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_range_incl)[pnmpi_level](group, n, ranges, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_range_incl at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_range_incl");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_range_incl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_range_incl");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_rank(MPI_Group group, int *rank)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_rank_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_rank[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_rank at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_rank));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_rank)[pnmpi_level](group, rank);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_rank at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_rank");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_rank(group, rank);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_rank");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_size(MPI_Group group, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_size)[pnmpi_level](group, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_size(group, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_translate_ranks_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_translate_ranks[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_translate_ranks at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_translate_ranks));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_translate_ranks)[pnmpi_level](group1, n, ranks1, group2, ranks2);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_translate_ranks at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_translate_ranks");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_translate_ranks");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Group_union_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Group_union[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Group_union at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Group_union));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Group_union)[pnmpi_level](group1, group2, newgroup);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Group_union at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Group_union");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Group_union(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Group_union");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iallgather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iallgather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iallgather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iallgather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iallgather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iallgather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iallgather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iallgather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iallgatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iallgatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iallgatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iallgatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iallgatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iallgatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iallgatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iallgatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iallreduce_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iallreduce[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iallreduce at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iallreduce));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iallreduce)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iallreduce at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iallreduce");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iallreduce");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ialltoall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ialltoall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ialltoall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoall)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ialltoall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ialltoall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ialltoall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ialltoallv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoallv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ialltoallv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ialltoallv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoallv)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ialltoallv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ialltoallv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ialltoallv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ialltoallw_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoallw[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ialltoallw at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ialltoallw));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ialltoallw)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ialltoallw at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ialltoallw");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ialltoallw");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ibarrier(MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ibarrier_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ibarrier[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ibarrier at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ibarrier));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ibarrier)[pnmpi_level](comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ibarrier at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ibarrier");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ibarrier(comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ibarrier");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ibcast_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ibcast[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ibcast at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ibcast));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ibcast)[pnmpi_level](buffer, count, datatype, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ibcast at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ibcast");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ibcast(buffer, count, datatype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ibcast");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ibsend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ibsend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ibsend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ibsend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ibsend)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ibsend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ibsend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ibsend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iexscan_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iexscan[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iexscan at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iexscan));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iexscan)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iexscan at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iexscan");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iexscan");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Igather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Igather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Igather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Igather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Igather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Igather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Igather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Igather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Igatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Igatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Igatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Igatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Igatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Igatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Igatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Igatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Improbe_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Improbe[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Improbe at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Improbe));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Improbe)[pnmpi_level](source, tag, comm, flag, message, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Improbe at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Improbe");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Improbe(source, tag, comm, flag, message, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Improbe");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Imrecv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Imrecv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Imrecv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Imrecv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Imrecv)[pnmpi_level](buf, count, type, message, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Imrecv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Imrecv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Imrecv(buf, count, type, message, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Imrecv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ineighbor_allgather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_allgather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ineighbor_allgather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ineighbor_allgather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_allgather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ineighbor_allgather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ineighbor_allgather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ineighbor_allgather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ineighbor_allgatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_allgatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ineighbor_allgatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ineighbor_allgatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_allgatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ineighbor_allgatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ineighbor_allgatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ineighbor_allgatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ineighbor_alltoall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ineighbor_alltoall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ineighbor_alltoall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoall)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ineighbor_alltoall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ineighbor_alltoall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ineighbor_alltoall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ineighbor_alltoallv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoallv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ineighbor_alltoallv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ineighbor_alltoallv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoallv)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ineighbor_alltoallv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ineighbor_alltoallv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ineighbor_alltoallv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ineighbor_alltoallw_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoallw[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ineighbor_alltoallw at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ineighbor_alltoallw));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ineighbor_alltoallw)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ineighbor_alltoallw at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ineighbor_alltoallw");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ineighbor_alltoallw");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_create(MPI_Info *info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_create)[pnmpi_level](info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_create(info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_delete(MPI_Info info, const char *key)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_delete_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_delete[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_delete at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_delete));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_delete)[pnmpi_level](info, key);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_delete at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_delete");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_delete(info, key);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_delete");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_dup(MPI_Info info, MPI_Info *newinfo)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_dup_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_dup[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_dup at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_dup));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_dup)[pnmpi_level](info, newinfo);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_dup at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_dup");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_dup(info, newinfo);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_dup");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_free(MPI_Info *info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_free)[pnmpi_level](info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_free(info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_get_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_get[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_get at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_get));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_get)[pnmpi_level](info, key, valuelen, value, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_get at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_get");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_get(info, key, valuelen, value, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_get");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_get_nkeys(MPI_Info info, int *nkeys)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_get_nkeys_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_nkeys[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_get_nkeys at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_get_nkeys));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_nkeys)[pnmpi_level](info, nkeys);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_get_nkeys at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_get_nkeys");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_get_nkeys(info, nkeys);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_get_nkeys");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_get_nthkey(MPI_Info info, int n, char *key)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_get_nthkey_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_nthkey[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_get_nthkey at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_get_nthkey));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_nthkey)[pnmpi_level](info, n, key);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_get_nthkey at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_get_nthkey");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_get_nthkey(info, n, key);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_get_nthkey");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_get_valuelen_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_valuelen[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_get_valuelen at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_get_valuelen));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_get_valuelen)[pnmpi_level](info, key, valuelen, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_get_valuelen at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_get_valuelen");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_get_valuelen(info, key, valuelen, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_get_valuelen");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Info_set(MPI_Info info, const char *key, const char *value)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Info_set_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Info_set[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Info_set at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Info_set));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Info_set)[pnmpi_level](info, key, value);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Info_set at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Info_set");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Info_set(info, key, value);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Info_set");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Initialized(int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Initialized_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Initialized[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Initialized at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Initialized));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Initialized)[pnmpi_level](flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Initialized at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Initialized");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Initialized(flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Initialized");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Intercomm_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Intercomm_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Intercomm_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Intercomm_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Intercomm_create)[pnmpi_level](local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Intercomm_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Intercomm_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Intercomm_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Intercomm_merge_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Intercomm_merge[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Intercomm_merge at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Intercomm_merge));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Intercomm_merge)[pnmpi_level](intercomm, high, newintercomm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Intercomm_merge at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Intercomm_merge");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Intercomm_merge(intercomm, high, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Intercomm_merge");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iprobe_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iprobe[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iprobe at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iprobe));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iprobe)[pnmpi_level](source, tag, comm, flag, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iprobe at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iprobe");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iprobe(source, tag, comm, flag, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iprobe");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Irecv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Irecv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Irecv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Irecv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Irecv)[pnmpi_level](buf, count, datatype, source, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Irecv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Irecv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Irecv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ireduce_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ireduce at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ireduce));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ireduce at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ireduce");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ireduce");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ireduce_scatter_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce_scatter[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ireduce_scatter at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ireduce_scatter));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce_scatter)[pnmpi_level](sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ireduce_scatter at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ireduce_scatter");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ireduce_scatter");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ireduce_scatter_block_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce_scatter_block[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ireduce_scatter_block at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ireduce_scatter_block));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ireduce_scatter_block)[pnmpi_level](sendbuf, recvbuf, recvcount, datatype, op, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ireduce_scatter_block at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ireduce_scatter_block");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ireduce_scatter_block");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Irsend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Irsend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Irsend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Irsend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Irsend)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Irsend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Irsend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Irsend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Is_thread_main(int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Is_thread_main_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Is_thread_main[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Is_thread_main at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Is_thread_main));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Is_thread_main)[pnmpi_level](flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Is_thread_main at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Is_thread_main");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Is_thread_main(flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Is_thread_main");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iscan_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iscan[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iscan at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iscan));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iscan)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iscan at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iscan");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iscan");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iscatter_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iscatter[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iscatter at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iscatter));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iscatter)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iscatter at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iscatter");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iscatter");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Iscatterv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Iscatterv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Iscatterv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Iscatterv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Iscatterv)[pnmpi_level](sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Iscatterv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Iscatterv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Iscatterv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Isend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Isend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Isend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Isend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Isend)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Isend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Isend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Isend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Issend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Issend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Issend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Issend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Issend)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Issend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Issend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Issend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Keyval_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Keyval_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Keyval_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Keyval_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Keyval_create)[pnmpi_level](copy_fn, delete_fn, keyval, extra_state);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Keyval_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Keyval_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Keyval_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Keyval_free(int *keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Keyval_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Keyval_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Keyval_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Keyval_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Keyval_free)[pnmpi_level](keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Keyval_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Keyval_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Keyval_free(keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Keyval_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Lookup_name(const char *service_name, MPI_Info info, char *port_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Lookup_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Lookup_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Lookup_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Lookup_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Lookup_name)[pnmpi_level](service_name, info, port_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Lookup_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Lookup_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Lookup_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Lookup_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Mprobe_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Mprobe[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Mprobe at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Mprobe));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Mprobe)[pnmpi_level](source, tag, comm, message, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Mprobe at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Mprobe");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Mprobe(source, tag, comm, message, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Mprobe");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Mrecv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Mrecv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Mrecv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Mrecv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Mrecv)[pnmpi_level](buf, count, type, message, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Mrecv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Mrecv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Mrecv(buf, count, type, message, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Mrecv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Neighbor_allgather_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_allgather[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Neighbor_allgather at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Neighbor_allgather));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_allgather)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Neighbor_allgather at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Neighbor_allgather");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Neighbor_allgather");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Neighbor_allgatherv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_allgatherv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Neighbor_allgatherv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Neighbor_allgatherv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_allgatherv)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Neighbor_allgatherv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Neighbor_allgatherv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Neighbor_allgatherv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Neighbor_alltoall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Neighbor_alltoall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Neighbor_alltoall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoall)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Neighbor_alltoall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Neighbor_alltoall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Neighbor_alltoall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Neighbor_alltoallv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoallv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Neighbor_alltoallv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Neighbor_alltoallv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoallv)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Neighbor_alltoallv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Neighbor_alltoallv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Neighbor_alltoallv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Neighbor_alltoallw_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoallw[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Neighbor_alltoallw at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Neighbor_alltoallw));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Neighbor_alltoallw)[pnmpi_level](sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Neighbor_alltoallw at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Neighbor_alltoallw");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Neighbor_alltoallw");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Op_commutative(MPI_Op op, int *commute)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Op_commutative_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Op_commutative[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Op_commutative at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Op_commutative));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Op_commutative)[pnmpi_level](op, commute);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Op_commutative at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Op_commutative");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Op_commutative(op, commute);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Op_commutative");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Op_create(MPI_User_function *function, int commute, MPI_Op *op)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Op_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Op_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Op_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Op_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Op_create)[pnmpi_level](function, commute, op);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Op_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Op_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Op_create(function, commute, op);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Op_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Op_free(MPI_Op *op)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Op_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Op_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Op_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Op_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Op_free)[pnmpi_level](op);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Op_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Op_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Op_free(op);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Op_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Open_port(MPI_Info info, char *port_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Open_port_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Open_port[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Open_port at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Open_port));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Open_port)[pnmpi_level](info, port_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Open_port at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Open_port");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Open_port(info, port_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Open_port");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Pack_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Pack[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Pack at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Pack));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Pack)[pnmpi_level](inbuf, incount, datatype, outbuf, outsize, position, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Pack at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Pack");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Pack");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Pack_external_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Pack_external[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Pack_external at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Pack_external));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Pack_external)[pnmpi_level](datarep, inbuf, incount, datatype, outbuf, outsize, position);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Pack_external at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Pack_external");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Pack_external");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Pack_external_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Pack_external_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Pack_external_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Pack_external_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Pack_external_size)[pnmpi_level](datarep, incount, datatype, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Pack_external_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Pack_external_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Pack_external_size(datarep, incount, datatype, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Pack_external_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Pack_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Pack_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Pack_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Pack_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Pack_size)[pnmpi_level](incount, datatype, comm, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Pack_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Pack_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Pack_size(incount, datatype, comm, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Pack_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Probe_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Probe[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Probe at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Probe));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Probe)[pnmpi_level](source, tag, comm, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Probe at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Probe");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Probe(source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Probe");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Publish_name(const char *service_name, MPI_Info info, const char *port_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Publish_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Publish_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Publish_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Publish_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Publish_name)[pnmpi_level](service_name, info, port_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Publish_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Publish_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Publish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Publish_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Put_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Put[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Put at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Put));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Put)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Put at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Put");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Put");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Query_thread(int *provided)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Query_thread_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Query_thread[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Query_thread at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Query_thread));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Query_thread)[pnmpi_level](provided);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Query_thread at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Query_thread");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Query_thread(provided);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Query_thread");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Raccumulate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Raccumulate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Raccumulate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Raccumulate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Raccumulate)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Raccumulate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Raccumulate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Raccumulate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Recv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Recv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Recv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Recv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Recv)[pnmpi_level](buf, count, datatype, source, tag, comm, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Recv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Recv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Recv(buf, count, datatype, source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Recv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Recv_init_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Recv_init[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Recv_init at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Recv_init));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Recv_init)[pnmpi_level](buf, count, datatype, source, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Recv_init at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Recv_init");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Recv_init");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Reduce_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Reduce[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Reduce at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Reduce));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Reduce)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Reduce at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Reduce");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Reduce");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Reduce_local_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_local[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Reduce_local at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Reduce_local));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_local)[pnmpi_level](inbuf, inoutbuf, count, datatype, op);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Reduce_local at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Reduce_local");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Reduce_local");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Reduce_scatter_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_scatter[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Reduce_scatter at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Reduce_scatter));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_scatter)[pnmpi_level](sendbuf, recvbuf, recvcounts, datatype, op, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Reduce_scatter at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Reduce_scatter");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Reduce_scatter");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Reduce_scatter_block_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_scatter_block[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Reduce_scatter_block at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Reduce_scatter_block));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Reduce_scatter_block)[pnmpi_level](sendbuf, recvbuf, recvcount, datatype, op, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Reduce_scatter_block at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Reduce_scatter_block");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Reduce_scatter_block");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Register_datarep_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Register_datarep[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Register_datarep at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Register_datarep));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Register_datarep)[pnmpi_level](datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Register_datarep at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Register_datarep");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Register_datarep");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Request_free(MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Request_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Request_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Request_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Request_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Request_free)[pnmpi_level](request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Request_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Request_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Request_free(request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Request_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Request_get_status(MPI_Request request, int *flag, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Request_get_status_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Request_get_status[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Request_get_status at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Request_get_status));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Request_get_status)[pnmpi_level](request, flag, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Request_get_status at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Request_get_status");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Request_get_status(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Request_get_status");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Rget_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Rget[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Rget at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Rget));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Rget)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Rget at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Rget");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Rget");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Rget_accumulate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Rget_accumulate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Rget_accumulate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Rget_accumulate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Rget_accumulate)[pnmpi_level](origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Rget_accumulate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Rget_accumulate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Rget_accumulate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Rput_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Rput[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Rput at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Rput));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Rput)[pnmpi_level](origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Rput at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Rput");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Rput");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Rsend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Rsend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Rsend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Rsend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Rsend)[pnmpi_level](ibuf, count, datatype, dest, tag, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Rsend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Rsend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Rsend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Rsend_init_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Rsend_init[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Rsend_init at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Rsend_init));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Rsend_init)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Rsend_init at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Rsend_init");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Rsend_init");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Scan_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Scan[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Scan at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Scan));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Scan)[pnmpi_level](sendbuf, recvbuf, count, datatype, op, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Scan at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Scan");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Scan");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Scatter_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Scatter[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Scatter at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Scatter));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Scatter)[pnmpi_level](sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Scatter at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Scatter");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Scatter");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Scatterv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Scatterv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Scatterv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Scatterv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Scatterv)[pnmpi_level](sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Scatterv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Scatterv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Scatterv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Send_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Send[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Send at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Send));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Send)[pnmpi_level](buf, count, datatype, dest, tag, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Send at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Send");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Send(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Send");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Send_init_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Send_init[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Send_init at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Send_init));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Send_init)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Send_init at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Send_init");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Send_init");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Sendrecv_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Sendrecv[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Sendrecv at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Sendrecv));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Sendrecv)[pnmpi_level](sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Sendrecv at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Sendrecv");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Sendrecv");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Sendrecv_replace_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Sendrecv_replace[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Sendrecv_replace at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Sendrecv_replace));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Sendrecv_replace)[pnmpi_level](buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Sendrecv_replace at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Sendrecv_replace");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Sendrecv_replace");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ssend_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ssend[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ssend at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ssend));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ssend)[pnmpi_level](buf, count, datatype, dest, tag, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ssend at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ssend");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ssend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ssend");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Ssend_init_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Ssend_init[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Ssend_init at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Ssend_init));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Ssend_init)[pnmpi_level](buf, count, datatype, dest, tag, comm, request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Ssend_init at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Ssend_init");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Ssend_init");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Start(MPI_Request *request)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Start_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Start[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Start at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Start));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Start)[pnmpi_level](request);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Start at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Start");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Start(request);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Start");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Startall(int count, MPI_Request array_of_requests[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Startall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Startall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Startall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Startall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Startall)[pnmpi_level](count, array_of_requests);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Startall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Startall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Startall(count, array_of_requests);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Startall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Status_set_cancelled(MPI_Status *status, int flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Status_set_cancelled_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_cancelled[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Status_set_cancelled at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Status_set_cancelled));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_cancelled)[pnmpi_level](status, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Status_set_cancelled at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Status_set_cancelled");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Status_set_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Status_set_cancelled");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Status_set_elements_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_elements[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Status_set_elements at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Status_set_elements));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_elements)[pnmpi_level](status, datatype, count);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Status_set_elements at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Status_set_elements");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Status_set_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Status_set_elements");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Status_set_elements_x_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_elements_x[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Status_set_elements_x at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Status_set_elements_x));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Status_set_elements_x)[pnmpi_level](status, datatype, count);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Status_set_elements_x at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Status_set_elements_x");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Status_set_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Status_set_elements_x");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Test(MPI_Request *request, int *flag, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Test_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Test[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Test at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Test));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Test)[pnmpi_level](request, flag, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Test at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Test");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Test(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Test");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Test_cancelled(const MPI_Status *status, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Test_cancelled_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Test_cancelled[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Test_cancelled at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Test_cancelled));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Test_cancelled)[pnmpi_level](status, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Test_cancelled at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Test_cancelled");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Test_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Test_cancelled");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Testall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Testall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Testall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Testall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Testall)[pnmpi_level](count, array_of_requests, flag, array_of_statuses);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Testall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Testall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Testall(count, array_of_requests, flag, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Testall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Testany_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Testany[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Testany at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Testany));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Testany)[pnmpi_level](count, array_of_requests, index, flag, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Testany at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Testany");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Testany(count, array_of_requests, index, flag, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Testany");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Testsome_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Testsome[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Testsome at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Testsome));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Testsome)[pnmpi_level](incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Testsome at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Testsome");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Testsome");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Topo_test(MPI_Comm comm, int *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Topo_test_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Topo_test[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Topo_test at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Topo_test));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Topo_test)[pnmpi_level](comm, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Topo_test at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Topo_test");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Topo_test(comm, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Topo_test");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_commit(MPI_Datatype *type)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_commit_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_commit[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_commit at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_commit));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_commit)[pnmpi_level](type);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_commit at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_commit");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_commit(type);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_commit");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_contiguous_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_contiguous[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_contiguous at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_contiguous));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_contiguous)[pnmpi_level](count, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_contiguous at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_contiguous");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_contiguous(count, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_contiguous");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_darray_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_darray[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_darray at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_darray));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_darray)[pnmpi_level](size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_darray at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_darray");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_darray");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_f90_complex_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_complex[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_f90_complex at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_f90_complex));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_complex)[pnmpi_level](p, r, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_f90_complex at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_f90_complex");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_f90_complex(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_f90_complex");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_f90_integer(int r, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_f90_integer_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_integer[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_f90_integer at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_f90_integer));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_integer)[pnmpi_level](r, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_f90_integer at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_f90_integer");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_f90_integer(r, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_f90_integer");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_f90_real_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_real[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_f90_real at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_f90_real));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_f90_real)[pnmpi_level](p, r, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_f90_real at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_f90_real");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_f90_real(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_f90_real");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_hindexed_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hindexed[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_hindexed at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_hindexed));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hindexed)[pnmpi_level](count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_hindexed at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_hindexed");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_hindexed");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_hindexed_block_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hindexed_block[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_hindexed_block at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_hindexed_block));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hindexed_block)[pnmpi_level](count, blocklength, array_of_displacements, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_hindexed_block at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_hindexed_block");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_hindexed_block");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_hvector_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hvector[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_hvector at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_hvector));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_hvector)[pnmpi_level](count, blocklength, stride, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_hvector at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_hvector");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_hvector");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_indexed_block_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_indexed_block[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_indexed_block at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_indexed_block));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_indexed_block)[pnmpi_level](count, blocklength, array_of_displacements, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_indexed_block at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_indexed_block");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_indexed_block");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_keyval)[pnmpi_level](type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_resized_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_resized[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_resized at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_resized));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_resized)[pnmpi_level](oldtype, lb, extent, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_resized at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_resized");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_resized(oldtype, lb, extent, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_resized");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_struct_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_struct[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_struct at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_struct));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_struct)[pnmpi_level](count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_struct at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_struct");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_struct");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_create_subarray_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_subarray[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_create_subarray at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_create_subarray));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_create_subarray)[pnmpi_level](ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_create_subarray at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_create_subarray");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_create_subarray");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_delete_attr(MPI_Datatype type, int type_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_delete_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_delete_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_delete_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_delete_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_delete_attr)[pnmpi_level](type, type_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_delete_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_delete_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_delete_attr(type, type_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_delete_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_dup(MPI_Datatype type, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_dup_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_dup[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_dup at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_dup));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_dup)[pnmpi_level](type, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_dup at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_dup");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_dup(type, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_dup");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_extent(MPI_Datatype type, MPI_Aint *extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_extent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_extent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_extent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_extent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_extent)[pnmpi_level](type, extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_extent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_extent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_extent(type, extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_extent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_free(MPI_Datatype *type)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_free)[pnmpi_level](type);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_free(type);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_free_keyval(int *type_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_free_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_free_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_free_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_free_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_free_keyval)[pnmpi_level](type_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_free_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_free_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_free_keyval(type_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_free_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_attr)[pnmpi_level](type, type_keyval, attribute_val, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_contents_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_contents[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_contents at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_contents));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_contents)[pnmpi_level](mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_contents at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_contents");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_contents");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_envelope_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_envelope[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_envelope at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_envelope));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_envelope)[pnmpi_level](type, num_integers, num_addresses, num_datatypes, combiner);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_envelope at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_envelope");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_envelope");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_extent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_extent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_extent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_extent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_extent)[pnmpi_level](type, lb, extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_extent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_extent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_extent(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_extent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_extent_x_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_extent_x[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_extent_x at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_extent_x));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_extent_x)[pnmpi_level](type, lb, extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_extent_x at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_extent_x");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_extent_x(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_extent_x");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_name)[pnmpi_level](type, type_name, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_name(type, type_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_true_extent_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_true_extent[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_true_extent at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_true_extent));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_true_extent)[pnmpi_level](datatype, true_lb, true_extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_true_extent at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_true_extent");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_true_extent(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_true_extent");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_get_true_extent_x_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_true_extent_x[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_get_true_extent_x at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_get_true_extent_x));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_get_true_extent_x)[pnmpi_level](datatype, true_lb, true_extent);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_get_true_extent_x at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_get_true_extent_x");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_get_true_extent_x");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_hindexed_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_hindexed[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_hindexed at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_hindexed));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_hindexed)[pnmpi_level](count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_hindexed at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_hindexed");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_hindexed");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_hvector_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_hvector[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_hvector at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_hvector));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_hvector)[pnmpi_level](count, blocklength, stride, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_hvector at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_hvector");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_hvector");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_indexed_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_indexed[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_indexed at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_indexed));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_indexed)[pnmpi_level](count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_indexed at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_indexed");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_indexed");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_lb(MPI_Datatype type, MPI_Aint *lb)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_lb_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_lb[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_lb at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_lb));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_lb)[pnmpi_level](type, lb);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_lb at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_lb");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_lb(type, lb);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_lb");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_match_size(int typeclass, int size, MPI_Datatype *type)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_match_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_match_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_match_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_match_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_match_size)[pnmpi_level](typeclass, size, type);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_match_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_match_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_match_size(typeclass, size, type);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_match_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_set_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_set_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_set_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_set_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_set_attr)[pnmpi_level](type, type_keyval, attr_val);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_set_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_set_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_set_attr(type, type_keyval, attr_val);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_set_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_set_name(MPI_Datatype type, const char *type_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_set_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_set_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_set_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_set_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_set_name)[pnmpi_level](type, type_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_set_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_set_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_set_name(type, type_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_set_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_size(MPI_Datatype type, int *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_size_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_size[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_size at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_size));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_size)[pnmpi_level](type, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_size at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_size");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_size(type, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_size");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_size_x(MPI_Datatype type, MPI_Count *size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_size_x_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_size_x[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_size_x at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_size_x));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_size_x)[pnmpi_level](type, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_size_x at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_size_x");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_size_x(type, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_size_x");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_struct_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_struct[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_struct at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_struct));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_struct)[pnmpi_level](count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_struct at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_struct");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_struct");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_ub(MPI_Datatype mtype, MPI_Aint *ub)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_ub_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_ub[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_ub at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_ub));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_ub)[pnmpi_level](mtype, ub);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_ub at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_ub");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_ub(mtype, ub);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_ub");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Type_vector_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Type_vector[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Type_vector at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Type_vector));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Type_vector)[pnmpi_level](count, blocklength, stride, oldtype, newtype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Type_vector at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Type_vector");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Type_vector");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Unpack_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Unpack[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Unpack at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Unpack));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Unpack)[pnmpi_level](inbuf, insize, position, outbuf, outcount, datatype, comm);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Unpack at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Unpack");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Unpack");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Unpack_external_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Unpack_external[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Unpack_external at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Unpack_external));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Unpack_external)[pnmpi_level](datarep, inbuf, insize, position, outbuf, outcount, datatype);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Unpack_external at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Unpack_external");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Unpack_external");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Unpublish_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Unpublish_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Unpublish_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Unpublish_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Unpublish_name)[pnmpi_level](service_name, info, port_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Unpublish_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Unpublish_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Unpublish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Unpublish_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Wait(MPI_Request *request, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Wait_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Wait[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Wait at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Wait));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Wait)[pnmpi_level](request, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Wait at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Wait");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Wait(request, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Wait");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Waitall_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Waitall[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Waitall at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Waitall));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Waitall)[pnmpi_level](count, array_of_requests, array_of_statuses);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Waitall at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Waitall");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Waitall(count, array_of_requests, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Waitall");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Waitany_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Waitany[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Waitany at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Waitany));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Waitany)[pnmpi_level](count, array_of_requests, index, status);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Waitany at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Waitany");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Waitany(count, array_of_requests, index, status);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Waitany");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Waitsome_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Waitsome[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Waitsome at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Waitsome));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Waitsome)[pnmpi_level](incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Waitsome at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Waitsome");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Waitsome");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_allocate_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_allocate[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_allocate at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_allocate));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_allocate)[pnmpi_level](size, disp_unit, info, comm, baseptr, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_allocate at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_allocate");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_allocate");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_allocate_shared_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_allocate_shared[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_allocate_shared at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_allocate_shared));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_allocate_shared)[pnmpi_level](size, disp_unit, info, comm, baseptr, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_allocate_shared at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_allocate_shared");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_allocate_shared");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_attach(MPI_Win win, void *base, MPI_Aint size)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_attach_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_attach[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_attach at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_attach));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_attach)[pnmpi_level](win, base, size);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_attach at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_attach");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_attach(win, base, size);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_attach");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_call_errhandler(MPI_Win win, int errorcode)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_call_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_call_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_call_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_call_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_call_errhandler)[pnmpi_level](win, errorcode);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_call_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_call_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_call_errhandler(win, errorcode);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_call_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_complete(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_complete_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_complete[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_complete at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_complete));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_complete)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_complete at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_complete");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_complete(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_complete");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_create_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_create[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_create at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_create));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_create)[pnmpi_level](base, size, disp_unit, info, comm, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_create at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_create");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_create(base, size, disp_unit, info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_create");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_create_dynamic_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_dynamic[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_create_dynamic at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_create_dynamic));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_dynamic)[pnmpi_level](info, comm, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_create_dynamic at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_create_dynamic");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_create_dynamic(info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_create_dynamic");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_create_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_create_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_create_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_errhandler)[pnmpi_level](function, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_create_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_create_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_create_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_create_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_create_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_create_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_create_keyval)[pnmpi_level](win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_create_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_create_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_create_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_delete_attr(MPI_Win win, int win_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_delete_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_delete_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_delete_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_delete_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_delete_attr)[pnmpi_level](win, win_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_delete_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_delete_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_delete_attr(win, win_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_delete_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_detach(MPI_Win win, const void *base)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_detach_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_detach[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_detach at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_detach));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_detach)[pnmpi_level](win, base);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_detach at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_detach");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_detach(win, base);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_detach");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_fence(int assert, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_fence_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_fence[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_fence at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_fence));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_fence)[pnmpi_level](assert, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_fence at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_fence");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_fence(assert, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_fence");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_flush(int rank, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_flush_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_flush at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_flush));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush)[pnmpi_level](rank, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_flush at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_flush");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_flush(rank, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_flush");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_flush_all(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_flush_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_flush_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_flush_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_all)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_flush_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_flush_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_flush_all(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_flush_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_flush_local(int rank, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_flush_local_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_local[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_flush_local at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_flush_local));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_local)[pnmpi_level](rank, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_flush_local at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_flush_local");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_flush_local(rank, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_flush_local");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_flush_local_all(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_flush_local_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_local_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_flush_local_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_flush_local_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_flush_local_all)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_flush_local_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_flush_local_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_flush_local_all(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_flush_local_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_free(MPI_Win *win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_free_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_free[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_free at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_free));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_free)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_free at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_free");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_free(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_free");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_free_keyval(int *win_keyval)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_free_keyval_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_free_keyval[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_free_keyval at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_free_keyval));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_free_keyval)[pnmpi_level](win_keyval);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_free_keyval at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_free_keyval");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_free_keyval(win_keyval);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_free_keyval");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_get_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_get_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_get_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_attr)[pnmpi_level](win, win_keyval, attribute_val, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_get_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_get_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_get_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_get_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_get_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_get_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_errhandler)[pnmpi_level](win, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_get_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_get_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_get_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_get_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_get_group(MPI_Win win, MPI_Group *group)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_get_group_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_group[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_get_group at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_get_group));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_group)[pnmpi_level](win, group);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_get_group at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_get_group");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_get_group(win, group);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_get_group");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_get_info(MPI_Win win, MPI_Info *info_used)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_get_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_get_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_get_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_info)[pnmpi_level](win, info_used);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_get_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_get_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_get_info(win, info_used);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_get_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_get_name(MPI_Win win, char *win_name, int *resultlen)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_get_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_get_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_get_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_get_name)[pnmpi_level](win, win_name, resultlen);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_get_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_get_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_get_name(win, win_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_get_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_lock(int lock_type, int rank, int assert, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_lock_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_lock[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_lock at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_lock));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_lock)[pnmpi_level](lock_type, rank, assert, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_lock at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_lock");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_lock(lock_type, rank, assert, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_lock");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_lock_all(int assert, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_lock_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_lock_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_lock_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_lock_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_lock_all)[pnmpi_level](assert, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_lock_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_lock_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_lock_all(assert, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_lock_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_post(MPI_Group group, int assert, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_post_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_post[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_post at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_post));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_post)[pnmpi_level](group, assert, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_post at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_post");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_post(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_post");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_set_attr_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_attr[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_set_attr at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_set_attr));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_attr)[pnmpi_level](win, win_keyval, attribute_val);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_set_attr at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_set_attr");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_set_attr(win, win_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_set_attr");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_set_errhandler_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_errhandler[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_set_errhandler at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_set_errhandler));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_errhandler)[pnmpi_level](win, errhandler);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_set_errhandler at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_set_errhandler");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_set_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_set_errhandler");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_set_info(MPI_Win win, MPI_Info info)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_set_info_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_info[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_set_info at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_set_info));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_info)[pnmpi_level](win, info);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_set_info at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_set_info");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_set_info(win, info);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_set_info");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_set_name(MPI_Win win, const char *win_name)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_set_name_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_name[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_set_name at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_set_name));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_set_name)[pnmpi_level](win, win_name);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_set_name at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_set_name");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_set_name(win, win_name);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_set_name");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_shared_query_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_shared_query[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_shared_query at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_shared_query));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_shared_query)[pnmpi_level](win, rank, size, disp_unit, baseptr);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_shared_query at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_shared_query");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_shared_query");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_start(MPI_Group group, int assert, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_start_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_start[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_start at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_start));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_start)[pnmpi_level](group, assert, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_start at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_start");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_start(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_start");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_sync(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_sync_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_sync[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_sync at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_sync));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_sync)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_sync at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_sync");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_sync(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_sync");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_test(MPI_Win win, int *flag)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_test_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_test[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_test at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_test));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_test)[pnmpi_level](win, flag);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_test at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_test");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_test(win, flag);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_test");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_unlock(int rank, MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_unlock_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_unlock[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_unlock at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_unlock));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_unlock)[pnmpi_level](rank, win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_unlock at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_unlock");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_unlock(rank, win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_unlock");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_unlock_all(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_unlock_all_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_unlock_all[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_unlock_all at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_unlock_all));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_unlock_all)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_unlock_all at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_unlock_all");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_unlock_all(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_unlock_all");
  pnmpi_level = start_level;
  return res;
}

int NQJ_Win_wait(MPI_Win win)
{
  int res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Win_wait_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Win_wait[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Win_wait at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Win_wait));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Win_wait)[pnmpi_level](win);
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Win_wait at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Win_wait");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Win_wait(win);
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Win_wait");
  pnmpi_level = start_level;
  return res;
}

double NQJ_Wtick()
{
  double res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Wtick_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Wtick[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Wtick at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Wtick));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Wtick)[pnmpi_level]();
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Wtick at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Wtick");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Wtick();
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Wtick");
  pnmpi_level = start_level;
  return res;
}

double NQJ_Wtime()
{
  double res;
  int start_level = pnmpi_level;

  if (IS_ACTIVATED(MPI_Wtime_ID) && (get_pnmpi_mpi_level() == 0) && (pnmpi_initialization_complete > 0))
    {
       while ((pnmpi_level<pnmpi_max_level) && (modules.module[pnmpi_level]->stack_delimiter==0))
        {
          if (pnmpi_function_ptrs.pnmpi_int_MPI_Wtime[pnmpi_level]!=NULL)
	    {
          WRAP_MPI_CALL_PREFIX
          DBGPRINT3("Calling a wrapper in MPI_Wtime at level %i FROM %px",pnmpi_level,&(Internal_XMPI_Wtime));
          res=(pnmpi_function_ptrs.pnmpi_int_MPI_Wtime)[pnmpi_level]();
          WRAP_MPI_CALL_POSTFIX
	      DBGPRINT3("Done with wrapper in MPI_Wtime at level %i - reseting to %i",pnmpi_level,start_level);
	      pnmpi_level = start_level;
	      return res;
            }
          ++pnmpi_level;
       }
    }

  DBGPRINT3("Calling a original MPI in MPI_Wtime");
  WRAP_MPI_CALL_PREFIX
  res=PMPI_Wtime();
  WRAP_MPI_CALL_POSTFIX
  DBGPRINT3("Done with original MPI in MPI_Wtime");
  pnmpi_level = start_level;
  return res;
}



/* ================== C Wrappers for MPI_Abort ================== */
_EXTERN_C_ int PMPI_Abort(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Abort(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Abort at base level - Location = %px",&(MPI_Abort));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Abort_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Abort(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Abort(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Abort(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Abort =============== */
static void MPI_Abort_fortran_wrapper(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Abort((MPI_Comm)(*comm), *errorcode);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Abort(MPI_Comm_f2c(*comm), *errorcode);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ABORT(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Abort_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_abort(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Abort_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_abort_(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Abort_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_abort__(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Abort_fortran_wrapper(comm, errorcode, ierr);
}

/* ================= End Wrappers for MPI_Abort ================= */


/* ================== C Wrappers for MPI_Accumulate ================== */
_EXTERN_C_ int PMPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Accumulate at base level - Location = %px",&(MPI_Accumulate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Accumulate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Accumulate =============== */
static void MPI_Accumulate_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Accumulate(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Op)(*op), (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Accumulate(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Op_f2c(*op), MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ACCUMULATE(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_accumulate(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_accumulate_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_accumulate__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

/* ================= End Wrappers for MPI_Accumulate ================= */


/* ================== C Wrappers for MPI_Add_error_class ================== */
_EXTERN_C_ int PMPI_Add_error_class(int *errorclass);
_EXTERN_C_ int MPI_Add_error_class(int *errorclass) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Add_error_class at base level - Location = %px",&(MPI_Add_error_class));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Add_error_class_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Add_error_class(errorclass);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Add_error_class(errorclass);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Add_error_class(errorclass);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Add_error_class =============== */
static void MPI_Add_error_class_fortran_wrapper(MPI_Fint *errorclass, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Add_error_class(((int*)errorclass));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ADD_ERROR_CLASS(MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Add_error_class_fortran_wrapper(errorclass, ierr);
}

_EXTERN_C_ void mpi_add_error_class(MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Add_error_class_fortran_wrapper(errorclass, ierr);
}

_EXTERN_C_ void mpi_add_error_class_(MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Add_error_class_fortran_wrapper(errorclass, ierr);
}

_EXTERN_C_ void mpi_add_error_class__(MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Add_error_class_fortran_wrapper(errorclass, ierr);
}

/* ================= End Wrappers for MPI_Add_error_class ================= */


/* ================== C Wrappers for MPI_Add_error_code ================== */
_EXTERN_C_ int PMPI_Add_error_code(int errorclass, int *errorcode);
_EXTERN_C_ int MPI_Add_error_code(int errorclass, int *errorcode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Add_error_code at base level - Location = %px",&(MPI_Add_error_code));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Add_error_code_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Add_error_code(errorclass, errorcode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Add_error_code(errorclass, errorcode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Add_error_code(errorclass, errorcode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Add_error_code =============== */
static void MPI_Add_error_code_fortran_wrapper(MPI_Fint *errorclass, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Add_error_code(*errorclass, ((int*)errorcode));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ADD_ERROR_CODE(MPI_Fint *errorclass, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Add_error_code_fortran_wrapper(errorclass, errorcode, ierr);
}

_EXTERN_C_ void mpi_add_error_code(MPI_Fint *errorclass, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Add_error_code_fortran_wrapper(errorclass, errorcode, ierr);
}

_EXTERN_C_ void mpi_add_error_code_(MPI_Fint *errorclass, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Add_error_code_fortran_wrapper(errorclass, errorcode, ierr);
}

_EXTERN_C_ void mpi_add_error_code__(MPI_Fint *errorclass, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Add_error_code_fortran_wrapper(errorclass, errorcode, ierr);
}

/* ================= End Wrappers for MPI_Add_error_code ================= */


/* ================== C Wrappers for MPI_Add_error_string ================== */
_EXTERN_C_ int PMPI_Add_error_string(int errorcode, const char *string);
_EXTERN_C_ int MPI_Add_error_string(int errorcode, const char *string) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Add_error_string at base level - Location = %px",&(MPI_Add_error_string));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Add_error_string_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Add_error_string(errorcode, string);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Add_error_string(errorcode, string);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Add_error_string(errorcode, string);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Add_error_string =============== */
static void MPI_Add_error_string_fortran_wrapper(MPI_Fint *errorcode, MPI_Fint *string, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Add_error_string(*errorcode, ((const char*)string));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ADD_ERROR_STRING(MPI_Fint *errorcode, MPI_Fint *string, MPI_Fint *ierr) { 
    MPI_Add_error_string_fortran_wrapper(errorcode, string, ierr);
}

_EXTERN_C_ void mpi_add_error_string(MPI_Fint *errorcode, MPI_Fint *string, MPI_Fint *ierr) { 
    MPI_Add_error_string_fortran_wrapper(errorcode, string, ierr);
}

_EXTERN_C_ void mpi_add_error_string_(MPI_Fint *errorcode, MPI_Fint *string, MPI_Fint *ierr) { 
    MPI_Add_error_string_fortran_wrapper(errorcode, string, ierr);
}

_EXTERN_C_ void mpi_add_error_string__(MPI_Fint *errorcode, MPI_Fint *string, MPI_Fint *ierr) { 
    MPI_Add_error_string_fortran_wrapper(errorcode, string, ierr);
}

/* ================= End Wrappers for MPI_Add_error_string ================= */


/* ================== C Wrappers for MPI_Address ================== */
_EXTERN_C_ int PMPI_Address(void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Address(void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Address at base level - Location = %px",&(MPI_Address));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Address_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Address(location, address);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Address(location, address);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Address(location, address);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Address =============== */
static void MPI_Address_fortran_wrapper(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Address(BufferC2F((void*)location), ((MPI_Aint*)address));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ADDRESS(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_address(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_address_(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_address__(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Address_fortran_wrapper(location, address, ierr);
}

/* ================= End Wrappers for MPI_Address ================= */


/* ================== C Wrappers for MPI_Allgather ================== */
_EXTERN_C_ int PMPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Allgather at base level - Location = %px",&(MPI_Allgather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Allgather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Allgather =============== */
static void MPI_Allgather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Allgather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Allgather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLGATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Allgather ================= */


/* ================== C Wrappers for MPI_Allgatherv ================== */
_EXTERN_C_ int PMPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Allgatherv at base level - Location = %px",&(MPI_Allgatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Allgatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Allgatherv =============== */
static void MPI_Allgatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Allgatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Allgatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLGATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_allgatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Allgatherv ================= */


/* ================== C Wrappers for MPI_Alloc_mem ================== */
_EXTERN_C_ int PMPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr);
_EXTERN_C_ int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Alloc_mem at base level - Location = %px",&(MPI_Alloc_mem));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Alloc_mem_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Alloc_mem(size, info, baseptr);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Alloc_mem(size, info, baseptr);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Alloc_mem(size, info, baseptr);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Alloc_mem =============== */
static void MPI_Alloc_mem_fortran_wrapper(MPI_Aint *size, MPI_Fint *info, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Alloc_mem(*size, (MPI_Info)(*info), BufferC2F((void*)baseptr));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Alloc_mem(*size, MPI_Info_f2c(*info), BufferC2F((void*)baseptr));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLOC_MEM(MPI_Aint *size, MPI_Fint *info, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Alloc_mem_fortran_wrapper(size, info, baseptr, ierr);
}

_EXTERN_C_ void mpi_alloc_mem(MPI_Aint *size, MPI_Fint *info, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Alloc_mem_fortran_wrapper(size, info, baseptr, ierr);
}

_EXTERN_C_ void mpi_alloc_mem_(MPI_Aint *size, MPI_Fint *info, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Alloc_mem_fortran_wrapper(size, info, baseptr, ierr);
}

_EXTERN_C_ void mpi_alloc_mem__(MPI_Aint *size, MPI_Fint *info, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Alloc_mem_fortran_wrapper(size, info, baseptr, ierr);
}

/* ================= End Wrappers for MPI_Alloc_mem ================= */


/* ================== C Wrappers for MPI_Allreduce ================== */
_EXTERN_C_ int PMPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Allreduce at base level - Location = %px",&(MPI_Allreduce));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Allreduce_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Allreduce =============== */
static void MPI_Allreduce_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Allreduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Allreduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLREDUCE(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_allreduce(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_allreduce_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_allreduce__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Allreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

/* ================= End Wrappers for MPI_Allreduce ================= */


/* ================== C Wrappers for MPI_Alltoall ================== */
_EXTERN_C_ int PMPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Alltoall at base level - Location = %px",&(MPI_Alltoall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Alltoall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Alltoall =============== */
static void MPI_Alltoall_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Alltoall(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Alltoall(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLTOALL(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoall(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoall_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoall__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Alltoall ================= */


/* ================== C Wrappers for MPI_Alltoallv ================== */
_EXTERN_C_ int PMPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Alltoallv at base level - Location = %px",&(MPI_Alltoallv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Alltoallv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Alltoallv =============== */
static void MPI_Alltoallv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLTOALLV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Alltoallv ================= */


/* ================== C Wrappers for MPI_Alltoallw ================== */
_EXTERN_C_ int PMPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Alltoallw at base level - Location = %px",&(MPI_Alltoallw));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Alltoallw_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Alltoallw =============== */
static void MPI_Alltoallw_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), ((const MPI_Datatype*)recvtypes), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), ((const MPI_Datatype*)recvtypes), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ALLTOALLW(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallw(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallw_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_alltoallw__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

/* ================= End Wrappers for MPI_Alltoallw ================= */


/* ================== C Wrappers for MPI_Attr_delete ================== */
_EXTERN_C_ int PMPI_Attr_delete(MPI_Comm comm, int keyval);
_EXTERN_C_ int MPI_Attr_delete(MPI_Comm comm, int keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Attr_delete at base level - Location = %px",&(MPI_Attr_delete));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Attr_delete_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Attr_delete(comm, keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Attr_delete(comm, keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Attr_delete(comm, keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Attr_delete =============== */
static void MPI_Attr_delete_fortran_wrapper(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Attr_delete((MPI_Comm)(*comm), *keyval);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Attr_delete(MPI_Comm_f2c(*comm), *keyval);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ATTR_DELETE(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Attr_delete_fortran_wrapper(comm, keyval, ierr);
}

_EXTERN_C_ void mpi_attr_delete(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Attr_delete_fortran_wrapper(comm, keyval, ierr);
}

_EXTERN_C_ void mpi_attr_delete_(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Attr_delete_fortran_wrapper(comm, keyval, ierr);
}

_EXTERN_C_ void mpi_attr_delete__(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Attr_delete_fortran_wrapper(comm, keyval, ierr);
}

/* ================= End Wrappers for MPI_Attr_delete ================= */


/* ================== C Wrappers for MPI_Attr_get ================== */
_EXTERN_C_ int PMPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Attr_get at base level - Location = %px",&(MPI_Attr_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Attr_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Attr_get(comm, keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Attr_get(comm, keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Attr_get(comm, keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Attr_get =============== */
static void MPI_Attr_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Attr_get((MPI_Comm)(*comm), *keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Attr_get(MPI_Comm_f2c(*comm), *keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ATTR_GET(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Attr_get_fortran_wrapper(comm, keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_attr_get(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Attr_get_fortran_wrapper(comm, keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_attr_get_(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Attr_get_fortran_wrapper(comm, keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_attr_get__(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Attr_get_fortran_wrapper(comm, keyval, attribute_val, flag, ierr);
}

/* ================= End Wrappers for MPI_Attr_get ================= */


/* ================== C Wrappers for MPI_Attr_put ================== */
_EXTERN_C_ int PMPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);
_EXTERN_C_ int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Attr_put at base level - Location = %px",&(MPI_Attr_put));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Attr_put_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Attr_put(comm, keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Attr_put(comm, keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Attr_put(comm, keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Attr_put =============== */
static void MPI_Attr_put_fortran_wrapper(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Attr_put((MPI_Comm)(*comm), *keyval, BufferC2F((void*)attribute_val));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Attr_put(MPI_Comm_f2c(*comm), *keyval, BufferC2F((void*)attribute_val));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ATTR_PUT(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Attr_put_fortran_wrapper(comm, keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_attr_put(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Attr_put_fortran_wrapper(comm, keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_attr_put_(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Attr_put_fortran_wrapper(comm, keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_attr_put__(MPI_Fint *comm, MPI_Fint *keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Attr_put_fortran_wrapper(comm, keyval, attribute_val, ierr);
}

/* ================= End Wrappers for MPI_Attr_put ================= */


/* ================== C Wrappers for MPI_Barrier ================== */
_EXTERN_C_ int PMPI_Barrier(MPI_Comm comm);
_EXTERN_C_ int MPI_Barrier(MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Barrier at base level - Location = %px",&(MPI_Barrier));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Barrier_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Barrier(comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Barrier(comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Barrier(comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Barrier =============== */
static void MPI_Barrier_fortran_wrapper(MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Barrier((MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Barrier(MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BARRIER(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Barrier_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_barrier(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Barrier_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_barrier_(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Barrier_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_barrier__(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Barrier_fortran_wrapper(comm, ierr);
}

/* ================= End Wrappers for MPI_Barrier ================= */


/* ================== C Wrappers for MPI_Bcast ================== */
_EXTERN_C_ int PMPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Bcast at base level - Location = %px",&(MPI_Bcast));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Bcast_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Bcast(buffer, count, datatype, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Bcast(buffer, count, datatype, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Bcast(buffer, count, datatype, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Bcast =============== */
static void MPI_Bcast_fortran_wrapper(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Bcast(BufferC2F((void*)buffer), *count, (MPI_Datatype)(*datatype), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Bcast(BufferC2F((void*)buffer), *count, MPI_Type_f2c(*datatype), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BCAST(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bcast_fortran_wrapper(buffer, count, datatype, root, comm, ierr);
}

_EXTERN_C_ void mpi_bcast(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bcast_fortran_wrapper(buffer, count, datatype, root, comm, ierr);
}

_EXTERN_C_ void mpi_bcast_(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bcast_fortran_wrapper(buffer, count, datatype, root, comm, ierr);
}

_EXTERN_C_ void mpi_bcast__(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bcast_fortran_wrapper(buffer, count, datatype, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Bcast ================= */


/* ================== C Wrappers for MPI_Bsend ================== */
_EXTERN_C_ int PMPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Bsend at base level - Location = %px",&(MPI_Bsend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Bsend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Bsend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Bsend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Bsend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Bsend =============== */
static void MPI_Bsend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Bsend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Bsend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BSEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_bsend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_bsend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_bsend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Bsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

/* ================= End Wrappers for MPI_Bsend ================= */


/* ================== C Wrappers for MPI_Bsend_init ================== */
_EXTERN_C_ int PMPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Bsend_init at base level - Location = %px",&(MPI_Bsend_init));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Bsend_init_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Bsend_init =============== */
static void MPI_Bsend_init_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Bsend_init(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Bsend_init(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BSEND_INIT(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Bsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_bsend_init(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Bsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_bsend_init_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Bsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_bsend_init__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Bsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Bsend_init ================= */


/* ================== C Wrappers for MPI_Buffer_attach ================== */
_EXTERN_C_ int PMPI_Buffer_attach(void *buffer, int size);
_EXTERN_C_ int MPI_Buffer_attach(void *buffer, int size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Buffer_attach at base level - Location = %px",&(MPI_Buffer_attach));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Buffer_attach_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Buffer_attach(buffer, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Buffer_attach(buffer, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Buffer_attach(buffer, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Buffer_attach =============== */
static void MPI_Buffer_attach_fortran_wrapper(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Buffer_attach(BufferC2F((void*)buffer), *size);
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BUFFER_ATTACH(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_attach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_attach(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_attach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_attach_(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_attach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_attach__(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_attach_fortran_wrapper(buffer, size, ierr);
}

/* ================= End Wrappers for MPI_Buffer_attach ================= */


/* ================== C Wrappers for MPI_Buffer_detach ================== */
_EXTERN_C_ int PMPI_Buffer_detach(void *buffer, int *size);
_EXTERN_C_ int MPI_Buffer_detach(void *buffer, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Buffer_detach at base level - Location = %px",&(MPI_Buffer_detach));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Buffer_detach_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Buffer_detach(buffer, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Buffer_detach(buffer, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Buffer_detach(buffer, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Buffer_detach =============== */
static void MPI_Buffer_detach_fortran_wrapper(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Buffer_detach(BufferC2F((void*)buffer), ((int*)size));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_BUFFER_DETACH(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_detach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_detach(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_detach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_detach_(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_detach_fortran_wrapper(buffer, size, ierr);
}

_EXTERN_C_ void mpi_buffer_detach__(MPI_Fint *buffer, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Buffer_detach_fortran_wrapper(buffer, size, ierr);
}

/* ================= End Wrappers for MPI_Buffer_detach ================= */


/* ================== C Wrappers for MPI_Cancel ================== */
_EXTERN_C_ int PMPI_Cancel(MPI_Request *request);
_EXTERN_C_ int MPI_Cancel(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cancel at base level - Location = %px",&(MPI_Cancel));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cancel_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cancel(request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cancel(request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cancel(request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cancel =============== */
static void MPI_Cancel_fortran_wrapper(MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cancel((MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Cancel(&temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CANCEL(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Cancel_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_cancel(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Cancel_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_cancel_(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Cancel_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_cancel__(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Cancel_fortran_wrapper(request, ierr);
}

/* ================= End Wrappers for MPI_Cancel ================= */


/* ================== C Wrappers for MPI_Cart_coords ================== */
_EXTERN_C_ int PMPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]);
_EXTERN_C_ int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_coords at base level - Location = %px",&(MPI_Cart_coords));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_coords_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_coords(comm, rank, maxdims, coords);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_coords(comm, rank, maxdims, coords);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_coords(comm, rank, maxdims, coords);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_coords =============== */
static void MPI_Cart_coords_fortran_wrapper(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxdims, MPI_Fint coords[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_coords((MPI_Comm)(*comm), *rank, *maxdims, ((int*)coords));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cart_coords(MPI_Comm_f2c(*comm), *rank, *maxdims, ((int*)coords));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_COORDS(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxdims, MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_coords_fortran_wrapper(comm, rank, maxdims, coords, ierr);
}

_EXTERN_C_ void mpi_cart_coords(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxdims, MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_coords_fortran_wrapper(comm, rank, maxdims, coords, ierr);
}

_EXTERN_C_ void mpi_cart_coords_(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxdims, MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_coords_fortran_wrapper(comm, rank, maxdims, coords, ierr);
}

_EXTERN_C_ void mpi_cart_coords__(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxdims, MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_coords_fortran_wrapper(comm, rank, maxdims, coords, ierr);
}

/* ================= End Wrappers for MPI_Cart_coords ================= */


/* ================== C Wrappers for MPI_Cart_create ================== */
_EXTERN_C_ int PMPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);
_EXTERN_C_ int MPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_create at base level - Location = %px",&(MPI_Cart_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_create =============== */
static void MPI_Cart_create_fortran_wrapper(MPI_Fint *old_comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *reorder, MPI_Fint *comm_cart, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_create((MPI_Comm)(*old_comm), *ndims, ((const int*)dims), ((const int*)periods), *reorder, (MPI_Comm*)comm_cart);
#else /* MPI-2 safe call */
    MPI_Comm temp_comm_cart;
    _wrap_py_return_val = MPI_Cart_create(MPI_Comm_f2c(*old_comm), *ndims, ((const int*)dims), ((const int*)periods), *reorder, &temp_comm_cart);
    *comm_cart = MPI_Comm_c2f(temp_comm_cart);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_CREATE(MPI_Fint *old_comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *reorder, MPI_Fint *comm_cart, MPI_Fint *ierr) { 
    MPI_Cart_create_fortran_wrapper(old_comm, ndims, dims, periods, reorder, comm_cart, ierr);
}

_EXTERN_C_ void mpi_cart_create(MPI_Fint *old_comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *reorder, MPI_Fint *comm_cart, MPI_Fint *ierr) { 
    MPI_Cart_create_fortran_wrapper(old_comm, ndims, dims, periods, reorder, comm_cart, ierr);
}

_EXTERN_C_ void mpi_cart_create_(MPI_Fint *old_comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *reorder, MPI_Fint *comm_cart, MPI_Fint *ierr) { 
    MPI_Cart_create_fortran_wrapper(old_comm, ndims, dims, periods, reorder, comm_cart, ierr);
}

_EXTERN_C_ void mpi_cart_create__(MPI_Fint *old_comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *reorder, MPI_Fint *comm_cart, MPI_Fint *ierr) { 
    MPI_Cart_create_fortran_wrapper(old_comm, ndims, dims, periods, reorder, comm_cart, ierr);
}

/* ================= End Wrappers for MPI_Cart_create ================= */


/* ================== C Wrappers for MPI_Cart_get ================== */
_EXTERN_C_ int PMPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);
_EXTERN_C_ int MPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_get at base level - Location = %px",&(MPI_Cart_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_get(comm, maxdims, dims, periods, coords);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_get(comm, maxdims, dims, periods, coords);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_get(comm, maxdims, dims, periods, coords);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_get =============== */
static void MPI_Cart_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *maxdims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint coords[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_get((MPI_Comm)(*comm), *maxdims, ((int*)dims), ((int*)periods), ((int*)coords));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cart_get(MPI_Comm_f2c(*comm), *maxdims, ((int*)dims), ((int*)periods), ((int*)coords));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_GET(MPI_Fint *comm, MPI_Fint *maxdims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_get_fortran_wrapper(comm, maxdims, dims, periods, coords, ierr);
}

_EXTERN_C_ void mpi_cart_get(MPI_Fint *comm, MPI_Fint *maxdims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_get_fortran_wrapper(comm, maxdims, dims, periods, coords, ierr);
}

_EXTERN_C_ void mpi_cart_get_(MPI_Fint *comm, MPI_Fint *maxdims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_get_fortran_wrapper(comm, maxdims, dims, periods, coords, ierr);
}

_EXTERN_C_ void mpi_cart_get__(MPI_Fint *comm, MPI_Fint *maxdims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint coords[], MPI_Fint *ierr) { 
    MPI_Cart_get_fortran_wrapper(comm, maxdims, dims, periods, coords, ierr);
}

/* ================= End Wrappers for MPI_Cart_get ================= */


/* ================== C Wrappers for MPI_Cart_map ================== */
_EXTERN_C_ int PMPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);
_EXTERN_C_ int MPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_map at base level - Location = %px",&(MPI_Cart_map));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_map_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_map(comm, ndims, dims, periods, newrank);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_map(comm, ndims, dims, periods, newrank);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_map(comm, ndims, dims, periods, newrank);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_map =============== */
static void MPI_Cart_map_fortran_wrapper(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_map((MPI_Comm)(*comm), *ndims, ((const int*)dims), ((const int*)periods), ((int*)newrank));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cart_map(MPI_Comm_f2c(*comm), *ndims, ((const int*)dims), ((const int*)periods), ((int*)newrank));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_MAP(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Cart_map_fortran_wrapper(comm, ndims, dims, periods, newrank, ierr);
}

_EXTERN_C_ void mpi_cart_map(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Cart_map_fortran_wrapper(comm, ndims, dims, periods, newrank, ierr);
}

_EXTERN_C_ void mpi_cart_map_(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Cart_map_fortran_wrapper(comm, ndims, dims, periods, newrank, ierr);
}

_EXTERN_C_ void mpi_cart_map__(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint periods[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Cart_map_fortran_wrapper(comm, ndims, dims, periods, newrank, ierr);
}

/* ================= End Wrappers for MPI_Cart_map ================= */


/* ================== C Wrappers for MPI_Cart_rank ================== */
_EXTERN_C_ int PMPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank);
_EXTERN_C_ int MPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_rank at base level - Location = %px",&(MPI_Cart_rank));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_rank_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_rank(comm, coords, rank);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_rank(comm, coords, rank);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_rank(comm, coords, rank);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_rank =============== */
static void MPI_Cart_rank_fortran_wrapper(MPI_Fint *comm, MPI_Fint coords[], MPI_Fint *rank, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_rank((MPI_Comm)(*comm), ((const int*)coords), ((int*)rank));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cart_rank(MPI_Comm_f2c(*comm), ((const int*)coords), ((int*)rank));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_RANK(MPI_Fint *comm, MPI_Fint coords[], MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Cart_rank_fortran_wrapper(comm, coords, rank, ierr);
}

_EXTERN_C_ void mpi_cart_rank(MPI_Fint *comm, MPI_Fint coords[], MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Cart_rank_fortran_wrapper(comm, coords, rank, ierr);
}

_EXTERN_C_ void mpi_cart_rank_(MPI_Fint *comm, MPI_Fint coords[], MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Cart_rank_fortran_wrapper(comm, coords, rank, ierr);
}

_EXTERN_C_ void mpi_cart_rank__(MPI_Fint *comm, MPI_Fint coords[], MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Cart_rank_fortran_wrapper(comm, coords, rank, ierr);
}

/* ================= End Wrappers for MPI_Cart_rank ================= */


/* ================== C Wrappers for MPI_Cart_shift ================== */
_EXTERN_C_ int PMPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);
_EXTERN_C_ int MPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_shift at base level - Location = %px",&(MPI_Cart_shift));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_shift_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_shift =============== */
static void MPI_Cart_shift_fortran_wrapper(MPI_Fint *comm, MPI_Fint *direction, MPI_Fint *disp, MPI_Fint *rank_source, MPI_Fint *rank_dest, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_shift((MPI_Comm)(*comm), *direction, *disp, ((int*)rank_source), ((int*)rank_dest));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cart_shift(MPI_Comm_f2c(*comm), *direction, *disp, ((int*)rank_source), ((int*)rank_dest));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_SHIFT(MPI_Fint *comm, MPI_Fint *direction, MPI_Fint *disp, MPI_Fint *rank_source, MPI_Fint *rank_dest, MPI_Fint *ierr) { 
    MPI_Cart_shift_fortran_wrapper(comm, direction, disp, rank_source, rank_dest, ierr);
}

_EXTERN_C_ void mpi_cart_shift(MPI_Fint *comm, MPI_Fint *direction, MPI_Fint *disp, MPI_Fint *rank_source, MPI_Fint *rank_dest, MPI_Fint *ierr) { 
    MPI_Cart_shift_fortran_wrapper(comm, direction, disp, rank_source, rank_dest, ierr);
}

_EXTERN_C_ void mpi_cart_shift_(MPI_Fint *comm, MPI_Fint *direction, MPI_Fint *disp, MPI_Fint *rank_source, MPI_Fint *rank_dest, MPI_Fint *ierr) { 
    MPI_Cart_shift_fortran_wrapper(comm, direction, disp, rank_source, rank_dest, ierr);
}

_EXTERN_C_ void mpi_cart_shift__(MPI_Fint *comm, MPI_Fint *direction, MPI_Fint *disp, MPI_Fint *rank_source, MPI_Fint *rank_dest, MPI_Fint *ierr) { 
    MPI_Cart_shift_fortran_wrapper(comm, direction, disp, rank_source, rank_dest, ierr);
}

/* ================= End Wrappers for MPI_Cart_shift ================= */


/* ================== C Wrappers for MPI_Cart_sub ================== */
_EXTERN_C_ int PMPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);
_EXTERN_C_ int MPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cart_sub at base level - Location = %px",&(MPI_Cart_sub));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cart_sub_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cart_sub(comm, remain_dims, new_comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cart_sub(comm, remain_dims, new_comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cart_sub(comm, remain_dims, new_comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cart_sub =============== */
static void MPI_Cart_sub_fortran_wrapper(MPI_Fint *comm, MPI_Fint remain_dims[], MPI_Fint *new_comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cart_sub((MPI_Comm)(*comm), ((const int*)remain_dims), (MPI_Comm*)new_comm);
#else /* MPI-2 safe call */
    MPI_Comm temp_new_comm;
    _wrap_py_return_val = MPI_Cart_sub(MPI_Comm_f2c(*comm), ((const int*)remain_dims), &temp_new_comm);
    *new_comm = MPI_Comm_c2f(temp_new_comm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CART_SUB(MPI_Fint *comm, MPI_Fint remain_dims[], MPI_Fint *new_comm, MPI_Fint *ierr) { 
    MPI_Cart_sub_fortran_wrapper(comm, remain_dims, new_comm, ierr);
}

_EXTERN_C_ void mpi_cart_sub(MPI_Fint *comm, MPI_Fint remain_dims[], MPI_Fint *new_comm, MPI_Fint *ierr) { 
    MPI_Cart_sub_fortran_wrapper(comm, remain_dims, new_comm, ierr);
}

_EXTERN_C_ void mpi_cart_sub_(MPI_Fint *comm, MPI_Fint remain_dims[], MPI_Fint *new_comm, MPI_Fint *ierr) { 
    MPI_Cart_sub_fortran_wrapper(comm, remain_dims, new_comm, ierr);
}

_EXTERN_C_ void mpi_cart_sub__(MPI_Fint *comm, MPI_Fint remain_dims[], MPI_Fint *new_comm, MPI_Fint *ierr) { 
    MPI_Cart_sub_fortran_wrapper(comm, remain_dims, new_comm, ierr);
}

/* ================= End Wrappers for MPI_Cart_sub ================= */


/* ================== C Wrappers for MPI_Cartdim_get ================== */
_EXTERN_C_ int PMPI_Cartdim_get(MPI_Comm comm, int *ndims);
_EXTERN_C_ int MPI_Cartdim_get(MPI_Comm comm, int *ndims) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Cartdim_get at base level - Location = %px",&(MPI_Cartdim_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Cartdim_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Cartdim_get(comm, ndims);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Cartdim_get(comm, ndims);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Cartdim_get(comm, ndims);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Cartdim_get =============== */
static void MPI_Cartdim_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Cartdim_get((MPI_Comm)(*comm), ((int*)ndims));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Cartdim_get(MPI_Comm_f2c(*comm), ((int*)ndims));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CARTDIM_GET(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint *ierr) { 
    MPI_Cartdim_get_fortran_wrapper(comm, ndims, ierr);
}

_EXTERN_C_ void mpi_cartdim_get(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint *ierr) { 
    MPI_Cartdim_get_fortran_wrapper(comm, ndims, ierr);
}

_EXTERN_C_ void mpi_cartdim_get_(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint *ierr) { 
    MPI_Cartdim_get_fortran_wrapper(comm, ndims, ierr);
}

_EXTERN_C_ void mpi_cartdim_get__(MPI_Fint *comm, MPI_Fint *ndims, MPI_Fint *ierr) { 
    MPI_Cartdim_get_fortran_wrapper(comm, ndims, ierr);
}

/* ================= End Wrappers for MPI_Cartdim_get ================= */


/* ================== C Wrappers for MPI_Close_port ================== */
_EXTERN_C_ int PMPI_Close_port(const char *port_name);
_EXTERN_C_ int MPI_Close_port(const char *port_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Close_port at base level - Location = %px",&(MPI_Close_port));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Close_port_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Close_port(port_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Close_port(port_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Close_port(port_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Close_port =============== */
static void MPI_Close_port_fortran_wrapper(MPI_Fint *port_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Close_port(((const char*)port_name));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_CLOSE_PORT(MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Close_port_fortran_wrapper(port_name, ierr);
}

_EXTERN_C_ void mpi_close_port(MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Close_port_fortran_wrapper(port_name, ierr);
}

_EXTERN_C_ void mpi_close_port_(MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Close_port_fortran_wrapper(port_name, ierr);
}

_EXTERN_C_ void mpi_close_port__(MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Close_port_fortran_wrapper(port_name, ierr);
}

/* ================= End Wrappers for MPI_Close_port ================= */


/* ================== C Wrappers for MPI_Comm_accept ================== */
_EXTERN_C_ int PMPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_accept at base level - Location = %px",&(MPI_Comm_accept));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_accept_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_accept(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_accept(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_accept(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_accept =============== */
static void MPI_Comm_accept_fortran_wrapper(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_accept(((const char*)port_name), (MPI_Info)(*info), *root, (MPI_Comm)(*comm), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_accept(((const char*)port_name), MPI_Info_f2c(*info), *root, MPI_Comm_f2c(*comm), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_ACCEPT(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_accept_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_accept(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_accept_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_accept_(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_accept_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_accept__(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_accept_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_accept ================= */


/* ================== C Wrappers for MPI_Comm_call_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_call_errhandler(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_call_errhandler at base level - Location = %px",&(MPI_Comm_call_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_call_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_call_errhandler(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_call_errhandler(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_call_errhandler(comm, errorcode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_call_errhandler =============== */
static void MPI_Comm_call_errhandler_fortran_wrapper(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_call_errhandler((MPI_Comm)(*comm), *errorcode);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_call_errhandler(MPI_Comm_f2c(*comm), *errorcode);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CALL_ERRHANDLER(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Comm_call_errhandler_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_comm_call_errhandler(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Comm_call_errhandler_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_comm_call_errhandler_(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Comm_call_errhandler_fortran_wrapper(comm, errorcode, ierr);
}

_EXTERN_C_ void mpi_comm_call_errhandler__(MPI_Fint *comm, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Comm_call_errhandler_fortran_wrapper(comm, errorcode, ierr);
}

/* ================= End Wrappers for MPI_Comm_call_errhandler ================= */


/* ================== C Wrappers for MPI_Comm_compare ================== */
_EXTERN_C_ int PMPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);
_EXTERN_C_ int MPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_compare at base level - Location = %px",&(MPI_Comm_compare));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_compare_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_compare(comm1, comm2, result);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_compare(comm1, comm2, result);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_compare(comm1, comm2, result);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_compare =============== */
static void MPI_Comm_compare_fortran_wrapper(MPI_Fint *comm1, MPI_Fint *comm2, MPI_Fint *result, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_compare((MPI_Comm)(*comm1), (MPI_Comm)(*comm2), ((int*)result));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_compare(MPI_Comm_f2c(*comm1), MPI_Comm_f2c(*comm2), ((int*)result));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_COMPARE(MPI_Fint *comm1, MPI_Fint *comm2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Comm_compare_fortran_wrapper(comm1, comm2, result, ierr);
}

_EXTERN_C_ void mpi_comm_compare(MPI_Fint *comm1, MPI_Fint *comm2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Comm_compare_fortran_wrapper(comm1, comm2, result, ierr);
}

_EXTERN_C_ void mpi_comm_compare_(MPI_Fint *comm1, MPI_Fint *comm2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Comm_compare_fortran_wrapper(comm1, comm2, result, ierr);
}

_EXTERN_C_ void mpi_comm_compare__(MPI_Fint *comm1, MPI_Fint *comm2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Comm_compare_fortran_wrapper(comm1, comm2, result, ierr);
}

/* ================= End Wrappers for MPI_Comm_compare ================= */


/* ================== C Wrappers for MPI_Comm_connect ================== */
_EXTERN_C_ int PMPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_connect at base level - Location = %px",&(MPI_Comm_connect));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_connect_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_connect(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_connect(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_connect(port_name, info, root, comm, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_connect =============== */
static void MPI_Comm_connect_fortran_wrapper(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_connect(((const char*)port_name), (MPI_Info)(*info), *root, (MPI_Comm)(*comm), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_connect(((const char*)port_name), MPI_Info_f2c(*info), *root, MPI_Comm_f2c(*comm), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CONNECT(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_connect_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_connect(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_connect_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_connect_(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_connect_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_connect__(MPI_Fint *port_name, MPI_Fint *info, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_connect_fortran_wrapper(port_name, info, root, comm, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_connect ================= */


/* ================== C Wrappers for MPI_Comm_create ================== */
_EXTERN_C_ int PMPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_create at base level - Location = %px",&(MPI_Comm_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_create(comm, group, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_create(comm, group, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_create(comm, group, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_create =============== */
static void MPI_Comm_create_fortran_wrapper(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_create((MPI_Comm)(*comm), (MPI_Group)(*group), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_create(MPI_Comm_f2c(*comm), MPI_Group_f2c(*group), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CREATE(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_fortran_wrapper(comm, group, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_fortran_wrapper(comm, group, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create_(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_fortran_wrapper(comm, group, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create__(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_fortran_wrapper(comm, group, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_create ================= */


/* ================== C Wrappers for MPI_Comm_create_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_create_errhandler at base level - Location = %px",&(MPI_Comm_create_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_create_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_create_errhandler =============== */
static void MPI_Comm_create_errhandler_fortran_wrapper(MPI_Comm_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_create_errhandler(((MPI_Comm_errhandler_function*)function), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_Comm_create_errhandler(((MPI_Comm_errhandler_function*)function), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CREATE_ERRHANDLER(MPI_Comm_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_create_errhandler_(MPI_Comm_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_create_errhandler__(MPI_Comm_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Comm_create_errhandler ================= */


/* ================== C Wrappers for MPI_Comm_create_group ================== */
_EXTERN_C_ int PMPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_create_group at base level - Location = %px",&(MPI_Comm_create_group));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_create_group_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_create_group(comm, group, tag, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_create_group(comm, group, tag, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_create_group(comm, group, tag, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_create_group =============== */
static void MPI_Comm_create_group_fortran_wrapper(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *tag, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_create_group((MPI_Comm)(*comm), (MPI_Group)(*group), *tag, (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_create_group(MPI_Comm_f2c(*comm), MPI_Group_f2c(*group), *tag, &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CREATE_GROUP(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *tag, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_group_fortran_wrapper(comm, group, tag, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create_group(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *tag, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_group_fortran_wrapper(comm, group, tag, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create_group_(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *tag, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_group_fortran_wrapper(comm, group, tag, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_create_group__(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *tag, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_create_group_fortran_wrapper(comm, group, tag, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_create_group ================= */


/* ================== C Wrappers for MPI_Comm_create_keyval ================== */
_EXTERN_C_ int PMPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);
_EXTERN_C_ int MPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_create_keyval at base level - Location = %px",&(MPI_Comm_create_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_create_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_create_keyval =============== */
static void MPI_Comm_create_keyval_fortran_wrapper(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, MPI_Fint *comm_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Comm_create_keyval(((MPI_Comm_copy_attr_function*)comm_copy_attr_fn), ((MPI_Comm_delete_attr_function*)comm_delete_attr_fn), ((int*)comm_keyval), BufferC2F((void*)extra_state));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_CREATE_KEYVAL(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, MPI_Fint *comm_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Comm_create_keyval_fortran_wrapper(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, MPI_Fint *comm_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Comm_create_keyval_fortran_wrapper(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_comm_create_keyval_(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, MPI_Fint *comm_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Comm_create_keyval_fortran_wrapper(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_comm_create_keyval__(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, MPI_Fint *comm_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Comm_create_keyval_fortran_wrapper(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state, ierr);
}

/* ================= End Wrappers for MPI_Comm_create_keyval ================= */


/* ================== C Wrappers for MPI_Comm_delete_attr ================== */
_EXTERN_C_ int PMPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval);
_EXTERN_C_ int MPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_delete_attr at base level - Location = %px",&(MPI_Comm_delete_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_delete_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_delete_attr(comm, comm_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_delete_attr(comm, comm_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_delete_attr(comm, comm_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_delete_attr =============== */
static void MPI_Comm_delete_attr_fortran_wrapper(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_delete_attr((MPI_Comm)(*comm), *comm_keyval);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_delete_attr(MPI_Comm_f2c(*comm), *comm_keyval);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_DELETE_ATTR(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_delete_attr_fortran_wrapper(comm, comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_delete_attr(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_delete_attr_fortran_wrapper(comm, comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_delete_attr_(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_delete_attr_fortran_wrapper(comm, comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_delete_attr__(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_delete_attr_fortran_wrapper(comm, comm_keyval, ierr);
}

/* ================= End Wrappers for MPI_Comm_delete_attr ================= */


/* ================== C Wrappers for MPI_Comm_disconnect ================== */
_EXTERN_C_ int PMPI_Comm_disconnect(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_disconnect(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_disconnect at base level - Location = %px",&(MPI_Comm_disconnect));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_disconnect_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_disconnect(comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_disconnect(comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_disconnect(comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_disconnect =============== */
static void MPI_Comm_disconnect_fortran_wrapper(MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_disconnect((MPI_Comm*)comm);
#else /* MPI-2 safe call */
    MPI_Comm temp_comm;
    _wrap_py_return_val = MPI_Comm_disconnect(&temp_comm);
    *comm = MPI_Comm_c2f(temp_comm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_DISCONNECT(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_disconnect_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_disconnect(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_disconnect_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_disconnect_(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_disconnect_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_disconnect__(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_disconnect_fortran_wrapper(comm, ierr);
}

/* ================= End Wrappers for MPI_Comm_disconnect ================= */


/* ================== C Wrappers for MPI_Comm_dup ================== */
_EXTERN_C_ int PMPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_dup at base level - Location = %px",&(MPI_Comm_dup));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_dup_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_dup(comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_dup(comm, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_dup(comm, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_dup =============== */
static void MPI_Comm_dup_fortran_wrapper(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_dup((MPI_Comm)(*comm), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_dup(MPI_Comm_f2c(*comm), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_DUP(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_fortran_wrapper(comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_fortran_wrapper(comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup_(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_fortran_wrapper(comm, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup__(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_fortran_wrapper(comm, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_dup ================= */


/* ================== C Wrappers for MPI_Comm_dup_with_info ================== */
_EXTERN_C_ int PMPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_dup_with_info at base level - Location = %px",&(MPI_Comm_dup_with_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_dup_with_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_dup_with_info(comm, info, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_dup_with_info(comm, info, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_dup_with_info(comm, info, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_dup_with_info =============== */
static void MPI_Comm_dup_with_info_fortran_wrapper(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_dup_with_info((MPI_Comm)(*comm), (MPI_Info)(*info), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_dup_with_info(MPI_Comm_f2c(*comm), MPI_Info_f2c(*info), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_DUP_WITH_INFO(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_with_info_fortran_wrapper(comm, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup_with_info(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_with_info_fortran_wrapper(comm, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup_with_info_(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_with_info_fortran_wrapper(comm, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_dup_with_info__(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_dup_with_info_fortran_wrapper(comm, info, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_dup_with_info ================= */


/* ================== C Wrappers for MPI_Comm_free ================== */
_EXTERN_C_ int PMPI_Comm_free(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_free(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_free at base level - Location = %px",&(MPI_Comm_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_free(comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_free(comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_free(comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_free =============== */
static void MPI_Comm_free_fortran_wrapper(MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_free((MPI_Comm*)comm);
#else /* MPI-2 safe call */
    MPI_Comm temp_comm;
    temp_comm = MPI_Comm_f2c(*comm);
    _wrap_py_return_val = MPI_Comm_free(&temp_comm);
    *comm = MPI_Comm_c2f(temp_comm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_FREE(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_free_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_free(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_free_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_free_(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_free_fortran_wrapper(comm, ierr);
}

_EXTERN_C_ void mpi_comm_free__(MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Comm_free_fortran_wrapper(comm, ierr);
}

/* ================= End Wrappers for MPI_Comm_free ================= */


/* ================== C Wrappers for MPI_Comm_free_keyval ================== */
_EXTERN_C_ int PMPI_Comm_free_keyval(int *comm_keyval);
_EXTERN_C_ int MPI_Comm_free_keyval(int *comm_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_free_keyval at base level - Location = %px",&(MPI_Comm_free_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_free_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_free_keyval(comm_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_free_keyval(comm_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_free_keyval(comm_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_free_keyval =============== */
static void MPI_Comm_free_keyval_fortran_wrapper(MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Comm_free_keyval(((int*)comm_keyval));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_FREE_KEYVAL(MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_free_keyval_fortran_wrapper(comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_free_keyval(MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_free_keyval_fortran_wrapper(comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_free_keyval_(MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_free_keyval_fortran_wrapper(comm_keyval, ierr);
}

_EXTERN_C_ void mpi_comm_free_keyval__(MPI_Fint *comm_keyval, MPI_Fint *ierr) { 
    MPI_Comm_free_keyval_fortran_wrapper(comm_keyval, ierr);
}

/* ================= End Wrappers for MPI_Comm_free_keyval ================= */


/* ================== C Wrappers for MPI_Comm_get_attr ================== */
_EXTERN_C_ int PMPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_get_attr at base level - Location = %px",&(MPI_Comm_get_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_get_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_get_attr =============== */
static void MPI_Comm_get_attr_fortran_wrapper(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_get_attr((MPI_Comm)(*comm), *comm_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_get_attr(MPI_Comm_f2c(*comm), *comm_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GET_ATTR(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_get_attr_fortran_wrapper(comm, comm_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_comm_get_attr(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_get_attr_fortran_wrapper(comm, comm_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_comm_get_attr_(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_get_attr_fortran_wrapper(comm, comm_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_comm_get_attr__(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_get_attr_fortran_wrapper(comm, comm_keyval, attribute_val, flag, ierr);
}

/* ================= End Wrappers for MPI_Comm_get_attr ================= */


/* ================== C Wrappers for MPI_Comm_get_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);
_EXTERN_C_ int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_get_errhandler at base level - Location = %px",&(MPI_Comm_get_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_get_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_get_errhandler(comm, erhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_get_errhandler(comm, erhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_get_errhandler(comm, erhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_get_errhandler =============== */
static void MPI_Comm_get_errhandler_fortran_wrapper(MPI_Fint *comm, MPI_Fint *erhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_get_errhandler((MPI_Comm)(*comm), (MPI_Errhandler*)erhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_erhandler;
    _wrap_py_return_val = MPI_Comm_get_errhandler(MPI_Comm_f2c(*comm), &temp_erhandler);
    *erhandler = MPI_Errhandler_c2f(temp_erhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GET_ERRHANDLER(MPI_Fint *comm, MPI_Fint *erhandler, MPI_Fint *ierr) { 
    MPI_Comm_get_errhandler_fortran_wrapper(comm, erhandler, ierr);
}

_EXTERN_C_ void mpi_comm_get_errhandler(MPI_Fint *comm, MPI_Fint *erhandler, MPI_Fint *ierr) { 
    MPI_Comm_get_errhandler_fortran_wrapper(comm, erhandler, ierr);
}

_EXTERN_C_ void mpi_comm_get_errhandler_(MPI_Fint *comm, MPI_Fint *erhandler, MPI_Fint *ierr) { 
    MPI_Comm_get_errhandler_fortran_wrapper(comm, erhandler, ierr);
}

_EXTERN_C_ void mpi_comm_get_errhandler__(MPI_Fint *comm, MPI_Fint *erhandler, MPI_Fint *ierr) { 
    MPI_Comm_get_errhandler_fortran_wrapper(comm, erhandler, ierr);
}

/* ================= End Wrappers for MPI_Comm_get_errhandler ================= */


/* ================== C Wrappers for MPI_Comm_get_info ================== */
_EXTERN_C_ int PMPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used);
_EXTERN_C_ int MPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_get_info at base level - Location = %px",&(MPI_Comm_get_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_get_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_get_info(comm, info_used);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_get_info(comm, info_used);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_get_info(comm, info_used);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_get_info =============== */
static void MPI_Comm_get_info_fortran_wrapper(MPI_Fint *comm, MPI_Fint *info_used, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_get_info((MPI_Comm)(*comm), (MPI_Info*)info_used);
#else /* MPI-2 safe call */
    MPI_Info temp_info_used;
    _wrap_py_return_val = MPI_Comm_get_info(MPI_Comm_f2c(*comm), &temp_info_used);
    *info_used = MPI_Info_c2f(temp_info_used);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GET_INFO(MPI_Fint *comm, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Comm_get_info_fortran_wrapper(comm, info_used, ierr);
}

_EXTERN_C_ void mpi_comm_get_info(MPI_Fint *comm, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Comm_get_info_fortran_wrapper(comm, info_used, ierr);
}

_EXTERN_C_ void mpi_comm_get_info_(MPI_Fint *comm, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Comm_get_info_fortran_wrapper(comm, info_used, ierr);
}

_EXTERN_C_ void mpi_comm_get_info__(MPI_Fint *comm, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Comm_get_info_fortran_wrapper(comm, info_used, ierr);
}

/* ================= End Wrappers for MPI_Comm_get_info ================= */


/* ================== C Wrappers for MPI_Comm_get_name ================== */
_EXTERN_C_ int PMPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);
_EXTERN_C_ int MPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_get_name at base level - Location = %px",&(MPI_Comm_get_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_get_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_get_name(comm, comm_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_get_name(comm, comm_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_get_name(comm, comm_name, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_get_name =============== */
static void MPI_Comm_get_name_fortran_wrapper(MPI_Fint *comm, char *comm_name, MPI_Fint *resultlen, MPI_Fint *ierr, int comm_name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_get_name((MPI_Comm)(*comm), comm_name, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_comm_name;
    temp_comm_name = (char*)malloc(sizeof(char) * (comm_name_len+1));
    char_p_f2c(comm_name,comm_name_len,&temp_comm_name);
    _wrap_py_return_val = MPI_Comm_get_name(MPI_Comm_f2c(*comm), comm_name, ((int*)resultlen));
    char_p_c2f(temp_comm_name,comm_name,comm_name_len);
 if(temp_comm_name) free(temp_comm_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GET_NAME(MPI_Fint *comm, char *comm_name, MPI_Fint *resultlen, MPI_Fint *ierr, int comm_name_len) { 
    MPI_Comm_get_name_fortran_wrapper(comm, comm_name, resultlen, ierr, comm_name_len);
}

_EXTERN_C_ void mpi_comm_get_name(MPI_Fint *comm, char *comm_name, MPI_Fint *resultlen, MPI_Fint *ierr, int comm_name_len) { 
    MPI_Comm_get_name_fortran_wrapper(comm, comm_name, resultlen, ierr, comm_name_len);
}

_EXTERN_C_ void mpi_comm_get_name_(MPI_Fint *comm, char *comm_name, MPI_Fint *resultlen, MPI_Fint *ierr, int comm_name_len) { 
    MPI_Comm_get_name_fortran_wrapper(comm, comm_name, resultlen, ierr, comm_name_len);
}

_EXTERN_C_ void mpi_comm_get_name__(MPI_Fint *comm, char *comm_name, MPI_Fint *resultlen, MPI_Fint *ierr, int comm_name_len) { 
    MPI_Comm_get_name_fortran_wrapper(comm, comm_name, resultlen, ierr, comm_name_len);
}

/* ================= End Wrappers for MPI_Comm_get_name ================= */


/* ================== C Wrappers for MPI_Comm_get_parent ================== */
_EXTERN_C_ int PMPI_Comm_get_parent(MPI_Comm *parent);
_EXTERN_C_ int MPI_Comm_get_parent(MPI_Comm *parent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_get_parent at base level - Location = %px",&(MPI_Comm_get_parent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_get_parent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_get_parent(parent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_get_parent(parent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_get_parent(parent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_get_parent =============== */
static void MPI_Comm_get_parent_fortran_wrapper(MPI_Fint *parent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_get_parent((MPI_Comm*)parent);
#else /* MPI-2 safe call */
    MPI_Comm temp_parent;
    _wrap_py_return_val = MPI_Comm_get_parent(&temp_parent);
    *parent = MPI_Comm_c2f(temp_parent);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GET_PARENT(MPI_Fint *parent, MPI_Fint *ierr) { 
    MPI_Comm_get_parent_fortran_wrapper(parent, ierr);
}

_EXTERN_C_ void mpi_comm_get_parent(MPI_Fint *parent, MPI_Fint *ierr) { 
    MPI_Comm_get_parent_fortran_wrapper(parent, ierr);
}

_EXTERN_C_ void mpi_comm_get_parent_(MPI_Fint *parent, MPI_Fint *ierr) { 
    MPI_Comm_get_parent_fortran_wrapper(parent, ierr);
}

_EXTERN_C_ void mpi_comm_get_parent__(MPI_Fint *parent, MPI_Fint *ierr) { 
    MPI_Comm_get_parent_fortran_wrapper(parent, ierr);
}

/* ================= End Wrappers for MPI_Comm_get_parent ================= */


/* ================== C Wrappers for MPI_Comm_group ================== */
_EXTERN_C_ int PMPI_Comm_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_group at base level - Location = %px",&(MPI_Comm_group));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_group_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_group(comm, group);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_group(comm, group);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_group(comm, group);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_group =============== */
static void MPI_Comm_group_fortran_wrapper(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_group((MPI_Comm)(*comm), (MPI_Group*)group);
#else /* MPI-2 safe call */
    MPI_Group temp_group;
    _wrap_py_return_val = MPI_Comm_group(MPI_Comm_f2c(*comm), &temp_group);
    *group = MPI_Group_c2f(temp_group);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_GROUP(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_group(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_group_(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_group__(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_group_fortran_wrapper(comm, group, ierr);
}

/* ================= End Wrappers for MPI_Comm_group ================= */


/* ================== C Wrappers for MPI_Comm_idup ================== */
_EXTERN_C_ int PMPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);
_EXTERN_C_ int MPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_idup at base level - Location = %px",&(MPI_Comm_idup));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_idup_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_idup(comm, newcomm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_idup(comm, newcomm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_idup(comm, newcomm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_idup =============== */
static void MPI_Comm_idup_fortran_wrapper(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_idup((MPI_Comm)(*comm), (MPI_Comm*)newcomm, (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_idup(MPI_Comm_f2c(*comm), &temp_newcomm, &temp_request);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_IDUP(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Comm_idup_fortran_wrapper(comm, newcomm, request, ierr);
}

_EXTERN_C_ void mpi_comm_idup(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Comm_idup_fortran_wrapper(comm, newcomm, request, ierr);
}

_EXTERN_C_ void mpi_comm_idup_(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Comm_idup_fortran_wrapper(comm, newcomm, request, ierr);
}

_EXTERN_C_ void mpi_comm_idup__(MPI_Fint *comm, MPI_Fint *newcomm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Comm_idup_fortran_wrapper(comm, newcomm, request, ierr);
}

/* ================= End Wrappers for MPI_Comm_idup ================= */


/* ================== C Wrappers for MPI_Comm_join ================== */
_EXTERN_C_ int PMPI_Comm_join(int fd, MPI_Comm *intercomm);
_EXTERN_C_ int MPI_Comm_join(int fd, MPI_Comm *intercomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_join at base level - Location = %px",&(MPI_Comm_join));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_join_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_join(fd, intercomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_join(fd, intercomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_join(fd, intercomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_join =============== */
static void MPI_Comm_join_fortran_wrapper(MPI_Fint *fd, MPI_Fint *intercomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_join(*fd, (MPI_Comm*)intercomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_intercomm;
    _wrap_py_return_val = MPI_Comm_join(*fd, &temp_intercomm);
    *intercomm = MPI_Comm_c2f(temp_intercomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_JOIN(MPI_Fint *fd, MPI_Fint *intercomm, MPI_Fint *ierr) { 
    MPI_Comm_join_fortran_wrapper(fd, intercomm, ierr);
}

_EXTERN_C_ void mpi_comm_join(MPI_Fint *fd, MPI_Fint *intercomm, MPI_Fint *ierr) { 
    MPI_Comm_join_fortran_wrapper(fd, intercomm, ierr);
}

_EXTERN_C_ void mpi_comm_join_(MPI_Fint *fd, MPI_Fint *intercomm, MPI_Fint *ierr) { 
    MPI_Comm_join_fortran_wrapper(fd, intercomm, ierr);
}

_EXTERN_C_ void mpi_comm_join__(MPI_Fint *fd, MPI_Fint *intercomm, MPI_Fint *ierr) { 
    MPI_Comm_join_fortran_wrapper(fd, intercomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_join ================= */


/* ================== C Wrappers for MPI_Comm_rank ================== */
_EXTERN_C_ int PMPI_Comm_rank(MPI_Comm comm, int *rank);
_EXTERN_C_ int MPI_Comm_rank(MPI_Comm comm, int *rank) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_rank at base level - Location = %px",&(MPI_Comm_rank));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_rank_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_rank(comm, rank);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_rank(comm, rank);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_rank(comm, rank);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_rank =============== */
static void MPI_Comm_rank_fortran_wrapper(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_rank((MPI_Comm)(*comm), ((int*)rank));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_rank(MPI_Comm_f2c(*comm), ((int*)rank));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_RANK(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Comm_rank_fortran_wrapper(comm, rank, ierr);
}

_EXTERN_C_ void mpi_comm_rank(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Comm_rank_fortran_wrapper(comm, rank, ierr);
}

_EXTERN_C_ void mpi_comm_rank_(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Comm_rank_fortran_wrapper(comm, rank, ierr);
}

_EXTERN_C_ void mpi_comm_rank__(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Comm_rank_fortran_wrapper(comm, rank, ierr);
}

/* ================= End Wrappers for MPI_Comm_rank ================= */


/* ================== C Wrappers for MPI_Comm_remote_group ================== */
_EXTERN_C_ int PMPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_remote_group at base level - Location = %px",&(MPI_Comm_remote_group));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_remote_group_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_remote_group(comm, group);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_remote_group(comm, group);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_remote_group(comm, group);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_remote_group =============== */
static void MPI_Comm_remote_group_fortran_wrapper(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_remote_group((MPI_Comm)(*comm), (MPI_Group*)group);
#else /* MPI-2 safe call */
    MPI_Group temp_group;
    _wrap_py_return_val = MPI_Comm_remote_group(MPI_Comm_f2c(*comm), &temp_group);
    *group = MPI_Group_c2f(temp_group);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_REMOTE_GROUP(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_remote_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_remote_group(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_remote_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_remote_group_(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_remote_group_fortran_wrapper(comm, group, ierr);
}

_EXTERN_C_ void mpi_comm_remote_group__(MPI_Fint *comm, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Comm_remote_group_fortran_wrapper(comm, group, ierr);
}

/* ================= End Wrappers for MPI_Comm_remote_group ================= */


/* ================== C Wrappers for MPI_Comm_remote_size ================== */
_EXTERN_C_ int PMPI_Comm_remote_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_remote_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_remote_size at base level - Location = %px",&(MPI_Comm_remote_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_remote_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_remote_size(comm, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_remote_size(comm, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_remote_size(comm, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_remote_size =============== */
static void MPI_Comm_remote_size_fortran_wrapper(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_remote_size((MPI_Comm)(*comm), ((int*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_remote_size(MPI_Comm_f2c(*comm), ((int*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_REMOTE_SIZE(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_remote_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_remote_size(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_remote_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_remote_size_(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_remote_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_remote_size__(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_remote_size_fortran_wrapper(comm, size, ierr);
}

/* ================= End Wrappers for MPI_Comm_remote_size ================= */


/* ================== C Wrappers for MPI_Comm_set_attr ================== */
_EXTERN_C_ int PMPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_set_attr at base level - Location = %px",&(MPI_Comm_set_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_set_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_set_attr =============== */
static void MPI_Comm_set_attr_fortran_wrapper(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_set_attr((MPI_Comm)(*comm), *comm_keyval, BufferC2F((void*)attribute_val));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_set_attr(MPI_Comm_f2c(*comm), *comm_keyval, BufferC2F((void*)attribute_val));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SET_ATTR(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Comm_set_attr_fortran_wrapper(comm, comm_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_comm_set_attr(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Comm_set_attr_fortran_wrapper(comm, comm_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_comm_set_attr_(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Comm_set_attr_fortran_wrapper(comm, comm_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_comm_set_attr__(MPI_Fint *comm, MPI_Fint *comm_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Comm_set_attr_fortran_wrapper(comm, comm_keyval, attribute_val, ierr);
}

/* ================= End Wrappers for MPI_Comm_set_attr ================= */


/* ================== C Wrappers for MPI_Comm_set_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_set_errhandler at base level - Location = %px",&(MPI_Comm_set_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_set_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_set_errhandler(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_set_errhandler(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_set_errhandler(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_set_errhandler =============== */
static void MPI_Comm_set_errhandler_fortran_wrapper(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_set_errhandler((MPI_Comm)(*comm), (MPI_Errhandler)(*errhandler));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_set_errhandler(MPI_Comm_f2c(*comm), MPI_Errhandler_f2c(*errhandler));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SET_ERRHANDLER(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_set_errhandler_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_set_errhandler(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_set_errhandler_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_set_errhandler_(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_set_errhandler_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_comm_set_errhandler__(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Comm_set_errhandler_fortran_wrapper(comm, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Comm_set_errhandler ================= */


/* ================== C Wrappers for MPI_Comm_set_info ================== */
_EXTERN_C_ int PMPI_Comm_set_info(MPI_Comm comm, MPI_Info info);
_EXTERN_C_ int MPI_Comm_set_info(MPI_Comm comm, MPI_Info info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_set_info at base level - Location = %px",&(MPI_Comm_set_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_set_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_set_info(comm, info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_set_info(comm, info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_set_info(comm, info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_set_info =============== */
static void MPI_Comm_set_info_fortran_wrapper(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_set_info((MPI_Comm)(*comm), (MPI_Info)(*info));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_set_info(MPI_Comm_f2c(*comm), MPI_Info_f2c(*info));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SET_INFO(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Comm_set_info_fortran_wrapper(comm, info, ierr);
}

_EXTERN_C_ void mpi_comm_set_info(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Comm_set_info_fortran_wrapper(comm, info, ierr);
}

_EXTERN_C_ void mpi_comm_set_info_(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Comm_set_info_fortran_wrapper(comm, info, ierr);
}

_EXTERN_C_ void mpi_comm_set_info__(MPI_Fint *comm, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Comm_set_info_fortran_wrapper(comm, info, ierr);
}

/* ================= End Wrappers for MPI_Comm_set_info ================= */


/* ================== C Wrappers for MPI_Comm_set_name ================== */
_EXTERN_C_ int PMPI_Comm_set_name(MPI_Comm comm, const char *comm_name);
_EXTERN_C_ int MPI_Comm_set_name(MPI_Comm comm, const char *comm_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_set_name at base level - Location = %px",&(MPI_Comm_set_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_set_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_set_name(comm, comm_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_set_name(comm, comm_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_set_name(comm, comm_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_set_name =============== */
static void MPI_Comm_set_name_fortran_wrapper(MPI_Fint *comm, MPI_Fint *comm_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_set_name((MPI_Comm)(*comm), ((const char*)comm_name));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_set_name(MPI_Comm_f2c(*comm), ((const char*)comm_name));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SET_NAME(MPI_Fint *comm, MPI_Fint *comm_name, MPI_Fint *ierr) { 
    MPI_Comm_set_name_fortran_wrapper(comm, comm_name, ierr);
}

_EXTERN_C_ void mpi_comm_set_name(MPI_Fint *comm, MPI_Fint *comm_name, MPI_Fint *ierr) { 
    MPI_Comm_set_name_fortran_wrapper(comm, comm_name, ierr);
}

_EXTERN_C_ void mpi_comm_set_name_(MPI_Fint *comm, MPI_Fint *comm_name, MPI_Fint *ierr) { 
    MPI_Comm_set_name_fortran_wrapper(comm, comm_name, ierr);
}

_EXTERN_C_ void mpi_comm_set_name__(MPI_Fint *comm, MPI_Fint *comm_name, MPI_Fint *ierr) { 
    MPI_Comm_set_name_fortran_wrapper(comm, comm_name, ierr);
}

/* ================= End Wrappers for MPI_Comm_set_name ================= */


/* ================== C Wrappers for MPI_Comm_size ================== */
_EXTERN_C_ int PMPI_Comm_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_size at base level - Location = %px",&(MPI_Comm_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_size(comm, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_size(comm, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_size(comm, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_size =============== */
static void MPI_Comm_size_fortran_wrapper(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_size((MPI_Comm)(*comm), ((int*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_size(MPI_Comm_f2c(*comm), ((int*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SIZE(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_size(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_size_(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_size_fortran_wrapper(comm, size, ierr);
}

_EXTERN_C_ void mpi_comm_size__(MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Comm_size_fortran_wrapper(comm, size, ierr);
}

/* ================= End Wrappers for MPI_Comm_size ================= */


/* ================== C Wrappers for MPI_Comm_split ================== */
_EXTERN_C_ int PMPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_split at base level - Location = %px",&(MPI_Comm_split));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_split_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_split(comm, color, key, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_split(comm, color, key, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_split(comm, color, key, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_split =============== */
static void MPI_Comm_split_fortran_wrapper(MPI_Fint *comm, MPI_Fint *color, MPI_Fint *key, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_split((MPI_Comm)(*comm), *color, *key, (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_split(MPI_Comm_f2c(*comm), *color, *key, &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SPLIT(MPI_Fint *comm, MPI_Fint *color, MPI_Fint *key, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_fortran_wrapper(comm, color, key, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split(MPI_Fint *comm, MPI_Fint *color, MPI_Fint *key, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_fortran_wrapper(comm, color, key, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split_(MPI_Fint *comm, MPI_Fint *color, MPI_Fint *key, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_fortran_wrapper(comm, color, key, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split__(MPI_Fint *comm, MPI_Fint *color, MPI_Fint *key, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_fortran_wrapper(comm, color, key, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_split ================= */


/* ================== C Wrappers for MPI_Comm_split_type ================== */
_EXTERN_C_ int PMPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_split_type at base level - Location = %px",&(MPI_Comm_split_type));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_split_type_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_split_type(comm, split_type, key, info, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_split_type(comm, split_type, key, info, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_split_type(comm, split_type, key, info, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_split_type =============== */
static void MPI_Comm_split_type_fortran_wrapper(MPI_Fint *comm, MPI_Fint *split_type, MPI_Fint *key, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_split_type((MPI_Comm)(*comm), *split_type, *key, (MPI_Info)(*info), (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Comm_split_type(MPI_Comm_f2c(*comm), *split_type, *key, MPI_Info_f2c(*info), &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_SPLIT_TYPE(MPI_Fint *comm, MPI_Fint *split_type, MPI_Fint *key, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_type_fortran_wrapper(comm, split_type, key, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split_type(MPI_Fint *comm, MPI_Fint *split_type, MPI_Fint *key, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_type_fortran_wrapper(comm, split_type, key, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split_type_(MPI_Fint *comm, MPI_Fint *split_type, MPI_Fint *key, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_type_fortran_wrapper(comm, split_type, key, info, newcomm, ierr);
}

_EXTERN_C_ void mpi_comm_split_type__(MPI_Fint *comm, MPI_Fint *split_type, MPI_Fint *key, MPI_Fint *info, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Comm_split_type_fortran_wrapper(comm, split_type, key, info, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Comm_split_type ================= */


/* ================== C Wrappers for MPI_Comm_test_inter ================== */
_EXTERN_C_ int PMPI_Comm_test_inter(MPI_Comm comm, int *flag);
_EXTERN_C_ int MPI_Comm_test_inter(MPI_Comm comm, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Comm_test_inter at base level - Location = %px",&(MPI_Comm_test_inter));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Comm_test_inter_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Comm_test_inter(comm, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Comm_test_inter(comm, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Comm_test_inter(comm, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Comm_test_inter =============== */
static void MPI_Comm_test_inter_fortran_wrapper(MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Comm_test_inter((MPI_Comm)(*comm), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Comm_test_inter(MPI_Comm_f2c(*comm), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMM_TEST_INTER(MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_test_inter_fortran_wrapper(comm, flag, ierr);
}

_EXTERN_C_ void mpi_comm_test_inter(MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_test_inter_fortran_wrapper(comm, flag, ierr);
}

_EXTERN_C_ void mpi_comm_test_inter_(MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_test_inter_fortran_wrapper(comm, flag, ierr);
}

_EXTERN_C_ void mpi_comm_test_inter__(MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Comm_test_inter_fortran_wrapper(comm, flag, ierr);
}

/* ================= End Wrappers for MPI_Comm_test_inter ================= */


/* ================== C Wrappers for MPI_Compare_and_swap ================== */
_EXTERN_C_ int PMPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);
_EXTERN_C_ int MPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Compare_and_swap at base level - Location = %px",&(MPI_Compare_and_swap));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Compare_and_swap_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Compare_and_swap =============== */
static void MPI_Compare_and_swap_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *compare_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Compare_and_swap(((const void*)origin_addr), ((const void*)compare_addr), BufferC2F((void*)result_addr), (MPI_Datatype)(*datatype), *target_rank, *target_disp, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Compare_and_swap(((const void*)origin_addr), ((const void*)compare_addr), BufferC2F((void*)result_addr), MPI_Type_f2c(*datatype), *target_rank, *target_disp, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_COMPARE_AND_SWAP(MPI_Fint *origin_addr, MPI_Fint *compare_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Compare_and_swap_fortran_wrapper(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win, ierr);
}

_EXTERN_C_ void mpi_compare_and_swap(MPI_Fint *origin_addr, MPI_Fint *compare_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Compare_and_swap_fortran_wrapper(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win, ierr);
}

_EXTERN_C_ void mpi_compare_and_swap_(MPI_Fint *origin_addr, MPI_Fint *compare_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Compare_and_swap_fortran_wrapper(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win, ierr);
}

_EXTERN_C_ void mpi_compare_and_swap__(MPI_Fint *origin_addr, MPI_Fint *compare_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Compare_and_swap_fortran_wrapper(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win, ierr);
}

/* ================= End Wrappers for MPI_Compare_and_swap ================= */


/* ================== C Wrappers for MPI_Dims_create ================== */
_EXTERN_C_ int PMPI_Dims_create(int nnodes, int ndims, int dims[]);
_EXTERN_C_ int MPI_Dims_create(int nnodes, int ndims, int dims[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Dims_create at base level - Location = %px",&(MPI_Dims_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Dims_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Dims_create(nnodes, ndims, dims);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Dims_create(nnodes, ndims, dims);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Dims_create(nnodes, ndims, dims);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Dims_create =============== */
static void MPI_Dims_create_fortran_wrapper(MPI_Fint *nnodes, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Dims_create(*nnodes, *ndims, ((int*)dims));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_DIMS_CREATE(MPI_Fint *nnodes, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint *ierr) { 
    MPI_Dims_create_fortran_wrapper(nnodes, ndims, dims, ierr);
}

_EXTERN_C_ void mpi_dims_create(MPI_Fint *nnodes, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint *ierr) { 
    MPI_Dims_create_fortran_wrapper(nnodes, ndims, dims, ierr);
}

_EXTERN_C_ void mpi_dims_create_(MPI_Fint *nnodes, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint *ierr) { 
    MPI_Dims_create_fortran_wrapper(nnodes, ndims, dims, ierr);
}

_EXTERN_C_ void mpi_dims_create__(MPI_Fint *nnodes, MPI_Fint *ndims, MPI_Fint dims[], MPI_Fint *ierr) { 
    MPI_Dims_create_fortran_wrapper(nnodes, ndims, dims, ierr);
}

/* ================= End Wrappers for MPI_Dims_create ================= */


/* ================== C Wrappers for MPI_Dist_graph_create ================== */
_EXTERN_C_ int PMPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Dist_graph_create at base level - Location = %px",&(MPI_Dist_graph_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Dist_graph_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Dist_graph_create =============== */
static void MPI_Dist_graph_create_fortran_wrapper(MPI_Fint *comm_old, MPI_Fint *n, MPI_Fint nodes[], MPI_Fint degrees[], MPI_Fint targets[], MPI_Fint weights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Dist_graph_create((MPI_Comm)(*comm_old), *n, ((const int*)nodes), ((const int*)degrees), ((const int*)targets), ((const int*)weights), (MPI_Info)(*info), *reorder, (MPI_Comm*)newcomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newcomm;
    _wrap_py_return_val = MPI_Dist_graph_create(MPI_Comm_f2c(*comm_old), *n, ((const int*)nodes), ((const int*)degrees), ((const int*)targets), ((const int*)weights), MPI_Info_f2c(*info), *reorder, &temp_newcomm);
    *newcomm = MPI_Comm_c2f(temp_newcomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_DIST_GRAPH_CREATE(MPI_Fint *comm_old, MPI_Fint *n, MPI_Fint nodes[], MPI_Fint degrees[], MPI_Fint targets[], MPI_Fint weights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_fortran_wrapper(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create(MPI_Fint *comm_old, MPI_Fint *n, MPI_Fint nodes[], MPI_Fint degrees[], MPI_Fint targets[], MPI_Fint weights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_fortran_wrapper(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create_(MPI_Fint *comm_old, MPI_Fint *n, MPI_Fint nodes[], MPI_Fint degrees[], MPI_Fint targets[], MPI_Fint weights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_fortran_wrapper(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create__(MPI_Fint *comm_old, MPI_Fint *n, MPI_Fint nodes[], MPI_Fint degrees[], MPI_Fint targets[], MPI_Fint weights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *newcomm, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_fortran_wrapper(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm, ierr);
}

/* ================= End Wrappers for MPI_Dist_graph_create ================= */


/* ================== C Wrappers for MPI_Dist_graph_create_adjacent ================== */
_EXTERN_C_ int PMPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);
_EXTERN_C_ int MPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Dist_graph_create_adjacent at base level - Location = %px",&(MPI_Dist_graph_create_adjacent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Dist_graph_create_adjacent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Dist_graph_create_adjacent =============== */
static void MPI_Dist_graph_create_adjacent_fortran_wrapper(MPI_Fint *comm_old, MPI_Fint *indegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *outdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *comm_dist_graph, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Dist_graph_create_adjacent((MPI_Comm)(*comm_old), *indegree, ((const int*)sources), ((const int*)sourceweights), *outdegree, ((const int*)destinations), ((const int*)destweights), (MPI_Info)(*info), *reorder, (MPI_Comm*)comm_dist_graph);
#else /* MPI-2 safe call */
    MPI_Comm temp_comm_dist_graph;
    _wrap_py_return_val = MPI_Dist_graph_create_adjacent(MPI_Comm_f2c(*comm_old), *indegree, ((const int*)sources), ((const int*)sourceweights), *outdegree, ((const int*)destinations), ((const int*)destweights), MPI_Info_f2c(*info), *reorder, &temp_comm_dist_graph);
    *comm_dist_graph = MPI_Comm_c2f(temp_comm_dist_graph);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_DIST_GRAPH_CREATE_ADJACENT(MPI_Fint *comm_old, MPI_Fint *indegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *outdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *comm_dist_graph, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_adjacent_fortran_wrapper(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create_adjacent(MPI_Fint *comm_old, MPI_Fint *indegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *outdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *comm_dist_graph, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_adjacent_fortran_wrapper(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create_adjacent_(MPI_Fint *comm_old, MPI_Fint *indegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *outdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *comm_dist_graph, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_adjacent_fortran_wrapper(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph, ierr);
}

_EXTERN_C_ void mpi_dist_graph_create_adjacent__(MPI_Fint *comm_old, MPI_Fint *indegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *outdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *info, MPI_Fint *reorder, MPI_Fint *comm_dist_graph, MPI_Fint *ierr) { 
    MPI_Dist_graph_create_adjacent_fortran_wrapper(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph, ierr);
}

/* ================= End Wrappers for MPI_Dist_graph_create_adjacent ================= */


/* ================== C Wrappers for MPI_Dist_graph_neighbors ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);
_EXTERN_C_ int MPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Dist_graph_neighbors at base level - Location = %px",&(MPI_Dist_graph_neighbors));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Dist_graph_neighbors_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Dist_graph_neighbors =============== */
static void MPI_Dist_graph_neighbors_fortran_wrapper(MPI_Fint *comm, MPI_Fint *maxindegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *maxoutdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Dist_graph_neighbors((MPI_Comm)(*comm), *maxindegree, ((int*)sources), ((int*)sourceweights), *maxoutdegree, ((int*)destinations), ((int*)destweights));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Dist_graph_neighbors(MPI_Comm_f2c(*comm), *maxindegree, ((int*)sources), ((int*)sourceweights), *maxoutdegree, ((int*)destinations), ((int*)destweights));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_DIST_GRAPH_NEIGHBORS(MPI_Fint *comm, MPI_Fint *maxindegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *maxoutdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_fortran_wrapper(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors(MPI_Fint *comm, MPI_Fint *maxindegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *maxoutdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_fortran_wrapper(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors_(MPI_Fint *comm, MPI_Fint *maxindegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *maxoutdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_fortran_wrapper(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors__(MPI_Fint *comm, MPI_Fint *maxindegree, MPI_Fint sources[], MPI_Fint sourceweights[], MPI_Fint *maxoutdegree, MPI_Fint destinations[], MPI_Fint destweights[], MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_fortran_wrapper(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights, ierr);
}

/* ================= End Wrappers for MPI_Dist_graph_neighbors ================= */


/* ================== C Wrappers for MPI_Dist_graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);
_EXTERN_C_ int MPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Dist_graph_neighbors_count at base level - Location = %px",&(MPI_Dist_graph_neighbors_count));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Dist_graph_neighbors_count_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Dist_graph_neighbors_count =============== */
static void MPI_Dist_graph_neighbors_count_fortran_wrapper(MPI_Fint *comm, MPI_Fint *inneighbors, MPI_Fint *outneighbors, MPI_Fint *weighted, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Dist_graph_neighbors_count((MPI_Comm)(*comm), ((int*)inneighbors), ((int*)outneighbors), ((int*)weighted));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Dist_graph_neighbors_count(MPI_Comm_f2c(*comm), ((int*)inneighbors), ((int*)outneighbors), ((int*)weighted));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_DIST_GRAPH_NEIGHBORS_COUNT(MPI_Fint *comm, MPI_Fint *inneighbors, MPI_Fint *outneighbors, MPI_Fint *weighted, MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_count_fortran_wrapper(comm, inneighbors, outneighbors, weighted, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors_count(MPI_Fint *comm, MPI_Fint *inneighbors, MPI_Fint *outneighbors, MPI_Fint *weighted, MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_count_fortran_wrapper(comm, inneighbors, outneighbors, weighted, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors_count_(MPI_Fint *comm, MPI_Fint *inneighbors, MPI_Fint *outneighbors, MPI_Fint *weighted, MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_count_fortran_wrapper(comm, inneighbors, outneighbors, weighted, ierr);
}

_EXTERN_C_ void mpi_dist_graph_neighbors_count__(MPI_Fint *comm, MPI_Fint *inneighbors, MPI_Fint *outneighbors, MPI_Fint *weighted, MPI_Fint *ierr) { 
    MPI_Dist_graph_neighbors_count_fortran_wrapper(comm, inneighbors, outneighbors, weighted, ierr);
}

/* ================= End Wrappers for MPI_Dist_graph_neighbors_count ================= */


/* ================== C Wrappers for MPI_Errhandler_create ================== */
_EXTERN_C_ int PMPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Errhandler_create at base level - Location = %px",&(MPI_Errhandler_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Errhandler_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Errhandler_create(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Errhandler_create(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Errhandler_create(function, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Errhandler_create =============== */
static void MPI_Errhandler_create_fortran_wrapper(MPI_Handler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Errhandler_create(((MPI_Handler_function*)function), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_Errhandler_create(((MPI_Handler_function*)function), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERRHANDLER_CREATE(MPI_Handler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_create_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_create(MPI_Handler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_create_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_create_(MPI_Handler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_create_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_create__(MPI_Handler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_create_fortran_wrapper(function, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Errhandler_create ================= */


/* ================== C Wrappers for MPI_Errhandler_free ================== */
_EXTERN_C_ int PMPI_Errhandler_free(MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_free(MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Errhandler_free at base level - Location = %px",&(MPI_Errhandler_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Errhandler_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Errhandler_free(errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Errhandler_free(errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Errhandler_free(errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Errhandler_free =============== */
static void MPI_Errhandler_free_fortran_wrapper(MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Errhandler_free((MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    temp_errhandler = MPI_Errhandler_f2c(*errhandler);
    _wrap_py_return_val = MPI_Errhandler_free(&temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERRHANDLER_FREE(MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_free_fortran_wrapper(errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_free(MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_free_fortran_wrapper(errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_free_(MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_free_fortran_wrapper(errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_free__(MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_free_fortran_wrapper(errhandler, ierr);
}

/* ================= End Wrappers for MPI_Errhandler_free ================= */


/* ================== C Wrappers for MPI_Errhandler_get ================== */
_EXTERN_C_ int PMPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Errhandler_get at base level - Location = %px",&(MPI_Errhandler_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Errhandler_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Errhandler_get(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Errhandler_get(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Errhandler_get(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Errhandler_get =============== */
static void MPI_Errhandler_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Errhandler_get((MPI_Comm)(*comm), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_Errhandler_get(MPI_Comm_f2c(*comm), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERRHANDLER_GET(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_get_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_get(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_get_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_get_(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_get_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_get__(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_get_fortran_wrapper(comm, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Errhandler_get ================= */


/* ================== C Wrappers for MPI_Errhandler_set ================== */
_EXTERN_C_ int PMPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Errhandler_set at base level - Location = %px",&(MPI_Errhandler_set));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Errhandler_set_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Errhandler_set(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Errhandler_set(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Errhandler_set(comm, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Errhandler_set =============== */
static void MPI_Errhandler_set_fortran_wrapper(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Errhandler_set((MPI_Comm)(*comm), (MPI_Errhandler)(*errhandler));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Errhandler_set(MPI_Comm_f2c(*comm), MPI_Errhandler_f2c(*errhandler));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERRHANDLER_SET(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_set_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_set(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_set_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_set_(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_set_fortran_wrapper(comm, errhandler, ierr);
}

_EXTERN_C_ void mpi_errhandler_set__(MPI_Fint *comm, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Errhandler_set_fortran_wrapper(comm, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Errhandler_set ================= */


/* ================== C Wrappers for MPI_Error_class ================== */
_EXTERN_C_ int PMPI_Error_class(int errorcode, int *errorclass);
_EXTERN_C_ int MPI_Error_class(int errorcode, int *errorclass) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Error_class at base level - Location = %px",&(MPI_Error_class));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Error_class_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Error_class(errorcode, errorclass);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Error_class(errorcode, errorclass);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Error_class(errorcode, errorclass);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Error_class =============== */
static void MPI_Error_class_fortran_wrapper(MPI_Fint *errorcode, MPI_Fint *errorclass, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Error_class(*errorcode, ((int*)errorclass));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERROR_CLASS(MPI_Fint *errorcode, MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Error_class_fortran_wrapper(errorcode, errorclass, ierr);
}

_EXTERN_C_ void mpi_error_class(MPI_Fint *errorcode, MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Error_class_fortran_wrapper(errorcode, errorclass, ierr);
}

_EXTERN_C_ void mpi_error_class_(MPI_Fint *errorcode, MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Error_class_fortran_wrapper(errorcode, errorclass, ierr);
}

_EXTERN_C_ void mpi_error_class__(MPI_Fint *errorcode, MPI_Fint *errorclass, MPI_Fint *ierr) { 
    MPI_Error_class_fortran_wrapper(errorcode, errorclass, ierr);
}

/* ================= End Wrappers for MPI_Error_class ================= */


/* ================== C Wrappers for MPI_Error_string ================== */
_EXTERN_C_ int PMPI_Error_string(int errorcode, char *string, int *resultlen);
_EXTERN_C_ int MPI_Error_string(int errorcode, char *string, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Error_string at base level - Location = %px",&(MPI_Error_string));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Error_string_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Error_string(errorcode, string, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Error_string(errorcode, string, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Error_string(errorcode, string, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Error_string =============== */
static void MPI_Error_string_fortran_wrapper(MPI_Fint *errorcode, char *string, MPI_Fint *resultlen, MPI_Fint *ierr, int string_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Error_string(*errorcode, string, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_string;
    temp_string = (char*)malloc(sizeof(char) * (string_len+1));
    char_p_f2c(string,string_len,&temp_string);
    _wrap_py_return_val = MPI_Error_string(*errorcode, string, ((int*)resultlen));
    char_p_c2f(temp_string,string,string_len);
 if(temp_string) free(temp_string);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ERROR_STRING(MPI_Fint *errorcode, char *string, MPI_Fint *resultlen, MPI_Fint *ierr, int string_len) { 
    MPI_Error_string_fortran_wrapper(errorcode, string, resultlen, ierr, string_len);
}

_EXTERN_C_ void mpi_error_string(MPI_Fint *errorcode, char *string, MPI_Fint *resultlen, MPI_Fint *ierr, int string_len) { 
    MPI_Error_string_fortran_wrapper(errorcode, string, resultlen, ierr, string_len);
}

_EXTERN_C_ void mpi_error_string_(MPI_Fint *errorcode, char *string, MPI_Fint *resultlen, MPI_Fint *ierr, int string_len) { 
    MPI_Error_string_fortran_wrapper(errorcode, string, resultlen, ierr, string_len);
}

_EXTERN_C_ void mpi_error_string__(MPI_Fint *errorcode, char *string, MPI_Fint *resultlen, MPI_Fint *ierr, int string_len) { 
    MPI_Error_string_fortran_wrapper(errorcode, string, resultlen, ierr, string_len);
}

/* ================= End Wrappers for MPI_Error_string ================= */


/* ================== C Wrappers for MPI_Exscan ================== */
_EXTERN_C_ int PMPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Exscan at base level - Location = %px",&(MPI_Exscan));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Exscan_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Exscan =============== */
static void MPI_Exscan_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Exscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Exscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_EXSCAN(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Exscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_exscan(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Exscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_exscan_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Exscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_exscan__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Exscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

/* ================= End Wrappers for MPI_Exscan ================= */


/* ================== C Wrappers for MPI_Fetch_and_op ================== */
_EXTERN_C_ int PMPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Fetch_and_op at base level - Location = %px",&(MPI_Fetch_and_op));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Fetch_and_op_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Fetch_and_op =============== */
static void MPI_Fetch_and_op_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Fetch_and_op(((const void*)origin_addr), BufferC2F((void*)result_addr), (MPI_Datatype)(*datatype), *target_rank, *target_disp, (MPI_Op)(*op), (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Fetch_and_op(((const void*)origin_addr), BufferC2F((void*)result_addr), MPI_Type_f2c(*datatype), *target_rank, *target_disp, MPI_Op_f2c(*op), MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FETCH_AND_OP(MPI_Fint *origin_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Fetch_and_op_fortran_wrapper(origin_addr, result_addr, datatype, target_rank, target_disp, op, win, ierr);
}

_EXTERN_C_ void mpi_fetch_and_op(MPI_Fint *origin_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Fetch_and_op_fortran_wrapper(origin_addr, result_addr, datatype, target_rank, target_disp, op, win, ierr);
}

_EXTERN_C_ void mpi_fetch_and_op_(MPI_Fint *origin_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Fetch_and_op_fortran_wrapper(origin_addr, result_addr, datatype, target_rank, target_disp, op, win, ierr);
}

_EXTERN_C_ void mpi_fetch_and_op__(MPI_Fint *origin_addr, MPI_Fint *result_addr, MPI_Fint *datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Fetch_and_op_fortran_wrapper(origin_addr, result_addr, datatype, target_rank, target_disp, op, win, ierr);
}

/* ================= End Wrappers for MPI_Fetch_and_op ================= */


/* ================== C Wrappers for MPI_File_call_errhandler ================== */
_EXTERN_C_ int PMPI_File_call_errhandler(MPI_File fh, int errorcode);
_EXTERN_C_ int MPI_File_call_errhandler(MPI_File fh, int errorcode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_call_errhandler at base level - Location = %px",&(MPI_File_call_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_call_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_call_errhandler(fh, errorcode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_call_errhandler(fh, errorcode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_call_errhandler(fh, errorcode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_call_errhandler =============== */
static void MPI_File_call_errhandler_fortran_wrapper(MPI_Fint *fh, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_call_errhandler((MPI_File)(*fh), *errorcode);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_call_errhandler(MPI_File_f2c(*fh), *errorcode);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_CALL_ERRHANDLER(MPI_Fint *fh, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_File_call_errhandler_fortran_wrapper(fh, errorcode, ierr);
}

_EXTERN_C_ void mpi_file_call_errhandler(MPI_Fint *fh, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_File_call_errhandler_fortran_wrapper(fh, errorcode, ierr);
}

_EXTERN_C_ void mpi_file_call_errhandler_(MPI_Fint *fh, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_File_call_errhandler_fortran_wrapper(fh, errorcode, ierr);
}

_EXTERN_C_ void mpi_file_call_errhandler__(MPI_Fint *fh, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_File_call_errhandler_fortran_wrapper(fh, errorcode, ierr);
}

/* ================= End Wrappers for MPI_File_call_errhandler ================= */


/* ================== C Wrappers for MPI_File_close ================== */
_EXTERN_C_ int PMPI_File_close(MPI_File *fh);
_EXTERN_C_ int MPI_File_close(MPI_File *fh) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_close at base level - Location = %px",&(MPI_File_close));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_close_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_close(fh);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_close(fh);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_close(fh);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_close =============== */
static void MPI_File_close_fortran_wrapper(MPI_Fint *fh, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_close((MPI_File*)fh);
#else /* MPI-2 safe call */
    MPI_File temp_fh;
    temp_fh = MPI_File_f2c(*fh);
    _wrap_py_return_val = MPI_File_close(&temp_fh);
    *fh = MPI_File_c2f(temp_fh);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_CLOSE(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_close_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_close(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_close_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_close_(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_close_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_close__(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_close_fortran_wrapper(fh, ierr);
}

/* ================= End Wrappers for MPI_File_close ================= */


/* ================== C Wrappers for MPI_File_create_errhandler ================== */
_EXTERN_C_ int PMPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_create_errhandler at base level - Location = %px",&(MPI_File_create_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_create_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_create_errhandler =============== */
static void MPI_File_create_errhandler_fortran_wrapper(MPI_File_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_create_errhandler(((MPI_File_errhandler_function*)function), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_File_create_errhandler(((MPI_File_errhandler_function*)function), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_CREATE_ERRHANDLER(MPI_File_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_create_errhandler(MPI_File_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_create_errhandler_(MPI_File_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_create_errhandler__(MPI_File_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

/* ================= End Wrappers for MPI_File_create_errhandler ================= */


/* ================== C Wrappers for MPI_File_delete ================== */
_EXTERN_C_ int PMPI_File_delete(const char *filename, MPI_Info info);
_EXTERN_C_ int MPI_File_delete(const char *filename, MPI_Info info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_delete at base level - Location = %px",&(MPI_File_delete));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_delete_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_delete(filename, info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_delete(filename, info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_delete(filename, info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_delete =============== */
static void MPI_File_delete_fortran_wrapper(MPI_Fint *filename, MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_delete(((const char*)filename), (MPI_Info)(*info));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_delete(((const char*)filename), MPI_Info_f2c(*info));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_DELETE(MPI_Fint *filename, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_delete_fortran_wrapper(filename, info, ierr);
}

_EXTERN_C_ void mpi_file_delete(MPI_Fint *filename, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_delete_fortran_wrapper(filename, info, ierr);
}

_EXTERN_C_ void mpi_file_delete_(MPI_Fint *filename, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_delete_fortran_wrapper(filename, info, ierr);
}

_EXTERN_C_ void mpi_file_delete__(MPI_Fint *filename, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_delete_fortran_wrapper(filename, info, ierr);
}

/* ================= End Wrappers for MPI_File_delete ================= */


/* ================== C Wrappers for MPI_File_get_amode ================== */
_EXTERN_C_ int PMPI_File_get_amode(MPI_File fh, int *amode);
_EXTERN_C_ int MPI_File_get_amode(MPI_File fh, int *amode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_amode at base level - Location = %px",&(MPI_File_get_amode));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_amode_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_amode(fh, amode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_amode(fh, amode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_amode(fh, amode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_amode =============== */
static void MPI_File_get_amode_fortran_wrapper(MPI_Fint *fh, MPI_Fint *amode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_amode((MPI_File)(*fh), ((int*)amode));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_amode(MPI_File_f2c(*fh), ((int*)amode));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_AMODE(MPI_Fint *fh, MPI_Fint *amode, MPI_Fint *ierr) { 
    MPI_File_get_amode_fortran_wrapper(fh, amode, ierr);
}

_EXTERN_C_ void mpi_file_get_amode(MPI_Fint *fh, MPI_Fint *amode, MPI_Fint *ierr) { 
    MPI_File_get_amode_fortran_wrapper(fh, amode, ierr);
}

_EXTERN_C_ void mpi_file_get_amode_(MPI_Fint *fh, MPI_Fint *amode, MPI_Fint *ierr) { 
    MPI_File_get_amode_fortran_wrapper(fh, amode, ierr);
}

_EXTERN_C_ void mpi_file_get_amode__(MPI_Fint *fh, MPI_Fint *amode, MPI_Fint *ierr) { 
    MPI_File_get_amode_fortran_wrapper(fh, amode, ierr);
}

/* ================= End Wrappers for MPI_File_get_amode ================= */


/* ================== C Wrappers for MPI_File_get_atomicity ================== */
_EXTERN_C_ int PMPI_File_get_atomicity(MPI_File fh, int *flag);
_EXTERN_C_ int MPI_File_get_atomicity(MPI_File fh, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_atomicity at base level - Location = %px",&(MPI_File_get_atomicity));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_atomicity_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_atomicity =============== */
static void MPI_File_get_atomicity_fortran_wrapper(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_atomicity((MPI_File)(*fh), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_atomicity(MPI_File_f2c(*fh), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_ATOMICITY(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_get_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_get_atomicity(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_get_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_get_atomicity_(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_get_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_get_atomicity__(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_get_atomicity_fortran_wrapper(fh, flag, ierr);
}

/* ================= End Wrappers for MPI_File_get_atomicity ================= */


/* ================== C Wrappers for MPI_File_get_byte_offset ================== */
_EXTERN_C_ int PMPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);
_EXTERN_C_ int MPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_byte_offset at base level - Location = %px",&(MPI_File_get_byte_offset));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_byte_offset_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_byte_offset(fh, offset, disp);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_byte_offset(fh, offset, disp);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_byte_offset(fh, offset, disp);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_byte_offset =============== */
static void MPI_File_get_byte_offset_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *disp, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_byte_offset((MPI_File)(*fh), *offset, ((MPI_Offset*)disp));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_byte_offset(MPI_File_f2c(*fh), *offset, ((MPI_Offset*)disp));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_BYTE_OFFSET(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *disp, MPI_Fint *ierr) { 
    MPI_File_get_byte_offset_fortran_wrapper(fh, offset, disp, ierr);
}

_EXTERN_C_ void mpi_file_get_byte_offset(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *disp, MPI_Fint *ierr) { 
    MPI_File_get_byte_offset_fortran_wrapper(fh, offset, disp, ierr);
}

_EXTERN_C_ void mpi_file_get_byte_offset_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *disp, MPI_Fint *ierr) { 
    MPI_File_get_byte_offset_fortran_wrapper(fh, offset, disp, ierr);
}

_EXTERN_C_ void mpi_file_get_byte_offset__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *disp, MPI_Fint *ierr) { 
    MPI_File_get_byte_offset_fortran_wrapper(fh, offset, disp, ierr);
}

/* ================= End Wrappers for MPI_File_get_byte_offset ================= */


/* ================== C Wrappers for MPI_File_get_errhandler ================== */
_EXTERN_C_ int PMPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_errhandler at base level - Location = %px",&(MPI_File_get_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_errhandler =============== */
static void MPI_File_get_errhandler_fortran_wrapper(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_errhandler((MPI_File)(*file), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_File_get_errhandler(MPI_File_f2c(*file), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_ERRHANDLER(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_get_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_get_errhandler(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_get_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_get_errhandler_(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_get_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_get_errhandler__(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_get_errhandler_fortran_wrapper(file, errhandler, ierr);
}

/* ================= End Wrappers for MPI_File_get_errhandler ================= */


/* ================== C Wrappers for MPI_File_get_group ================== */
_EXTERN_C_ int PMPI_File_get_group(MPI_File fh, MPI_Group *group);
_EXTERN_C_ int MPI_File_get_group(MPI_File fh, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_group at base level - Location = %px",&(MPI_File_get_group));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_group_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_group(fh, group);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_group(fh, group);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_group(fh, group);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_group =============== */
static void MPI_File_get_group_fortran_wrapper(MPI_Fint *fh, MPI_Fint *group, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_group((MPI_File)(*fh), (MPI_Group*)group);
#else /* MPI-2 safe call */
    MPI_Group temp_group;
    _wrap_py_return_val = MPI_File_get_group(MPI_File_f2c(*fh), &temp_group);
    *group = MPI_Group_c2f(temp_group);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_GROUP(MPI_Fint *fh, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_File_get_group_fortran_wrapper(fh, group, ierr);
}

_EXTERN_C_ void mpi_file_get_group(MPI_Fint *fh, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_File_get_group_fortran_wrapper(fh, group, ierr);
}

_EXTERN_C_ void mpi_file_get_group_(MPI_Fint *fh, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_File_get_group_fortran_wrapper(fh, group, ierr);
}

_EXTERN_C_ void mpi_file_get_group__(MPI_Fint *fh, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_File_get_group_fortran_wrapper(fh, group, ierr);
}

/* ================= End Wrappers for MPI_File_get_group ================= */


/* ================== C Wrappers for MPI_File_get_info ================== */
_EXTERN_C_ int PMPI_File_get_info(MPI_File fh, MPI_Info *info_used);
_EXTERN_C_ int MPI_File_get_info(MPI_File fh, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_info at base level - Location = %px",&(MPI_File_get_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_info(fh, info_used);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_info(fh, info_used);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_info(fh, info_used);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_info =============== */
static void MPI_File_get_info_fortran_wrapper(MPI_Fint *fh, MPI_Fint *info_used, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_info((MPI_File)(*fh), (MPI_Info*)info_used);
#else /* MPI-2 safe call */
    MPI_Info temp_info_used;
    _wrap_py_return_val = MPI_File_get_info(MPI_File_f2c(*fh), &temp_info_used);
    *info_used = MPI_Info_c2f(temp_info_used);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_INFO(MPI_Fint *fh, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_File_get_info_fortran_wrapper(fh, info_used, ierr);
}

_EXTERN_C_ void mpi_file_get_info(MPI_Fint *fh, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_File_get_info_fortran_wrapper(fh, info_used, ierr);
}

_EXTERN_C_ void mpi_file_get_info_(MPI_Fint *fh, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_File_get_info_fortran_wrapper(fh, info_used, ierr);
}

_EXTERN_C_ void mpi_file_get_info__(MPI_Fint *fh, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_File_get_info_fortran_wrapper(fh, info_used, ierr);
}

/* ================= End Wrappers for MPI_File_get_info ================= */


/* ================== C Wrappers for MPI_File_get_position ================== */
_EXTERN_C_ int PMPI_File_get_position(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_position at base level - Location = %px",&(MPI_File_get_position));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_position_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_position(fh, offset);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_position(fh, offset);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_position(fh, offset);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_position =============== */
static void MPI_File_get_position_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_position((MPI_File)(*fh), ((MPI_Offset*)offset));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_position(MPI_File_f2c(*fh), ((MPI_Offset*)offset));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_POSITION(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_fortran_wrapper(fh, offset, ierr);
}

/* ================= End Wrappers for MPI_File_get_position ================= */


/* ================== C Wrappers for MPI_File_get_position_shared ================== */
_EXTERN_C_ int PMPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_position_shared at base level - Location = %px",&(MPI_File_get_position_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_position_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_position_shared(fh, offset);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_position_shared(fh, offset);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_position_shared(fh, offset);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_position_shared =============== */
static void MPI_File_get_position_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_position_shared((MPI_File)(*fh), ((MPI_Offset*)offset));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_position_shared(MPI_File_f2c(*fh), ((MPI_Offset*)offset));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_POSITION_SHARED(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_shared_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position_shared(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_shared_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position_shared_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_shared_fortran_wrapper(fh, offset, ierr);
}

_EXTERN_C_ void mpi_file_get_position_shared__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *ierr) { 
    MPI_File_get_position_shared_fortran_wrapper(fh, offset, ierr);
}

/* ================= End Wrappers for MPI_File_get_position_shared ================= */


/* ================== C Wrappers for MPI_File_get_size ================== */
_EXTERN_C_ int PMPI_File_get_size(MPI_File fh, MPI_Offset *size);
_EXTERN_C_ int MPI_File_get_size(MPI_File fh, MPI_Offset *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_size at base level - Location = %px",&(MPI_File_get_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_size(fh, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_size(fh, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_size(fh, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_size =============== */
static void MPI_File_get_size_fortran_wrapper(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_size((MPI_File)(*fh), ((MPI_Offset*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_size(MPI_File_f2c(*fh), ((MPI_Offset*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_SIZE(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_get_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_get_size(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_get_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_get_size_(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_get_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_get_size__(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_get_size_fortran_wrapper(fh, size, ierr);
}

/* ================= End Wrappers for MPI_File_get_size ================= */


/* ================== C Wrappers for MPI_File_get_type_extent ================== */
_EXTERN_C_ int PMPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);
_EXTERN_C_ int MPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_type_extent at base level - Location = %px",&(MPI_File_get_type_extent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_type_extent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_type_extent(fh, datatype, extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_type_extent(fh, datatype, extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_type_extent(fh, datatype, extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_type_extent =============== */
static void MPI_File_get_type_extent_fortran_wrapper(MPI_Fint *fh, MPI_Fint *datatype, MPI_Aint *extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_type_extent((MPI_File)(*fh), (MPI_Datatype)(*datatype), ((MPI_Aint*)extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_get_type_extent(MPI_File_f2c(*fh), MPI_Type_f2c(*datatype), ((MPI_Aint*)extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_TYPE_EXTENT(MPI_Fint *fh, MPI_Fint *datatype, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_File_get_type_extent_fortran_wrapper(fh, datatype, extent, ierr);
}

_EXTERN_C_ void mpi_file_get_type_extent(MPI_Fint *fh, MPI_Fint *datatype, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_File_get_type_extent_fortran_wrapper(fh, datatype, extent, ierr);
}

_EXTERN_C_ void mpi_file_get_type_extent_(MPI_Fint *fh, MPI_Fint *datatype, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_File_get_type_extent_fortran_wrapper(fh, datatype, extent, ierr);
}

_EXTERN_C_ void mpi_file_get_type_extent__(MPI_Fint *fh, MPI_Fint *datatype, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_File_get_type_extent_fortran_wrapper(fh, datatype, extent, ierr);
}

/* ================= End Wrappers for MPI_File_get_type_extent ================= */


/* ================== C Wrappers for MPI_File_get_view ================== */
_EXTERN_C_ int PMPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);
_EXTERN_C_ int MPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_get_view at base level - Location = %px",&(MPI_File_get_view));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_get_view_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_get_view(fh, disp, etype, filetype, datarep);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_get_view(fh, disp, etype, filetype, datarep);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_get_view(fh, disp, etype, filetype, datarep);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_get_view =============== */
static void MPI_File_get_view_fortran_wrapper(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, char *datarep, MPI_Fint *ierr, int datarep_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_get_view((MPI_File)(*fh), ((MPI_Offset*)disp), (MPI_Datatype*)etype, (MPI_Datatype*)filetype, datarep);
#else /* MPI-2 safe call */
    char* temp_datarep;
    MPI_Datatype temp_filetype;
    MPI_Datatype temp_etype;
    temp_datarep = (char*)malloc(sizeof(char) * (datarep_len+1));
    char_p_f2c(datarep,datarep_len,&temp_datarep);
    _wrap_py_return_val = MPI_File_get_view(MPI_File_f2c(*fh), ((MPI_Offset*)disp), &temp_etype, &temp_filetype, datarep);
    *etype = MPI_Type_c2f(temp_etype);
    *filetype = MPI_Type_c2f(temp_filetype);
    char_p_c2f(temp_datarep,datarep,datarep_len);
 if(temp_datarep) free(temp_datarep);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_GET_VIEW(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, char *datarep, MPI_Fint *ierr, int datarep_len) { 
    MPI_File_get_view_fortran_wrapper(fh, disp, etype, filetype, datarep, ierr, datarep_len);
}

_EXTERN_C_ void mpi_file_get_view(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, char *datarep, MPI_Fint *ierr, int datarep_len) { 
    MPI_File_get_view_fortran_wrapper(fh, disp, etype, filetype, datarep, ierr, datarep_len);
}

_EXTERN_C_ void mpi_file_get_view_(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, char *datarep, MPI_Fint *ierr, int datarep_len) { 
    MPI_File_get_view_fortran_wrapper(fh, disp, etype, filetype, datarep, ierr, datarep_len);
}

_EXTERN_C_ void mpi_file_get_view__(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, char *datarep, MPI_Fint *ierr, int datarep_len) { 
    MPI_File_get_view_fortran_wrapper(fh, disp, etype, filetype, datarep, ierr, datarep_len);
}

/* ================= End Wrappers for MPI_File_get_view ================= */


/* ================== C Wrappers for MPI_File_iread ================== */
_EXTERN_C_ int PMPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iread at base level - Location = %px",&(MPI_File_iread));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iread_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iread(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iread(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iread(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iread =============== */
static void MPI_File_iread_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iread((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iread(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IREAD(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iread ================= */


/* ================== C Wrappers for MPI_File_iread_at ================== */
_EXTERN_C_ int PMPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iread_at at base level - Location = %px",&(MPI_File_iread_at));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iread_at_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iread_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iread_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iread_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iread_at =============== */
static void MPI_File_iread_at_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iread_at((MPI_File)(*fh), *offset, BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iread_at(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IREAD_AT(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_at(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_at_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_at__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iread_at ================= */


/* ================== C Wrappers for MPI_File_iread_shared ================== */
_EXTERN_C_ int PMPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iread_shared at base level - Location = %px",&(MPI_File_iread_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iread_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iread_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iread_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iread_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iread_shared =============== */
static void MPI_File_iread_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iread_shared((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iread_shared(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IREAD_SHARED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_shared(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_shared_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iread_shared__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iread_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iread_shared ================= */


/* ================== C Wrappers for MPI_File_iwrite ================== */
_EXTERN_C_ int PMPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iwrite at base level - Location = %px",&(MPI_File_iwrite));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iwrite_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iwrite(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iwrite(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iwrite(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iwrite =============== */
static void MPI_File_iwrite_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iwrite((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iwrite(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IWRITE(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iwrite ================= */


/* ================== C Wrappers for MPI_File_iwrite_at ================== */
_EXTERN_C_ int PMPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iwrite_at at base level - Location = %px",&(MPI_File_iwrite_at));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iwrite_at_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iwrite_at =============== */
static void MPI_File_iwrite_at_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iwrite_at((MPI_File)(*fh), *offset, ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iwrite_at(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IWRITE_AT(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_at(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_at_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_at__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_at_fortran_wrapper(fh, offset, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iwrite_at ================= */


/* ================== C Wrappers for MPI_File_iwrite_shared ================== */
_EXTERN_C_ int PMPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_iwrite_shared at base level - Location = %px",&(MPI_File_iwrite_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_iwrite_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_iwrite_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_iwrite_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_iwrite_shared(fh, buf, count, datatype, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_iwrite_shared =============== */
static void MPI_File_iwrite_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_iwrite_shared((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_File_iwrite_shared(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_IWRITE_SHARED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_shared(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_shared_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

_EXTERN_C_ void mpi_file_iwrite_shared__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_File_iwrite_shared_fortran_wrapper(fh, buf, count, datatype, request, ierr);
}

/* ================= End Wrappers for MPI_File_iwrite_shared ================= */


/* ================== C Wrappers for MPI_File_open ================== */
_EXTERN_C_ int PMPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);
_EXTERN_C_ int MPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_open at base level - Location = %px",&(MPI_File_open));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_open_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_open(comm, filename, amode, info, fh);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_open(comm, filename, amode, info, fh);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_open(comm, filename, amode, info, fh);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_open =============== */
static void MPI_File_open_fortran_wrapper(MPI_Fint *comm, MPI_Fint *filename, MPI_Fint *amode, MPI_Fint *info, MPI_Fint *fh, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_open((MPI_Comm)(*comm), ((const char*)filename), *amode, (MPI_Info)(*info), (MPI_File*)fh);
#else /* MPI-2 safe call */
    MPI_File temp_fh;
    temp_fh = MPI_File_f2c(*fh);
    _wrap_py_return_val = MPI_File_open(MPI_Comm_f2c(*comm), ((const char*)filename), *amode, MPI_Info_f2c(*info), &temp_fh);
    *fh = MPI_File_c2f(temp_fh);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_OPEN(MPI_Fint *comm, MPI_Fint *filename, MPI_Fint *amode, MPI_Fint *info, MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_open_fortran_wrapper(comm, filename, amode, info, fh, ierr);
}

_EXTERN_C_ void mpi_file_open(MPI_Fint *comm, MPI_Fint *filename, MPI_Fint *amode, MPI_Fint *info, MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_open_fortran_wrapper(comm, filename, amode, info, fh, ierr);
}

_EXTERN_C_ void mpi_file_open_(MPI_Fint *comm, MPI_Fint *filename, MPI_Fint *amode, MPI_Fint *info, MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_open_fortran_wrapper(comm, filename, amode, info, fh, ierr);
}

_EXTERN_C_ void mpi_file_open__(MPI_Fint *comm, MPI_Fint *filename, MPI_Fint *amode, MPI_Fint *info, MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_open_fortran_wrapper(comm, filename, amode, info, fh, ierr);
}

/* ================= End Wrappers for MPI_File_open ================= */


/* ================== C Wrappers for MPI_File_preallocate ================== */
_EXTERN_C_ int PMPI_File_preallocate(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_preallocate(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_preallocate at base level - Location = %px",&(MPI_File_preallocate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_preallocate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_preallocate(fh, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_preallocate(fh, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_preallocate(fh, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_preallocate =============== */
static void MPI_File_preallocate_fortran_wrapper(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_preallocate((MPI_File)(*fh), *size);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_preallocate(MPI_File_f2c(*fh), *size);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_PREALLOCATE(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_preallocate_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_preallocate(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_preallocate_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_preallocate_(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_preallocate_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_preallocate__(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_preallocate_fortran_wrapper(fh, size, ierr);
}

/* ================= End Wrappers for MPI_File_preallocate ================= */


/* ================== C Wrappers for MPI_File_read ================== */
_EXTERN_C_ int PMPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read at base level - Location = %px",&(MPI_File_read));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read =============== */
static void MPI_File_read_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read ================= */


/* ================== C Wrappers for MPI_File_read_all ================== */
_EXTERN_C_ int PMPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_all at base level - Location = %px",&(MPI_File_read_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_all =============== */
static void MPI_File_read_all_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_all((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_all(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_all(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ALL(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_all ================= */


/* ================== C Wrappers for MPI_File_read_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_all_begin at base level - Location = %px",&(MPI_File_read_all_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_all_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_all_begin =============== */
static void MPI_File_read_all_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_all_begin((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_read_all_begin(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ALL_BEGIN(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_all_begin(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_all_begin_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_all_begin__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_read_all_begin ================= */


/* ================== C Wrappers for MPI_File_read_all_end ================== */
_EXTERN_C_ int PMPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_all_end at base level - Location = %px",&(MPI_File_read_all_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_all_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_all_end =============== */
static void MPI_File_read_all_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_all_end((MPI_File)(*fh), BufferC2F((void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_all_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_all_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ALL_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_all_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_all_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_all_end ================= */


/* ================== C Wrappers for MPI_File_read_at ================== */
_EXTERN_C_ int PMPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_at at base level - Location = %px",&(MPI_File_read_at));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_at_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_at =============== */
static void MPI_File_read_at_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_at((MPI_File)(*fh), *offset, BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_AT(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_at ================= */


/* ================== C Wrappers for MPI_File_read_at_all ================== */
_EXTERN_C_ int PMPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_at_all at base level - Location = %px",&(MPI_File_read_at_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_at_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_at_all =============== */
static void MPI_File_read_at_all_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_at_all((MPI_File)(*fh), *offset, BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at_all(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at_all(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_AT_ALL(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_at_all ================= */


/* ================== C Wrappers for MPI_File_read_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_at_all_begin at base level - Location = %px",&(MPI_File_read_at_all_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_at_all_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_at_all_begin =============== */
static void MPI_File_read_at_all_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_at_all_begin((MPI_File)(*fh), *offset, BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_read_at_all_begin(MPI_File_f2c(*fh), *offset, BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_AT_ALL_BEGIN(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_begin(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_begin_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_begin__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_read_at_all_begin ================= */


/* ================== C Wrappers for MPI_File_read_at_all_end ================== */
_EXTERN_C_ int PMPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_at_all_end at base level - Location = %px",&(MPI_File_read_at_all_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_at_all_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_at_all_end =============== */
static void MPI_File_read_at_all_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_at_all_end((MPI_File)(*fh), BufferC2F((void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at_all_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_at_all_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_AT_ALL_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_at_all_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_at_all_end ================= */


/* ================== C Wrappers for MPI_File_read_ordered ================== */
_EXTERN_C_ int PMPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_ordered at base level - Location = %px",&(MPI_File_read_ordered));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_ordered_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_ordered =============== */
static void MPI_File_read_ordered_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_ordered((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_ordered(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_ordered(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ORDERED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_ordered ================= */


/* ================== C Wrappers for MPI_File_read_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_ordered_begin at base level - Location = %px",&(MPI_File_read_ordered_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_ordered_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_ordered_begin =============== */
static void MPI_File_read_ordered_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_ordered_begin((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_read_ordered_begin(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ORDERED_BEGIN(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_begin(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_begin_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_begin__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_read_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_read_ordered_begin ================= */


/* ================== C Wrappers for MPI_File_read_ordered_end ================== */
_EXTERN_C_ int PMPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_ordered_end at base level - Location = %px",&(MPI_File_read_ordered_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_ordered_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_ordered_end =============== */
static void MPI_File_read_ordered_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_ordered_end((MPI_File)(*fh), BufferC2F((void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_ordered_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_ordered_end(MPI_File_f2c(*fh), BufferC2F((void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_ORDERED_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_read_ordered_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_ordered_end ================= */


/* ================== C Wrappers for MPI_File_read_shared ================== */
_EXTERN_C_ int PMPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_read_shared at base level - Location = %px",&(MPI_File_read_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_read_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_read_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_read_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_read_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_read_shared =============== */
static void MPI_File_read_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_read_shared((MPI_File)(*fh), BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_shared(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_read_shared(MPI_File_f2c(*fh), BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_READ_SHARED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_shared(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_shared_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_read_shared__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_read_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_read_shared ================= */


/* ================== C Wrappers for MPI_File_seek ================== */
_EXTERN_C_ int PMPI_File_seek(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_seek at base level - Location = %px",&(MPI_File_seek));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_seek_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_seek(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_seek(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_seek(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_seek =============== */
static void MPI_File_seek_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_seek((MPI_File)(*fh), *offset, *whence);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_seek(MPI_File_f2c(*fh), *offset, *whence);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SEEK(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_fortran_wrapper(fh, offset, whence, ierr);
}

/* ================= End Wrappers for MPI_File_seek ================= */


/* ================== C Wrappers for MPI_File_seek_shared ================== */
_EXTERN_C_ int PMPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_seek_shared at base level - Location = %px",&(MPI_File_seek_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_seek_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_seek_shared(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_seek_shared(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_seek_shared(fh, offset, whence);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_seek_shared =============== */
static void MPI_File_seek_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_seek_shared((MPI_File)(*fh), *offset, *whence);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_seek_shared(MPI_File_f2c(*fh), *offset, *whence);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SEEK_SHARED(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_shared_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek_shared(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_shared_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek_shared_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_shared_fortran_wrapper(fh, offset, whence, ierr);
}

_EXTERN_C_ void mpi_file_seek_shared__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *whence, MPI_Fint *ierr) { 
    MPI_File_seek_shared_fortran_wrapper(fh, offset, whence, ierr);
}

/* ================= End Wrappers for MPI_File_seek_shared ================= */


/* ================== C Wrappers for MPI_File_set_atomicity ================== */
_EXTERN_C_ int PMPI_File_set_atomicity(MPI_File fh, int flag);
_EXTERN_C_ int MPI_File_set_atomicity(MPI_File fh, int flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_set_atomicity at base level - Location = %px",&(MPI_File_set_atomicity));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_set_atomicity_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_set_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_set_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_set_atomicity(fh, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_set_atomicity =============== */
static void MPI_File_set_atomicity_fortran_wrapper(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_set_atomicity((MPI_File)(*fh), *flag);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_set_atomicity(MPI_File_f2c(*fh), *flag);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SET_ATOMICITY(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_set_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_set_atomicity(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_set_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_set_atomicity_(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_set_atomicity_fortran_wrapper(fh, flag, ierr);
}

_EXTERN_C_ void mpi_file_set_atomicity__(MPI_Fint *fh, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_File_set_atomicity_fortran_wrapper(fh, flag, ierr);
}

/* ================= End Wrappers for MPI_File_set_atomicity ================= */


/* ================== C Wrappers for MPI_File_set_errhandler ================== */
_EXTERN_C_ int PMPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_set_errhandler at base level - Location = %px",&(MPI_File_set_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_set_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_set_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_set_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_set_errhandler(file, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_set_errhandler =============== */
static void MPI_File_set_errhandler_fortran_wrapper(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_set_errhandler((MPI_File)(*file), (MPI_Errhandler)(*errhandler));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_set_errhandler(MPI_File_f2c(*file), MPI_Errhandler_f2c(*errhandler));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SET_ERRHANDLER(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_set_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_set_errhandler(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_set_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_set_errhandler_(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_set_errhandler_fortran_wrapper(file, errhandler, ierr);
}

_EXTERN_C_ void mpi_file_set_errhandler__(MPI_Fint *file, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_File_set_errhandler_fortran_wrapper(file, errhandler, ierr);
}

/* ================= End Wrappers for MPI_File_set_errhandler ================= */


/* ================== C Wrappers for MPI_File_set_info ================== */
_EXTERN_C_ int PMPI_File_set_info(MPI_File fh, MPI_Info info);
_EXTERN_C_ int MPI_File_set_info(MPI_File fh, MPI_Info info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_set_info at base level - Location = %px",&(MPI_File_set_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_set_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_set_info(fh, info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_set_info(fh, info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_set_info(fh, info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_set_info =============== */
static void MPI_File_set_info_fortran_wrapper(MPI_Fint *fh, MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_set_info((MPI_File)(*fh), (MPI_Info)(*info));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_set_info(MPI_File_f2c(*fh), MPI_Info_f2c(*info));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SET_INFO(MPI_Fint *fh, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_info_fortran_wrapper(fh, info, ierr);
}

_EXTERN_C_ void mpi_file_set_info(MPI_Fint *fh, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_info_fortran_wrapper(fh, info, ierr);
}

_EXTERN_C_ void mpi_file_set_info_(MPI_Fint *fh, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_info_fortran_wrapper(fh, info, ierr);
}

_EXTERN_C_ void mpi_file_set_info__(MPI_Fint *fh, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_info_fortran_wrapper(fh, info, ierr);
}

/* ================= End Wrappers for MPI_File_set_info ================= */


/* ================== C Wrappers for MPI_File_set_size ================== */
_EXTERN_C_ int PMPI_File_set_size(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_set_size(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_set_size at base level - Location = %px",&(MPI_File_set_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_set_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_set_size(fh, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_set_size(fh, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_set_size(fh, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_set_size =============== */
static void MPI_File_set_size_fortran_wrapper(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_set_size((MPI_File)(*fh), *size);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_set_size(MPI_File_f2c(*fh), *size);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SET_SIZE(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_set_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_set_size(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_set_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_set_size_(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_set_size_fortran_wrapper(fh, size, ierr);
}

_EXTERN_C_ void mpi_file_set_size__(MPI_Fint *fh, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_File_set_size_fortran_wrapper(fh, size, ierr);
}

/* ================= End Wrappers for MPI_File_set_size ================= */


/* ================== C Wrappers for MPI_File_set_view ================== */
_EXTERN_C_ int PMPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);
_EXTERN_C_ int MPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_set_view at base level - Location = %px",&(MPI_File_set_view));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_set_view_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_set_view =============== */
static void MPI_File_set_view_fortran_wrapper(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, MPI_Fint *datarep, MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_set_view((MPI_File)(*fh), *disp, (MPI_Datatype)(*etype), (MPI_Datatype)(*filetype), ((const char*)datarep), (MPI_Info)(*info));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_set_view(MPI_File_f2c(*fh), *disp, MPI_Type_f2c(*etype), MPI_Type_f2c(*filetype), ((const char*)datarep), MPI_Info_f2c(*info));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SET_VIEW(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, MPI_Fint *datarep, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_view_fortran_wrapper(fh, disp, etype, filetype, datarep, info, ierr);
}

_EXTERN_C_ void mpi_file_set_view(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, MPI_Fint *datarep, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_view_fortran_wrapper(fh, disp, etype, filetype, datarep, info, ierr);
}

_EXTERN_C_ void mpi_file_set_view_(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, MPI_Fint *datarep, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_view_fortran_wrapper(fh, disp, etype, filetype, datarep, info, ierr);
}

_EXTERN_C_ void mpi_file_set_view__(MPI_Fint *fh, MPI_Fint *disp, MPI_Fint *etype, MPI_Fint *filetype, MPI_Fint *datarep, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_File_set_view_fortran_wrapper(fh, disp, etype, filetype, datarep, info, ierr);
}

/* ================= End Wrappers for MPI_File_set_view ================= */


/* ================== C Wrappers for MPI_File_sync ================== */
_EXTERN_C_ int PMPI_File_sync(MPI_File fh);
_EXTERN_C_ int MPI_File_sync(MPI_File fh) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_sync at base level - Location = %px",&(MPI_File_sync));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_sync_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_sync(fh);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_sync(fh);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_sync(fh);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_sync =============== */
static void MPI_File_sync_fortran_wrapper(MPI_Fint *fh, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_sync((MPI_File)(*fh));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_sync(MPI_File_f2c(*fh));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_SYNC(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_sync_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_sync(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_sync_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_sync_(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_sync_fortran_wrapper(fh, ierr);
}

_EXTERN_C_ void mpi_file_sync__(MPI_Fint *fh, MPI_Fint *ierr) { 
    MPI_File_sync_fortran_wrapper(fh, ierr);
}

/* ================= End Wrappers for MPI_File_sync ================= */


/* ================== C Wrappers for MPI_File_write ================== */
_EXTERN_C_ int PMPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write at base level - Location = %px",&(MPI_File_write));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write =============== */
static void MPI_File_write_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write ================= */


/* ================== C Wrappers for MPI_File_write_all ================== */
_EXTERN_C_ int PMPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_all at base level - Location = %px",&(MPI_File_write_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_all(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_all =============== */
static void MPI_File_write_all_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_all((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_all(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_all(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ALL(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_all ================= */


/* ================== C Wrappers for MPI_File_write_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_all_begin at base level - Location = %px",&(MPI_File_write_all_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_all_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_all_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_all_begin =============== */
static void MPI_File_write_all_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_all_begin((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_write_all_begin(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ALL_BEGIN(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_all_begin(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_all_begin_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_all_begin__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_all_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_write_all_begin ================= */


/* ================== C Wrappers for MPI_File_write_all_end ================== */
_EXTERN_C_ int PMPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_all_end at base level - Location = %px",&(MPI_File_write_all_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_all_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_all_end =============== */
static void MPI_File_write_all_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_all_end((MPI_File)(*fh), ((const void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_all_end(MPI_File_f2c(*fh), ((const void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_all_end(MPI_File_f2c(*fh), ((const void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ALL_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_all_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_all_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_all_end ================= */


/* ================== C Wrappers for MPI_File_write_at ================== */
_EXTERN_C_ int PMPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_at at base level - Location = %px",&(MPI_File_write_at));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_at_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_at(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_at =============== */
static void MPI_File_write_at_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_at((MPI_File)(*fh), *offset, ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_AT(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_at ================= */


/* ================== C Wrappers for MPI_File_write_at_all ================== */
_EXTERN_C_ int PMPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_at_all at base level - Location = %px",&(MPI_File_write_at_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_at_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_at_all =============== */
static void MPI_File_write_at_all_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_at_all((MPI_File)(*fh), *offset, ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at_all(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at_all(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_AT_ALL(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_fortran_wrapper(fh, offset, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_at_all ================= */


/* ================== C Wrappers for MPI_File_write_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_at_all_begin at base level - Location = %px",&(MPI_File_write_at_all_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_at_all_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_at_all_begin =============== */
static void MPI_File_write_at_all_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_at_all_begin((MPI_File)(*fh), *offset, ((const void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_write_at_all_begin(MPI_File_f2c(*fh), *offset, ((const void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_AT_ALL_BEGIN(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_begin(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_begin_(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_begin__(MPI_Fint *fh, MPI_Fint *offset, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_at_all_begin_fortran_wrapper(fh, offset, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_write_at_all_begin ================= */


/* ================== C Wrappers for MPI_File_write_at_all_end ================== */
_EXTERN_C_ int PMPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_at_all_end at base level - Location = %px",&(MPI_File_write_at_all_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_at_all_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_at_all_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_at_all_end =============== */
static void MPI_File_write_at_all_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_at_all_end((MPI_File)(*fh), ((const void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at_all_end(MPI_File_f2c(*fh), ((const void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_at_all_end(MPI_File_f2c(*fh), ((const void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_AT_ALL_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_at_all_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_at_all_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_at_all_end ================= */


/* ================== C Wrappers for MPI_File_write_ordered ================== */
_EXTERN_C_ int PMPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_ordered at base level - Location = %px",&(MPI_File_write_ordered));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_ordered_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_ordered(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_ordered =============== */
static void MPI_File_write_ordered_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_ordered((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_ordered(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_ordered(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ORDERED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_ordered ================= */


/* ================== C Wrappers for MPI_File_write_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_ordered_begin at base level - Location = %px",&(MPI_File_write_ordered_begin));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_ordered_begin_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_ordered_begin(fh, buf, count, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_ordered_begin =============== */
static void MPI_File_write_ordered_begin_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_ordered_begin((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_File_write_ordered_begin(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ORDERED_BEGIN(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_begin(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_begin_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_begin__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_File_write_ordered_begin_fortran_wrapper(fh, buf, count, datatype, ierr);
}

/* ================= End Wrappers for MPI_File_write_ordered_begin ================= */


/* ================== C Wrappers for MPI_File_write_ordered_end ================== */
_EXTERN_C_ int PMPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_ordered_end at base level - Location = %px",&(MPI_File_write_ordered_end));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_ordered_end_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_ordered_end(fh, buf, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_ordered_end =============== */
static void MPI_File_write_ordered_end_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_ordered_end((MPI_File)(*fh), ((const void*)buf), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_ordered_end(MPI_File_f2c(*fh), ((const void*)buf), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_ordered_end(MPI_File_f2c(*fh), ((const void*)buf), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_ORDERED_END(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_end(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_end_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

_EXTERN_C_ void mpi_file_write_ordered_end__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_ordered_end_fortran_wrapper(fh, buf, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_ordered_end ================= */


/* ================== C Wrappers for MPI_File_write_shared ================== */
_EXTERN_C_ int PMPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_File_write_shared at base level - Location = %px",&(MPI_File_write_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_File_write_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_File_write_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_File_write_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_File_write_shared(fh, buf, count, datatype, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_File_write_shared =============== */
static void MPI_File_write_shared_fortran_wrapper(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_File_write_shared((MPI_File)(*fh), ((const void*)buf), *count, (MPI_Datatype)(*datatype), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_shared(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_File_write_shared(MPI_File_f2c(*fh), ((const void*)buf), *count, MPI_Type_f2c(*datatype), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FILE_WRITE_SHARED(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_shared(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_shared_(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

_EXTERN_C_ void mpi_file_write_shared__(MPI_Fint *fh, MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_File_write_shared_fortran_wrapper(fh, buf, count, datatype, status, ierr);
}

/* ================= End Wrappers for MPI_File_write_shared ================= */


/* ================== C Wrappers for MPI_Finalized ================== */
_EXTERN_C_ int PMPI_Finalized(int *flag);
_EXTERN_C_ int MPI_Finalized(int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Finalized at base level - Location = %px",&(MPI_Finalized));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Finalized_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Finalized(flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Finalized(flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Finalized(flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Finalized =============== */
static void MPI_Finalized_fortran_wrapper(MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Finalized(((int*)flag));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FINALIZED(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Finalized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_finalized(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Finalized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_finalized_(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Finalized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_finalized__(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Finalized_fortran_wrapper(flag, ierr);
}

/* ================= End Wrappers for MPI_Finalized ================= */


/* ================== C Wrappers for MPI_Free_mem ================== */
_EXTERN_C_ int PMPI_Free_mem(void *base);
_EXTERN_C_ int MPI_Free_mem(void *base) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Free_mem at base level - Location = %px",&(MPI_Free_mem));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Free_mem_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Free_mem(base);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Free_mem(base);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Free_mem(base);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Free_mem =============== */
static void MPI_Free_mem_fortran_wrapper(MPI_Fint *base, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Free_mem(BufferC2F((void*)base));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_FREE_MEM(MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Free_mem_fortran_wrapper(base, ierr);
}

_EXTERN_C_ void mpi_free_mem(MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Free_mem_fortran_wrapper(base, ierr);
}

_EXTERN_C_ void mpi_free_mem_(MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Free_mem_fortran_wrapper(base, ierr);
}

_EXTERN_C_ void mpi_free_mem__(MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Free_mem_fortran_wrapper(base, ierr);
}

/* ================= End Wrappers for MPI_Free_mem ================= */


/* ================== C Wrappers for MPI_Gather ================== */
_EXTERN_C_ int PMPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Gather at base level - Location = %px",&(MPI_Gather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Gather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Gather =============== */
static void MPI_Gather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Gather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Gather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Gather ================= */


/* ================== C Wrappers for MPI_Gatherv ================== */
_EXTERN_C_ int PMPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Gatherv at base level - Location = %px",&(MPI_Gatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Gatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Gatherv =============== */
static void MPI_Gatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Gatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Gatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_gatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Gatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Gatherv ================= */


/* ================== C Wrappers for MPI_Get ================== */
_EXTERN_C_ int PMPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get at base level - Location = %px",&(MPI_Get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get =============== */
static void MPI_Get_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get(BufferC2F((void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Get(BufferC2F((void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_get(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_get_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_get__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

/* ================= End Wrappers for MPI_Get ================= */


/* ================== C Wrappers for MPI_Get_accumulate ================== */
_EXTERN_C_ int PMPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_accumulate at base level - Location = %px",&(MPI_Get_accumulate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_accumulate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_accumulate =============== */
static void MPI_Get_accumulate_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_accumulate(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), BufferC2F((void*)result_addr), *result_count, (MPI_Datatype)(*result_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Op)(*op), (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Get_accumulate(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), BufferC2F((void*)result_addr), *result_count, MPI_Type_f2c(*result_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Op_f2c(*op), MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_ACCUMULATE(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_get_accumulate(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_get_accumulate_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

_EXTERN_C_ void mpi_get_accumulate__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Get_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, ierr);
}

/* ================= End Wrappers for MPI_Get_accumulate ================= */


/* ================== C Wrappers for MPI_Get_address ================== */
_EXTERN_C_ int PMPI_Get_address(const void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Get_address(const void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_address at base level - Location = %px",&(MPI_Get_address));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_address_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_address(location, address);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_address(location, address);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_address(location, address);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_address =============== */
static void MPI_Get_address_fortran_wrapper(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Get_address(((const void*)location), ((MPI_Aint*)address));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_ADDRESS(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Get_address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_get_address(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Get_address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_get_address_(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Get_address_fortran_wrapper(location, address, ierr);
}

_EXTERN_C_ void mpi_get_address__(MPI_Fint *location, MPI_Aint *address, MPI_Fint *ierr) { 
    MPI_Get_address_fortran_wrapper(location, address, ierr);
}

/* ================= End Wrappers for MPI_Get_address ================= */


/* ================== C Wrappers for MPI_Get_count ================== */
_EXTERN_C_ int PMPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_count at base level - Location = %px",&(MPI_Get_count));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_count_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_count(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_count(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_count(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_count =============== */
static void MPI_Get_count_fortran_wrapper(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_count(((const MPI_Status*)status), (MPI_Datatype)(*datatype), ((int*)count));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Get_count(((const MPI_Status*)status), MPI_Type_f2c(*datatype), ((int*)count));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_COUNT(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_count_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_count(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_count_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_count_(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_count_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_count__(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_count_fortran_wrapper(status, datatype, count, ierr);
}

/* ================= End Wrappers for MPI_Get_count ================= */


/* ================== C Wrappers for MPI_Get_elements ================== */
_EXTERN_C_ int PMPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_elements at base level - Location = %px",&(MPI_Get_elements));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_elements_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_elements =============== */
static void MPI_Get_elements_fortran_wrapper(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_elements(((const MPI_Status*)status), (MPI_Datatype)(*datatype), ((int*)count));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Get_elements(((const MPI_Status*)status), MPI_Type_f2c(*datatype), ((int*)count));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_ELEMENTS(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements_(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements__(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_fortran_wrapper(status, datatype, count, ierr);
}

/* ================= End Wrappers for MPI_Get_elements ================= */


/* ================== C Wrappers for MPI_Get_elements_x ================== */
_EXTERN_C_ int PMPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);
_EXTERN_C_ int MPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_elements_x at base level - Location = %px",&(MPI_Get_elements_x));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_elements_x_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_elements_x =============== */
static void MPI_Get_elements_x_fortran_wrapper(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_elements_x(((const MPI_Status*)status), (MPI_Datatype)(*datatype), ((MPI_Count*)count));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Get_elements_x(((const MPI_Status*)status), MPI_Type_f2c(*datatype), ((MPI_Count*)count));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_ELEMENTS_X(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements_x(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements_x_(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_get_elements_x__(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Get_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

/* ================= End Wrappers for MPI_Get_elements_x ================= */


/* ================== C Wrappers for MPI_Get_library_version ================== */
_EXTERN_C_ int PMPI_Get_library_version(char *version, int *resultlen);
_EXTERN_C_ int MPI_Get_library_version(char *version, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_library_version at base level - Location = %px",&(MPI_Get_library_version));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_library_version_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_library_version(version, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_library_version(version, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_library_version(version, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_library_version =============== */
static void MPI_Get_library_version_fortran_wrapper(char *version, MPI_Fint *resultlen, MPI_Fint *ierr, int version_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_library_version(version, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_version;
    temp_version = (char*)malloc(sizeof(char) * (version_len+1));
    char_p_f2c(version,version_len,&temp_version);
    _wrap_py_return_val = MPI_Get_library_version(version, ((int*)resultlen));
    char_p_c2f(temp_version,version,version_len);
 if(temp_version) free(temp_version);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_LIBRARY_VERSION(char *version, MPI_Fint *resultlen, MPI_Fint *ierr, int version_len) { 
    MPI_Get_library_version_fortran_wrapper(version, resultlen, ierr, version_len);
}

_EXTERN_C_ void mpi_get_library_version(char *version, MPI_Fint *resultlen, MPI_Fint *ierr, int version_len) { 
    MPI_Get_library_version_fortran_wrapper(version, resultlen, ierr, version_len);
}

_EXTERN_C_ void mpi_get_library_version_(char *version, MPI_Fint *resultlen, MPI_Fint *ierr, int version_len) { 
    MPI_Get_library_version_fortran_wrapper(version, resultlen, ierr, version_len);
}

_EXTERN_C_ void mpi_get_library_version__(char *version, MPI_Fint *resultlen, MPI_Fint *ierr, int version_len) { 
    MPI_Get_library_version_fortran_wrapper(version, resultlen, ierr, version_len);
}

/* ================= End Wrappers for MPI_Get_library_version ================= */


/* ================== C Wrappers for MPI_Get_processor_name ================== */
_EXTERN_C_ int PMPI_Get_processor_name(char *name, int *resultlen);
_EXTERN_C_ int MPI_Get_processor_name(char *name, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_processor_name at base level - Location = %px",&(MPI_Get_processor_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_processor_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_processor_name(name, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_processor_name(name, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_processor_name(name, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_processor_name =============== */
static void MPI_Get_processor_name_fortran_wrapper(char *name, MPI_Fint *resultlen, MPI_Fint *ierr, int name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Get_processor_name(name, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_name;
    temp_name = (char*)malloc(sizeof(char) * (name_len+1));
    char_p_f2c(name,name_len,&temp_name);
    _wrap_py_return_val = MPI_Get_processor_name(name, ((int*)resultlen));
    char_p_c2f(temp_name,name,name_len);
 if(temp_name) free(temp_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_PROCESSOR_NAME(char *name, MPI_Fint *resultlen, MPI_Fint *ierr, int name_len) { 
    MPI_Get_processor_name_fortran_wrapper(name, resultlen, ierr, name_len);
}

_EXTERN_C_ void mpi_get_processor_name(char *name, MPI_Fint *resultlen, MPI_Fint *ierr, int name_len) { 
    MPI_Get_processor_name_fortran_wrapper(name, resultlen, ierr, name_len);
}

_EXTERN_C_ void mpi_get_processor_name_(char *name, MPI_Fint *resultlen, MPI_Fint *ierr, int name_len) { 
    MPI_Get_processor_name_fortran_wrapper(name, resultlen, ierr, name_len);
}

_EXTERN_C_ void mpi_get_processor_name__(char *name, MPI_Fint *resultlen, MPI_Fint *ierr, int name_len) { 
    MPI_Get_processor_name_fortran_wrapper(name, resultlen, ierr, name_len);
}

/* ================= End Wrappers for MPI_Get_processor_name ================= */


/* ================== C Wrappers for MPI_Get_version ================== */
_EXTERN_C_ int PMPI_Get_version(int *version, int *subversion);
_EXTERN_C_ int MPI_Get_version(int *version, int *subversion) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Get_version at base level - Location = %px",&(MPI_Get_version));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Get_version_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Get_version(version, subversion);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Get_version(version, subversion);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Get_version(version, subversion);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Get_version =============== */
static void MPI_Get_version_fortran_wrapper(MPI_Fint *version, MPI_Fint *subversion, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Get_version(((int*)version), ((int*)subversion));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GET_VERSION(MPI_Fint *version, MPI_Fint *subversion, MPI_Fint *ierr) { 
    MPI_Get_version_fortran_wrapper(version, subversion, ierr);
}

_EXTERN_C_ void mpi_get_version(MPI_Fint *version, MPI_Fint *subversion, MPI_Fint *ierr) { 
    MPI_Get_version_fortran_wrapper(version, subversion, ierr);
}

_EXTERN_C_ void mpi_get_version_(MPI_Fint *version, MPI_Fint *subversion, MPI_Fint *ierr) { 
    MPI_Get_version_fortran_wrapper(version, subversion, ierr);
}

_EXTERN_C_ void mpi_get_version__(MPI_Fint *version, MPI_Fint *subversion, MPI_Fint *ierr) { 
    MPI_Get_version_fortran_wrapper(version, subversion, ierr);
}

/* ================= End Wrappers for MPI_Get_version ================= */


/* ================== C Wrappers for MPI_Graph_create ================== */
_EXTERN_C_ int PMPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);
_EXTERN_C_ int MPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graph_create at base level - Location = %px",&(MPI_Graph_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graph_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graph_create =============== */
static void MPI_Graph_create_fortran_wrapper(MPI_Fint *comm_old, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *reorder, MPI_Fint *comm_graph, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graph_create((MPI_Comm)(*comm_old), *nnodes, ((const int*)index), ((const int*)edges), *reorder, (MPI_Comm*)comm_graph);
#else /* MPI-2 safe call */
    MPI_Comm temp_comm_graph;
    _wrap_py_return_val = MPI_Graph_create(MPI_Comm_f2c(*comm_old), *nnodes, ((const int*)index), ((const int*)edges), *reorder, &temp_comm_graph);
    *comm_graph = MPI_Comm_c2f(temp_comm_graph);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPH_CREATE(MPI_Fint *comm_old, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *reorder, MPI_Fint *comm_graph, MPI_Fint *ierr) { 
    MPI_Graph_create_fortran_wrapper(comm_old, nnodes, index, edges, reorder, comm_graph, ierr);
}

_EXTERN_C_ void mpi_graph_create(MPI_Fint *comm_old, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *reorder, MPI_Fint *comm_graph, MPI_Fint *ierr) { 
    MPI_Graph_create_fortran_wrapper(comm_old, nnodes, index, edges, reorder, comm_graph, ierr);
}

_EXTERN_C_ void mpi_graph_create_(MPI_Fint *comm_old, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *reorder, MPI_Fint *comm_graph, MPI_Fint *ierr) { 
    MPI_Graph_create_fortran_wrapper(comm_old, nnodes, index, edges, reorder, comm_graph, ierr);
}

_EXTERN_C_ void mpi_graph_create__(MPI_Fint *comm_old, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *reorder, MPI_Fint *comm_graph, MPI_Fint *ierr) { 
    MPI_Graph_create_fortran_wrapper(comm_old, nnodes, index, edges, reorder, comm_graph, ierr);
}

/* ================= End Wrappers for MPI_Graph_create ================= */


/* ================== C Wrappers for MPI_Graph_get ================== */
_EXTERN_C_ int PMPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);
_EXTERN_C_ int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graph_get at base level - Location = %px",&(MPI_Graph_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graph_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graph_get(comm, maxindex, maxedges, index, edges);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graph_get =============== */
static void MPI_Graph_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *maxindex, MPI_Fint *maxedges, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graph_get((MPI_Comm)(*comm), *maxindex, *maxedges, ((int*)index), ((int*)edges));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Graph_get(MPI_Comm_f2c(*comm), *maxindex, *maxedges, ((int*)index), ((int*)edges));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPH_GET(MPI_Fint *comm, MPI_Fint *maxindex, MPI_Fint *maxedges, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *ierr) { 
    MPI_Graph_get_fortran_wrapper(comm, maxindex, maxedges, index, edges, ierr);
}

_EXTERN_C_ void mpi_graph_get(MPI_Fint *comm, MPI_Fint *maxindex, MPI_Fint *maxedges, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *ierr) { 
    MPI_Graph_get_fortran_wrapper(comm, maxindex, maxedges, index, edges, ierr);
}

_EXTERN_C_ void mpi_graph_get_(MPI_Fint *comm, MPI_Fint *maxindex, MPI_Fint *maxedges, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *ierr) { 
    MPI_Graph_get_fortran_wrapper(comm, maxindex, maxedges, index, edges, ierr);
}

_EXTERN_C_ void mpi_graph_get__(MPI_Fint *comm, MPI_Fint *maxindex, MPI_Fint *maxedges, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *ierr) { 
    MPI_Graph_get_fortran_wrapper(comm, maxindex, maxedges, index, edges, ierr);
}

/* ================= End Wrappers for MPI_Graph_get ================= */


/* ================== C Wrappers for MPI_Graph_map ================== */
_EXTERN_C_ int PMPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);
_EXTERN_C_ int MPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graph_map at base level - Location = %px",&(MPI_Graph_map));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graph_map_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graph_map(comm, nnodes, index, edges, newrank);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graph_map(comm, nnodes, index, edges, newrank);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graph_map(comm, nnodes, index, edges, newrank);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graph_map =============== */
static void MPI_Graph_map_fortran_wrapper(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graph_map((MPI_Comm)(*comm), *nnodes, ((const int*)index), ((const int*)edges), ((int*)newrank));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Graph_map(MPI_Comm_f2c(*comm), *nnodes, ((const int*)index), ((const int*)edges), ((int*)newrank));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPH_MAP(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Graph_map_fortran_wrapper(comm, nnodes, index, edges, newrank, ierr);
}

_EXTERN_C_ void mpi_graph_map(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Graph_map_fortran_wrapper(comm, nnodes, index, edges, newrank, ierr);
}

_EXTERN_C_ void mpi_graph_map_(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Graph_map_fortran_wrapper(comm, nnodes, index, edges, newrank, ierr);
}

_EXTERN_C_ void mpi_graph_map__(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint index[], MPI_Fint edges[], MPI_Fint *newrank, MPI_Fint *ierr) { 
    MPI_Graph_map_fortran_wrapper(comm, nnodes, index, edges, newrank, ierr);
}

/* ================= End Wrappers for MPI_Graph_map ================= */


/* ================== C Wrappers for MPI_Graph_neighbors ================== */
_EXTERN_C_ int PMPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);
_EXTERN_C_ int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graph_neighbors at base level - Location = %px",&(MPI_Graph_neighbors));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graph_neighbors_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graph_neighbors =============== */
static void MPI_Graph_neighbors_fortran_wrapper(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxneighbors, MPI_Fint neighbors[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graph_neighbors((MPI_Comm)(*comm), *rank, *maxneighbors, ((int*)neighbors));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Graph_neighbors(MPI_Comm_f2c(*comm), *rank, *maxneighbors, ((int*)neighbors));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPH_NEIGHBORS(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxneighbors, MPI_Fint neighbors[], MPI_Fint *ierr) { 
    MPI_Graph_neighbors_fortran_wrapper(comm, rank, maxneighbors, neighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxneighbors, MPI_Fint neighbors[], MPI_Fint *ierr) { 
    MPI_Graph_neighbors_fortran_wrapper(comm, rank, maxneighbors, neighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors_(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxneighbors, MPI_Fint neighbors[], MPI_Fint *ierr) { 
    MPI_Graph_neighbors_fortran_wrapper(comm, rank, maxneighbors, neighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors__(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *maxneighbors, MPI_Fint neighbors[], MPI_Fint *ierr) { 
    MPI_Graph_neighbors_fortran_wrapper(comm, rank, maxneighbors, neighbors, ierr);
}

/* ================= End Wrappers for MPI_Graph_neighbors ================= */


/* ================== C Wrappers for MPI_Graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);
_EXTERN_C_ int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graph_neighbors_count at base level - Location = %px",&(MPI_Graph_neighbors_count));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graph_neighbors_count_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graph_neighbors_count(comm, rank, nneighbors);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graph_neighbors_count(comm, rank, nneighbors);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graph_neighbors_count(comm, rank, nneighbors);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graph_neighbors_count =============== */
static void MPI_Graph_neighbors_count_fortran_wrapper(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *nneighbors, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graph_neighbors_count((MPI_Comm)(*comm), *rank, ((int*)nneighbors));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Graph_neighbors_count(MPI_Comm_f2c(*comm), *rank, ((int*)nneighbors));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPH_NEIGHBORS_COUNT(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *nneighbors, MPI_Fint *ierr) { 
    MPI_Graph_neighbors_count_fortran_wrapper(comm, rank, nneighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors_count(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *nneighbors, MPI_Fint *ierr) { 
    MPI_Graph_neighbors_count_fortran_wrapper(comm, rank, nneighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors_count_(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *nneighbors, MPI_Fint *ierr) { 
    MPI_Graph_neighbors_count_fortran_wrapper(comm, rank, nneighbors, ierr);
}

_EXTERN_C_ void mpi_graph_neighbors_count__(MPI_Fint *comm, MPI_Fint *rank, MPI_Fint *nneighbors, MPI_Fint *ierr) { 
    MPI_Graph_neighbors_count_fortran_wrapper(comm, rank, nneighbors, ierr);
}

/* ================= End Wrappers for MPI_Graph_neighbors_count ================= */


/* ================== C Wrappers for MPI_Graphdims_get ================== */
_EXTERN_C_ int PMPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);
_EXTERN_C_ int MPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Graphdims_get at base level - Location = %px",&(MPI_Graphdims_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Graphdims_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Graphdims_get(comm, nnodes, nedges);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Graphdims_get(comm, nnodes, nedges);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Graphdims_get(comm, nnodes, nedges);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Graphdims_get =============== */
static void MPI_Graphdims_get_fortran_wrapper(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint *nedges, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Graphdims_get((MPI_Comm)(*comm), ((int*)nnodes), ((int*)nedges));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Graphdims_get(MPI_Comm_f2c(*comm), ((int*)nnodes), ((int*)nedges));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GRAPHDIMS_GET(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint *nedges, MPI_Fint *ierr) { 
    MPI_Graphdims_get_fortran_wrapper(comm, nnodes, nedges, ierr);
}

_EXTERN_C_ void mpi_graphdims_get(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint *nedges, MPI_Fint *ierr) { 
    MPI_Graphdims_get_fortran_wrapper(comm, nnodes, nedges, ierr);
}

_EXTERN_C_ void mpi_graphdims_get_(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint *nedges, MPI_Fint *ierr) { 
    MPI_Graphdims_get_fortran_wrapper(comm, nnodes, nedges, ierr);
}

_EXTERN_C_ void mpi_graphdims_get__(MPI_Fint *comm, MPI_Fint *nnodes, MPI_Fint *nedges, MPI_Fint *ierr) { 
    MPI_Graphdims_get_fortran_wrapper(comm, nnodes, nedges, ierr);
}

/* ================= End Wrappers for MPI_Graphdims_get ================= */


/* ================== C Wrappers for MPI_Grequest_complete ================== */
_EXTERN_C_ int PMPI_Grequest_complete(MPI_Request request);
_EXTERN_C_ int MPI_Grequest_complete(MPI_Request request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Grequest_complete at base level - Location = %px",&(MPI_Grequest_complete));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Grequest_complete_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Grequest_complete(request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Grequest_complete(request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Grequest_complete(request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Grequest_complete =============== */
static void MPI_Grequest_complete_fortran_wrapper(MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Grequest_complete((MPI_Request)(*request));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Grequest_complete(MPI_Request_f2c(*request));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GREQUEST_COMPLETE(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_complete_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_grequest_complete(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_complete_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_grequest_complete_(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_complete_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_grequest_complete__(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_complete_fortran_wrapper(request, ierr);
}

/* ================= End Wrappers for MPI_Grequest_complete ================= */


/* ================== C Wrappers for MPI_Grequest_start ================== */
_EXTERN_C_ int PMPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);
_EXTERN_C_ int MPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Grequest_start at base level - Location = %px",&(MPI_Grequest_start));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Grequest_start_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Grequest_start =============== */
static void MPI_Grequest_start_fortran_wrapper(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, MPI_Fint *extra_state, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Grequest_start(((MPI_Grequest_query_function*)query_fn), ((MPI_Grequest_free_function*)free_fn), ((MPI_Grequest_cancel_function*)cancel_fn), BufferC2F((void*)extra_state), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Grequest_start(((MPI_Grequest_query_function*)query_fn), ((MPI_Grequest_free_function*)free_fn), ((MPI_Grequest_cancel_function*)cancel_fn), BufferC2F((void*)extra_state), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GREQUEST_START(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, MPI_Fint *extra_state, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_start_fortran_wrapper(query_fn, free_fn, cancel_fn, extra_state, request, ierr);
}

_EXTERN_C_ void mpi_grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, MPI_Fint *extra_state, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_start_fortran_wrapper(query_fn, free_fn, cancel_fn, extra_state, request, ierr);
}

_EXTERN_C_ void mpi_grequest_start_(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, MPI_Fint *extra_state, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_start_fortran_wrapper(query_fn, free_fn, cancel_fn, extra_state, request, ierr);
}

_EXTERN_C_ void mpi_grequest_start__(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, MPI_Fint *extra_state, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Grequest_start_fortran_wrapper(query_fn, free_fn, cancel_fn, extra_state, request, ierr);
}

/* ================= End Wrappers for MPI_Grequest_start ================= */


/* ================== C Wrappers for MPI_Group_compare ================== */
_EXTERN_C_ int PMPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result);
_EXTERN_C_ int MPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_compare at base level - Location = %px",&(MPI_Group_compare));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_compare_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_compare(group1, group2, result);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_compare(group1, group2, result);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_compare(group1, group2, result);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_compare =============== */
static void MPI_Group_compare_fortran_wrapper(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *result, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_compare((MPI_Group)(*group1), (MPI_Group)(*group2), ((int*)result));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Group_compare(MPI_Group_f2c(*group1), MPI_Group_f2c(*group2), ((int*)result));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_COMPARE(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Group_compare_fortran_wrapper(group1, group2, result, ierr);
}

_EXTERN_C_ void mpi_group_compare(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Group_compare_fortran_wrapper(group1, group2, result, ierr);
}

_EXTERN_C_ void mpi_group_compare_(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Group_compare_fortran_wrapper(group1, group2, result, ierr);
}

_EXTERN_C_ void mpi_group_compare__(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *result, MPI_Fint *ierr) { 
    MPI_Group_compare_fortran_wrapper(group1, group2, result, ierr);
}

/* ================= End Wrappers for MPI_Group_compare ================= */


/* ================== C Wrappers for MPI_Group_difference ================== */
_EXTERN_C_ int PMPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_difference at base level - Location = %px",&(MPI_Group_difference));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_difference_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_difference(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_difference(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_difference(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_difference =============== */
static void MPI_Group_difference_fortran_wrapper(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_difference((MPI_Group)(*group1), (MPI_Group)(*group2), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_difference(MPI_Group_f2c(*group1), MPI_Group_f2c(*group2), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_DIFFERENCE(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_difference_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_difference(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_difference_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_difference_(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_difference_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_difference__(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_difference_fortran_wrapper(group1, group2, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_difference ================= */


/* ================== C Wrappers for MPI_Group_excl ================== */
_EXTERN_C_ int PMPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_excl at base level - Location = %px",&(MPI_Group_excl));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_excl_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_excl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_excl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_excl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_excl =============== */
static void MPI_Group_excl_fortran_wrapper(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_excl((MPI_Group)(*group), *n, ((const int*)ranks), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_excl(MPI_Group_f2c(*group), *n, ((const int*)ranks), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_EXCL(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_excl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_excl(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_excl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_excl_(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_excl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_excl__(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_excl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_excl ================= */


/* ================== C Wrappers for MPI_Group_free ================== */
_EXTERN_C_ int PMPI_Group_free(MPI_Group *group);
_EXTERN_C_ int MPI_Group_free(MPI_Group *group) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_free at base level - Location = %px",&(MPI_Group_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_free(group);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_free(group);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_free(group);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_free =============== */
static void MPI_Group_free_fortran_wrapper(MPI_Fint *group, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_free((MPI_Group*)group);
#else /* MPI-2 safe call */
    MPI_Group temp_group;
    temp_group = MPI_Group_f2c(*group);
    _wrap_py_return_val = MPI_Group_free(&temp_group);
    *group = MPI_Group_c2f(temp_group);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_FREE(MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Group_free_fortran_wrapper(group, ierr);
}

_EXTERN_C_ void mpi_group_free(MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Group_free_fortran_wrapper(group, ierr);
}

_EXTERN_C_ void mpi_group_free_(MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Group_free_fortran_wrapper(group, ierr);
}

_EXTERN_C_ void mpi_group_free__(MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Group_free_fortran_wrapper(group, ierr);
}

/* ================= End Wrappers for MPI_Group_free ================= */


/* ================== C Wrappers for MPI_Group_incl ================== */
_EXTERN_C_ int PMPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_incl at base level - Location = %px",&(MPI_Group_incl));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_incl_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_incl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_incl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_incl(group, n, ranks, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_incl =============== */
static void MPI_Group_incl_fortran_wrapper(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_incl((MPI_Group)(*group), *n, ((const int*)ranks), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_incl(MPI_Group_f2c(*group), *n, ((const int*)ranks), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_INCL(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_incl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_incl(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_incl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_incl_(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_incl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_incl__(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranks[], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_incl_fortran_wrapper(group, n, ranks, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_incl ================= */


/* ================== C Wrappers for MPI_Group_intersection ================== */
_EXTERN_C_ int PMPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_intersection at base level - Location = %px",&(MPI_Group_intersection));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_intersection_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_intersection(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_intersection(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_intersection(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_intersection =============== */
static void MPI_Group_intersection_fortran_wrapper(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_intersection((MPI_Group)(*group1), (MPI_Group)(*group2), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_intersection(MPI_Group_f2c(*group1), MPI_Group_f2c(*group2), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_INTERSECTION(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_intersection_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_intersection(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_intersection_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_intersection_(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_intersection_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_intersection__(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_intersection_fortran_wrapper(group1, group2, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_intersection ================= */


/* ================== C Wrappers for MPI_Group_range_excl ================== */
_EXTERN_C_ int PMPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_range_excl at base level - Location = %px",&(MPI_Group_range_excl));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_range_excl_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_range_excl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_range_excl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_range_excl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_range_excl =============== */
static void MPI_Group_range_excl_fortran_wrapper(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_range_excl((MPI_Group)(*group), *n, ((int(*)[3])ranges), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_range_excl(MPI_Group_f2c(*group), *n, ((int(*)[3])ranges), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_RANGE_EXCL(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_excl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_excl(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_excl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_excl_(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_excl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_excl__(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_excl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_range_excl ================= */


/* ================== C Wrappers for MPI_Group_range_incl ================== */
_EXTERN_C_ int PMPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_range_incl at base level - Location = %px",&(MPI_Group_range_incl));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_range_incl_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_range_incl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_range_incl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_range_incl(group, n, ranges, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_range_incl =============== */
static void MPI_Group_range_incl_fortran_wrapper(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_range_incl((MPI_Group)(*group), *n, ((int(*)[3])ranges), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_range_incl(MPI_Group_f2c(*group), *n, ((int(*)[3])ranges), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_RANGE_INCL(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_incl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_incl(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_incl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_incl_(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_incl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_range_incl__(MPI_Fint *group, MPI_Fint *n, MPI_Fint ranges[][3], MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_range_incl_fortran_wrapper(group, n, ranges, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_range_incl ================= */


/* ================== C Wrappers for MPI_Group_rank ================== */
_EXTERN_C_ int PMPI_Group_rank(MPI_Group group, int *rank);
_EXTERN_C_ int MPI_Group_rank(MPI_Group group, int *rank) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_rank at base level - Location = %px",&(MPI_Group_rank));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_rank_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_rank(group, rank);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_rank(group, rank);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_rank(group, rank);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_rank =============== */
static void MPI_Group_rank_fortran_wrapper(MPI_Fint *group, MPI_Fint *rank, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_rank((MPI_Group)(*group), ((int*)rank));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Group_rank(MPI_Group_f2c(*group), ((int*)rank));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_RANK(MPI_Fint *group, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Group_rank_fortran_wrapper(group, rank, ierr);
}

_EXTERN_C_ void mpi_group_rank(MPI_Fint *group, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Group_rank_fortran_wrapper(group, rank, ierr);
}

_EXTERN_C_ void mpi_group_rank_(MPI_Fint *group, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Group_rank_fortran_wrapper(group, rank, ierr);
}

_EXTERN_C_ void mpi_group_rank__(MPI_Fint *group, MPI_Fint *rank, MPI_Fint *ierr) { 
    MPI_Group_rank_fortran_wrapper(group, rank, ierr);
}

/* ================= End Wrappers for MPI_Group_rank ================= */


/* ================== C Wrappers for MPI_Group_size ================== */
_EXTERN_C_ int PMPI_Group_size(MPI_Group group, int *size);
_EXTERN_C_ int MPI_Group_size(MPI_Group group, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_size at base level - Location = %px",&(MPI_Group_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_size(group, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_size(group, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_size(group, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_size =============== */
static void MPI_Group_size_fortran_wrapper(MPI_Fint *group, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_size((MPI_Group)(*group), ((int*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Group_size(MPI_Group_f2c(*group), ((int*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_SIZE(MPI_Fint *group, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Group_size_fortran_wrapper(group, size, ierr);
}

_EXTERN_C_ void mpi_group_size(MPI_Fint *group, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Group_size_fortran_wrapper(group, size, ierr);
}

_EXTERN_C_ void mpi_group_size_(MPI_Fint *group, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Group_size_fortran_wrapper(group, size, ierr);
}

_EXTERN_C_ void mpi_group_size__(MPI_Fint *group, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Group_size_fortran_wrapper(group, size, ierr);
}

/* ================= End Wrappers for MPI_Group_size ================= */


/* ================== C Wrappers for MPI_Group_translate_ranks ================== */
_EXTERN_C_ int PMPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);
_EXTERN_C_ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_translate_ranks at base level - Location = %px",&(MPI_Group_translate_ranks));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_translate_ranks_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_translate_ranks =============== */
static void MPI_Group_translate_ranks_fortran_wrapper(MPI_Fint *group1, MPI_Fint *n, MPI_Fint ranks1[], MPI_Fint *group2, MPI_Fint ranks2[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_translate_ranks((MPI_Group)(*group1), *n, ((const int*)ranks1), (MPI_Group)(*group2), ((int*)ranks2));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Group_translate_ranks(MPI_Group_f2c(*group1), *n, ((const int*)ranks1), MPI_Group_f2c(*group2), ((int*)ranks2));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_TRANSLATE_RANKS(MPI_Fint *group1, MPI_Fint *n, MPI_Fint ranks1[], MPI_Fint *group2, MPI_Fint ranks2[], MPI_Fint *ierr) { 
    MPI_Group_translate_ranks_fortran_wrapper(group1, n, ranks1, group2, ranks2, ierr);
}

_EXTERN_C_ void mpi_group_translate_ranks(MPI_Fint *group1, MPI_Fint *n, MPI_Fint ranks1[], MPI_Fint *group2, MPI_Fint ranks2[], MPI_Fint *ierr) { 
    MPI_Group_translate_ranks_fortran_wrapper(group1, n, ranks1, group2, ranks2, ierr);
}

_EXTERN_C_ void mpi_group_translate_ranks_(MPI_Fint *group1, MPI_Fint *n, MPI_Fint ranks1[], MPI_Fint *group2, MPI_Fint ranks2[], MPI_Fint *ierr) { 
    MPI_Group_translate_ranks_fortran_wrapper(group1, n, ranks1, group2, ranks2, ierr);
}

_EXTERN_C_ void mpi_group_translate_ranks__(MPI_Fint *group1, MPI_Fint *n, MPI_Fint ranks1[], MPI_Fint *group2, MPI_Fint ranks2[], MPI_Fint *ierr) { 
    MPI_Group_translate_ranks_fortran_wrapper(group1, n, ranks1, group2, ranks2, ierr);
}

/* ================= End Wrappers for MPI_Group_translate_ranks ================= */


/* ================== C Wrappers for MPI_Group_union ================== */
_EXTERN_C_ int PMPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Group_union at base level - Location = %px",&(MPI_Group_union));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Group_union_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Group_union(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Group_union(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Group_union(group1, group2, newgroup);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Group_union =============== */
static void MPI_Group_union_fortran_wrapper(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Group_union((MPI_Group)(*group1), (MPI_Group)(*group2), (MPI_Group*)newgroup);
#else /* MPI-2 safe call */
    MPI_Group temp_newgroup;
    _wrap_py_return_val = MPI_Group_union(MPI_Group_f2c(*group1), MPI_Group_f2c(*group2), &temp_newgroup);
    *newgroup = MPI_Group_c2f(temp_newgroup);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_GROUP_UNION(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_union_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_union(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_union_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_union_(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_union_fortran_wrapper(group1, group2, newgroup, ierr);
}

_EXTERN_C_ void mpi_group_union__(MPI_Fint *group1, MPI_Fint *group2, MPI_Fint *newgroup, MPI_Fint *ierr) { 
    MPI_Group_union_fortran_wrapper(group1, group2, newgroup, ierr);
}

/* ================= End Wrappers for MPI_Group_union ================= */


/* ================== C Wrappers for MPI_Iallgather ================== */
_EXTERN_C_ int PMPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iallgather at base level - Location = %px",&(MPI_Iallgather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iallgather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iallgather =============== */
static void MPI_Iallgather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iallgather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iallgather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLGATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iallgather ================= */


/* ================== C Wrappers for MPI_Iallgatherv ================== */
_EXTERN_C_ int PMPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iallgatherv at base level - Location = %px",&(MPI_Iallgatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iallgatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iallgatherv =============== */
static void MPI_Iallgatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iallgatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iallgatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLGATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallgatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iallgatherv ================= */


/* ================== C Wrappers for MPI_Iallreduce ================== */
_EXTERN_C_ int PMPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iallreduce at base level - Location = %px",&(MPI_Iallreduce));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iallreduce_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iallreduce =============== */
static void MPI_Iallreduce_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iallreduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iallreduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLREDUCE(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallreduce(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallreduce_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iallreduce__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iallreduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iallreduce ================= */


/* ================== C Wrappers for MPI_Ialltoall ================== */
_EXTERN_C_ int PMPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ialltoall at base level - Location = %px",&(MPI_Ialltoall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ialltoall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ialltoall =============== */
static void MPI_Ialltoall_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ialltoall(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ialltoall(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLTOALL(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoall(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoall_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoall__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ialltoall ================= */


/* ================== C Wrappers for MPI_Ialltoallv ================== */
_EXTERN_C_ int PMPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ialltoallv at base level - Location = %px",&(MPI_Ialltoallv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ialltoallv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ialltoallv =============== */
static void MPI_Ialltoallv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ialltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ialltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLTOALLV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ialltoallv ================= */


/* ================== C Wrappers for MPI_Ialltoallw ================== */
_EXTERN_C_ int PMPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ialltoallw at base level - Location = %px",&(MPI_Ialltoallw));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ialltoallw_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ialltoallw =============== */
static void MPI_Ialltoallw_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ialltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), ((const MPI_Datatype*)recvtypes), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ialltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), ((const MPI_Datatype*)recvtypes), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IALLTOALLW(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallw(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallw_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ialltoallw__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ialltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ialltoallw ================= */


/* ================== C Wrappers for MPI_Ibarrier ================== */
_EXTERN_C_ int PMPI_Ibarrier(MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibarrier(MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ibarrier at base level - Location = %px",&(MPI_Ibarrier));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ibarrier_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ibarrier(comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ibarrier(comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ibarrier(comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ibarrier =============== */
static void MPI_Ibarrier_fortran_wrapper(MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ibarrier((MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ibarrier(MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IBARRIER(MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibarrier_fortran_wrapper(comm, request, ierr);
}

_EXTERN_C_ void mpi_ibarrier(MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibarrier_fortran_wrapper(comm, request, ierr);
}

_EXTERN_C_ void mpi_ibarrier_(MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibarrier_fortran_wrapper(comm, request, ierr);
}

_EXTERN_C_ void mpi_ibarrier__(MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibarrier_fortran_wrapper(comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ibarrier ================= */


/* ================== C Wrappers for MPI_Ibcast ================== */
_EXTERN_C_ int PMPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ibcast at base level - Location = %px",&(MPI_Ibcast));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ibcast_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ibcast(buffer, count, datatype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ibcast(buffer, count, datatype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ibcast(buffer, count, datatype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ibcast =============== */
static void MPI_Ibcast_fortran_wrapper(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ibcast(BufferC2F((void*)buffer), *count, (MPI_Datatype)(*datatype), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ibcast(BufferC2F((void*)buffer), *count, MPI_Type_f2c(*datatype), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IBCAST(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibcast_fortran_wrapper(buffer, count, datatype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibcast(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibcast_fortran_wrapper(buffer, count, datatype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibcast_(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibcast_fortran_wrapper(buffer, count, datatype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibcast__(MPI_Fint *buffer, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibcast_fortran_wrapper(buffer, count, datatype, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ibcast ================= */


/* ================== C Wrappers for MPI_Ibsend ================== */
_EXTERN_C_ int PMPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ibsend at base level - Location = %px",&(MPI_Ibsend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ibsend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ibsend =============== */
static void MPI_Ibsend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ibsend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ibsend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IBSEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibsend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibsend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ibsend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ibsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ibsend ================= */


/* ================== C Wrappers for MPI_Iexscan ================== */
_EXTERN_C_ int PMPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iexscan at base level - Location = %px",&(MPI_Iexscan));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iexscan_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iexscan =============== */
static void MPI_Iexscan_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iexscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iexscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IEXSCAN(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iexscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iexscan(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iexscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iexscan_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iexscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iexscan__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iexscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iexscan ================= */


/* ================== C Wrappers for MPI_Igather ================== */
_EXTERN_C_ int PMPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Igather at base level - Location = %px",&(MPI_Igather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Igather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Igather =============== */
static void MPI_Igather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Igather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Igather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IGATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Igather ================= */


/* ================== C Wrappers for MPI_Igatherv ================== */
_EXTERN_C_ int PMPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Igatherv at base level - Location = %px",&(MPI_Igatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Igatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Igatherv =============== */
static void MPI_Igatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Igatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Igatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IGATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_igatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Igatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Igatherv ================= */


/* ================== C Wrappers for MPI_Improbe ================== */
_EXTERN_C_ int PMPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Improbe at base level - Location = %px",&(MPI_Improbe));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Improbe_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Improbe(source, tag, comm, flag, message, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Improbe(source, tag, comm, flag, message, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Improbe(source, tag, comm, flag, message, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Improbe =============== */
static void MPI_Improbe_fortran_wrapper(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Improbe(*source, *tag, (MPI_Comm)(*comm), ((int*)flag), ((MPI_Message*)message), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Improbe(*source, *tag, MPI_Comm_f2c(*comm), ((int*)flag), ((MPI_Message*)message), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Improbe(*source, *tag, MPI_Comm_f2c(*comm), ((int*)flag), ((MPI_Message*)message), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IMPROBE(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Improbe_fortran_wrapper(source, tag, comm, flag, message, status, ierr);
}

_EXTERN_C_ void mpi_improbe(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Improbe_fortran_wrapper(source, tag, comm, flag, message, status, ierr);
}

_EXTERN_C_ void mpi_improbe_(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Improbe_fortran_wrapper(source, tag, comm, flag, message, status, ierr);
}

_EXTERN_C_ void mpi_improbe__(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Improbe_fortran_wrapper(source, tag, comm, flag, message, status, ierr);
}

/* ================= End Wrappers for MPI_Improbe ================= */


/* ================== C Wrappers for MPI_Imrecv ================== */
_EXTERN_C_ int PMPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);
_EXTERN_C_ int MPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Imrecv at base level - Location = %px",&(MPI_Imrecv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Imrecv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Imrecv(buf, count, type, message, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Imrecv(buf, count, type, message, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Imrecv(buf, count, type, message, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Imrecv =============== */
static void MPI_Imrecv_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Imrecv(BufferC2F((void*)buf), *count, (MPI_Datatype)(*type), ((MPI_Message*)message), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Imrecv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*type), ((MPI_Message*)message), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IMRECV(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Imrecv_fortran_wrapper(buf, count, type, message, request, ierr);
}

_EXTERN_C_ void mpi_imrecv(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Imrecv_fortran_wrapper(buf, count, type, message, request, ierr);
}

_EXTERN_C_ void mpi_imrecv_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Imrecv_fortran_wrapper(buf, count, type, message, request, ierr);
}

_EXTERN_C_ void mpi_imrecv__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Imrecv_fortran_wrapper(buf, count, type, message, request, ierr);
}

/* ================= End Wrappers for MPI_Imrecv ================= */


/* ================== C Wrappers for MPI_Ineighbor_allgather ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ineighbor_allgather at base level - Location = %px",&(MPI_Ineighbor_allgather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ineighbor_allgather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ineighbor_allgather =============== */
static void MPI_Ineighbor_allgather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ineighbor_allgather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ineighbor_allgather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INEIGHBOR_ALLGATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ineighbor_allgather ================= */


/* ================== C Wrappers for MPI_Ineighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ineighbor_allgatherv at base level - Location = %px",&(MPI_Ineighbor_allgatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ineighbor_allgatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ineighbor_allgatherv =============== */
static void MPI_Ineighbor_allgatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ineighbor_allgatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ineighbor_allgatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INEIGHBOR_ALLGATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_allgatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ineighbor_allgatherv ================= */


/* ================== C Wrappers for MPI_Ineighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ineighbor_alltoall at base level - Location = %px",&(MPI_Ineighbor_alltoall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ineighbor_alltoall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ineighbor_alltoall =============== */
static void MPI_Ineighbor_alltoall_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ineighbor_alltoall(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ineighbor_alltoall(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INEIGHBOR_ALLTOALL(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoall(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoall_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoall__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ineighbor_alltoall ================= */


/* ================== C Wrappers for MPI_Ineighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ineighbor_alltoallv at base level - Location = %px",&(MPI_Ineighbor_alltoallv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ineighbor_alltoallv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ineighbor_alltoallv =============== */
static void MPI_Ineighbor_alltoallv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ineighbor_alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ineighbor_alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INEIGHBOR_ALLTOALLV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ineighbor_alltoallv ================= */


/* ================== C Wrappers for MPI_Ineighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ineighbor_alltoallw at base level - Location = %px",&(MPI_Ineighbor_alltoallw));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ineighbor_alltoallw_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ineighbor_alltoallw =============== */
static void MPI_Ineighbor_alltoallw_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ineighbor_alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const MPI_Aint*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const MPI_Aint*)rdispls), ((const MPI_Datatype*)recvtypes), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ineighbor_alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const MPI_Aint*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const MPI_Aint*)rdispls), ((const MPI_Datatype*)recvtypes), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INEIGHBOR_ALLTOALLW(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallw(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallw_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

_EXTERN_C_ void mpi_ineighbor_alltoallw__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ineighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ineighbor_alltoallw ================= */


/* ================== C Wrappers for MPI_Info_create ================== */
_EXTERN_C_ int PMPI_Info_create(MPI_Info *info);
_EXTERN_C_ int MPI_Info_create(MPI_Info *info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_create at base level - Location = %px",&(MPI_Info_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_create(info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_create(info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_create(info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_create =============== */
static void MPI_Info_create_fortran_wrapper(MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_create((MPI_Info*)info);
#else /* MPI-2 safe call */
    MPI_Info temp_info;
    _wrap_py_return_val = MPI_Info_create(&temp_info);
    *info = MPI_Info_c2f(temp_info);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_CREATE(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_create_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_create(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_create_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_create_(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_create_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_create__(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_create_fortran_wrapper(info, ierr);
}

/* ================= End Wrappers for MPI_Info_create ================= */


/* ================== C Wrappers for MPI_Info_delete ================== */
_EXTERN_C_ int PMPI_Info_delete(MPI_Info info, const char *key);
_EXTERN_C_ int MPI_Info_delete(MPI_Info info, const char *key) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_delete at base level - Location = %px",&(MPI_Info_delete));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_delete_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_delete(info, key);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_delete(info, key);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_delete(info, key);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_delete =============== */
static void MPI_Info_delete_fortran_wrapper(MPI_Fint *info, MPI_Fint *key, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_delete((MPI_Info)(*info), ((const char*)key));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Info_delete(MPI_Info_f2c(*info), ((const char*)key));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_DELETE(MPI_Fint *info, MPI_Fint *key, MPI_Fint *ierr) { 
    MPI_Info_delete_fortran_wrapper(info, key, ierr);
}

_EXTERN_C_ void mpi_info_delete(MPI_Fint *info, MPI_Fint *key, MPI_Fint *ierr) { 
    MPI_Info_delete_fortran_wrapper(info, key, ierr);
}

_EXTERN_C_ void mpi_info_delete_(MPI_Fint *info, MPI_Fint *key, MPI_Fint *ierr) { 
    MPI_Info_delete_fortran_wrapper(info, key, ierr);
}

_EXTERN_C_ void mpi_info_delete__(MPI_Fint *info, MPI_Fint *key, MPI_Fint *ierr) { 
    MPI_Info_delete_fortran_wrapper(info, key, ierr);
}

/* ================= End Wrappers for MPI_Info_delete ================= */


/* ================== C Wrappers for MPI_Info_dup ================== */
_EXTERN_C_ int PMPI_Info_dup(MPI_Info info, MPI_Info *newinfo);
_EXTERN_C_ int MPI_Info_dup(MPI_Info info, MPI_Info *newinfo) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_dup at base level - Location = %px",&(MPI_Info_dup));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_dup_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_dup(info, newinfo);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_dup(info, newinfo);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_dup(info, newinfo);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_dup =============== */
static void MPI_Info_dup_fortran_wrapper(MPI_Fint *info, MPI_Fint *newinfo, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_dup((MPI_Info)(*info), (MPI_Info*)newinfo);
#else /* MPI-2 safe call */
    MPI_Info temp_newinfo;
    _wrap_py_return_val = MPI_Info_dup(MPI_Info_f2c(*info), &temp_newinfo);
    *newinfo = MPI_Info_c2f(temp_newinfo);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_DUP(MPI_Fint *info, MPI_Fint *newinfo, MPI_Fint *ierr) { 
    MPI_Info_dup_fortran_wrapper(info, newinfo, ierr);
}

_EXTERN_C_ void mpi_info_dup(MPI_Fint *info, MPI_Fint *newinfo, MPI_Fint *ierr) { 
    MPI_Info_dup_fortran_wrapper(info, newinfo, ierr);
}

_EXTERN_C_ void mpi_info_dup_(MPI_Fint *info, MPI_Fint *newinfo, MPI_Fint *ierr) { 
    MPI_Info_dup_fortran_wrapper(info, newinfo, ierr);
}

_EXTERN_C_ void mpi_info_dup__(MPI_Fint *info, MPI_Fint *newinfo, MPI_Fint *ierr) { 
    MPI_Info_dup_fortran_wrapper(info, newinfo, ierr);
}

/* ================= End Wrappers for MPI_Info_dup ================= */


/* ================== C Wrappers for MPI_Info_free ================== */
_EXTERN_C_ int PMPI_Info_free(MPI_Info *info);
_EXTERN_C_ int MPI_Info_free(MPI_Info *info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_free at base level - Location = %px",&(MPI_Info_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_free(info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_free(info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_free(info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_free =============== */
static void MPI_Info_free_fortran_wrapper(MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_free((MPI_Info*)info);
#else /* MPI-2 safe call */
    MPI_Info temp_info;
    temp_info = MPI_Info_f2c(*info);
    _wrap_py_return_val = MPI_Info_free(&temp_info);
    *info = MPI_Info_c2f(temp_info);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_FREE(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_free_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_free(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_free_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_free_(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_free_fortran_wrapper(info, ierr);
}

_EXTERN_C_ void mpi_info_free__(MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Info_free_fortran_wrapper(info, ierr);
}

/* ================= End Wrappers for MPI_Info_free ================= */


/* ================== C Wrappers for MPI_Info_get ================== */
_EXTERN_C_ int PMPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag);
_EXTERN_C_ int MPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_get at base level - Location = %px",&(MPI_Info_get));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_get_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_get(info, key, valuelen, value, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_get(info, key, valuelen, value, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_get(info, key, valuelen, value, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_get =============== */
static void MPI_Info_get_fortran_wrapper(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, char *value, MPI_Fint *flag, MPI_Fint *ierr, int value_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_get((MPI_Info)(*info), ((const char*)key), *valuelen, value, ((int*)flag));
#else /* MPI-2 safe call */
    char* temp_value;
    temp_value = (char*)malloc(sizeof(char) * (value_len+1));
    char_p_f2c(value,value_len,&temp_value);
    _wrap_py_return_val = MPI_Info_get(MPI_Info_f2c(*info), ((const char*)key), *valuelen, value, ((int*)flag));
    char_p_c2f(temp_value,value,value_len);
 if(temp_value) free(temp_value);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_GET(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, char *value, MPI_Fint *flag, MPI_Fint *ierr, int value_len) { 
    MPI_Info_get_fortran_wrapper(info, key, valuelen, value, flag, ierr, value_len);
}

_EXTERN_C_ void mpi_info_get(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, char *value, MPI_Fint *flag, MPI_Fint *ierr, int value_len) { 
    MPI_Info_get_fortran_wrapper(info, key, valuelen, value, flag, ierr, value_len);
}

_EXTERN_C_ void mpi_info_get_(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, char *value, MPI_Fint *flag, MPI_Fint *ierr, int value_len) { 
    MPI_Info_get_fortran_wrapper(info, key, valuelen, value, flag, ierr, value_len);
}

_EXTERN_C_ void mpi_info_get__(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, char *value, MPI_Fint *flag, MPI_Fint *ierr, int value_len) { 
    MPI_Info_get_fortran_wrapper(info, key, valuelen, value, flag, ierr, value_len);
}

/* ================= End Wrappers for MPI_Info_get ================= */


/* ================== C Wrappers for MPI_Info_get_nkeys ================== */
_EXTERN_C_ int PMPI_Info_get_nkeys(MPI_Info info, int *nkeys);
_EXTERN_C_ int MPI_Info_get_nkeys(MPI_Info info, int *nkeys) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_get_nkeys at base level - Location = %px",&(MPI_Info_get_nkeys));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_get_nkeys_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_get_nkeys(info, nkeys);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_get_nkeys(info, nkeys);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_get_nkeys(info, nkeys);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_get_nkeys =============== */
static void MPI_Info_get_nkeys_fortran_wrapper(MPI_Fint *info, MPI_Fint *nkeys, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_get_nkeys((MPI_Info)(*info), ((int*)nkeys));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Info_get_nkeys(MPI_Info_f2c(*info), ((int*)nkeys));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_GET_NKEYS(MPI_Fint *info, MPI_Fint *nkeys, MPI_Fint *ierr) { 
    MPI_Info_get_nkeys_fortran_wrapper(info, nkeys, ierr);
}

_EXTERN_C_ void mpi_info_get_nkeys(MPI_Fint *info, MPI_Fint *nkeys, MPI_Fint *ierr) { 
    MPI_Info_get_nkeys_fortran_wrapper(info, nkeys, ierr);
}

_EXTERN_C_ void mpi_info_get_nkeys_(MPI_Fint *info, MPI_Fint *nkeys, MPI_Fint *ierr) { 
    MPI_Info_get_nkeys_fortran_wrapper(info, nkeys, ierr);
}

_EXTERN_C_ void mpi_info_get_nkeys__(MPI_Fint *info, MPI_Fint *nkeys, MPI_Fint *ierr) { 
    MPI_Info_get_nkeys_fortran_wrapper(info, nkeys, ierr);
}

/* ================= End Wrappers for MPI_Info_get_nkeys ================= */


/* ================== C Wrappers for MPI_Info_get_nthkey ================== */
_EXTERN_C_ int PMPI_Info_get_nthkey(MPI_Info info, int n, char *key);
_EXTERN_C_ int MPI_Info_get_nthkey(MPI_Info info, int n, char *key) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_get_nthkey at base level - Location = %px",&(MPI_Info_get_nthkey));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_get_nthkey_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_get_nthkey(info, n, key);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_get_nthkey(info, n, key);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_get_nthkey(info, n, key);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_get_nthkey =============== */
static void MPI_Info_get_nthkey_fortran_wrapper(MPI_Fint *info, MPI_Fint *n, char *key, MPI_Fint *ierr, int key_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_get_nthkey((MPI_Info)(*info), *n, key);
#else /* MPI-2 safe call */
    char* temp_key;
    temp_key = (char*)malloc(sizeof(char) * (key_len+1));
    char_p_f2c(key,key_len,&temp_key);
    _wrap_py_return_val = MPI_Info_get_nthkey(MPI_Info_f2c(*info), *n, key);
    char_p_c2f(temp_key,key,key_len);
 if(temp_key) free(temp_key);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_GET_NTHKEY(MPI_Fint *info, MPI_Fint *n, char *key, MPI_Fint *ierr, int key_len) { 
    MPI_Info_get_nthkey_fortran_wrapper(info, n, key, ierr, key_len);
}

_EXTERN_C_ void mpi_info_get_nthkey(MPI_Fint *info, MPI_Fint *n, char *key, MPI_Fint *ierr, int key_len) { 
    MPI_Info_get_nthkey_fortran_wrapper(info, n, key, ierr, key_len);
}

_EXTERN_C_ void mpi_info_get_nthkey_(MPI_Fint *info, MPI_Fint *n, char *key, MPI_Fint *ierr, int key_len) { 
    MPI_Info_get_nthkey_fortran_wrapper(info, n, key, ierr, key_len);
}

_EXTERN_C_ void mpi_info_get_nthkey__(MPI_Fint *info, MPI_Fint *n, char *key, MPI_Fint *ierr, int key_len) { 
    MPI_Info_get_nthkey_fortran_wrapper(info, n, key, ierr, key_len);
}

/* ================= End Wrappers for MPI_Info_get_nthkey ================= */


/* ================== C Wrappers for MPI_Info_get_valuelen ================== */
_EXTERN_C_ int PMPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag);
_EXTERN_C_ int MPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_get_valuelen at base level - Location = %px",&(MPI_Info_get_valuelen));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_get_valuelen_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_get_valuelen(info, key, valuelen, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_get_valuelen(info, key, valuelen, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_get_valuelen(info, key, valuelen, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_get_valuelen =============== */
static void MPI_Info_get_valuelen_fortran_wrapper(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_get_valuelen((MPI_Info)(*info), ((const char*)key), ((int*)valuelen), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Info_get_valuelen(MPI_Info_f2c(*info), ((const char*)key), ((int*)valuelen), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_GET_VALUELEN(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Info_get_valuelen_fortran_wrapper(info, key, valuelen, flag, ierr);
}

_EXTERN_C_ void mpi_info_get_valuelen(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Info_get_valuelen_fortran_wrapper(info, key, valuelen, flag, ierr);
}

_EXTERN_C_ void mpi_info_get_valuelen_(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Info_get_valuelen_fortran_wrapper(info, key, valuelen, flag, ierr);
}

_EXTERN_C_ void mpi_info_get_valuelen__(MPI_Fint *info, MPI_Fint *key, MPI_Fint *valuelen, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Info_get_valuelen_fortran_wrapper(info, key, valuelen, flag, ierr);
}

/* ================= End Wrappers for MPI_Info_get_valuelen ================= */


/* ================== C Wrappers for MPI_Info_set ================== */
_EXTERN_C_ int PMPI_Info_set(MPI_Info info, const char *key, const char *value);
_EXTERN_C_ int MPI_Info_set(MPI_Info info, const char *key, const char *value) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Info_set at base level - Location = %px",&(MPI_Info_set));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Info_set_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Info_set(info, key, value);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Info_set(info, key, value);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Info_set(info, key, value);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Info_set =============== */
static void MPI_Info_set_fortran_wrapper(MPI_Fint *info, MPI_Fint *key, MPI_Fint *value, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Info_set((MPI_Info)(*info), ((const char*)key), ((const char*)value));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Info_set(MPI_Info_f2c(*info), ((const char*)key), ((const char*)value));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INFO_SET(MPI_Fint *info, MPI_Fint *key, MPI_Fint *value, MPI_Fint *ierr) { 
    MPI_Info_set_fortran_wrapper(info, key, value, ierr);
}

_EXTERN_C_ void mpi_info_set(MPI_Fint *info, MPI_Fint *key, MPI_Fint *value, MPI_Fint *ierr) { 
    MPI_Info_set_fortran_wrapper(info, key, value, ierr);
}

_EXTERN_C_ void mpi_info_set_(MPI_Fint *info, MPI_Fint *key, MPI_Fint *value, MPI_Fint *ierr) { 
    MPI_Info_set_fortran_wrapper(info, key, value, ierr);
}

_EXTERN_C_ void mpi_info_set__(MPI_Fint *info, MPI_Fint *key, MPI_Fint *value, MPI_Fint *ierr) { 
    MPI_Info_set_fortran_wrapper(info, key, value, ierr);
}

/* ================= End Wrappers for MPI_Info_set ================= */


/* ================== C Wrappers for MPI_Initialized ================== */
_EXTERN_C_ int PMPI_Initialized(int *flag);
_EXTERN_C_ int MPI_Initialized(int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Initialized at base level - Location = %px",&(MPI_Initialized));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Initialized_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Initialized(flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Initialized(flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Initialized(flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Initialized =============== */
static void MPI_Initialized_fortran_wrapper(MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Initialized(((int*)flag));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INITIALIZED(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Initialized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_initialized(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Initialized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_initialized_(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Initialized_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_initialized__(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Initialized_fortran_wrapper(flag, ierr);
}

/* ================= End Wrappers for MPI_Initialized ================= */


/* ================== C Wrappers for MPI_Intercomm_create ================== */
_EXTERN_C_ int PMPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Intercomm_create at base level - Location = %px",&(MPI_Intercomm_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Intercomm_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Intercomm_create =============== */
static void MPI_Intercomm_create_fortran_wrapper(MPI_Fint *local_comm, MPI_Fint *local_leader, MPI_Fint *bridge_comm, MPI_Fint *remote_leader, MPI_Fint *tag, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Intercomm_create((MPI_Comm)(*local_comm), *local_leader, (MPI_Comm)(*bridge_comm), *remote_leader, *tag, (MPI_Comm*)newintercomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newintercomm;
    _wrap_py_return_val = MPI_Intercomm_create(MPI_Comm_f2c(*local_comm), *local_leader, MPI_Comm_f2c(*bridge_comm), *remote_leader, *tag, &temp_newintercomm);
    *newintercomm = MPI_Comm_c2f(temp_newintercomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INTERCOMM_CREATE(MPI_Fint *local_comm, MPI_Fint *local_leader, MPI_Fint *bridge_comm, MPI_Fint *remote_leader, MPI_Fint *tag, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_create_fortran_wrapper(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_create(MPI_Fint *local_comm, MPI_Fint *local_leader, MPI_Fint *bridge_comm, MPI_Fint *remote_leader, MPI_Fint *tag, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_create_fortran_wrapper(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_create_(MPI_Fint *local_comm, MPI_Fint *local_leader, MPI_Fint *bridge_comm, MPI_Fint *remote_leader, MPI_Fint *tag, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_create_fortran_wrapper(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_create__(MPI_Fint *local_comm, MPI_Fint *local_leader, MPI_Fint *bridge_comm, MPI_Fint *remote_leader, MPI_Fint *tag, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_create_fortran_wrapper(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm, ierr);
}

/* ================= End Wrappers for MPI_Intercomm_create ================= */


/* ================== C Wrappers for MPI_Intercomm_merge ================== */
_EXTERN_C_ int PMPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Intercomm_merge at base level - Location = %px",&(MPI_Intercomm_merge));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Intercomm_merge_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Intercomm_merge(intercomm, high, newintercomm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Intercomm_merge(intercomm, high, newintercomm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Intercomm_merge(intercomm, high, newintercomm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Intercomm_merge =============== */
static void MPI_Intercomm_merge_fortran_wrapper(MPI_Fint *intercomm, MPI_Fint *high, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Intercomm_merge((MPI_Comm)(*intercomm), *high, (MPI_Comm*)newintercomm);
#else /* MPI-2 safe call */
    MPI_Comm temp_newintercomm;
    _wrap_py_return_val = MPI_Intercomm_merge(MPI_Comm_f2c(*intercomm), *high, &temp_newintercomm);
    *newintercomm = MPI_Comm_c2f(temp_newintercomm);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_INTERCOMM_MERGE(MPI_Fint *intercomm, MPI_Fint *high, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_merge_fortran_wrapper(intercomm, high, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_merge(MPI_Fint *intercomm, MPI_Fint *high, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_merge_fortran_wrapper(intercomm, high, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_merge_(MPI_Fint *intercomm, MPI_Fint *high, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_merge_fortran_wrapper(intercomm, high, newintercomm, ierr);
}

_EXTERN_C_ void mpi_intercomm_merge__(MPI_Fint *intercomm, MPI_Fint *high, MPI_Fint *newintercomm, MPI_Fint *ierr) { 
    MPI_Intercomm_merge_fortran_wrapper(intercomm, high, newintercomm, ierr);
}

/* ================= End Wrappers for MPI_Intercomm_merge ================= */


/* ================== C Wrappers for MPI_Iprobe ================== */
_EXTERN_C_ int PMPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iprobe at base level - Location = %px",&(MPI_Iprobe));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iprobe_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iprobe(source, tag, comm, flag, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iprobe(source, tag, comm, flag, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iprobe(source, tag, comm, flag, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iprobe =============== */
static void MPI_Iprobe_fortran_wrapper(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iprobe(*source, *tag, (MPI_Comm)(*comm), ((int*)flag), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Iprobe(*source, *tag, MPI_Comm_f2c(*comm), ((int*)flag), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Iprobe(*source, *tag, MPI_Comm_f2c(*comm), ((int*)flag), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IPROBE(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Iprobe_fortran_wrapper(source, tag, comm, flag, status, ierr);
}

_EXTERN_C_ void mpi_iprobe(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Iprobe_fortran_wrapper(source, tag, comm, flag, status, ierr);
}

_EXTERN_C_ void mpi_iprobe_(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Iprobe_fortran_wrapper(source, tag, comm, flag, status, ierr);
}

_EXTERN_C_ void mpi_iprobe__(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Iprobe_fortran_wrapper(source, tag, comm, flag, status, ierr);
}

/* ================= End Wrappers for MPI_Iprobe ================= */


/* ================== C Wrappers for MPI_Irecv ================== */
_EXTERN_C_ int PMPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Irecv at base level - Location = %px",&(MPI_Irecv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Irecv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Irecv(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Irecv =============== */
static void MPI_Irecv_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Irecv(BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), *source, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Irecv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *source, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IRECV(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irecv_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irecv(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irecv_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irecv_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irecv_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irecv__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irecv_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Irecv ================= */


/* ================== C Wrappers for MPI_Ireduce ================== */
_EXTERN_C_ int PMPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ireduce at base level - Location = %px",&(MPI_Ireduce));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ireduce_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ireduce =============== */
static void MPI_Ireduce_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ireduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ireduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IREDUCE(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ireduce ================= */


/* ================== C Wrappers for MPI_Ireduce_scatter ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ireduce_scatter at base level - Location = %px",&(MPI_Ireduce_scatter));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ireduce_scatter_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ireduce_scatter =============== */
static void MPI_Ireduce_scatter_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ireduce_scatter(((const void*)sendbuf), BufferC2F((void*)recvbuf), ((const int*)recvcounts), (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ireduce_scatter(((const void*)sendbuf), BufferC2F((void*)recvbuf), ((const int*)recvcounts), MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IREDUCE_SCATTER(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ireduce_scatter ================= */


/* ================== C Wrappers for MPI_Ireduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ireduce_scatter_block at base level - Location = %px",&(MPI_Ireduce_scatter_block));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ireduce_scatter_block_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ireduce_scatter_block =============== */
static void MPI_Ireduce_scatter_block_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ireduce_scatter_block(((const void*)sendbuf), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ireduce_scatter_block(((const void*)sendbuf), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IREDUCE_SCATTER_BLOCK(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter_block(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter_block_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_ireduce_scatter_block__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ireduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ireduce_scatter_block ================= */


/* ================== C Wrappers for MPI_Irsend ================== */
_EXTERN_C_ int PMPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Irsend at base level - Location = %px",&(MPI_Irsend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Irsend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Irsend =============== */
static void MPI_Irsend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Irsend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Irsend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IRSEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irsend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irsend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_irsend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Irsend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Irsend ================= */


/* ================== C Wrappers for MPI_Is_thread_main ================== */
_EXTERN_C_ int PMPI_Is_thread_main(int *flag);
_EXTERN_C_ int MPI_Is_thread_main(int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Is_thread_main at base level - Location = %px",&(MPI_Is_thread_main));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Is_thread_main_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Is_thread_main(flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Is_thread_main(flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Is_thread_main(flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Is_thread_main =============== */
static void MPI_Is_thread_main_fortran_wrapper(MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Is_thread_main(((int*)flag));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_IS_THREAD_MAIN(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Is_thread_main_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_is_thread_main(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Is_thread_main_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_is_thread_main_(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Is_thread_main_fortran_wrapper(flag, ierr);
}

_EXTERN_C_ void mpi_is_thread_main__(MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Is_thread_main_fortran_wrapper(flag, ierr);
}

/* ================= End Wrappers for MPI_Is_thread_main ================= */


/* ================== C Wrappers for MPI_Iscan ================== */
_EXTERN_C_ int PMPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iscan at base level - Location = %px",&(MPI_Iscan));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iscan_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iscan =============== */
static void MPI_Iscan_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iscan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ISCAN(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscan(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscan_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscan__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iscan ================= */


/* ================== C Wrappers for MPI_Iscatter ================== */
_EXTERN_C_ int PMPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iscatter at base level - Location = %px",&(MPI_Iscatter));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iscatter_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iscatter =============== */
static void MPI_Iscatter_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iscatter(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iscatter(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ISCATTER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatter(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatter_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatter__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iscatter ================= */


/* ================== C Wrappers for MPI_Iscatterv ================== */
_EXTERN_C_ int PMPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Iscatterv at base level - Location = %px",&(MPI_Iscatterv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Iscatterv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Iscatterv =============== */
static void MPI_Iscatterv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Iscatterv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)displs), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Iscatterv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)displs), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ISCATTERV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatterv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatterv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

_EXTERN_C_ void mpi_iscatterv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Iscatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Iscatterv ================= */


/* ================== C Wrappers for MPI_Isend ================== */
_EXTERN_C_ int PMPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Isend at base level - Location = %px",&(MPI_Isend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Isend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Isend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Isend =============== */
static void MPI_Isend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Isend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Isend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ISEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Isend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_isend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Isend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_isend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Isend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_isend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Isend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Isend ================= */


/* ================== C Wrappers for MPI_Issend ================== */
_EXTERN_C_ int PMPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Issend at base level - Location = %px",&(MPI_Issend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Issend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Issend(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Issend =============== */
static void MPI_Issend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Issend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Issend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_ISSEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Issend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_issend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Issend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_issend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Issend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_issend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Issend_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Issend ================= */


/* ================== C Wrappers for MPI_Keyval_create ================== */
_EXTERN_C_ int PMPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);
_EXTERN_C_ int MPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Keyval_create at base level - Location = %px",&(MPI_Keyval_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Keyval_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Keyval_create =============== */
static void MPI_Keyval_create_fortran_wrapper(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, MPI_Fint *keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Keyval_create(((MPI_Copy_function*)copy_fn), ((MPI_Delete_function*)delete_fn), ((int*)keyval), BufferC2F((void*)extra_state));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_KEYVAL_CREATE(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, MPI_Fint *keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Keyval_create_fortran_wrapper(copy_fn, delete_fn, keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, MPI_Fint *keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Keyval_create_fortran_wrapper(copy_fn, delete_fn, keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_keyval_create_(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, MPI_Fint *keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Keyval_create_fortran_wrapper(copy_fn, delete_fn, keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_keyval_create__(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, MPI_Fint *keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Keyval_create_fortran_wrapper(copy_fn, delete_fn, keyval, extra_state, ierr);
}

/* ================= End Wrappers for MPI_Keyval_create ================= */


/* ================== C Wrappers for MPI_Keyval_free ================== */
_EXTERN_C_ int PMPI_Keyval_free(int *keyval);
_EXTERN_C_ int MPI_Keyval_free(int *keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Keyval_free at base level - Location = %px",&(MPI_Keyval_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Keyval_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Keyval_free(keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Keyval_free(keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Keyval_free(keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Keyval_free =============== */
static void MPI_Keyval_free_fortran_wrapper(MPI_Fint *keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Keyval_free(((int*)keyval));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_KEYVAL_FREE(MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Keyval_free_fortran_wrapper(keyval, ierr);
}

_EXTERN_C_ void mpi_keyval_free(MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Keyval_free_fortran_wrapper(keyval, ierr);
}

_EXTERN_C_ void mpi_keyval_free_(MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Keyval_free_fortran_wrapper(keyval, ierr);
}

_EXTERN_C_ void mpi_keyval_free__(MPI_Fint *keyval, MPI_Fint *ierr) { 
    MPI_Keyval_free_fortran_wrapper(keyval, ierr);
}

/* ================= End Wrappers for MPI_Keyval_free ================= */


/* ================== C Wrappers for MPI_Lookup_name ================== */
_EXTERN_C_ int PMPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Lookup_name at base level - Location = %px",&(MPI_Lookup_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Lookup_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Lookup_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Lookup_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Lookup_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Lookup_name =============== */
static void MPI_Lookup_name_fortran_wrapper(MPI_Fint *service_name, MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Lookup_name(((const char*)service_name), (MPI_Info)(*info), port_name);
#else /* MPI-2 safe call */
    char* temp_port_name;
    temp_port_name = (char*)malloc(sizeof(char) * (port_name_len+1));
    char_p_f2c(port_name,port_name_len,&temp_port_name);
    _wrap_py_return_val = MPI_Lookup_name(((const char*)service_name), MPI_Info_f2c(*info), port_name);
    char_p_c2f(temp_port_name,port_name,port_name_len);
 if(temp_port_name) free(temp_port_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_LOOKUP_NAME(MPI_Fint *service_name, MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Lookup_name_fortran_wrapper(service_name, info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_lookup_name(MPI_Fint *service_name, MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Lookup_name_fortran_wrapper(service_name, info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_lookup_name_(MPI_Fint *service_name, MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Lookup_name_fortran_wrapper(service_name, info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_lookup_name__(MPI_Fint *service_name, MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Lookup_name_fortran_wrapper(service_name, info, port_name, ierr, port_name_len);
}

/* ================= End Wrappers for MPI_Lookup_name ================= */


/* ================== C Wrappers for MPI_Mprobe ================== */
_EXTERN_C_ int PMPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Mprobe at base level - Location = %px",&(MPI_Mprobe));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Mprobe_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Mprobe(source, tag, comm, message, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Mprobe(source, tag, comm, message, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Mprobe(source, tag, comm, message, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Mprobe =============== */
static void MPI_Mprobe_fortran_wrapper(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Mprobe(*source, *tag, (MPI_Comm)(*comm), ((MPI_Message*)message), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Mprobe(*source, *tag, MPI_Comm_f2c(*comm), ((MPI_Message*)message), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Mprobe(*source, *tag, MPI_Comm_f2c(*comm), ((MPI_Message*)message), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_MPROBE(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mprobe_fortran_wrapper(source, tag, comm, message, status, ierr);
}

_EXTERN_C_ void mpi_mprobe(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mprobe_fortran_wrapper(source, tag, comm, message, status, ierr);
}

_EXTERN_C_ void mpi_mprobe_(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mprobe_fortran_wrapper(source, tag, comm, message, status, ierr);
}

_EXTERN_C_ void mpi_mprobe__(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mprobe_fortran_wrapper(source, tag, comm, message, status, ierr);
}

/* ================= End Wrappers for MPI_Mprobe ================= */


/* ================== C Wrappers for MPI_Mrecv ================== */
_EXTERN_C_ int PMPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Mrecv at base level - Location = %px",&(MPI_Mrecv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Mrecv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Mrecv(buf, count, type, message, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Mrecv(buf, count, type, message, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Mrecv(buf, count, type, message, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Mrecv =============== */
static void MPI_Mrecv_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Mrecv(BufferC2F((void*)buf), *count, (MPI_Datatype)(*type), ((MPI_Message*)message), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Mrecv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*type), ((MPI_Message*)message), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Mrecv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*type), ((MPI_Message*)message), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_MRECV(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mrecv_fortran_wrapper(buf, count, type, message, status, ierr);
}

_EXTERN_C_ void mpi_mrecv(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mrecv_fortran_wrapper(buf, count, type, message, status, ierr);
}

_EXTERN_C_ void mpi_mrecv_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mrecv_fortran_wrapper(buf, count, type, message, status, ierr);
}

_EXTERN_C_ void mpi_mrecv__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *type, MPI_Fint *message, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Mrecv_fortran_wrapper(buf, count, type, message, status, ierr);
}

/* ================= End Wrappers for MPI_Mrecv ================= */


/* ================== C Wrappers for MPI_Neighbor_allgather ================== */
_EXTERN_C_ int PMPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Neighbor_allgather at base level - Location = %px",&(MPI_Neighbor_allgather));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Neighbor_allgather_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Neighbor_allgather =============== */
static void MPI_Neighbor_allgather_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Neighbor_allgather(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Neighbor_allgather(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_NEIGHBOR_ALLGATHER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgather(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgather_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgather__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgather_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Neighbor_allgather ================= */


/* ================== C Wrappers for MPI_Neighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Neighbor_allgatherv at base level - Location = %px",&(MPI_Neighbor_allgatherv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Neighbor_allgatherv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Neighbor_allgatherv =============== */
static void MPI_Neighbor_allgatherv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Neighbor_allgatherv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Neighbor_allgatherv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)displs), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_NEIGHBOR_ALLGATHERV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgatherv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgatherv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_allgatherv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint displs[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_allgatherv_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Neighbor_allgatherv ================= */


/* ================== C Wrappers for MPI_Neighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Neighbor_alltoall at base level - Location = %px",&(MPI_Neighbor_alltoall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Neighbor_alltoall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Neighbor_alltoall =============== */
static void MPI_Neighbor_alltoall_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Neighbor_alltoall(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Neighbor_alltoall(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_NEIGHBOR_ALLTOALL(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoall(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoall_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoall__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoall_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Neighbor_alltoall ================= */


/* ================== C Wrappers for MPI_Neighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Neighbor_alltoallv at base level - Location = %px",&(MPI_Neighbor_alltoallv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Neighbor_alltoallv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Neighbor_alltoallv =============== */
static void MPI_Neighbor_alltoallv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Neighbor_alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), (MPI_Datatype)(*recvtype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Neighbor_alltoallv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)sdispls), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const int*)rdispls), MPI_Type_f2c(*recvtype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_NEIGHBOR_ALLTOALLV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint *recvtype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallv_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, ierr);
}

/* ================= End Wrappers for MPI_Neighbor_alltoallv ================= */


/* ================== C Wrappers for MPI_Neighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Neighbor_alltoallw at base level - Location = %px",&(MPI_Neighbor_alltoallw));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Neighbor_alltoallw_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Neighbor_alltoallw =============== */
static void MPI_Neighbor_alltoallw_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Neighbor_alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const MPI_Aint*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const MPI_Aint*)rdispls), ((const MPI_Datatype*)recvtypes), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Neighbor_alltoallw(((const void*)sendbuf), ((const int*)sendcounts), ((const MPI_Aint*)sdispls), ((const MPI_Datatype*)sendtypes), BufferC2F((void*)recvbuf), ((const int*)recvcounts), ((const MPI_Aint*)rdispls), ((const MPI_Datatype*)recvtypes), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_NEIGHBOR_ALLTOALLW(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallw(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallw_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

_EXTERN_C_ void mpi_neighbor_alltoallw__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint sdispls[], MPI_Fint sendtypes[], MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint rdispls[], MPI_Fint recvtypes[], MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Neighbor_alltoallw_fortran_wrapper(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, ierr);
}

/* ================= End Wrappers for MPI_Neighbor_alltoallw ================= */


/* ================== C Wrappers for MPI_Op_commutative ================== */
_EXTERN_C_ int PMPI_Op_commutative(MPI_Op op, int *commute);
_EXTERN_C_ int MPI_Op_commutative(MPI_Op op, int *commute) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Op_commutative at base level - Location = %px",&(MPI_Op_commutative));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Op_commutative_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Op_commutative(op, commute);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Op_commutative(op, commute);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Op_commutative(op, commute);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Op_commutative =============== */
static void MPI_Op_commutative_fortran_wrapper(MPI_Fint *op, MPI_Fint *commute, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Op_commutative((MPI_Op)(*op), ((int*)commute));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Op_commutative(MPI_Op_f2c(*op), ((int*)commute));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_OP_COMMUTATIVE(MPI_Fint *op, MPI_Fint *commute, MPI_Fint *ierr) { 
    MPI_Op_commutative_fortran_wrapper(op, commute, ierr);
}

_EXTERN_C_ void mpi_op_commutative(MPI_Fint *op, MPI_Fint *commute, MPI_Fint *ierr) { 
    MPI_Op_commutative_fortran_wrapper(op, commute, ierr);
}

_EXTERN_C_ void mpi_op_commutative_(MPI_Fint *op, MPI_Fint *commute, MPI_Fint *ierr) { 
    MPI_Op_commutative_fortran_wrapper(op, commute, ierr);
}

_EXTERN_C_ void mpi_op_commutative__(MPI_Fint *op, MPI_Fint *commute, MPI_Fint *ierr) { 
    MPI_Op_commutative_fortran_wrapper(op, commute, ierr);
}

/* ================= End Wrappers for MPI_Op_commutative ================= */


/* ================== C Wrappers for MPI_Op_create ================== */
_EXTERN_C_ int PMPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op);
_EXTERN_C_ int MPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Op_create at base level - Location = %px",&(MPI_Op_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Op_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Op_create(function, commute, op);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Op_create(function, commute, op);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Op_create(function, commute, op);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Op_create =============== */
static void MPI_Op_create_fortran_wrapper(MPI_User_function *function, MPI_Fint *commute, MPI_Fint *op, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Op_create(((MPI_User_function*)function), *commute, (MPI_Op*)op);
#else /* MPI-2 safe call */
    MPI_Op temp_op;
    _wrap_py_return_val = MPI_Op_create(((MPI_User_function*)function), *commute, &temp_op);
    *op = MPI_Op_c2f(temp_op);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_OP_CREATE(MPI_User_function *function, MPI_Fint *commute, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_create_fortran_wrapper(function, commute, op, ierr);
}

_EXTERN_C_ void mpi_op_create(MPI_User_function *function, MPI_Fint *commute, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_create_fortran_wrapper(function, commute, op, ierr);
}

_EXTERN_C_ void mpi_op_create_(MPI_User_function *function, MPI_Fint *commute, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_create_fortran_wrapper(function, commute, op, ierr);
}

_EXTERN_C_ void mpi_op_create__(MPI_User_function *function, MPI_Fint *commute, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_create_fortran_wrapper(function, commute, op, ierr);
}

/* ================= End Wrappers for MPI_Op_create ================= */


/* ================== C Wrappers for MPI_Op_free ================== */
_EXTERN_C_ int PMPI_Op_free(MPI_Op *op);
_EXTERN_C_ int MPI_Op_free(MPI_Op *op) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Op_free at base level - Location = %px",&(MPI_Op_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Op_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Op_free(op);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Op_free(op);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Op_free(op);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Op_free =============== */
static void MPI_Op_free_fortran_wrapper(MPI_Fint *op, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Op_free((MPI_Op*)op);
#else /* MPI-2 safe call */
    MPI_Op temp_op;
    temp_op = MPI_Op_f2c(*op);
    _wrap_py_return_val = MPI_Op_free(&temp_op);
    *op = MPI_Op_c2f(temp_op);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_OP_FREE(MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_free_fortran_wrapper(op, ierr);
}

_EXTERN_C_ void mpi_op_free(MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_free_fortran_wrapper(op, ierr);
}

_EXTERN_C_ void mpi_op_free_(MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_free_fortran_wrapper(op, ierr);
}

_EXTERN_C_ void mpi_op_free__(MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Op_free_fortran_wrapper(op, ierr);
}

/* ================= End Wrappers for MPI_Op_free ================= */


/* ================== C Wrappers for MPI_Open_port ================== */
_EXTERN_C_ int PMPI_Open_port(MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Open_port(MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Open_port at base level - Location = %px",&(MPI_Open_port));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Open_port_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Open_port(info, port_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Open_port(info, port_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Open_port(info, port_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Open_port =============== */
static void MPI_Open_port_fortran_wrapper(MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Open_port((MPI_Info)(*info), port_name);
#else /* MPI-2 safe call */
    char* temp_port_name;
    temp_port_name = (char*)malloc(sizeof(char) * (port_name_len+1));
    char_p_f2c(port_name,port_name_len,&temp_port_name);
    _wrap_py_return_val = MPI_Open_port(MPI_Info_f2c(*info), port_name);
    char_p_c2f(temp_port_name,port_name,port_name_len);
 if(temp_port_name) free(temp_port_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_OPEN_PORT(MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Open_port_fortran_wrapper(info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_open_port(MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Open_port_fortran_wrapper(info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_open_port_(MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Open_port_fortran_wrapper(info, port_name, ierr, port_name_len);
}

_EXTERN_C_ void mpi_open_port__(MPI_Fint *info, char *port_name, MPI_Fint *ierr, int port_name_len) { 
    MPI_Open_port_fortran_wrapper(info, port_name, ierr, port_name_len);
}

/* ================= End Wrappers for MPI_Open_port ================= */


/* ================== C Wrappers for MPI_Pack ================== */
_EXTERN_C_ int PMPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);
_EXTERN_C_ int MPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Pack at base level - Location = %px",&(MPI_Pack));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Pack_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Pack =============== */
static void MPI_Pack_fortran_wrapper(MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Fint *outsize, MPI_Fint *position, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Pack(((const void*)inbuf), *incount, (MPI_Datatype)(*datatype), BufferC2F((void*)outbuf), *outsize, ((int*)position), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Pack(((const void*)inbuf), *incount, MPI_Type_f2c(*datatype), BufferC2F((void*)outbuf), *outsize, ((int*)position), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PACK(MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Fint *outsize, MPI_Fint *position, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Pack_fortran_wrapper(inbuf, incount, datatype, outbuf, outsize, position, comm, ierr);
}

_EXTERN_C_ void mpi_pack(MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Fint *outsize, MPI_Fint *position, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Pack_fortran_wrapper(inbuf, incount, datatype, outbuf, outsize, position, comm, ierr);
}

_EXTERN_C_ void mpi_pack_(MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Fint *outsize, MPI_Fint *position, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Pack_fortran_wrapper(inbuf, incount, datatype, outbuf, outsize, position, comm, ierr);
}

_EXTERN_C_ void mpi_pack__(MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Fint *outsize, MPI_Fint *position, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Pack_fortran_wrapper(inbuf, incount, datatype, outbuf, outsize, position, comm, ierr);
}

/* ================= End Wrappers for MPI_Pack ================= */


/* ================== C Wrappers for MPI_Pack_external ================== */
_EXTERN_C_ int PMPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);
_EXTERN_C_ int MPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Pack_external at base level - Location = %px",&(MPI_Pack_external));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Pack_external_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Pack_external =============== */
static void MPI_Pack_external_fortran_wrapper(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Aint *outsize, MPI_Aint *position, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Pack_external(((const char*)datarep), ((const void*)inbuf), *incount, (MPI_Datatype)(*datatype), BufferC2F((void*)outbuf), *outsize, ((MPI_Aint*)position));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Pack_external(((const char*)datarep), ((const void*)inbuf), *incount, MPI_Type_f2c(*datatype), BufferC2F((void*)outbuf), *outsize, ((MPI_Aint*)position));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PACK_EXTERNAL(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Aint *outsize, MPI_Aint *position, MPI_Fint *ierr) { 
    MPI_Pack_external_fortran_wrapper(datarep, inbuf, incount, datatype, outbuf, outsize, position, ierr);
}

_EXTERN_C_ void mpi_pack_external(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Aint *outsize, MPI_Aint *position, MPI_Fint *ierr) { 
    MPI_Pack_external_fortran_wrapper(datarep, inbuf, incount, datatype, outbuf, outsize, position, ierr);
}

_EXTERN_C_ void mpi_pack_external_(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Aint *outsize, MPI_Aint *position, MPI_Fint *ierr) { 
    MPI_Pack_external_fortran_wrapper(datarep, inbuf, incount, datatype, outbuf, outsize, position, ierr);
}

_EXTERN_C_ void mpi_pack_external__(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *outbuf, MPI_Aint *outsize, MPI_Aint *position, MPI_Fint *ierr) { 
    MPI_Pack_external_fortran_wrapper(datarep, inbuf, incount, datatype, outbuf, outsize, position, ierr);
}

/* ================= End Wrappers for MPI_Pack_external ================= */


/* ================== C Wrappers for MPI_Pack_external_size ================== */
_EXTERN_C_ int PMPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);
_EXTERN_C_ int MPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Pack_external_size at base level - Location = %px",&(MPI_Pack_external_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Pack_external_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Pack_external_size(datarep, incount, datatype, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Pack_external_size(datarep, incount, datatype, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Pack_external_size(datarep, incount, datatype, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Pack_external_size =============== */
static void MPI_Pack_external_size_fortran_wrapper(MPI_Fint datarep[], MPI_Fint *incount, MPI_Fint *datatype, MPI_Aint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Pack_external_size(((const char*)datarep), *incount, (MPI_Datatype)(*datatype), ((MPI_Aint*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Pack_external_size(((const char*)datarep), *incount, MPI_Type_f2c(*datatype), ((MPI_Aint*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PACK_EXTERNAL_SIZE(MPI_Fint datarep[], MPI_Fint *incount, MPI_Fint *datatype, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Pack_external_size_fortran_wrapper(datarep, incount, datatype, size, ierr);
}

_EXTERN_C_ void mpi_pack_external_size(MPI_Fint datarep[], MPI_Fint *incount, MPI_Fint *datatype, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Pack_external_size_fortran_wrapper(datarep, incount, datatype, size, ierr);
}

_EXTERN_C_ void mpi_pack_external_size_(MPI_Fint datarep[], MPI_Fint *incount, MPI_Fint *datatype, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Pack_external_size_fortran_wrapper(datarep, incount, datatype, size, ierr);
}

_EXTERN_C_ void mpi_pack_external_size__(MPI_Fint datarep[], MPI_Fint *incount, MPI_Fint *datatype, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Pack_external_size_fortran_wrapper(datarep, incount, datatype, size, ierr);
}

/* ================= End Wrappers for MPI_Pack_external_size ================= */


/* ================== C Wrappers for MPI_Pack_size ================== */
_EXTERN_C_ int PMPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Pack_size at base level - Location = %px",&(MPI_Pack_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Pack_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Pack_size(incount, datatype, comm, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Pack_size(incount, datatype, comm, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Pack_size(incount, datatype, comm, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Pack_size =============== */
static void MPI_Pack_size_fortran_wrapper(MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Pack_size(*incount, (MPI_Datatype)(*datatype), (MPI_Comm)(*comm), ((int*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Pack_size(*incount, MPI_Type_f2c(*datatype), MPI_Comm_f2c(*comm), ((int*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PACK_SIZE(MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Pack_size_fortran_wrapper(incount, datatype, comm, size, ierr);
}

_EXTERN_C_ void mpi_pack_size(MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Pack_size_fortran_wrapper(incount, datatype, comm, size, ierr);
}

_EXTERN_C_ void mpi_pack_size_(MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Pack_size_fortran_wrapper(incount, datatype, comm, size, ierr);
}

_EXTERN_C_ void mpi_pack_size__(MPI_Fint *incount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Pack_size_fortran_wrapper(incount, datatype, comm, size, ierr);
}

/* ================= End Wrappers for MPI_Pack_size ================= */


/* ================== C Wrappers for MPI_Probe ================== */
_EXTERN_C_ int PMPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Probe at base level - Location = %px",&(MPI_Probe));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Probe_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Probe(source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Probe(source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Probe(source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Probe =============== */
static void MPI_Probe_fortran_wrapper(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Probe(*source, *tag, (MPI_Comm)(*comm), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Probe(*source, *tag, MPI_Comm_f2c(*comm), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Probe(*source, *tag, MPI_Comm_f2c(*comm), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PROBE(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Probe_fortran_wrapper(source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_probe(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Probe_fortran_wrapper(source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_probe_(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Probe_fortran_wrapper(source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_probe__(MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Probe_fortran_wrapper(source, tag, comm, status, ierr);
}

/* ================= End Wrappers for MPI_Probe ================= */


/* ================== C Wrappers for MPI_Publish_name ================== */
_EXTERN_C_ int PMPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Publish_name at base level - Location = %px",&(MPI_Publish_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Publish_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Publish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Publish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Publish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Publish_name =============== */
static void MPI_Publish_name_fortran_wrapper(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Publish_name(((const char*)service_name), (MPI_Info)(*info), ((const char*)port_name));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Publish_name(((const char*)service_name), MPI_Info_f2c(*info), ((const char*)port_name));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PUBLISH_NAME(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Publish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_publish_name(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Publish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_publish_name_(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Publish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_publish_name__(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Publish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

/* ================= End Wrappers for MPI_Publish_name ================= */


/* ================== C Wrappers for MPI_Put ================== */
_EXTERN_C_ int PMPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Put at base level - Location = %px",&(MPI_Put));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Put_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Put =============== */
static void MPI_Put_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Put(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Put(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_PUT(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Put_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_put(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Put_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_put_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Put_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

_EXTERN_C_ void mpi_put__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Put_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, ierr);
}

/* ================= End Wrappers for MPI_Put ================= */


/* ================== C Wrappers for MPI_Query_thread ================== */
_EXTERN_C_ int PMPI_Query_thread(int *provided);
_EXTERN_C_ int MPI_Query_thread(int *provided) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Query_thread at base level - Location = %px",&(MPI_Query_thread));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Query_thread_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Query_thread(provided);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Query_thread(provided);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Query_thread(provided);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Query_thread =============== */
static void MPI_Query_thread_fortran_wrapper(MPI_Fint *provided, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Query_thread(((int*)provided));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_QUERY_THREAD(MPI_Fint *provided, MPI_Fint *ierr) { 
    MPI_Query_thread_fortran_wrapper(provided, ierr);
}

_EXTERN_C_ void mpi_query_thread(MPI_Fint *provided, MPI_Fint *ierr) { 
    MPI_Query_thread_fortran_wrapper(provided, ierr);
}

_EXTERN_C_ void mpi_query_thread_(MPI_Fint *provided, MPI_Fint *ierr) { 
    MPI_Query_thread_fortran_wrapper(provided, ierr);
}

_EXTERN_C_ void mpi_query_thread__(MPI_Fint *provided, MPI_Fint *ierr) { 
    MPI_Query_thread_fortran_wrapper(provided, ierr);
}

/* ================= End Wrappers for MPI_Query_thread ================= */


/* ================== C Wrappers for MPI_Raccumulate ================== */
_EXTERN_C_ int PMPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Raccumulate at base level - Location = %px",&(MPI_Raccumulate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Raccumulate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Raccumulate =============== */
static void MPI_Raccumulate_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Raccumulate(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Op)(*op), (MPI_Win)(*win), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Raccumulate(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Op_f2c(*op), MPI_Win_f2c(*win), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RACCUMULATE(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Raccumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_raccumulate(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Raccumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_raccumulate_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Raccumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_raccumulate__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Raccumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

/* ================= End Wrappers for MPI_Raccumulate ================= */


/* ================== C Wrappers for MPI_Recv ================== */
_EXTERN_C_ int PMPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Recv at base level - Location = %px",&(MPI_Recv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Recv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Recv(buf, count, datatype, source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Recv(buf, count, datatype, source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Recv(buf, count, datatype, source, tag, comm, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Recv =============== */
static void MPI_Recv_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Recv(BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), *source, *tag, (MPI_Comm)(*comm), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Recv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *source, *tag, MPI_Comm_f2c(*comm), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Recv(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *source, *tag, MPI_Comm_f2c(*comm), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RECV(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Recv_fortran_wrapper(buf, count, datatype, source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_recv(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Recv_fortran_wrapper(buf, count, datatype, source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_recv_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Recv_fortran_wrapper(buf, count, datatype, source, tag, comm, status, ierr);
}

_EXTERN_C_ void mpi_recv__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Recv_fortran_wrapper(buf, count, datatype, source, tag, comm, status, ierr);
}

/* ================= End Wrappers for MPI_Recv ================= */


/* ================== C Wrappers for MPI_Recv_init ================== */
_EXTERN_C_ int PMPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Recv_init at base level - Location = %px",&(MPI_Recv_init));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Recv_init_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Recv_init =============== */
static void MPI_Recv_init_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Recv_init(BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), *source, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Recv_init(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *source, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RECV_INIT(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Recv_init_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_recv_init(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Recv_init_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_recv_init_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Recv_init_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_recv_init__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *source, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Recv_init_fortran_wrapper(buf, count, datatype, source, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Recv_init ================= */


/* ================== C Wrappers for MPI_Reduce ================== */
_EXTERN_C_ int PMPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Reduce at base level - Location = %px",&(MPI_Reduce));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Reduce_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Reduce =============== */
static void MPI_Reduce_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Reduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Reduce(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REDUCE(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, ierr);
}

_EXTERN_C_ void mpi_reduce(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, ierr);
}

_EXTERN_C_ void mpi_reduce__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Reduce ================= */


/* ================== C Wrappers for MPI_Reduce_local ================== */
_EXTERN_C_ int PMPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);
_EXTERN_C_ int MPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Reduce_local at base level - Location = %px",&(MPI_Reduce_local));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Reduce_local_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Reduce_local =============== */
static void MPI_Reduce_local_fortran_wrapper(MPI_Fint *inbuf, MPI_Fint *inoutbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Reduce_local(((const void*)inbuf), BufferC2F((void*)inoutbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Reduce_local(((const void*)inbuf), BufferC2F((void*)inoutbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REDUCE_LOCAL(MPI_Fint *inbuf, MPI_Fint *inoutbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Reduce_local_fortran_wrapper(inbuf, inoutbuf, count, datatype, op, ierr);
}

_EXTERN_C_ void mpi_reduce_local(MPI_Fint *inbuf, MPI_Fint *inoutbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Reduce_local_fortran_wrapper(inbuf, inoutbuf, count, datatype, op, ierr);
}

_EXTERN_C_ void mpi_reduce_local_(MPI_Fint *inbuf, MPI_Fint *inoutbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Reduce_local_fortran_wrapper(inbuf, inoutbuf, count, datatype, op, ierr);
}

_EXTERN_C_ void mpi_reduce_local__(MPI_Fint *inbuf, MPI_Fint *inoutbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *ierr) { 
    MPI_Reduce_local_fortran_wrapper(inbuf, inoutbuf, count, datatype, op, ierr);
}

/* ================= End Wrappers for MPI_Reduce_local ================= */


/* ================== C Wrappers for MPI_Reduce_scatter ================== */
_EXTERN_C_ int PMPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Reduce_scatter at base level - Location = %px",&(MPI_Reduce_scatter));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Reduce_scatter_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Reduce_scatter =============== */
static void MPI_Reduce_scatter_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Reduce_scatter(((const void*)sendbuf), BufferC2F((void*)recvbuf), ((const int*)recvcounts), (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Reduce_scatter(((const void*)sendbuf), BufferC2F((void*)recvbuf), ((const int*)recvcounts), MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REDUCE_SCATTER(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint recvcounts[], MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_fortran_wrapper(sendbuf, recvbuf, recvcounts, datatype, op, comm, ierr);
}

/* ================= End Wrappers for MPI_Reduce_scatter ================= */


/* ================== C Wrappers for MPI_Reduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Reduce_scatter_block at base level - Location = %px",&(MPI_Reduce_scatter_block));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Reduce_scatter_block_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Reduce_scatter_block =============== */
static void MPI_Reduce_scatter_block_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Reduce_scatter_block(((const void*)sendbuf), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Reduce_scatter_block(((const void*)sendbuf), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REDUCE_SCATTER_BLOCK(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter_block(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter_block_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_reduce_scatter_block__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Reduce_scatter_block_fortran_wrapper(sendbuf, recvbuf, recvcount, datatype, op, comm, ierr);
}

/* ================= End Wrappers for MPI_Reduce_scatter_block ================= */


/* ================== C Wrappers for MPI_Register_datarep ================== */
_EXTERN_C_ int PMPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);
_EXTERN_C_ int MPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Register_datarep at base level - Location = %px",&(MPI_Register_datarep));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Register_datarep_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Register_datarep =============== */
static void MPI_Register_datarep_fortran_wrapper(MPI_Fint *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Register_datarep(((const char*)datarep), ((MPI_Datarep_conversion_function*)read_conversion_fn), ((MPI_Datarep_conversion_function*)write_conversion_fn), ((MPI_Datarep_extent_function*)dtype_file_extent_fn), BufferC2F((void*)extra_state));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REGISTER_DATAREP(MPI_Fint *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Register_datarep_fortran_wrapper(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state, ierr);
}

_EXTERN_C_ void mpi_register_datarep(MPI_Fint *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Register_datarep_fortran_wrapper(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state, ierr);
}

_EXTERN_C_ void mpi_register_datarep_(MPI_Fint *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Register_datarep_fortran_wrapper(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state, ierr);
}

_EXTERN_C_ void mpi_register_datarep__(MPI_Fint *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Register_datarep_fortran_wrapper(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state, ierr);
}

/* ================= End Wrappers for MPI_Register_datarep ================= */


/* ================== C Wrappers for MPI_Request_free ================== */
_EXTERN_C_ int PMPI_Request_free(MPI_Request *request);
_EXTERN_C_ int MPI_Request_free(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Request_free at base level - Location = %px",&(MPI_Request_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Request_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Request_free(request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Request_free(request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Request_free(request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Request_free =============== */
static void MPI_Request_free_fortran_wrapper(MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Request_free((MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    temp_request = MPI_Request_f2c(*request);
    _wrap_py_return_val = MPI_Request_free(&temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REQUEST_FREE(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Request_free_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_request_free(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Request_free_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_request_free_(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Request_free_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_request_free__(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Request_free_fortran_wrapper(request, ierr);
}

/* ================= End Wrappers for MPI_Request_free ================= */


/* ================== C Wrappers for MPI_Request_get_status ================== */
_EXTERN_C_ int PMPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Request_get_status at base level - Location = %px",&(MPI_Request_get_status));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Request_get_status_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Request_get_status(request, flag, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Request_get_status(request, flag, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Request_get_status(request, flag, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Request_get_status =============== */
static void MPI_Request_get_status_fortran_wrapper(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Request_get_status((MPI_Request)(*request), ((int*)flag), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Request_get_status(MPI_Request_f2c(*request), ((int*)flag), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Request_get_status(MPI_Request_f2c(*request), ((int*)flag), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_REQUEST_GET_STATUS(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Request_get_status_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_request_get_status(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Request_get_status_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_request_get_status_(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Request_get_status_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_request_get_status__(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Request_get_status_fortran_wrapper(request, flag, status, ierr);
}

/* ================= End Wrappers for MPI_Request_get_status ================= */


/* ================== C Wrappers for MPI_Rget ================== */
_EXTERN_C_ int PMPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Rget at base level - Location = %px",&(MPI_Rget));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Rget_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Rget =============== */
static void MPI_Rget_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Rget(BufferC2F((void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Win)(*win), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Rget(BufferC2F((void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Win_f2c(*win), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RGET(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rget(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rget_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rget__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request, ierr);
}

/* ================= End Wrappers for MPI_Rget ================= */


/* ================== C Wrappers for MPI_Rget_accumulate ================== */
_EXTERN_C_ int PMPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Rget_accumulate at base level - Location = %px",&(MPI_Rget_accumulate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Rget_accumulate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Rget_accumulate =============== */
static void MPI_Rget_accumulate_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Rget_accumulate(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), BufferC2F((void*)result_addr), *result_count, (MPI_Datatype)(*result_datatype), *target_rank, *target_disp, *target_count, (MPI_Datatype)(*target_datatype), (MPI_Op)(*op), (MPI_Win)(*win), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Rget_accumulate(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), BufferC2F((void*)result_addr), *result_count, MPI_Type_f2c(*result_datatype), *target_rank, *target_disp, *target_count, MPI_Type_f2c(*target_datatype), MPI_Op_f2c(*op), MPI_Win_f2c(*win), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RGET_ACCUMULATE(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_rget_accumulate(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_rget_accumulate_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

_EXTERN_C_ void mpi_rget_accumulate__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *result_addr, MPI_Fint *result_count, MPI_Fint *result_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_count, MPI_Fint *target_datatype, MPI_Fint *op, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rget_accumulate_fortran_wrapper(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request, ierr);
}

/* ================= End Wrappers for MPI_Rget_accumulate ================= */


/* ================== C Wrappers for MPI_Rput ================== */
_EXTERN_C_ int PMPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Rput at base level - Location = %px",&(MPI_Rput));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Rput_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Rput =============== */
static void MPI_Rput_fortran_wrapper(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_cout, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Rput(((const void*)origin_addr), *origin_count, (MPI_Datatype)(*origin_datatype), *target_rank, *target_disp, *target_cout, (MPI_Datatype)(*target_datatype), (MPI_Win)(*win), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    temp_request = MPI_Request_f2c(*request);
    _wrap_py_return_val = MPI_Rput(((const void*)origin_addr), *origin_count, MPI_Type_f2c(*origin_datatype), *target_rank, *target_disp, *target_cout, MPI_Type_f2c(*target_datatype), MPI_Win_f2c(*win), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RPUT(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_cout, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rput_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rput(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_cout, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rput_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rput_(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_cout, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rput_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request, ierr);
}

_EXTERN_C_ void mpi_rput__(MPI_Fint *origin_addr, MPI_Fint *origin_count, MPI_Fint *origin_datatype, MPI_Fint *target_rank, MPI_Aint *target_disp, MPI_Fint *target_cout, MPI_Fint *target_datatype, MPI_Fint *win, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rput_fortran_wrapper(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request, ierr);
}

/* ================= End Wrappers for MPI_Rput ================= */


/* ================== C Wrappers for MPI_Rsend ================== */
_EXTERN_C_ int PMPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Rsend at base level - Location = %px",&(MPI_Rsend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Rsend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Rsend =============== */
static void MPI_Rsend_fortran_wrapper(MPI_Fint *ibuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Rsend(((const void*)ibuf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Rsend(((const void*)ibuf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RSEND(MPI_Fint *ibuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Rsend_fortran_wrapper(ibuf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_rsend(MPI_Fint *ibuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Rsend_fortran_wrapper(ibuf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_rsend_(MPI_Fint *ibuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Rsend_fortran_wrapper(ibuf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_rsend__(MPI_Fint *ibuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Rsend_fortran_wrapper(ibuf, count, datatype, dest, tag, comm, ierr);
}

/* ================= End Wrappers for MPI_Rsend ================= */


/* ================== C Wrappers for MPI_Rsend_init ================== */
_EXTERN_C_ int PMPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Rsend_init at base level - Location = %px",&(MPI_Rsend_init));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Rsend_init_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Rsend_init =============== */
static void MPI_Rsend_init_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Rsend_init(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Rsend_init(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_RSEND_INIT(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_rsend_init(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_rsend_init_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_rsend_init__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Rsend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Rsend_init ================= */


/* ================== C Wrappers for MPI_Scan ================== */
_EXTERN_C_ int PMPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Scan at base level - Location = %px",&(MPI_Scan));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Scan_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Scan =============== */
static void MPI_Scan_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Scan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, (MPI_Datatype)(*datatype), (MPI_Op)(*op), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Scan(((const void*)sendbuf), BufferC2F((void*)recvbuf), *count, MPI_Type_f2c(*datatype), MPI_Op_f2c(*op), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SCAN(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_scan(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_scan_(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

_EXTERN_C_ void mpi_scan__(MPI_Fint *sendbuf, MPI_Fint *recvbuf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *op, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scan_fortran_wrapper(sendbuf, recvbuf, count, datatype, op, comm, ierr);
}

/* ================= End Wrappers for MPI_Scan ================= */


/* ================== C Wrappers for MPI_Scatter ================== */
_EXTERN_C_ int PMPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Scatter at base level - Location = %px",&(MPI_Scatter));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Scatter_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Scatter =============== */
static void MPI_Scatter_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Scatter(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Scatter(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SCATTER(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatter(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatter_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatter__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatter_fortran_wrapper(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Scatter ================= */


/* ================== C Wrappers for MPI_Scatterv ================== */
_EXTERN_C_ int PMPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Scatterv at base level - Location = %px",&(MPI_Scatterv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Scatterv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Scatterv =============== */
static void MPI_Scatterv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Scatterv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)displs), (MPI_Datatype)(*sendtype), BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *root, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Scatterv(((const void*)sendbuf), ((const int*)sendcounts), ((const int*)displs), MPI_Type_f2c(*sendtype), BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *root, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SCATTERV(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatterv(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatterv_(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

_EXTERN_C_ void mpi_scatterv__(MPI_Fint *sendbuf, MPI_Fint sendcounts[], MPI_Fint displs[], MPI_Fint *sendtype, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *root, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Scatterv_fortran_wrapper(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, ierr);
}

/* ================= End Wrappers for MPI_Scatterv ================= */


/* ================== C Wrappers for MPI_Send ================== */
_EXTERN_C_ int PMPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Send at base level - Location = %px",&(MPI_Send));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Send_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Send(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Send(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Send(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Send =============== */
static void MPI_Send_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Send(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Send(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Send_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_send(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Send_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_send_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Send_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_send__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Send_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

/* ================= End Wrappers for MPI_Send ================= */


/* ================== C Wrappers for MPI_Send_init ================== */
_EXTERN_C_ int PMPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Send_init at base level - Location = %px",&(MPI_Send_init));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Send_init_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Send_init =============== */
static void MPI_Send_init_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Send_init(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Send_init(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SEND_INIT(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Send_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_send_init(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Send_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_send_init_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Send_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_send_init__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Send_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Send_init ================= */


/* ================== C Wrappers for MPI_Sendrecv ================== */
_EXTERN_C_ int PMPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Sendrecv at base level - Location = %px",&(MPI_Sendrecv));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Sendrecv_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Sendrecv =============== */
static void MPI_Sendrecv_fortran_wrapper(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Sendrecv(((const void*)sendbuf), *sendcount, (MPI_Datatype)(*sendtype), *dest, *sendtag, BufferC2F((void*)recvbuf), *recvcount, (MPI_Datatype)(*recvtype), *source, *recvtag, (MPI_Comm)(*comm), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Sendrecv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), *dest, *sendtag, BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *source, *recvtag, MPI_Comm_f2c(*comm), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Sendrecv(((const void*)sendbuf), *sendcount, MPI_Type_f2c(*sendtype), *dest, *sendtag, BufferC2F((void*)recvbuf), *recvcount, MPI_Type_f2c(*recvtype), *source, *recvtag, MPI_Comm_f2c(*comm), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SENDRECV(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_fortran_wrapper(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_fortran_wrapper(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv_(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_fortran_wrapper(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv__(MPI_Fint *sendbuf, MPI_Fint *sendcount, MPI_Fint *sendtype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *recvbuf, MPI_Fint *recvcount, MPI_Fint *recvtype, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_fortran_wrapper(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status, ierr);
}

/* ================= End Wrappers for MPI_Sendrecv ================= */


/* ================== C Wrappers for MPI_Sendrecv_replace ================== */
_EXTERN_C_ int PMPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Sendrecv_replace at base level - Location = %px",&(MPI_Sendrecv_replace));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Sendrecv_replace_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Sendrecv_replace =============== */
static void MPI_Sendrecv_replace_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Sendrecv_replace(BufferC2F((void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *sendtag, *source, *recvtag, (MPI_Comm)(*comm), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Sendrecv_replace(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *sendtag, *source, *recvtag, MPI_Comm_f2c(*comm), &temp_status);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Sendrecv_replace(BufferC2F((void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *sendtag, *source, *recvtag, MPI_Comm_f2c(*comm), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SENDRECV_REPLACE(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_replace_fortran_wrapper(buf, count, datatype, dest, sendtag, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv_replace(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_replace_fortran_wrapper(buf, count, datatype, dest, sendtag, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv_replace_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_replace_fortran_wrapper(buf, count, datatype, dest, sendtag, source, recvtag, comm, status, ierr);
}

_EXTERN_C_ void mpi_sendrecv_replace__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *sendtag, MPI_Fint *source, MPI_Fint *recvtag, MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Sendrecv_replace_fortran_wrapper(buf, count, datatype, dest, sendtag, source, recvtag, comm, status, ierr);
}

/* ================= End Wrappers for MPI_Sendrecv_replace ================= */


/* ================== C Wrappers for MPI_Ssend ================== */
_EXTERN_C_ int PMPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ssend at base level - Location = %px",&(MPI_Ssend));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ssend_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ssend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ssend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ssend(buf, count, datatype, dest, tag, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ssend =============== */
static void MPI_Ssend_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ssend(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Ssend(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SSEND(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Ssend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_ssend(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Ssend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_ssend_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Ssend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

_EXTERN_C_ void mpi_ssend__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Ssend_fortran_wrapper(buf, count, datatype, dest, tag, comm, ierr);
}

/* ================= End Wrappers for MPI_Ssend ================= */


/* ================== C Wrappers for MPI_Ssend_init ================== */
_EXTERN_C_ int PMPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Ssend_init at base level - Location = %px",&(MPI_Ssend_init));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Ssend_init_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Ssend_init =============== */
static void MPI_Ssend_init_fortran_wrapper(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Ssend_init(((const void*)buf), *count, (MPI_Datatype)(*datatype), *dest, *tag, (MPI_Comm)(*comm), (MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    _wrap_py_return_val = MPI_Ssend_init(((const void*)buf), *count, MPI_Type_f2c(*datatype), *dest, *tag, MPI_Comm_f2c(*comm), &temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_SSEND_INIT(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ssend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ssend_init(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ssend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ssend_init_(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ssend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

_EXTERN_C_ void mpi_ssend_init__(MPI_Fint *buf, MPI_Fint *count, MPI_Fint *datatype, MPI_Fint *dest, MPI_Fint *tag, MPI_Fint *comm, MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Ssend_init_fortran_wrapper(buf, count, datatype, dest, tag, comm, request, ierr);
}

/* ================= End Wrappers for MPI_Ssend_init ================= */


/* ================== C Wrappers for MPI_Start ================== */
_EXTERN_C_ int PMPI_Start(MPI_Request *request);
_EXTERN_C_ int MPI_Start(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Start at base level - Location = %px",&(MPI_Start));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Start_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Start(request);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Start(request);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Start(request);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Start =============== */
static void MPI_Start_fortran_wrapper(MPI_Fint *request, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Start((MPI_Request*)request);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    temp_request = MPI_Request_f2c(*request);
    _wrap_py_return_val = MPI_Start(&temp_request);
    *request = MPI_Request_c2f(temp_request);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_START(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Start_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_start(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Start_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_start_(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Start_fortran_wrapper(request, ierr);
}

_EXTERN_C_ void mpi_start__(MPI_Fint *request, MPI_Fint *ierr) { 
    MPI_Start_fortran_wrapper(request, ierr);
}

/* ================= End Wrappers for MPI_Start ================= */


/* ================== C Wrappers for MPI_Startall ================== */
_EXTERN_C_ int PMPI_Startall(int count, MPI_Request array_of_requests[]);
_EXTERN_C_ int MPI_Startall(int count, MPI_Request array_of_requests[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Startall at base level - Location = %px",&(MPI_Startall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Startall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Startall(count, array_of_requests);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Startall(count, array_of_requests);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Startall(count, array_of_requests);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Startall =============== */
static void MPI_Startall_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Startall(*count, (MPI_Request*)array_of_requests);
#else /* MPI-2 safe call */
    MPI_Request* temp_array_of_requests;
    int i;
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    _wrap_py_return_val = MPI_Startall(*count, temp_array_of_requests);
 if(temp_array_of_requests) free(temp_array_of_requests);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_STARTALL(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *ierr) { 
    MPI_Startall_fortran_wrapper(count, array_of_requests, ierr);
}

_EXTERN_C_ void mpi_startall(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *ierr) { 
    MPI_Startall_fortran_wrapper(count, array_of_requests, ierr);
}

_EXTERN_C_ void mpi_startall_(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *ierr) { 
    MPI_Startall_fortran_wrapper(count, array_of_requests, ierr);
}

_EXTERN_C_ void mpi_startall__(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *ierr) { 
    MPI_Startall_fortran_wrapper(count, array_of_requests, ierr);
}

/* ================= End Wrappers for MPI_Startall ================= */


/* ================== C Wrappers for MPI_Status_set_cancelled ================== */
_EXTERN_C_ int PMPI_Status_set_cancelled(MPI_Status *status, int flag);
_EXTERN_C_ int MPI_Status_set_cancelled(MPI_Status *status, int flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Status_set_cancelled at base level - Location = %px",&(MPI_Status_set_cancelled));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Status_set_cancelled_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Status_set_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Status_set_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Status_set_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Status_set_cancelled =============== */
static void MPI_Status_set_cancelled_fortran_wrapper(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Status_set_cancelled((MPI_Status*)status, *flag);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_cancelled(&temp_status, *flag);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_cancelled(((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status), *flag);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_STATUS_SET_CANCELLED(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Status_set_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_status_set_cancelled(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Status_set_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_status_set_cancelled_(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Status_set_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_status_set_cancelled__(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Status_set_cancelled_fortran_wrapper(status, flag, ierr);
}

/* ================= End Wrappers for MPI_Status_set_cancelled ================= */


/* ================== C Wrappers for MPI_Status_set_elements ================== */
_EXTERN_C_ int PMPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count);
_EXTERN_C_ int MPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Status_set_elements at base level - Location = %px",&(MPI_Status_set_elements));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Status_set_elements_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Status_set_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Status_set_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Status_set_elements(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Status_set_elements =============== */
static void MPI_Status_set_elements_fortran_wrapper(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Status_set_elements((MPI_Status*)status, (MPI_Datatype)(*datatype), *count);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_elements(&temp_status, MPI_Type_f2c(*datatype), *count);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_elements(((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status), MPI_Type_f2c(*datatype), *count);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_STATUS_SET_ELEMENTS(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements_(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements__(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_fortran_wrapper(status, datatype, count, ierr);
}

/* ================= End Wrappers for MPI_Status_set_elements ================= */


/* ================== C Wrappers for MPI_Status_set_elements_x ================== */
_EXTERN_C_ int PMPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);
_EXTERN_C_ int MPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Status_set_elements_x at base level - Location = %px",&(MPI_Status_set_elements_x));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Status_set_elements_x_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Status_set_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Status_set_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Status_set_elements_x(status, datatype, count);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Status_set_elements_x =============== */
static void MPI_Status_set_elements_x_fortran_wrapper(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Status_set_elements_x((MPI_Status*)status, (MPI_Datatype)(*datatype), *count);
#else /* MPI-2 safe call */
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_elements_x(&temp_status, MPI_Type_f2c(*datatype), *count);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Status_set_elements_x(((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status), MPI_Type_f2c(*datatype), *count);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_STATUS_SET_ELEMENTS_X(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements_x(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements_x_(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

_EXTERN_C_ void mpi_status_set_elements_x__(MPI_Fint *status, MPI_Fint *datatype, MPI_Fint *count, MPI_Fint *ierr) { 
    MPI_Status_set_elements_x_fortran_wrapper(status, datatype, count, ierr);
}

/* ================= End Wrappers for MPI_Status_set_elements_x ================= */


/* ================== C Wrappers for MPI_Test ================== */
_EXTERN_C_ int PMPI_Test(MPI_Request *request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Test(MPI_Request *request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Test at base level - Location = %px",&(MPI_Test));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Test_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Test(request, flag, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Test(request, flag, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Test(request, flag, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Test =============== */
static void MPI_Test_fortran_wrapper(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Test((MPI_Request*)request, ((int*)flag), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_request = MPI_Request_f2c(*request);
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Test(&temp_request, ((int*)flag), &temp_status);
    *request = MPI_Request_c2f(temp_request);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    temp_request = MPI_Request_f2c(*request);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Test(&temp_request, ((int*)flag), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    *request = MPI_Request_c2f(temp_request);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TEST(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Test_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_test(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Test_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_test_(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Test_fortran_wrapper(request, flag, status, ierr);
}

_EXTERN_C_ void mpi_test__(MPI_Fint *request, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Test_fortran_wrapper(request, flag, status, ierr);
}

/* ================= End Wrappers for MPI_Test ================= */


/* ================== C Wrappers for MPI_Test_cancelled ================== */
_EXTERN_C_ int PMPI_Test_cancelled(const MPI_Status *status, int *flag);
_EXTERN_C_ int MPI_Test_cancelled(const MPI_Status *status, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Test_cancelled at base level - Location = %px",&(MPI_Test_cancelled));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Test_cancelled_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Test_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Test_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Test_cancelled(status, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Test_cancelled =============== */
static void MPI_Test_cancelled_fortran_wrapper(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Test_cancelled(((const MPI_Status*)status), ((int*)flag));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TEST_CANCELLED(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Test_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_test_cancelled(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Test_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_test_cancelled_(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Test_cancelled_fortran_wrapper(status, flag, ierr);
}

_EXTERN_C_ void mpi_test_cancelled__(MPI_Fint *status, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Test_cancelled_fortran_wrapper(status, flag, ierr);
}

/* ================= End Wrappers for MPI_Test_cancelled ================= */


/* ================== C Wrappers for MPI_Testall ================== */
_EXTERN_C_ int PMPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Testall at base level - Location = %px",&(MPI_Testall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Testall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Testall(count, array_of_requests, flag, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Testall(count, array_of_requests, flag, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Testall(count, array_of_requests, flag, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Testall =============== */
static void MPI_Testall_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *flag, MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Testall(*count, (MPI_Request*)array_of_requests, ((int*)flag), (MPI_Status*)array_of_statuses);
#else /* MPI-2 safe call */
    MPI_Status* temp_array_of_statuses;
    MPI_Request* temp_array_of_requests;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *count);
    _wrap_py_return_val = MPI_Testall(*count, temp_array_of_requests, ((int*)flag), temp_array_of_statuses);
      for (i=0; i < *count; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *count);
    _wrap_py_return_val = MPI_Testall(*count, temp_array_of_requests, ((int*)flag), ((array_of_statuses == MPI_F_STATUSES_IGNORE) ? MPI_STATUSES_IGNORE : temp_array_of_statuses));
    if (array_of_statuses != MPI_F_STATUSES_IGNORE)
      for (i=0; i < *count; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
 if(temp_array_of_statuses) free(temp_array_of_statuses);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TESTALL(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *flag, MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testall_fortran_wrapper(count, array_of_requests, flag, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testall(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *flag, MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testall_fortran_wrapper(count, array_of_requests, flag, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testall_(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *flag, MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testall_fortran_wrapper(count, array_of_requests, flag, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testall__(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *flag, MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testall_fortran_wrapper(count, array_of_requests, flag, array_of_statuses, ierr);
}

/* ================= End Wrappers for MPI_Testall ================= */


/* ================== C Wrappers for MPI_Testany ================== */
_EXTERN_C_ int PMPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Testany at base level - Location = %px",&(MPI_Testany));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Testany_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Testany(count, array_of_requests, index, flag, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Testany(count, array_of_requests, index, flag, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Testany(count, array_of_requests, index, flag, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Testany =============== */
static void MPI_Testany_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Testany(*count, (MPI_Request*)array_of_requests, index, ((int*)flag), (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Request* temp_array_of_requests;
    MPI_Status temp_status;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Testany(*count, temp_array_of_requests, index, ((int*)flag), &temp_status);
    if (*index != MPI_UNDEFINED){
    ++(*index);
    MPI_Status_c2f(&temp_status, status);
    }
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Testany(*count, temp_array_of_requests, index, ((int*)flag), ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (*index != MPI_UNDEFINED){
    ++(*index);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
    }
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TESTANY(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Testany_fortran_wrapper(count, array_of_requests, index, flag, status, ierr);
}

_EXTERN_C_ void mpi_testany(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Testany_fortran_wrapper(count, array_of_requests, index, flag, status, ierr);
}

_EXTERN_C_ void mpi_testany_(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Testany_fortran_wrapper(count, array_of_requests, index, flag, status, ierr);
}

_EXTERN_C_ void mpi_testany__(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *flag, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Testany_fortran_wrapper(count, array_of_requests, index, flag, status, ierr);
}

/* ================= End Wrappers for MPI_Testany ================= */


/* ================== C Wrappers for MPI_Testsome ================== */
_EXTERN_C_ int PMPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Testsome at base level - Location = %px",&(MPI_Testsome));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Testsome_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Testsome =============== */
static void MPI_Testsome_fortran_wrapper(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Testsome(*incount, (MPI_Request*)array_of_requests, ((int*)outcount), array_of_indices, (MPI_Status*)array_of_statuses);
#else /* MPI-2 safe call */
    MPI_Status* temp_array_of_statuses;
    MPI_Request* temp_array_of_requests;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *incount);
    for (i=0; i < *incount; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *incount);
    _wrap_py_return_val = MPI_Testsome(*incount, temp_array_of_requests, ((int*)outcount), array_of_indices, temp_array_of_statuses);
    if (*outcount != MPI_UNDEFINED){
        for (i=0; i < *outcount; ++i)
            ++array_of_indices[i];
      for (i=0; i < *incount; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
    }
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *incount);
    for (i=0; i < *incount; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *incount);
    _wrap_py_return_val = MPI_Testsome(*incount, temp_array_of_requests, ((int*)outcount), array_of_indices, ((array_of_statuses == MPI_F_STATUSES_IGNORE) ? MPI_STATUSES_IGNORE : temp_array_of_statuses));
    if (*outcount != MPI_UNDEFINED){
        for (i=0; i < *outcount; ++i)
            ++array_of_indices[i];
    if (array_of_statuses != MPI_F_STATUSES_IGNORE)
      for (i=0; i < *incount; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
    }
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
 if(temp_array_of_statuses) free(temp_array_of_statuses);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TESTSOME(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testsome(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testsome_(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_testsome__(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Testsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

/* ================= End Wrappers for MPI_Testsome ================= */


/* ================== C Wrappers for MPI_Topo_test ================== */
_EXTERN_C_ int PMPI_Topo_test(MPI_Comm comm, int *status);
_EXTERN_C_ int MPI_Topo_test(MPI_Comm comm, int *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Topo_test at base level - Location = %px",&(MPI_Topo_test));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Topo_test_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Topo_test(comm, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Topo_test(comm, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Topo_test(comm, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Topo_test =============== */
static void MPI_Topo_test_fortran_wrapper(MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Topo_test((MPI_Comm)(*comm), ((int*)status));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Topo_test(MPI_Comm_f2c(*comm), ((int*)status));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TOPO_TEST(MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Topo_test_fortran_wrapper(comm, status, ierr);
}

_EXTERN_C_ void mpi_topo_test(MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Topo_test_fortran_wrapper(comm, status, ierr);
}

_EXTERN_C_ void mpi_topo_test_(MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Topo_test_fortran_wrapper(comm, status, ierr);
}

_EXTERN_C_ void mpi_topo_test__(MPI_Fint *comm, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Topo_test_fortran_wrapper(comm, status, ierr);
}

/* ================= End Wrappers for MPI_Topo_test ================= */


/* ================== C Wrappers for MPI_Type_commit ================== */
_EXTERN_C_ int PMPI_Type_commit(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_commit(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_commit at base level - Location = %px",&(MPI_Type_commit));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_commit_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_commit(type);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_commit(type);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_commit(type);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_commit =============== */
static void MPI_Type_commit_fortran_wrapper(MPI_Fint *type, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_commit((MPI_Datatype*)type);
#else /* MPI-2 safe call */
    MPI_Datatype temp_type;
    temp_type = MPI_Type_f2c(*type);
    _wrap_py_return_val = MPI_Type_commit(&temp_type);
    *type = MPI_Type_c2f(temp_type);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_COMMIT(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_commit_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_commit(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_commit_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_commit_(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_commit_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_commit__(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_commit_fortran_wrapper(type, ierr);
}

/* ================= End Wrappers for MPI_Type_commit ================= */


/* ================== C Wrappers for MPI_Type_contiguous ================== */
_EXTERN_C_ int PMPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_contiguous at base level - Location = %px",&(MPI_Type_contiguous));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_contiguous_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_contiguous(count, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_contiguous(count, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_contiguous(count, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_contiguous =============== */
static void MPI_Type_contiguous_fortran_wrapper(MPI_Fint *count, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_contiguous(*count, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_contiguous(*count, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CONTIGUOUS(MPI_Fint *count, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_contiguous_fortran_wrapper(count, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_contiguous(MPI_Fint *count, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_contiguous_fortran_wrapper(count, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_contiguous_(MPI_Fint *count, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_contiguous_fortran_wrapper(count, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_contiguous__(MPI_Fint *count, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_contiguous_fortran_wrapper(count, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_contiguous ================= */


/* ================== C Wrappers for MPI_Type_create_darray ================== */
_EXTERN_C_ int PMPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_darray at base level - Location = %px",&(MPI_Type_create_darray));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_darray_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_darray =============== */
static void MPI_Type_create_darray_fortran_wrapper(MPI_Fint *size, MPI_Fint *rank, MPI_Fint *ndims, MPI_Fint gsize_array[], MPI_Fint distrib_array[], MPI_Fint darg_array[], MPI_Fint psize_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_darray(*size, *rank, *ndims, ((const int*)gsize_array), ((const int*)distrib_array), ((const int*)darg_array), ((const int*)psize_array), *order, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_darray(*size, *rank, *ndims, ((const int*)gsize_array), ((const int*)distrib_array), ((const int*)darg_array), ((const int*)psize_array), *order, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_DARRAY(MPI_Fint *size, MPI_Fint *rank, MPI_Fint *ndims, MPI_Fint gsize_array[], MPI_Fint distrib_array[], MPI_Fint darg_array[], MPI_Fint psize_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_darray_fortran_wrapper(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_darray(MPI_Fint *size, MPI_Fint *rank, MPI_Fint *ndims, MPI_Fint gsize_array[], MPI_Fint distrib_array[], MPI_Fint darg_array[], MPI_Fint psize_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_darray_fortran_wrapper(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_darray_(MPI_Fint *size, MPI_Fint *rank, MPI_Fint *ndims, MPI_Fint gsize_array[], MPI_Fint distrib_array[], MPI_Fint darg_array[], MPI_Fint psize_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_darray_fortran_wrapper(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_darray__(MPI_Fint *size, MPI_Fint *rank, MPI_Fint *ndims, MPI_Fint gsize_array[], MPI_Fint distrib_array[], MPI_Fint darg_array[], MPI_Fint psize_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_darray_fortran_wrapper(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_darray ================= */


/* ================== C Wrappers for MPI_Type_create_f90_complex ================== */
_EXTERN_C_ int PMPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_f90_complex at base level - Location = %px",&(MPI_Type_create_f90_complex));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_f90_complex_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_f90_complex(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_f90_complex(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_f90_complex(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_f90_complex =============== */
static void MPI_Type_create_f90_complex_fortran_wrapper(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_f90_complex(*p, *r, (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_f90_complex(*p, *r, &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_F90_COMPLEX(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_complex_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_complex(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_complex_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_complex_(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_complex_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_complex__(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_complex_fortran_wrapper(p, r, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_f90_complex ================= */


/* ================== C Wrappers for MPI_Type_create_f90_integer ================== */
_EXTERN_C_ int PMPI_Type_create_f90_integer(int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_f90_integer at base level - Location = %px",&(MPI_Type_create_f90_integer));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_f90_integer_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_f90_integer(r, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_f90_integer(r, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_f90_integer(r, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_f90_integer =============== */
static void MPI_Type_create_f90_integer_fortran_wrapper(MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_f90_integer(*r, (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_f90_integer(*r, &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_F90_INTEGER(MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_integer_fortran_wrapper(r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_integer(MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_integer_fortran_wrapper(r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_integer_(MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_integer_fortran_wrapper(r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_integer__(MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_integer_fortran_wrapper(r, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_f90_integer ================= */


/* ================== C Wrappers for MPI_Type_create_f90_real ================== */
_EXTERN_C_ int PMPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_f90_real at base level - Location = %px",&(MPI_Type_create_f90_real));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_f90_real_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_f90_real(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_f90_real(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_f90_real(p, r, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_f90_real =============== */
static void MPI_Type_create_f90_real_fortran_wrapper(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_f90_real(*p, *r, (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_f90_real(*p, *r, &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_F90_REAL(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_real_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_real(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_real_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_real_(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_real_fortran_wrapper(p, r, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_f90_real__(MPI_Fint *p, MPI_Fint *r, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_f90_real_fortran_wrapper(p, r, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_f90_real ================= */


/* ================== C Wrappers for MPI_Type_create_hindexed ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_hindexed at base level - Location = %px",&(MPI_Type_create_hindexed));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_hindexed_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_hindexed =============== */
static void MPI_Type_create_hindexed_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_hindexed(*count, ((const int*)array_of_blocklengths), ((const MPI_Aint*)array_of_displacements), (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_hindexed(*count, ((const int*)array_of_blocklengths), ((const MPI_Aint*)array_of_displacements), MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_HINDEXED(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed_(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed__(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_hindexed ================= */


/* ================== C Wrappers for MPI_Type_create_hindexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_hindexed_block at base level - Location = %px",&(MPI_Type_create_hindexed_block));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_hindexed_block_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_hindexed_block =============== */
static void MPI_Type_create_hindexed_block_fortran_wrapper(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_hindexed_block(*count, *blocklength, ((const MPI_Aint*)array_of_displacements), (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_hindexed_block(*count, *blocklength, ((const MPI_Aint*)array_of_displacements), MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_HINDEXED_BLOCK(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed_block(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed_block_(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hindexed_block__(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hindexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_hindexed_block ================= */


/* ================== C Wrappers for MPI_Type_create_hvector ================== */
_EXTERN_C_ int PMPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_hvector at base level - Location = %px",&(MPI_Type_create_hvector));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_hvector_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_hvector =============== */
static void MPI_Type_create_hvector_fortran_wrapper(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_hvector(*count, *blocklength, *stride, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_hvector(*count, *blocklength, *stride, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_HVECTOR(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hvector(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hvector_(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_hvector__(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_hvector ================= */


/* ================== C Wrappers for MPI_Type_create_indexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_indexed_block at base level - Location = %px",&(MPI_Type_create_indexed_block));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_indexed_block_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_indexed_block =============== */
static void MPI_Type_create_indexed_block_fortran_wrapper(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_indexed_block(*count, *blocklength, ((const int*)array_of_displacements), (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_indexed_block(*count, *blocklength, ((const int*)array_of_displacements), MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_INDEXED_BLOCK(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_indexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_indexed_block(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_indexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_indexed_block_(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_indexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_indexed_block__(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_indexed_block_fortran_wrapper(count, blocklength, array_of_displacements, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_indexed_block ================= */


/* ================== C Wrappers for MPI_Type_create_keyval ================== */
_EXTERN_C_ int PMPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);
_EXTERN_C_ int MPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_keyval at base level - Location = %px",&(MPI_Type_create_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_keyval =============== */
static void MPI_Type_create_keyval_fortran_wrapper(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Type_create_keyval(((MPI_Type_copy_attr_function*)type_copy_attr_fn), ((MPI_Type_delete_attr_function*)type_delete_attr_fn), ((int*)type_keyval), BufferC2F((void*)extra_state));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_KEYVAL(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Type_create_keyval_fortran_wrapper(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Type_create_keyval_fortran_wrapper(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_type_create_keyval_(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Type_create_keyval_fortran_wrapper(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_type_create_keyval__(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Type_create_keyval_fortran_wrapper(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr);
}

/* ================= End Wrappers for MPI_Type_create_keyval ================= */


/* ================== C Wrappers for MPI_Type_create_resized ================== */
_EXTERN_C_ int PMPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_resized at base level - Location = %px",&(MPI_Type_create_resized));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_resized_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_resized(oldtype, lb, extent, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_resized(oldtype, lb, extent, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_resized(oldtype, lb, extent, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_resized =============== */
static void MPI_Type_create_resized_fortran_wrapper(MPI_Fint *oldtype, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_resized((MPI_Datatype)(*oldtype), *lb, *extent, (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_resized(MPI_Type_f2c(*oldtype), *lb, *extent, &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_RESIZED(MPI_Fint *oldtype, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_resized_fortran_wrapper(oldtype, lb, extent, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_resized(MPI_Fint *oldtype, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_resized_fortran_wrapper(oldtype, lb, extent, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_resized_(MPI_Fint *oldtype, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_resized_fortran_wrapper(oldtype, lb, extent, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_resized__(MPI_Fint *oldtype, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_resized_fortran_wrapper(oldtype, lb, extent, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_resized ================= */


/* ================== C Wrappers for MPI_Type_create_struct ================== */
_EXTERN_C_ int PMPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_struct at base level - Location = %px",&(MPI_Type_create_struct));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_struct_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_struct =============== */
static void MPI_Type_create_struct_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_block_lengths[], MPI_Fint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_struct(*count, ((const int*)array_of_block_lengths), ((const MPI_Aint*)array_of_displacements), ((const MPI_Datatype*)array_of_types), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_struct(*count, ((const int*)array_of_block_lengths), ((const MPI_Aint*)array_of_displacements), ((const MPI_Datatype*)array_of_types), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_STRUCT(MPI_Fint *count, MPI_Fint array_of_block_lengths[], MPI_Fint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_struct_fortran_wrapper(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_struct(MPI_Fint *count, MPI_Fint array_of_block_lengths[], MPI_Fint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_struct_fortran_wrapper(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_struct_(MPI_Fint *count, MPI_Fint array_of_block_lengths[], MPI_Fint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_struct_fortran_wrapper(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_struct__(MPI_Fint *count, MPI_Fint array_of_block_lengths[], MPI_Fint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_struct_fortran_wrapper(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_struct ================= */


/* ================== C Wrappers for MPI_Type_create_subarray ================== */
_EXTERN_C_ int PMPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_create_subarray at base level - Location = %px",&(MPI_Type_create_subarray));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_create_subarray_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_create_subarray =============== */
static void MPI_Type_create_subarray_fortran_wrapper(MPI_Fint *ndims, MPI_Fint size_array[], MPI_Fint subsize_array[], MPI_Fint start_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_create_subarray(*ndims, ((const int*)size_array), ((const int*)subsize_array), ((const int*)start_array), *order, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_create_subarray(*ndims, ((const int*)size_array), ((const int*)subsize_array), ((const int*)start_array), *order, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_CREATE_SUBARRAY(MPI_Fint *ndims, MPI_Fint size_array[], MPI_Fint subsize_array[], MPI_Fint start_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_subarray_fortran_wrapper(ndims, size_array, subsize_array, start_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_subarray(MPI_Fint *ndims, MPI_Fint size_array[], MPI_Fint subsize_array[], MPI_Fint start_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_subarray_fortran_wrapper(ndims, size_array, subsize_array, start_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_subarray_(MPI_Fint *ndims, MPI_Fint size_array[], MPI_Fint subsize_array[], MPI_Fint start_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_subarray_fortran_wrapper(ndims, size_array, subsize_array, start_array, order, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_create_subarray__(MPI_Fint *ndims, MPI_Fint size_array[], MPI_Fint subsize_array[], MPI_Fint start_array[], MPI_Fint *order, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_create_subarray_fortran_wrapper(ndims, size_array, subsize_array, start_array, order, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_create_subarray ================= */


/* ================== C Wrappers for MPI_Type_delete_attr ================== */
_EXTERN_C_ int PMPI_Type_delete_attr(MPI_Datatype type, int type_keyval);
_EXTERN_C_ int MPI_Type_delete_attr(MPI_Datatype type, int type_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_delete_attr at base level - Location = %px",&(MPI_Type_delete_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_delete_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_delete_attr(type, type_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_delete_attr(type, type_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_delete_attr(type, type_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_delete_attr =============== */
static void MPI_Type_delete_attr_fortran_wrapper(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_delete_attr((MPI_Datatype)(*type), *type_keyval);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_delete_attr(MPI_Type_f2c(*type), *type_keyval);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_DELETE_ATTR(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_delete_attr_fortran_wrapper(type, type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_delete_attr(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_delete_attr_fortran_wrapper(type, type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_delete_attr_(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_delete_attr_fortran_wrapper(type, type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_delete_attr__(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_delete_attr_fortran_wrapper(type, type_keyval, ierr);
}

/* ================= End Wrappers for MPI_Type_delete_attr ================= */


/* ================== C Wrappers for MPI_Type_dup ================== */
_EXTERN_C_ int PMPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_dup at base level - Location = %px",&(MPI_Type_dup));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_dup_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_dup(type, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_dup(type, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_dup(type, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_dup =============== */
static void MPI_Type_dup_fortran_wrapper(MPI_Fint *type, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_dup((MPI_Datatype)(*type), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_dup(MPI_Type_f2c(*type), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_DUP(MPI_Fint *type, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_dup_fortran_wrapper(type, newtype, ierr);
}

_EXTERN_C_ void mpi_type_dup(MPI_Fint *type, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_dup_fortran_wrapper(type, newtype, ierr);
}

_EXTERN_C_ void mpi_type_dup_(MPI_Fint *type, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_dup_fortran_wrapper(type, newtype, ierr);
}

_EXTERN_C_ void mpi_type_dup__(MPI_Fint *type, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_dup_fortran_wrapper(type, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_dup ================= */


/* ================== C Wrappers for MPI_Type_extent ================== */
_EXTERN_C_ int PMPI_Type_extent(MPI_Datatype type, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_extent(MPI_Datatype type, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_extent at base level - Location = %px",&(MPI_Type_extent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_extent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_extent(type, extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_extent(type, extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_extent(type, extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_extent =============== */
static void MPI_Type_extent_fortran_wrapper(MPI_Fint *type, MPI_Aint *extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_extent((MPI_Datatype)(*type), ((MPI_Aint*)extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_extent(MPI_Type_f2c(*type), ((MPI_Aint*)extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_EXTENT(MPI_Fint *type, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_extent_fortran_wrapper(type, extent, ierr);
}

_EXTERN_C_ void mpi_type_extent(MPI_Fint *type, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_extent_fortran_wrapper(type, extent, ierr);
}

_EXTERN_C_ void mpi_type_extent_(MPI_Fint *type, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_extent_fortran_wrapper(type, extent, ierr);
}

_EXTERN_C_ void mpi_type_extent__(MPI_Fint *type, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_extent_fortran_wrapper(type, extent, ierr);
}

/* ================= End Wrappers for MPI_Type_extent ================= */


/* ================== C Wrappers for MPI_Type_free ================== */
_EXTERN_C_ int PMPI_Type_free(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_free(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_free at base level - Location = %px",&(MPI_Type_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_free(type);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_free(type);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_free(type);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_free =============== */
static void MPI_Type_free_fortran_wrapper(MPI_Fint *type, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_free((MPI_Datatype*)type);
#else /* MPI-2 safe call */
    MPI_Datatype temp_type;
    temp_type = MPI_Type_f2c(*type);
    _wrap_py_return_val = MPI_Type_free(&temp_type);
    *type = MPI_Type_c2f(temp_type);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_FREE(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_free_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_free(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_free_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_free_(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_free_fortran_wrapper(type, ierr);
}

_EXTERN_C_ void mpi_type_free__(MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_free_fortran_wrapper(type, ierr);
}

/* ================= End Wrappers for MPI_Type_free ================= */


/* ================== C Wrappers for MPI_Type_free_keyval ================== */
_EXTERN_C_ int PMPI_Type_free_keyval(int *type_keyval);
_EXTERN_C_ int MPI_Type_free_keyval(int *type_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_free_keyval at base level - Location = %px",&(MPI_Type_free_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_free_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_free_keyval(type_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_free_keyval(type_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_free_keyval(type_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_free_keyval =============== */
static void MPI_Type_free_keyval_fortran_wrapper(MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Type_free_keyval(((int*)type_keyval));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_FREE_KEYVAL(MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_free_keyval_fortran_wrapper(type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_free_keyval(MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_free_keyval_fortran_wrapper(type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_free_keyval_(MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_free_keyval_fortran_wrapper(type_keyval, ierr);
}

_EXTERN_C_ void mpi_type_free_keyval__(MPI_Fint *type_keyval, MPI_Fint *ierr) { 
    MPI_Type_free_keyval_fortran_wrapper(type_keyval, ierr);
}

/* ================= End Wrappers for MPI_Type_free_keyval ================= */


/* ================== C Wrappers for MPI_Type_get_attr ================== */
_EXTERN_C_ int PMPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_attr at base level - Location = %px",&(MPI_Type_get_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_attr =============== */
static void MPI_Type_get_attr_fortran_wrapper(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_attr((MPI_Datatype)(*type), *type_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_attr(MPI_Type_f2c(*type), *type_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_ATTR(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Type_get_attr_fortran_wrapper(type, type_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_type_get_attr(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Type_get_attr_fortran_wrapper(type, type_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_type_get_attr_(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Type_get_attr_fortran_wrapper(type, type_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_type_get_attr__(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Type_get_attr_fortran_wrapper(type, type_keyval, attribute_val, flag, ierr);
}

/* ================= End Wrappers for MPI_Type_get_attr ================= */


/* ================== C Wrappers for MPI_Type_get_contents ================== */
_EXTERN_C_ int PMPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);
_EXTERN_C_ int MPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_contents at base level - Location = %px",&(MPI_Type_get_contents));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_contents_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_contents =============== */
static void MPI_Type_get_contents_fortran_wrapper(MPI_Fint *mtype, MPI_Fint *max_integers, MPI_Fint *max_addresses, MPI_Fint *max_datatypes, MPI_Fint array_of_integers[], MPI_Aint array_of_addresses[], MPI_Fint array_of_datatypes[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_contents((MPI_Datatype)(*mtype), *max_integers, *max_addresses, *max_datatypes, ((int*)array_of_integers), ((MPI_Aint*)array_of_addresses), (MPI_Datatype*)array_of_datatypes);
#else /* MPI-2 safe call */
    MPI_Datatype* temp_array_of_datatypes;
    int i;
    temp_array_of_datatypes = (MPI_Datatype*)malloc(sizeof(MPI_Datatype) * *max_integers);
    for (i=0; i < *max_integers; i++)
        temp_array_of_datatypes[i] = MPI_Type_f2c(array_of_datatypes[i]);
    _wrap_py_return_val = MPI_Type_get_contents(MPI_Type_f2c(*mtype), *max_integers, *max_addresses, *max_datatypes, ((int*)array_of_integers), ((MPI_Aint*)array_of_addresses), temp_array_of_datatypes);
 if(temp_array_of_datatypes) free(temp_array_of_datatypes);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_CONTENTS(MPI_Fint *mtype, MPI_Fint *max_integers, MPI_Fint *max_addresses, MPI_Fint *max_datatypes, MPI_Fint array_of_integers[], MPI_Aint array_of_addresses[], MPI_Fint array_of_datatypes[], MPI_Fint *ierr) { 
    MPI_Type_get_contents_fortran_wrapper(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes, ierr);
}

_EXTERN_C_ void mpi_type_get_contents(MPI_Fint *mtype, MPI_Fint *max_integers, MPI_Fint *max_addresses, MPI_Fint *max_datatypes, MPI_Fint array_of_integers[], MPI_Aint array_of_addresses[], MPI_Fint array_of_datatypes[], MPI_Fint *ierr) { 
    MPI_Type_get_contents_fortran_wrapper(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes, ierr);
}

_EXTERN_C_ void mpi_type_get_contents_(MPI_Fint *mtype, MPI_Fint *max_integers, MPI_Fint *max_addresses, MPI_Fint *max_datatypes, MPI_Fint array_of_integers[], MPI_Aint array_of_addresses[], MPI_Fint array_of_datatypes[], MPI_Fint *ierr) { 
    MPI_Type_get_contents_fortran_wrapper(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes, ierr);
}

_EXTERN_C_ void mpi_type_get_contents__(MPI_Fint *mtype, MPI_Fint *max_integers, MPI_Fint *max_addresses, MPI_Fint *max_datatypes, MPI_Fint array_of_integers[], MPI_Aint array_of_addresses[], MPI_Fint array_of_datatypes[], MPI_Fint *ierr) { 
    MPI_Type_get_contents_fortran_wrapper(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes, ierr);
}

/* ================= End Wrappers for MPI_Type_get_contents ================= */


/* ================== C Wrappers for MPI_Type_get_envelope ================== */
_EXTERN_C_ int PMPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);
_EXTERN_C_ int MPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_envelope at base level - Location = %px",&(MPI_Type_get_envelope));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_envelope_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_envelope =============== */
static void MPI_Type_get_envelope_fortran_wrapper(MPI_Fint *type, MPI_Fint *num_integers, MPI_Fint *num_addresses, MPI_Fint *num_datatypes, MPI_Fint *combiner, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_envelope((MPI_Datatype)(*type), ((int*)num_integers), ((int*)num_addresses), ((int*)num_datatypes), ((int*)combiner));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_envelope(MPI_Type_f2c(*type), ((int*)num_integers), ((int*)num_addresses), ((int*)num_datatypes), ((int*)combiner));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_ENVELOPE(MPI_Fint *type, MPI_Fint *num_integers, MPI_Fint *num_addresses, MPI_Fint *num_datatypes, MPI_Fint *combiner, MPI_Fint *ierr) { 
    MPI_Type_get_envelope_fortran_wrapper(type, num_integers, num_addresses, num_datatypes, combiner, ierr);
}

_EXTERN_C_ void mpi_type_get_envelope(MPI_Fint *type, MPI_Fint *num_integers, MPI_Fint *num_addresses, MPI_Fint *num_datatypes, MPI_Fint *combiner, MPI_Fint *ierr) { 
    MPI_Type_get_envelope_fortran_wrapper(type, num_integers, num_addresses, num_datatypes, combiner, ierr);
}

_EXTERN_C_ void mpi_type_get_envelope_(MPI_Fint *type, MPI_Fint *num_integers, MPI_Fint *num_addresses, MPI_Fint *num_datatypes, MPI_Fint *combiner, MPI_Fint *ierr) { 
    MPI_Type_get_envelope_fortran_wrapper(type, num_integers, num_addresses, num_datatypes, combiner, ierr);
}

_EXTERN_C_ void mpi_type_get_envelope__(MPI_Fint *type, MPI_Fint *num_integers, MPI_Fint *num_addresses, MPI_Fint *num_datatypes, MPI_Fint *combiner, MPI_Fint *ierr) { 
    MPI_Type_get_envelope_fortran_wrapper(type, num_integers, num_addresses, num_datatypes, combiner, ierr);
}

/* ================= End Wrappers for MPI_Type_get_envelope ================= */


/* ================== C Wrappers for MPI_Type_get_extent ================== */
_EXTERN_C_ int PMPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_extent at base level - Location = %px",&(MPI_Type_get_extent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_extent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_extent(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_extent(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_extent(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_extent =============== */
static void MPI_Type_get_extent_fortran_wrapper(MPI_Fint *type, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_extent((MPI_Datatype)(*type), ((MPI_Aint*)lb), ((MPI_Aint*)extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_extent(MPI_Type_f2c(*type), ((MPI_Aint*)lb), ((MPI_Aint*)extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_EXTENT(MPI_Fint *type, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent(MPI_Fint *type, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent_(MPI_Fint *type, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent__(MPI_Fint *type, MPI_Aint *lb, MPI_Aint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_fortran_wrapper(type, lb, extent, ierr);
}

/* ================= End Wrappers for MPI_Type_get_extent ================= */


/* ================== C Wrappers for MPI_Type_get_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);
_EXTERN_C_ int MPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_extent_x at base level - Location = %px",&(MPI_Type_get_extent_x));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_extent_x_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_extent_x(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_extent_x(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_extent_x(type, lb, extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_extent_x =============== */
static void MPI_Type_get_extent_x_fortran_wrapper(MPI_Fint *type, MPI_Fint *lb, MPI_Fint *extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_extent_x((MPI_Datatype)(*type), ((MPI_Count*)lb), ((MPI_Count*)extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_extent_x(MPI_Type_f2c(*type), ((MPI_Count*)lb), ((MPI_Count*)extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_EXTENT_X(MPI_Fint *type, MPI_Fint *lb, MPI_Fint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_x_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent_x(MPI_Fint *type, MPI_Fint *lb, MPI_Fint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_x_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent_x_(MPI_Fint *type, MPI_Fint *lb, MPI_Fint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_x_fortran_wrapper(type, lb, extent, ierr);
}

_EXTERN_C_ void mpi_type_get_extent_x__(MPI_Fint *type, MPI_Fint *lb, MPI_Fint *extent, MPI_Fint *ierr) { 
    MPI_Type_get_extent_x_fortran_wrapper(type, lb, extent, ierr);
}

/* ================= End Wrappers for MPI_Type_get_extent_x ================= */


/* ================== C Wrappers for MPI_Type_get_name ================== */
_EXTERN_C_ int PMPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen);
_EXTERN_C_ int MPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_name at base level - Location = %px",&(MPI_Type_get_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_name(type, type_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_name(type, type_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_name(type, type_name, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_name =============== */
static void MPI_Type_get_name_fortran_wrapper(MPI_Fint *type, char *type_name, MPI_Fint *resultlen, MPI_Fint *ierr, int type_name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_name((MPI_Datatype)(*type), type_name, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_type_name;
    temp_type_name = (char*)malloc(sizeof(char) * (type_name_len+1));
    char_p_f2c(type_name,type_name_len,&temp_type_name);
    _wrap_py_return_val = MPI_Type_get_name(MPI_Type_f2c(*type), type_name, ((int*)resultlen));
    char_p_c2f(temp_type_name,type_name,type_name_len);
 if(temp_type_name) free(temp_type_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_NAME(MPI_Fint *type, char *type_name, MPI_Fint *resultlen, MPI_Fint *ierr, int type_name_len) { 
    MPI_Type_get_name_fortran_wrapper(type, type_name, resultlen, ierr, type_name_len);
}

_EXTERN_C_ void mpi_type_get_name(MPI_Fint *type, char *type_name, MPI_Fint *resultlen, MPI_Fint *ierr, int type_name_len) { 
    MPI_Type_get_name_fortran_wrapper(type, type_name, resultlen, ierr, type_name_len);
}

_EXTERN_C_ void mpi_type_get_name_(MPI_Fint *type, char *type_name, MPI_Fint *resultlen, MPI_Fint *ierr, int type_name_len) { 
    MPI_Type_get_name_fortran_wrapper(type, type_name, resultlen, ierr, type_name_len);
}

_EXTERN_C_ void mpi_type_get_name__(MPI_Fint *type, char *type_name, MPI_Fint *resultlen, MPI_Fint *ierr, int type_name_len) { 
    MPI_Type_get_name_fortran_wrapper(type, type_name, resultlen, ierr, type_name_len);
}

/* ================= End Wrappers for MPI_Type_get_name ================= */


/* ================== C Wrappers for MPI_Type_get_true_extent ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_true_extent at base level - Location = %px",&(MPI_Type_get_true_extent));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_true_extent_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_true_extent(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_true_extent(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_true_extent(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_true_extent =============== */
static void MPI_Type_get_true_extent_fortran_wrapper(MPI_Fint *datatype, MPI_Aint *true_lb, MPI_Aint *true_extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_true_extent((MPI_Datatype)(*datatype), ((MPI_Aint*)true_lb), ((MPI_Aint*)true_extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_true_extent(MPI_Type_f2c(*datatype), ((MPI_Aint*)true_lb), ((MPI_Aint*)true_extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_TRUE_EXTENT(MPI_Fint *datatype, MPI_Aint *true_lb, MPI_Aint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent(MPI_Fint *datatype, MPI_Aint *true_lb, MPI_Aint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent_(MPI_Fint *datatype, MPI_Aint *true_lb, MPI_Aint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent__(MPI_Fint *datatype, MPI_Aint *true_lb, MPI_Aint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

/* ================= End Wrappers for MPI_Type_get_true_extent ================= */


/* ================== C Wrappers for MPI_Type_get_true_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_get_true_extent_x at base level - Location = %px",&(MPI_Type_get_true_extent_x));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_get_true_extent_x_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_get_true_extent_x =============== */
static void MPI_Type_get_true_extent_x_fortran_wrapper(MPI_Fint *datatype, MPI_Fint *true_lb, MPI_Fint *true_extent, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_get_true_extent_x((MPI_Datatype)(*datatype), ((MPI_Count*)true_lb), ((MPI_Count*)true_extent));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_get_true_extent_x(MPI_Type_f2c(*datatype), ((MPI_Count*)true_lb), ((MPI_Count*)true_extent));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_GET_TRUE_EXTENT_X(MPI_Fint *datatype, MPI_Fint *true_lb, MPI_Fint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_x_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent_x(MPI_Fint *datatype, MPI_Fint *true_lb, MPI_Fint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_x_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent_x_(MPI_Fint *datatype, MPI_Fint *true_lb, MPI_Fint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_x_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

_EXTERN_C_ void mpi_type_get_true_extent_x__(MPI_Fint *datatype, MPI_Fint *true_lb, MPI_Fint *true_extent, MPI_Fint *ierr) { 
    MPI_Type_get_true_extent_x_fortran_wrapper(datatype, true_lb, true_extent, ierr);
}

/* ================= End Wrappers for MPI_Type_get_true_extent_x ================= */


/* ================== C Wrappers for MPI_Type_hindexed ================== */
_EXTERN_C_ int PMPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_hindexed at base level - Location = %px",&(MPI_Type_hindexed));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_hindexed_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_hindexed =============== */
static void MPI_Type_hindexed_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_hindexed(*count, ((int*)array_of_blocklengths), ((MPI_Aint*)array_of_displacements), (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_hindexed(*count, ((int*)array_of_blocklengths), ((MPI_Aint*)array_of_displacements), MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_HINDEXED(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hindexed(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hindexed_(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hindexed__(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hindexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_hindexed ================= */


/* ================== C Wrappers for MPI_Type_hvector ================== */
_EXTERN_C_ int PMPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_hvector at base level - Location = %px",&(MPI_Type_hvector));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_hvector_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_hvector =============== */
static void MPI_Type_hvector_fortran_wrapper(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_hvector(*count, *blocklength, *stride, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_hvector(*count, *blocklength, *stride, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_HVECTOR(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hvector(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hvector_(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_hvector__(MPI_Fint *count, MPI_Fint *blocklength, MPI_Aint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_hvector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_hvector ================= */


/* ================== C Wrappers for MPI_Type_indexed ================== */
_EXTERN_C_ int PMPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_indexed at base level - Location = %px",&(MPI_Type_indexed));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_indexed_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_indexed =============== */
static void MPI_Type_indexed_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_indexed(*count, ((const int*)array_of_blocklengths), ((const int*)array_of_displacements), (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_indexed(*count, ((const int*)array_of_blocklengths), ((const int*)array_of_displacements), MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_INDEXED(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_indexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_indexed(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_indexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_indexed_(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_indexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_indexed__(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Fint array_of_displacements[], MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_indexed_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_indexed ================= */


/* ================== C Wrappers for MPI_Type_lb ================== */
_EXTERN_C_ int PMPI_Type_lb(MPI_Datatype type, MPI_Aint *lb);
_EXTERN_C_ int MPI_Type_lb(MPI_Datatype type, MPI_Aint *lb) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_lb at base level - Location = %px",&(MPI_Type_lb));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_lb_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_lb(type, lb);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_lb(type, lb);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_lb(type, lb);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_lb =============== */
static void MPI_Type_lb_fortran_wrapper(MPI_Fint *type, MPI_Aint *lb, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_lb((MPI_Datatype)(*type), ((MPI_Aint*)lb));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_lb(MPI_Type_f2c(*type), ((MPI_Aint*)lb));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_LB(MPI_Fint *type, MPI_Aint *lb, MPI_Fint *ierr) { 
    MPI_Type_lb_fortran_wrapper(type, lb, ierr);
}

_EXTERN_C_ void mpi_type_lb(MPI_Fint *type, MPI_Aint *lb, MPI_Fint *ierr) { 
    MPI_Type_lb_fortran_wrapper(type, lb, ierr);
}

_EXTERN_C_ void mpi_type_lb_(MPI_Fint *type, MPI_Aint *lb, MPI_Fint *ierr) { 
    MPI_Type_lb_fortran_wrapper(type, lb, ierr);
}

_EXTERN_C_ void mpi_type_lb__(MPI_Fint *type, MPI_Aint *lb, MPI_Fint *ierr) { 
    MPI_Type_lb_fortran_wrapper(type, lb, ierr);
}

/* ================= End Wrappers for MPI_Type_lb ================= */


/* ================== C Wrappers for MPI_Type_match_size ================== */
_EXTERN_C_ int PMPI_Type_match_size(int typeclass, int size, MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_match_size(int typeclass, int size, MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_match_size at base level - Location = %px",&(MPI_Type_match_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_match_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_match_size(typeclass, size, type);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_match_size(typeclass, size, type);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_match_size(typeclass, size, type);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_match_size =============== */
static void MPI_Type_match_size_fortran_wrapper(MPI_Fint *typeclass, MPI_Fint *size, MPI_Fint *type, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_match_size(*typeclass, *size, (MPI_Datatype*)type);
#else /* MPI-2 safe call */
    MPI_Datatype temp_type;
    _wrap_py_return_val = MPI_Type_match_size(*typeclass, *size, &temp_type);
    *type = MPI_Type_c2f(temp_type);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_MATCH_SIZE(MPI_Fint *typeclass, MPI_Fint *size, MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_match_size_fortran_wrapper(typeclass, size, type, ierr);
}

_EXTERN_C_ void mpi_type_match_size(MPI_Fint *typeclass, MPI_Fint *size, MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_match_size_fortran_wrapper(typeclass, size, type, ierr);
}

_EXTERN_C_ void mpi_type_match_size_(MPI_Fint *typeclass, MPI_Fint *size, MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_match_size_fortran_wrapper(typeclass, size, type, ierr);
}

_EXTERN_C_ void mpi_type_match_size__(MPI_Fint *typeclass, MPI_Fint *size, MPI_Fint *type, MPI_Fint *ierr) { 
    MPI_Type_match_size_fortran_wrapper(typeclass, size, type, ierr);
}

/* ================= End Wrappers for MPI_Type_match_size ================= */


/* ================== C Wrappers for MPI_Type_set_attr ================== */
_EXTERN_C_ int PMPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val);
_EXTERN_C_ int MPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_set_attr at base level - Location = %px",&(MPI_Type_set_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_set_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_set_attr(type, type_keyval, attr_val);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_set_attr(type, type_keyval, attr_val);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_set_attr(type, type_keyval, attr_val);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_set_attr =============== */
static void MPI_Type_set_attr_fortran_wrapper(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attr_val, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_set_attr((MPI_Datatype)(*type), *type_keyval, BufferC2F((void*)attr_val));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_set_attr(MPI_Type_f2c(*type), *type_keyval, BufferC2F((void*)attr_val));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_SET_ATTR(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attr_val, MPI_Fint *ierr) { 
    MPI_Type_set_attr_fortran_wrapper(type, type_keyval, attr_val, ierr);
}

_EXTERN_C_ void mpi_type_set_attr(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attr_val, MPI_Fint *ierr) { 
    MPI_Type_set_attr_fortran_wrapper(type, type_keyval, attr_val, ierr);
}

_EXTERN_C_ void mpi_type_set_attr_(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attr_val, MPI_Fint *ierr) { 
    MPI_Type_set_attr_fortran_wrapper(type, type_keyval, attr_val, ierr);
}

_EXTERN_C_ void mpi_type_set_attr__(MPI_Fint *type, MPI_Fint *type_keyval, MPI_Fint *attr_val, MPI_Fint *ierr) { 
    MPI_Type_set_attr_fortran_wrapper(type, type_keyval, attr_val, ierr);
}

/* ================= End Wrappers for MPI_Type_set_attr ================= */


/* ================== C Wrappers for MPI_Type_set_name ================== */
_EXTERN_C_ int PMPI_Type_set_name(MPI_Datatype type, const char *type_name);
_EXTERN_C_ int MPI_Type_set_name(MPI_Datatype type, const char *type_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_set_name at base level - Location = %px",&(MPI_Type_set_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_set_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_set_name(type, type_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_set_name(type, type_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_set_name(type, type_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_set_name =============== */
static void MPI_Type_set_name_fortran_wrapper(MPI_Fint *type, MPI_Fint *type_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_set_name((MPI_Datatype)(*type), ((const char*)type_name));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_set_name(MPI_Type_f2c(*type), ((const char*)type_name));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_SET_NAME(MPI_Fint *type, MPI_Fint *type_name, MPI_Fint *ierr) { 
    MPI_Type_set_name_fortran_wrapper(type, type_name, ierr);
}

_EXTERN_C_ void mpi_type_set_name(MPI_Fint *type, MPI_Fint *type_name, MPI_Fint *ierr) { 
    MPI_Type_set_name_fortran_wrapper(type, type_name, ierr);
}

_EXTERN_C_ void mpi_type_set_name_(MPI_Fint *type, MPI_Fint *type_name, MPI_Fint *ierr) { 
    MPI_Type_set_name_fortran_wrapper(type, type_name, ierr);
}

_EXTERN_C_ void mpi_type_set_name__(MPI_Fint *type, MPI_Fint *type_name, MPI_Fint *ierr) { 
    MPI_Type_set_name_fortran_wrapper(type, type_name, ierr);
}

/* ================= End Wrappers for MPI_Type_set_name ================= */


/* ================== C Wrappers for MPI_Type_size ================== */
_EXTERN_C_ int PMPI_Type_size(MPI_Datatype type, int *size);
_EXTERN_C_ int MPI_Type_size(MPI_Datatype type, int *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_size at base level - Location = %px",&(MPI_Type_size));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_size_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_size(type, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_size(type, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_size(type, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_size =============== */
static void MPI_Type_size_fortran_wrapper(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_size((MPI_Datatype)(*type), ((int*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_size(MPI_Type_f2c(*type), ((int*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_SIZE(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size_(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size__(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_fortran_wrapper(type, size, ierr);
}

/* ================= End Wrappers for MPI_Type_size ================= */


/* ================== C Wrappers for MPI_Type_size_x ================== */
_EXTERN_C_ int PMPI_Type_size_x(MPI_Datatype type, MPI_Count *size);
_EXTERN_C_ int MPI_Type_size_x(MPI_Datatype type, MPI_Count *size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_size_x at base level - Location = %px",&(MPI_Type_size_x));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_size_x_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_size_x(type, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_size_x(type, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_size_x(type, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_size_x =============== */
static void MPI_Type_size_x_fortran_wrapper(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_size_x((MPI_Datatype)(*type), ((MPI_Count*)size));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_size_x(MPI_Type_f2c(*type), ((MPI_Count*)size));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_SIZE_X(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_x_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size_x(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_x_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size_x_(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_x_fortran_wrapper(type, size, ierr);
}

_EXTERN_C_ void mpi_type_size_x__(MPI_Fint *type, MPI_Fint *size, MPI_Fint *ierr) { 
    MPI_Type_size_x_fortran_wrapper(type, size, ierr);
}

/* ================= End Wrappers for MPI_Type_size_x ================= */


/* ================== C Wrappers for MPI_Type_struct ================== */
_EXTERN_C_ int PMPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_struct at base level - Location = %px",&(MPI_Type_struct));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_struct_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_struct =============== */
static void MPI_Type_struct_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_struct(*count, ((int*)array_of_blocklengths), ((MPI_Aint*)array_of_displacements), (MPI_Datatype*)array_of_types, (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    MPI_Datatype* temp_array_of_types;
    int i;
    temp_array_of_types = (MPI_Datatype*)malloc(sizeof(MPI_Datatype) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_types[i] = MPI_Type_f2c(array_of_types[i]);
    _wrap_py_return_val = MPI_Type_struct(*count, ((int*)array_of_blocklengths), ((MPI_Aint*)array_of_displacements), temp_array_of_types, &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
 if(temp_array_of_types) free(temp_array_of_types);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_STRUCT(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_struct_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_struct(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_struct_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_struct_(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_struct_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype, ierr);
}

_EXTERN_C_ void mpi_type_struct__(MPI_Fint *count, MPI_Fint array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Fint array_of_types[], MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_struct_fortran_wrapper(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_struct ================= */


/* ================== C Wrappers for MPI_Type_ub ================== */
_EXTERN_C_ int PMPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);
_EXTERN_C_ int MPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_ub at base level - Location = %px",&(MPI_Type_ub));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_ub_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_ub(mtype, ub);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_ub(mtype, ub);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_ub(mtype, ub);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_ub =============== */
static void MPI_Type_ub_fortran_wrapper(MPI_Fint *mtype, MPI_Aint *ub, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_ub((MPI_Datatype)(*mtype), ((MPI_Aint*)ub));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Type_ub(MPI_Type_f2c(*mtype), ((MPI_Aint*)ub));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_UB(MPI_Fint *mtype, MPI_Aint *ub, MPI_Fint *ierr) { 
    MPI_Type_ub_fortran_wrapper(mtype, ub, ierr);
}

_EXTERN_C_ void mpi_type_ub(MPI_Fint *mtype, MPI_Aint *ub, MPI_Fint *ierr) { 
    MPI_Type_ub_fortran_wrapper(mtype, ub, ierr);
}

_EXTERN_C_ void mpi_type_ub_(MPI_Fint *mtype, MPI_Aint *ub, MPI_Fint *ierr) { 
    MPI_Type_ub_fortran_wrapper(mtype, ub, ierr);
}

_EXTERN_C_ void mpi_type_ub__(MPI_Fint *mtype, MPI_Aint *ub, MPI_Fint *ierr) { 
    MPI_Type_ub_fortran_wrapper(mtype, ub, ierr);
}

/* ================= End Wrappers for MPI_Type_ub ================= */


/* ================== C Wrappers for MPI_Type_vector ================== */
_EXTERN_C_ int PMPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Type_vector at base level - Location = %px",&(MPI_Type_vector));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Type_vector_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Type_vector =============== */
static void MPI_Type_vector_fortran_wrapper(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Type_vector(*count, *blocklength, *stride, (MPI_Datatype)(*oldtype), (MPI_Datatype*)newtype);
#else /* MPI-2 safe call */
    MPI_Datatype temp_newtype;
    _wrap_py_return_val = MPI_Type_vector(*count, *blocklength, *stride, MPI_Type_f2c(*oldtype), &temp_newtype);
    *newtype = MPI_Type_c2f(temp_newtype);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_TYPE_VECTOR(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_vector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_vector(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_vector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_vector_(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_vector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

_EXTERN_C_ void mpi_type_vector__(MPI_Fint *count, MPI_Fint *blocklength, MPI_Fint *stride, MPI_Fint *oldtype, MPI_Fint *newtype, MPI_Fint *ierr) { 
    MPI_Type_vector_fortran_wrapper(count, blocklength, stride, oldtype, newtype, ierr);
}

/* ================= End Wrappers for MPI_Type_vector ================= */


/* ================== C Wrappers for MPI_Unpack ================== */
_EXTERN_C_ int PMPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);
_EXTERN_C_ int MPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Unpack at base level - Location = %px",&(MPI_Unpack));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Unpack_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Unpack =============== */
static void MPI_Unpack_fortran_wrapper(MPI_Fint *inbuf, MPI_Fint *insize, MPI_Fint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Unpack(((const void*)inbuf), *insize, ((int*)position), BufferC2F((void*)outbuf), *outcount, (MPI_Datatype)(*datatype), (MPI_Comm)(*comm));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Unpack(((const void*)inbuf), *insize, ((int*)position), BufferC2F((void*)outbuf), *outcount, MPI_Type_f2c(*datatype), MPI_Comm_f2c(*comm));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_UNPACK(MPI_Fint *inbuf, MPI_Fint *insize, MPI_Fint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Unpack_fortran_wrapper(inbuf, insize, position, outbuf, outcount, datatype, comm, ierr);
}

_EXTERN_C_ void mpi_unpack(MPI_Fint *inbuf, MPI_Fint *insize, MPI_Fint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Unpack_fortran_wrapper(inbuf, insize, position, outbuf, outcount, datatype, comm, ierr);
}

_EXTERN_C_ void mpi_unpack_(MPI_Fint *inbuf, MPI_Fint *insize, MPI_Fint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Unpack_fortran_wrapper(inbuf, insize, position, outbuf, outcount, datatype, comm, ierr);
}

_EXTERN_C_ void mpi_unpack__(MPI_Fint *inbuf, MPI_Fint *insize, MPI_Fint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *comm, MPI_Fint *ierr) { 
    MPI_Unpack_fortran_wrapper(inbuf, insize, position, outbuf, outcount, datatype, comm, ierr);
}

/* ================= End Wrappers for MPI_Unpack ================= */


/* ================== C Wrappers for MPI_Unpack_external ================== */
_EXTERN_C_ int PMPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);
_EXTERN_C_ int MPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Unpack_external at base level - Location = %px",&(MPI_Unpack_external));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Unpack_external_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Unpack_external =============== */
static void MPI_Unpack_external_fortran_wrapper(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Aint *insize, MPI_Aint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Unpack_external(((const char*)datarep), ((const void*)inbuf), *insize, ((MPI_Aint*)position), BufferC2F((void*)outbuf), *outcount, (MPI_Datatype)(*datatype));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Unpack_external(((const char*)datarep), ((const void*)inbuf), *insize, ((MPI_Aint*)position), BufferC2F((void*)outbuf), *outcount, MPI_Type_f2c(*datatype));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_UNPACK_EXTERNAL(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Aint *insize, MPI_Aint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_Unpack_external_fortran_wrapper(datarep, inbuf, insize, position, outbuf, outcount, datatype, ierr);
}

_EXTERN_C_ void mpi_unpack_external(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Aint *insize, MPI_Aint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_Unpack_external_fortran_wrapper(datarep, inbuf, insize, position, outbuf, outcount, datatype, ierr);
}

_EXTERN_C_ void mpi_unpack_external_(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Aint *insize, MPI_Aint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_Unpack_external_fortran_wrapper(datarep, inbuf, insize, position, outbuf, outcount, datatype, ierr);
}

_EXTERN_C_ void mpi_unpack_external__(MPI_Fint datarep[], MPI_Fint *inbuf, MPI_Aint *insize, MPI_Aint *position, MPI_Fint *outbuf, MPI_Fint *outcount, MPI_Fint *datatype, MPI_Fint *ierr) { 
    MPI_Unpack_external_fortran_wrapper(datarep, inbuf, insize, position, outbuf, outcount, datatype, ierr);
}

/* ================= End Wrappers for MPI_Unpack_external ================= */


/* ================== C Wrappers for MPI_Unpublish_name ================== */
_EXTERN_C_ int PMPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Unpublish_name at base level - Location = %px",&(MPI_Unpublish_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Unpublish_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Unpublish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Unpublish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Unpublish_name(service_name, info, port_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Unpublish_name =============== */
static void MPI_Unpublish_name_fortran_wrapper(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Unpublish_name(((const char*)service_name), (MPI_Info)(*info), ((const char*)port_name));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Unpublish_name(((const char*)service_name), MPI_Info_f2c(*info), ((const char*)port_name));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_UNPUBLISH_NAME(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Unpublish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_unpublish_name(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Unpublish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_unpublish_name_(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Unpublish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

_EXTERN_C_ void mpi_unpublish_name__(MPI_Fint *service_name, MPI_Fint *info, MPI_Fint *port_name, MPI_Fint *ierr) { 
    MPI_Unpublish_name_fortran_wrapper(service_name, info, port_name, ierr);
}

/* ================= End Wrappers for MPI_Unpublish_name ================= */


/* ================== C Wrappers for MPI_Wait ================== */
_EXTERN_C_ int PMPI_Wait(MPI_Request *request, MPI_Status *status);
_EXTERN_C_ int MPI_Wait(MPI_Request *request, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Wait at base level - Location = %px",&(MPI_Wait));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Wait_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Wait(request, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Wait(request, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Wait(request, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Wait =============== */
static void MPI_Wait_fortran_wrapper(MPI_Fint *request, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Wait((MPI_Request*)request, (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Request temp_request;
    MPI_Status temp_status;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_request = MPI_Request_f2c(*request);
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Wait(&temp_request, &temp_status);
    *request = MPI_Request_c2f(temp_request);
    MPI_Status_c2f(&temp_status, status);
# else /* MPI-2 safe call */
    temp_request = MPI_Request_f2c(*request);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Wait(&temp_request, ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    *request = MPI_Request_c2f(temp_request);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
# endif /* MPICH test */
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WAIT(MPI_Fint *request, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Wait_fortran_wrapper(request, status, ierr);
}

_EXTERN_C_ void mpi_wait(MPI_Fint *request, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Wait_fortran_wrapper(request, status, ierr);
}

_EXTERN_C_ void mpi_wait_(MPI_Fint *request, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Wait_fortran_wrapper(request, status, ierr);
}

_EXTERN_C_ void mpi_wait__(MPI_Fint *request, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Wait_fortran_wrapper(request, status, ierr);
}

/* ================= End Wrappers for MPI_Wait ================= */


/* ================== C Wrappers for MPI_Waitall ================== */
_EXTERN_C_ int PMPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);
_EXTERN_C_ int MPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Waitall at base level - Location = %px",&(MPI_Waitall));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Waitall_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Waitall(count, array_of_requests, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Waitall(count, array_of_requests, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Waitall(count, array_of_requests, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Waitall =============== */
static void MPI_Waitall_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *array_of_statuses, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Waitall(*count, (MPI_Request*)array_of_requests, (MPI_Status*)array_of_statuses);
#else /* MPI-2 safe call */
    MPI_Status* temp_array_of_statuses;
    MPI_Request* temp_array_of_requests;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *count);
    _wrap_py_return_val = MPI_Waitall(*count, temp_array_of_requests, temp_array_of_statuses);
      for (i=0; i < *count; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *count);
    _wrap_py_return_val = MPI_Waitall(*count, temp_array_of_requests, ((array_of_statuses == MPI_F_STATUSES_IGNORE) ? MPI_STATUSES_IGNORE : temp_array_of_statuses));
    if (array_of_statuses != MPI_F_STATUSES_IGNORE)
      for (i=0; i < *count; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
 if(temp_array_of_statuses) free(temp_array_of_statuses);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WAITALL(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *array_of_statuses, MPI_Fint *ierr) { 
    MPI_Waitall_fortran_wrapper(count, array_of_requests, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitall(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *array_of_statuses, MPI_Fint *ierr) { 
    MPI_Waitall_fortran_wrapper(count, array_of_requests, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitall_(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *array_of_statuses, MPI_Fint *ierr) { 
    MPI_Waitall_fortran_wrapper(count, array_of_requests, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitall__(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *array_of_statuses, MPI_Fint *ierr) { 
    MPI_Waitall_fortran_wrapper(count, array_of_requests, array_of_statuses, ierr);
}

/* ================= End Wrappers for MPI_Waitall ================= */


/* ================== C Wrappers for MPI_Waitany ================== */
_EXTERN_C_ int PMPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);
_EXTERN_C_ int MPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Waitany at base level - Location = %px",&(MPI_Waitany));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Waitany_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Waitany(count, array_of_requests, index, status);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Waitany(count, array_of_requests, index, status);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Waitany(count, array_of_requests, index, status);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Waitany =============== */
static void MPI_Waitany_fortran_wrapper(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *status, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Waitany(*count, (MPI_Request*)array_of_requests, index, (MPI_Status*)status);
#else /* MPI-2 safe call */
    MPI_Request* temp_array_of_requests;
    MPI_Status temp_status;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Waitany(*count, temp_array_of_requests, index, &temp_status);
    if (*index != MPI_UNDEFINED){
    ++(*index);
    MPI_Status_c2f(&temp_status, status);
    }
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *count);
    for (i=0; i < *count; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_f2c(status, &temp_status);
    _wrap_py_return_val = MPI_Waitany(*count, temp_array_of_requests, index, ((status == MPI_F_STATUS_IGNORE) ? MPI_STATUS_IGNORE : &temp_status));
    if (*index != MPI_UNDEFINED){
    ++(*index);
    if (status != MPI_F_STATUS_IGNORE) MPI_Status_c2f(&temp_status, status);
    }
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WAITANY(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Waitany_fortran_wrapper(count, array_of_requests, index, status, ierr);
}

_EXTERN_C_ void mpi_waitany(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Waitany_fortran_wrapper(count, array_of_requests, index, status, ierr);
}

_EXTERN_C_ void mpi_waitany_(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Waitany_fortran_wrapper(count, array_of_requests, index, status, ierr);
}

_EXTERN_C_ void mpi_waitany__(MPI_Fint *count, MPI_Fint array_of_requests[], MPI_Fint *index, MPI_Fint *status, MPI_Fint *ierr) { 
    MPI_Waitany_fortran_wrapper(count, array_of_requests, index, status, ierr);
}

/* ================= End Wrappers for MPI_Waitany ================= */


/* ================== C Wrappers for MPI_Waitsome ================== */
_EXTERN_C_ int PMPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Waitsome at base level - Location = %px",&(MPI_Waitsome));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Waitsome_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Waitsome =============== */
static void MPI_Waitsome_fortran_wrapper(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Waitsome(*incount, (MPI_Request*)array_of_requests, ((int*)outcount), array_of_indices, (MPI_Status*)array_of_statuses);
#else /* MPI-2 safe call */
    MPI_Status* temp_array_of_statuses;
    MPI_Request* temp_array_of_requests;
    int i;
# if defined(MPICH_NAME) && (MPICH_NAME == 1) /* MPICH test */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *incount);
    for (i=0; i < *incount; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *incount);
    _wrap_py_return_val = MPI_Waitsome(*incount, temp_array_of_requests, ((int*)outcount), array_of_indices, temp_array_of_statuses);
    if (*outcount != MPI_UNDEFINED){
        for (i=0; i < *outcount; ++i)
            ++array_of_indices[i];
      for (i=0; i < *incount; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
    }
# else /* MPI-2 safe call */
    temp_array_of_requests = (MPI_Request*)malloc(sizeof(MPI_Request) * *incount);
    for (i=0; i < *incount; i++)
        temp_array_of_requests[i] = MPI_Request_f2c(array_of_requests[i]);
    temp_array_of_statuses = (MPI_Status*)malloc(sizeof(MPI_Status) * *incount);
    _wrap_py_return_val = MPI_Waitsome(*incount, temp_array_of_requests, ((int*)outcount), array_of_indices, ((array_of_statuses == MPI_F_STATUSES_IGNORE) ? MPI_STATUSES_IGNORE : temp_array_of_statuses));
    if (*outcount != MPI_UNDEFINED){
        for (i=0; i < *outcount; ++i)
            ++array_of_indices[i];
    if (array_of_statuses != MPI_F_STATUSES_IGNORE)
      for (i=0; i < *incount; i++)
        MPI_Status_c2f(&temp_array_of_statuses[i], &array_of_statuses[i * MPI_F_STATUS_SIZE]);
    }
# endif /* MPICH test */
 if(temp_array_of_requests) free(temp_array_of_requests);
 if(temp_array_of_statuses) free(temp_array_of_statuses);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WAITSOME(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Waitsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitsome(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Waitsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitsome_(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Waitsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

_EXTERN_C_ void mpi_waitsome__(MPI_Fint *incount, MPI_Fint array_of_requests[], MPI_Fint *outcount, MPI_Fint array_of_indices[], MPI_Fint array_of_statuses[], MPI_Fint *ierr) { 
    MPI_Waitsome_fortran_wrapper(incount, array_of_requests, outcount, array_of_indices, array_of_statuses, ierr);
}

/* ================= End Wrappers for MPI_Waitsome ================= */


/* ================== C Wrappers for MPI_Win_allocate ================== */
_EXTERN_C_ int PMPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_allocate at base level - Location = %px",&(MPI_Win_allocate));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_allocate_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_allocate =============== */
static void MPI_Win_allocate_fortran_wrapper(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_allocate(*size, *disp_unit, (MPI_Info)(*info), (MPI_Comm)(*comm), BufferC2F((void*)baseptr), (MPI_Win*)win);
#else /* MPI-2 safe call */
    MPI_Win temp_win;
    _wrap_py_return_val = MPI_Win_allocate(*size, *disp_unit, MPI_Info_f2c(*info), MPI_Comm_f2c(*comm), BufferC2F((void*)baseptr), &temp_win);
    *win = MPI_Win_c2f(temp_win);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_ALLOCATE(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate_(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate__(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

/* ================= End Wrappers for MPI_Win_allocate ================= */


/* ================== C Wrappers for MPI_Win_allocate_shared ================== */
_EXTERN_C_ int PMPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_allocate_shared at base level - Location = %px",&(MPI_Win_allocate_shared));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_allocate_shared_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_allocate_shared =============== */
static void MPI_Win_allocate_shared_fortran_wrapper(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_allocate_shared(*size, *disp_unit, (MPI_Info)(*info), (MPI_Comm)(*comm), BufferC2F((void*)baseptr), (MPI_Win*)win);
#else /* MPI-2 safe call */
    MPI_Win temp_win;
    _wrap_py_return_val = MPI_Win_allocate_shared(*size, *disp_unit, MPI_Info_f2c(*info), MPI_Comm_f2c(*comm), BufferC2F((void*)baseptr), &temp_win);
    *win = MPI_Win_c2f(temp_win);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_ALLOCATE_SHARED(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_shared_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate_shared(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_shared_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate_shared_(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_shared_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

_EXTERN_C_ void mpi_win_allocate_shared__(MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *baseptr, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_allocate_shared_fortran_wrapper(size, disp_unit, info, comm, baseptr, win, ierr);
}

/* ================= End Wrappers for MPI_Win_allocate_shared ================= */


/* ================== C Wrappers for MPI_Win_attach ================== */
_EXTERN_C_ int PMPI_Win_attach(MPI_Win win, void *base, MPI_Aint size);
_EXTERN_C_ int MPI_Win_attach(MPI_Win win, void *base, MPI_Aint size) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_attach at base level - Location = %px",&(MPI_Win_attach));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_attach_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_attach(win, base, size);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_attach(win, base, size);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_attach(win, base, size);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_attach =============== */
static void MPI_Win_attach_fortran_wrapper(MPI_Fint *win, MPI_Fint *base, MPI_Aint *size, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_attach((MPI_Win)(*win), BufferC2F((void*)base), *size);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_attach(MPI_Win_f2c(*win), BufferC2F((void*)base), *size);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_ATTACH(MPI_Fint *win, MPI_Fint *base, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Win_attach_fortran_wrapper(win, base, size, ierr);
}

_EXTERN_C_ void mpi_win_attach(MPI_Fint *win, MPI_Fint *base, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Win_attach_fortran_wrapper(win, base, size, ierr);
}

_EXTERN_C_ void mpi_win_attach_(MPI_Fint *win, MPI_Fint *base, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Win_attach_fortran_wrapper(win, base, size, ierr);
}

_EXTERN_C_ void mpi_win_attach__(MPI_Fint *win, MPI_Fint *base, MPI_Aint *size, MPI_Fint *ierr) { 
    MPI_Win_attach_fortran_wrapper(win, base, size, ierr);
}

/* ================= End Wrappers for MPI_Win_attach ================= */


/* ================== C Wrappers for MPI_Win_call_errhandler ================== */
_EXTERN_C_ int PMPI_Win_call_errhandler(MPI_Win win, int errorcode);
_EXTERN_C_ int MPI_Win_call_errhandler(MPI_Win win, int errorcode) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_call_errhandler at base level - Location = %px",&(MPI_Win_call_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_call_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_call_errhandler(win, errorcode);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_call_errhandler(win, errorcode);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_call_errhandler(win, errorcode);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_call_errhandler =============== */
static void MPI_Win_call_errhandler_fortran_wrapper(MPI_Fint *win, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_call_errhandler((MPI_Win)(*win), *errorcode);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_call_errhandler(MPI_Win_f2c(*win), *errorcode);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_CALL_ERRHANDLER(MPI_Fint *win, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Win_call_errhandler_fortran_wrapper(win, errorcode, ierr);
}

_EXTERN_C_ void mpi_win_call_errhandler(MPI_Fint *win, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Win_call_errhandler_fortran_wrapper(win, errorcode, ierr);
}

_EXTERN_C_ void mpi_win_call_errhandler_(MPI_Fint *win, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Win_call_errhandler_fortran_wrapper(win, errorcode, ierr);
}

_EXTERN_C_ void mpi_win_call_errhandler__(MPI_Fint *win, MPI_Fint *errorcode, MPI_Fint *ierr) { 
    MPI_Win_call_errhandler_fortran_wrapper(win, errorcode, ierr);
}

/* ================= End Wrappers for MPI_Win_call_errhandler ================= */


/* ================== C Wrappers for MPI_Win_complete ================== */
_EXTERN_C_ int PMPI_Win_complete(MPI_Win win);
_EXTERN_C_ int MPI_Win_complete(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_complete at base level - Location = %px",&(MPI_Win_complete));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_complete_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_complete(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_complete(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_complete(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_complete =============== */
static void MPI_Win_complete_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_complete((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_complete(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_COMPLETE(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_complete_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_complete(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_complete_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_complete_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_complete_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_complete__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_complete_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_complete ================= */


/* ================== C Wrappers for MPI_Win_create ================== */
_EXTERN_C_ int PMPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_create at base level - Location = %px",&(MPI_Win_create));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_create_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_create(base, size, disp_unit, info, comm, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_create(base, size, disp_unit, info, comm, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_create(base, size, disp_unit, info, comm, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_create =============== */
static void MPI_Win_create_fortran_wrapper(MPI_Fint *base, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_create(BufferC2F((void*)base), *size, *disp_unit, (MPI_Info)(*info), (MPI_Comm)(*comm), (MPI_Win*)win);
#else /* MPI-2 safe call */
    MPI_Win temp_win;
    _wrap_py_return_val = MPI_Win_create(BufferC2F((void*)base), *size, *disp_unit, MPI_Info_f2c(*info), MPI_Comm_f2c(*comm), &temp_win);
    *win = MPI_Win_c2f(temp_win);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_CREATE(MPI_Fint *base, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_fortran_wrapper(base, size, disp_unit, info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create(MPI_Fint *base, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_fortran_wrapper(base, size, disp_unit, info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create_(MPI_Fint *base, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_fortran_wrapper(base, size, disp_unit, info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create__(MPI_Fint *base, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_fortran_wrapper(base, size, disp_unit, info, comm, win, ierr);
}

/* ================= End Wrappers for MPI_Win_create ================= */


/* ================== C Wrappers for MPI_Win_create_dynamic ================== */
_EXTERN_C_ int PMPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_create_dynamic at base level - Location = %px",&(MPI_Win_create_dynamic));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_create_dynamic_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_create_dynamic(info, comm, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_create_dynamic(info, comm, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_create_dynamic(info, comm, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_create_dynamic =============== */
static void MPI_Win_create_dynamic_fortran_wrapper(MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_create_dynamic((MPI_Info)(*info), (MPI_Comm)(*comm), (MPI_Win*)win);
#else /* MPI-2 safe call */
    MPI_Win temp_win;
    _wrap_py_return_val = MPI_Win_create_dynamic(MPI_Info_f2c(*info), MPI_Comm_f2c(*comm), &temp_win);
    *win = MPI_Win_c2f(temp_win);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_CREATE_DYNAMIC(MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_dynamic_fortran_wrapper(info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create_dynamic(MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_dynamic_fortran_wrapper(info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create_dynamic_(MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_dynamic_fortran_wrapper(info, comm, win, ierr);
}

_EXTERN_C_ void mpi_win_create_dynamic__(MPI_Fint *info, MPI_Fint *comm, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_create_dynamic_fortran_wrapper(info, comm, win, ierr);
}

/* ================= End Wrappers for MPI_Win_create_dynamic ================= */


/* ================== C Wrappers for MPI_Win_create_errhandler ================== */
_EXTERN_C_ int PMPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_create_errhandler at base level - Location = %px",&(MPI_Win_create_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_create_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_create_errhandler(function, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_create_errhandler =============== */
static void MPI_Win_create_errhandler_fortran_wrapper(MPI_Win_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_create_errhandler(((MPI_Win_errhandler_function*)function), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_Win_create_errhandler(((MPI_Win_errhandler_function*)function), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_CREATE_ERRHANDLER(MPI_Win_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_create_errhandler_(MPI_Win_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_create_errhandler__(MPI_Win_errhandler_function *function, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_create_errhandler_fortran_wrapper(function, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Win_create_errhandler ================= */


/* ================== C Wrappers for MPI_Win_create_keyval ================== */
_EXTERN_C_ int PMPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);
_EXTERN_C_ int MPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_create_keyval at base level - Location = %px",&(MPI_Win_create_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_create_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_create_keyval =============== */
static void MPI_Win_create_keyval_fortran_wrapper(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Win_create_keyval(((MPI_Win_copy_attr_function*)win_copy_attr_fn), ((MPI_Win_delete_attr_function*)win_delete_attr_fn), ((int*)win_keyval), BufferC2F((void*)extra_state));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_CREATE_KEYVAL(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Win_create_keyval_fortran_wrapper(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Win_create_keyval_fortran_wrapper(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_win_create_keyval_(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Win_create_keyval_fortran_wrapper(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr);
}

_EXTERN_C_ void mpi_win_create_keyval__(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Fint *extra_state, MPI_Fint *ierr) { 
    MPI_Win_create_keyval_fortran_wrapper(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr);
}

/* ================= End Wrappers for MPI_Win_create_keyval ================= */


/* ================== C Wrappers for MPI_Win_delete_attr ================== */
_EXTERN_C_ int PMPI_Win_delete_attr(MPI_Win win, int win_keyval);
_EXTERN_C_ int MPI_Win_delete_attr(MPI_Win win, int win_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_delete_attr at base level - Location = %px",&(MPI_Win_delete_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_delete_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_delete_attr(win, win_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_delete_attr(win, win_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_delete_attr(win, win_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_delete_attr =============== */
static void MPI_Win_delete_attr_fortran_wrapper(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_delete_attr((MPI_Win)(*win), *win_keyval);
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_delete_attr(MPI_Win_f2c(*win), *win_keyval);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_DELETE_ATTR(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_delete_attr_fortran_wrapper(win, win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_delete_attr(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_delete_attr_fortran_wrapper(win, win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_delete_attr_(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_delete_attr_fortran_wrapper(win, win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_delete_attr__(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_delete_attr_fortran_wrapper(win, win_keyval, ierr);
}

/* ================= End Wrappers for MPI_Win_delete_attr ================= */


/* ================== C Wrappers for MPI_Win_detach ================== */
_EXTERN_C_ int PMPI_Win_detach(MPI_Win win, const void *base);
_EXTERN_C_ int MPI_Win_detach(MPI_Win win, const void *base) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_detach at base level - Location = %px",&(MPI_Win_detach));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_detach_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_detach(win, base);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_detach(win, base);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_detach(win, base);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_detach =============== */
static void MPI_Win_detach_fortran_wrapper(MPI_Fint *win, MPI_Fint *base, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_detach((MPI_Win)(*win), ((const void*)base));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_detach(MPI_Win_f2c(*win), ((const void*)base));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_DETACH(MPI_Fint *win, MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Win_detach_fortran_wrapper(win, base, ierr);
}

_EXTERN_C_ void mpi_win_detach(MPI_Fint *win, MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Win_detach_fortran_wrapper(win, base, ierr);
}

_EXTERN_C_ void mpi_win_detach_(MPI_Fint *win, MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Win_detach_fortran_wrapper(win, base, ierr);
}

_EXTERN_C_ void mpi_win_detach__(MPI_Fint *win, MPI_Fint *base, MPI_Fint *ierr) { 
    MPI_Win_detach_fortran_wrapper(win, base, ierr);
}

/* ================= End Wrappers for MPI_Win_detach ================= */


/* ================== C Wrappers for MPI_Win_fence ================== */
_EXTERN_C_ int PMPI_Win_fence(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_fence(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_fence at base level - Location = %px",&(MPI_Win_fence));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_fence_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_fence(assert, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_fence(assert, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_fence(assert, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_fence =============== */
static void MPI_Win_fence_fortran_wrapper(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_fence(*assert, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_fence(*assert, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FENCE(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_fence_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_fence(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_fence_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_fence_(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_fence_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_fence__(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_fence_fortran_wrapper(assert, win, ierr);
}

/* ================= End Wrappers for MPI_Win_fence ================= */


/* ================== C Wrappers for MPI_Win_flush ================== */
_EXTERN_C_ int PMPI_Win_flush(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_flush at base level - Location = %px",&(MPI_Win_flush));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_flush_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_flush(rank, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_flush(rank, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_flush(rank, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_flush =============== */
static void MPI_Win_flush_fortran_wrapper(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_flush(*rank, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_flush(*rank, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FLUSH(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush_(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush__(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_fortran_wrapper(rank, win, ierr);
}

/* ================= End Wrappers for MPI_Win_flush ================= */


/* ================== C Wrappers for MPI_Win_flush_all ================== */
_EXTERN_C_ int PMPI_Win_flush_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_flush_all at base level - Location = %px",&(MPI_Win_flush_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_flush_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_flush_all(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_flush_all(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_flush_all(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_flush_all =============== */
static void MPI_Win_flush_all_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_flush_all((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_flush_all(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FLUSH_ALL(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_all(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_all_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_all__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_all_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_flush_all ================= */


/* ================== C Wrappers for MPI_Win_flush_local ================== */
_EXTERN_C_ int PMPI_Win_flush_local(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_flush_local at base level - Location = %px",&(MPI_Win_flush_local));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_flush_local_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_flush_local(rank, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_flush_local(rank, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_flush_local(rank, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_flush_local =============== */
static void MPI_Win_flush_local_fortran_wrapper(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_flush_local(*rank, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_flush_local(*rank, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FLUSH_LOCAL(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local_(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local__(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_fortran_wrapper(rank, win, ierr);
}

/* ================= End Wrappers for MPI_Win_flush_local ================= */


/* ================== C Wrappers for MPI_Win_flush_local_all ================== */
_EXTERN_C_ int PMPI_Win_flush_local_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_flush_local_all at base level - Location = %px",&(MPI_Win_flush_local_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_flush_local_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_flush_local_all(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_flush_local_all(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_flush_local_all(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_flush_local_all =============== */
static void MPI_Win_flush_local_all_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_flush_local_all((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_flush_local_all(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FLUSH_LOCAL_ALL(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local_all(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local_all_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_flush_local_all__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_flush_local_all_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_flush_local_all ================= */


/* ================== C Wrappers for MPI_Win_free ================== */
_EXTERN_C_ int PMPI_Win_free(MPI_Win *win);
_EXTERN_C_ int MPI_Win_free(MPI_Win *win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_free at base level - Location = %px",&(MPI_Win_free));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_free_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_free(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_free(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_free(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_free =============== */
static void MPI_Win_free_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_free((MPI_Win*)win);
#else /* MPI-2 safe call */
    MPI_Win temp_win;
    temp_win = MPI_Win_f2c(*win);
    _wrap_py_return_val = MPI_Win_free(&temp_win);
    *win = MPI_Win_c2f(temp_win);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FREE(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_free_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_free(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_free_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_free_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_free_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_free__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_free_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_free ================= */


/* ================== C Wrappers for MPI_Win_free_keyval ================== */
_EXTERN_C_ int PMPI_Win_free_keyval(int *win_keyval);
_EXTERN_C_ int MPI_Win_free_keyval(int *win_keyval) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_free_keyval at base level - Location = %px",&(MPI_Win_free_keyval));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_free_keyval_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_free_keyval(win_keyval);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_free_keyval(win_keyval);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_free_keyval(win_keyval);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_free_keyval =============== */
static void MPI_Win_free_keyval_fortran_wrapper(MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val = MPI_Win_free_keyval(((int*)win_keyval));
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_FREE_KEYVAL(MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_free_keyval_fortran_wrapper(win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_free_keyval(MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_free_keyval_fortran_wrapper(win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_free_keyval_(MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_free_keyval_fortran_wrapper(win_keyval, ierr);
}

_EXTERN_C_ void mpi_win_free_keyval__(MPI_Fint *win_keyval, MPI_Fint *ierr) { 
    MPI_Win_free_keyval_fortran_wrapper(win_keyval, ierr);
}

/* ================= End Wrappers for MPI_Win_free_keyval ================= */


/* ================== C Wrappers for MPI_Win_get_attr ================== */
_EXTERN_C_ int PMPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_get_attr at base level - Location = %px",&(MPI_Win_get_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_get_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_get_attr =============== */
static void MPI_Win_get_attr_fortran_wrapper(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_get_attr((MPI_Win)(*win), *win_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_get_attr(MPI_Win_f2c(*win), *win_keyval, BufferC2F((void*)attribute_val), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_GET_ATTR(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_get_attr_fortran_wrapper(win, win_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_win_get_attr(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_get_attr_fortran_wrapper(win, win_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_win_get_attr_(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_get_attr_fortran_wrapper(win, win_keyval, attribute_val, flag, ierr);
}

_EXTERN_C_ void mpi_win_get_attr__(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_get_attr_fortran_wrapper(win, win_keyval, attribute_val, flag, ierr);
}

/* ================= End Wrappers for MPI_Win_get_attr ================= */


/* ================== C Wrappers for MPI_Win_get_errhandler ================== */
_EXTERN_C_ int PMPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_get_errhandler at base level - Location = %px",&(MPI_Win_get_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_get_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_get_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_get_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_get_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_get_errhandler =============== */
static void MPI_Win_get_errhandler_fortran_wrapper(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_get_errhandler((MPI_Win)(*win), (MPI_Errhandler*)errhandler);
#else /* MPI-2 safe call */
    MPI_Errhandler temp_errhandler;
    _wrap_py_return_val = MPI_Win_get_errhandler(MPI_Win_f2c(*win), &temp_errhandler);
    *errhandler = MPI_Errhandler_c2f(temp_errhandler);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_GET_ERRHANDLER(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_get_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_get_errhandler(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_get_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_get_errhandler_(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_get_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_get_errhandler__(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_get_errhandler_fortran_wrapper(win, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Win_get_errhandler ================= */


/* ================== C Wrappers for MPI_Win_get_group ================== */
_EXTERN_C_ int PMPI_Win_get_group(MPI_Win win, MPI_Group *group);
_EXTERN_C_ int MPI_Win_get_group(MPI_Win win, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_get_group at base level - Location = %px",&(MPI_Win_get_group));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_get_group_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_get_group(win, group);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_get_group(win, group);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_get_group(win, group);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_get_group =============== */
static void MPI_Win_get_group_fortran_wrapper(MPI_Fint *win, MPI_Fint *group, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_get_group((MPI_Win)(*win), (MPI_Group*)group);
#else /* MPI-2 safe call */
    MPI_Group temp_group;
    _wrap_py_return_val = MPI_Win_get_group(MPI_Win_f2c(*win), &temp_group);
    *group = MPI_Group_c2f(temp_group);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_GET_GROUP(MPI_Fint *win, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Win_get_group_fortran_wrapper(win, group, ierr);
}

_EXTERN_C_ void mpi_win_get_group(MPI_Fint *win, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Win_get_group_fortran_wrapper(win, group, ierr);
}

_EXTERN_C_ void mpi_win_get_group_(MPI_Fint *win, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Win_get_group_fortran_wrapper(win, group, ierr);
}

_EXTERN_C_ void mpi_win_get_group__(MPI_Fint *win, MPI_Fint *group, MPI_Fint *ierr) { 
    MPI_Win_get_group_fortran_wrapper(win, group, ierr);
}

/* ================= End Wrappers for MPI_Win_get_group ================= */


/* ================== C Wrappers for MPI_Win_get_info ================== */
_EXTERN_C_ int PMPI_Win_get_info(MPI_Win win, MPI_Info *info_used);
_EXTERN_C_ int MPI_Win_get_info(MPI_Win win, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_get_info at base level - Location = %px",&(MPI_Win_get_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_get_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_get_info(win, info_used);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_get_info(win, info_used);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_get_info(win, info_used);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_get_info =============== */
static void MPI_Win_get_info_fortran_wrapper(MPI_Fint *win, MPI_Fint *info_used, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_get_info((MPI_Win)(*win), (MPI_Info*)info_used);
#else /* MPI-2 safe call */
    MPI_Info temp_info_used;
    _wrap_py_return_val = MPI_Win_get_info(MPI_Win_f2c(*win), &temp_info_used);
    *info_used = MPI_Info_c2f(temp_info_used);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_GET_INFO(MPI_Fint *win, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Win_get_info_fortran_wrapper(win, info_used, ierr);
}

_EXTERN_C_ void mpi_win_get_info(MPI_Fint *win, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Win_get_info_fortran_wrapper(win, info_used, ierr);
}

_EXTERN_C_ void mpi_win_get_info_(MPI_Fint *win, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Win_get_info_fortran_wrapper(win, info_used, ierr);
}

_EXTERN_C_ void mpi_win_get_info__(MPI_Fint *win, MPI_Fint *info_used, MPI_Fint *ierr) { 
    MPI_Win_get_info_fortran_wrapper(win, info_used, ierr);
}

/* ================= End Wrappers for MPI_Win_get_info ================= */


/* ================== C Wrappers for MPI_Win_get_name ================== */
_EXTERN_C_ int PMPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen);
_EXTERN_C_ int MPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_get_name at base level - Location = %px",&(MPI_Win_get_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_get_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_get_name(win, win_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_get_name(win, win_name, resultlen);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_get_name(win, win_name, resultlen);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_get_name =============== */
static void MPI_Win_get_name_fortran_wrapper(MPI_Fint *win, char *win_name, MPI_Fint *resultlen, MPI_Fint *ierr, int win_name_len) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_get_name((MPI_Win)(*win), win_name, ((int*)resultlen));
#else /* MPI-2 safe call */
    char* temp_win_name;
    temp_win_name = (char*)malloc(sizeof(char) * (win_name_len+1));
    char_p_f2c(win_name,win_name_len,&temp_win_name);
    _wrap_py_return_val = MPI_Win_get_name(MPI_Win_f2c(*win), win_name, ((int*)resultlen));
    char_p_c2f(temp_win_name,win_name,win_name_len);
 if(temp_win_name) free(temp_win_name);
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_GET_NAME(MPI_Fint *win, char *win_name, MPI_Fint *resultlen, MPI_Fint *ierr, int win_name_len) { 
    MPI_Win_get_name_fortran_wrapper(win, win_name, resultlen, ierr, win_name_len);
}

_EXTERN_C_ void mpi_win_get_name(MPI_Fint *win, char *win_name, MPI_Fint *resultlen, MPI_Fint *ierr, int win_name_len) { 
    MPI_Win_get_name_fortran_wrapper(win, win_name, resultlen, ierr, win_name_len);
}

_EXTERN_C_ void mpi_win_get_name_(MPI_Fint *win, char *win_name, MPI_Fint *resultlen, MPI_Fint *ierr, int win_name_len) { 
    MPI_Win_get_name_fortran_wrapper(win, win_name, resultlen, ierr, win_name_len);
}

_EXTERN_C_ void mpi_win_get_name__(MPI_Fint *win, char *win_name, MPI_Fint *resultlen, MPI_Fint *ierr, int win_name_len) { 
    MPI_Win_get_name_fortran_wrapper(win, win_name, resultlen, ierr, win_name_len);
}

/* ================= End Wrappers for MPI_Win_get_name ================= */


/* ================== C Wrappers for MPI_Win_lock ================== */
_EXTERN_C_ int PMPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_lock at base level - Location = %px",&(MPI_Win_lock));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_lock_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_lock(lock_type, rank, assert, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_lock(lock_type, rank, assert, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_lock(lock_type, rank, assert, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_lock =============== */
static void MPI_Win_lock_fortran_wrapper(MPI_Fint *lock_type, MPI_Fint *rank, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_lock(*lock_type, *rank, *assert, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_lock(*lock_type, *rank, *assert, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_LOCK(MPI_Fint *lock_type, MPI_Fint *rank, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_fortran_wrapper(lock_type, rank, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock(MPI_Fint *lock_type, MPI_Fint *rank, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_fortran_wrapper(lock_type, rank, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock_(MPI_Fint *lock_type, MPI_Fint *rank, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_fortran_wrapper(lock_type, rank, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock__(MPI_Fint *lock_type, MPI_Fint *rank, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_fortran_wrapper(lock_type, rank, assert, win, ierr);
}

/* ================= End Wrappers for MPI_Win_lock ================= */


/* ================== C Wrappers for MPI_Win_lock_all ================== */
_EXTERN_C_ int PMPI_Win_lock_all(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock_all(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_lock_all at base level - Location = %px",&(MPI_Win_lock_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_lock_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_lock_all(assert, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_lock_all(assert, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_lock_all(assert, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_lock_all =============== */
static void MPI_Win_lock_all_fortran_wrapper(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_lock_all(*assert, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_lock_all(*assert, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_LOCK_ALL(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_all_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock_all(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_all_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock_all_(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_all_fortran_wrapper(assert, win, ierr);
}

_EXTERN_C_ void mpi_win_lock_all__(MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_lock_all_fortran_wrapper(assert, win, ierr);
}

/* ================= End Wrappers for MPI_Win_lock_all ================= */


/* ================== C Wrappers for MPI_Win_post ================== */
_EXTERN_C_ int PMPI_Win_post(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_post(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_post at base level - Location = %px",&(MPI_Win_post));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_post_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_post(group, assert, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_post(group, assert, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_post(group, assert, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_post =============== */
static void MPI_Win_post_fortran_wrapper(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_post((MPI_Group)(*group), *assert, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_post(MPI_Group_f2c(*group), *assert, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_POST(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_post_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_post(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_post_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_post_(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_post_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_post__(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_post_fortran_wrapper(group, assert, win, ierr);
}

/* ================= End Wrappers for MPI_Win_post ================= */


/* ================== C Wrappers for MPI_Win_set_attr ================== */
_EXTERN_C_ int PMPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_set_attr at base level - Location = %px",&(MPI_Win_set_attr));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_set_attr_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_set_attr(win, win_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_set_attr(win, win_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_set_attr(win, win_keyval, attribute_val);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_set_attr =============== */
static void MPI_Win_set_attr_fortran_wrapper(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_set_attr((MPI_Win)(*win), *win_keyval, BufferC2F((void*)attribute_val));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_set_attr(MPI_Win_f2c(*win), *win_keyval, BufferC2F((void*)attribute_val));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SET_ATTR(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Win_set_attr_fortran_wrapper(win, win_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_win_set_attr(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Win_set_attr_fortran_wrapper(win, win_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_win_set_attr_(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Win_set_attr_fortran_wrapper(win, win_keyval, attribute_val, ierr);
}

_EXTERN_C_ void mpi_win_set_attr__(MPI_Fint *win, MPI_Fint *win_keyval, MPI_Fint *attribute_val, MPI_Fint *ierr) { 
    MPI_Win_set_attr_fortran_wrapper(win, win_keyval, attribute_val, ierr);
}

/* ================= End Wrappers for MPI_Win_set_attr ================= */


/* ================== C Wrappers for MPI_Win_set_errhandler ================== */
_EXTERN_C_ int PMPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_set_errhandler at base level - Location = %px",&(MPI_Win_set_errhandler));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_set_errhandler_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_set_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_set_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_set_errhandler(win, errhandler);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_set_errhandler =============== */
static void MPI_Win_set_errhandler_fortran_wrapper(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_set_errhandler((MPI_Win)(*win), (MPI_Errhandler)(*errhandler));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_set_errhandler(MPI_Win_f2c(*win), MPI_Errhandler_f2c(*errhandler));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SET_ERRHANDLER(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_set_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_set_errhandler(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_set_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_set_errhandler_(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_set_errhandler_fortran_wrapper(win, errhandler, ierr);
}

_EXTERN_C_ void mpi_win_set_errhandler__(MPI_Fint *win, MPI_Fint *errhandler, MPI_Fint *ierr) { 
    MPI_Win_set_errhandler_fortran_wrapper(win, errhandler, ierr);
}

/* ================= End Wrappers for MPI_Win_set_errhandler ================= */


/* ================== C Wrappers for MPI_Win_set_info ================== */
_EXTERN_C_ int PMPI_Win_set_info(MPI_Win win, MPI_Info info);
_EXTERN_C_ int MPI_Win_set_info(MPI_Win win, MPI_Info info) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_set_info at base level - Location = %px",&(MPI_Win_set_info));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_set_info_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_set_info(win, info);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_set_info(win, info);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_set_info(win, info);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_set_info =============== */
static void MPI_Win_set_info_fortran_wrapper(MPI_Fint *win, MPI_Fint *info, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_set_info((MPI_Win)(*win), (MPI_Info)(*info));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_set_info(MPI_Win_f2c(*win), MPI_Info_f2c(*info));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SET_INFO(MPI_Fint *win, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Win_set_info_fortran_wrapper(win, info, ierr);
}

_EXTERN_C_ void mpi_win_set_info(MPI_Fint *win, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Win_set_info_fortran_wrapper(win, info, ierr);
}

_EXTERN_C_ void mpi_win_set_info_(MPI_Fint *win, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Win_set_info_fortran_wrapper(win, info, ierr);
}

_EXTERN_C_ void mpi_win_set_info__(MPI_Fint *win, MPI_Fint *info, MPI_Fint *ierr) { 
    MPI_Win_set_info_fortran_wrapper(win, info, ierr);
}

/* ================= End Wrappers for MPI_Win_set_info ================= */


/* ================== C Wrappers for MPI_Win_set_name ================== */
_EXTERN_C_ int PMPI_Win_set_name(MPI_Win win, const char *win_name);
_EXTERN_C_ int MPI_Win_set_name(MPI_Win win, const char *win_name) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_set_name at base level - Location = %px",&(MPI_Win_set_name));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_set_name_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_set_name(win, win_name);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_set_name(win, win_name);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_set_name(win, win_name);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_set_name =============== */
static void MPI_Win_set_name_fortran_wrapper(MPI_Fint *win, MPI_Fint *win_name, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_set_name((MPI_Win)(*win), ((const char*)win_name));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_set_name(MPI_Win_f2c(*win), ((const char*)win_name));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SET_NAME(MPI_Fint *win, MPI_Fint *win_name, MPI_Fint *ierr) { 
    MPI_Win_set_name_fortran_wrapper(win, win_name, ierr);
}

_EXTERN_C_ void mpi_win_set_name(MPI_Fint *win, MPI_Fint *win_name, MPI_Fint *ierr) { 
    MPI_Win_set_name_fortran_wrapper(win, win_name, ierr);
}

_EXTERN_C_ void mpi_win_set_name_(MPI_Fint *win, MPI_Fint *win_name, MPI_Fint *ierr) { 
    MPI_Win_set_name_fortran_wrapper(win, win_name, ierr);
}

_EXTERN_C_ void mpi_win_set_name__(MPI_Fint *win, MPI_Fint *win_name, MPI_Fint *ierr) { 
    MPI_Win_set_name_fortran_wrapper(win, win_name, ierr);
}

/* ================= End Wrappers for MPI_Win_set_name ================= */


/* ================== C Wrappers for MPI_Win_shared_query ================== */
_EXTERN_C_ int PMPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);
_EXTERN_C_ int MPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_shared_query at base level - Location = %px",&(MPI_Win_shared_query));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_shared_query_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_shared_query =============== */
static void MPI_Win_shared_query_fortran_wrapper(MPI_Fint *win, MPI_Fint *rank, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_shared_query((MPI_Win)(*win), *rank, ((MPI_Aint*)size), ((int*)disp_unit), BufferC2F((void*)baseptr));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_shared_query(MPI_Win_f2c(*win), *rank, ((MPI_Aint*)size), ((int*)disp_unit), BufferC2F((void*)baseptr));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SHARED_QUERY(MPI_Fint *win, MPI_Fint *rank, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Win_shared_query_fortran_wrapper(win, rank, size, disp_unit, baseptr, ierr);
}

_EXTERN_C_ void mpi_win_shared_query(MPI_Fint *win, MPI_Fint *rank, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Win_shared_query_fortran_wrapper(win, rank, size, disp_unit, baseptr, ierr);
}

_EXTERN_C_ void mpi_win_shared_query_(MPI_Fint *win, MPI_Fint *rank, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Win_shared_query_fortran_wrapper(win, rank, size, disp_unit, baseptr, ierr);
}

_EXTERN_C_ void mpi_win_shared_query__(MPI_Fint *win, MPI_Fint *rank, MPI_Aint *size, MPI_Fint *disp_unit, MPI_Fint *baseptr, MPI_Fint *ierr) { 
    MPI_Win_shared_query_fortran_wrapper(win, rank, size, disp_unit, baseptr, ierr);
}

/* ================= End Wrappers for MPI_Win_shared_query ================= */


/* ================== C Wrappers for MPI_Win_start ================== */
_EXTERN_C_ int PMPI_Win_start(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_start(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_start at base level - Location = %px",&(MPI_Win_start));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_start_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_start(group, assert, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_start(group, assert, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_start(group, assert, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_start =============== */
static void MPI_Win_start_fortran_wrapper(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_start((MPI_Group)(*group), *assert, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_start(MPI_Group_f2c(*group), *assert, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_START(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_start_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_start(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_start_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_start_(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_start_fortran_wrapper(group, assert, win, ierr);
}

_EXTERN_C_ void mpi_win_start__(MPI_Fint *group, MPI_Fint *assert, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_start_fortran_wrapper(group, assert, win, ierr);
}

/* ================= End Wrappers for MPI_Win_start ================= */


/* ================== C Wrappers for MPI_Win_sync ================== */
_EXTERN_C_ int PMPI_Win_sync(MPI_Win win);
_EXTERN_C_ int MPI_Win_sync(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_sync at base level - Location = %px",&(MPI_Win_sync));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_sync_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_sync(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_sync(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_sync(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_sync =============== */
static void MPI_Win_sync_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_sync((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_sync(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_SYNC(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_sync_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_sync(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_sync_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_sync_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_sync_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_sync__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_sync_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_sync ================= */


/* ================== C Wrappers for MPI_Win_test ================== */
_EXTERN_C_ int PMPI_Win_test(MPI_Win win, int *flag);
_EXTERN_C_ int MPI_Win_test(MPI_Win win, int *flag) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_test at base level - Location = %px",&(MPI_Win_test));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_test_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_test(win, flag);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_test(win, flag);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_test(win, flag);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_test =============== */
static void MPI_Win_test_fortran_wrapper(MPI_Fint *win, MPI_Fint *flag, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_test((MPI_Win)(*win), ((int*)flag));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_test(MPI_Win_f2c(*win), ((int*)flag));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_TEST(MPI_Fint *win, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_test_fortran_wrapper(win, flag, ierr);
}

_EXTERN_C_ void mpi_win_test(MPI_Fint *win, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_test_fortran_wrapper(win, flag, ierr);
}

_EXTERN_C_ void mpi_win_test_(MPI_Fint *win, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_test_fortran_wrapper(win, flag, ierr);
}

_EXTERN_C_ void mpi_win_test__(MPI_Fint *win, MPI_Fint *flag, MPI_Fint *ierr) { 
    MPI_Win_test_fortran_wrapper(win, flag, ierr);
}

/* ================= End Wrappers for MPI_Win_test ================= */


/* ================== C Wrappers for MPI_Win_unlock ================== */
_EXTERN_C_ int PMPI_Win_unlock(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_unlock at base level - Location = %px",&(MPI_Win_unlock));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_unlock_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_unlock(rank, win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_unlock(rank, win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_unlock(rank, win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_unlock =============== */
static void MPI_Win_unlock_fortran_wrapper(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_unlock(*rank, (MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_unlock(*rank, MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_UNLOCK(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_unlock(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_unlock_(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_fortran_wrapper(rank, win, ierr);
}

_EXTERN_C_ void mpi_win_unlock__(MPI_Fint *rank, MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_fortran_wrapper(rank, win, ierr);
}

/* ================= End Wrappers for MPI_Win_unlock ================= */


/* ================== C Wrappers for MPI_Win_unlock_all ================== */
_EXTERN_C_ int PMPI_Win_unlock_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_unlock_all at base level - Location = %px",&(MPI_Win_unlock_all));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_unlock_all_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_unlock_all(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_unlock_all(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_unlock_all(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_unlock_all =============== */
static void MPI_Win_unlock_all_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_unlock_all((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_unlock_all(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_UNLOCK_ALL(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_unlock_all(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_unlock_all_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_all_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_unlock_all__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_unlock_all_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_unlock_all ================= */


/* ================== C Wrappers for MPI_Win_wait ================== */
_EXTERN_C_ int PMPI_Win_wait(MPI_Win win);
_EXTERN_C_ int MPI_Win_wait(MPI_Win win) { 
    int _wrap_py_return_val = 0;

{
  WRAP_MPI_CALL_PREFIX
  DBGPRINT3("Entering Old MPI_Win_wait at base level - Location = %px",&(MPI_Win_wait));
  WRAP_MPI_CALL_POSTFIX

  if (NOT_ACTIVATED(MPI_Win_wait_ID))
  { // in case we have no registered function of this type and we call PMPI directly, a subsequent call from the mpi library starts at level1
    int start_level = pnmpi_level;
    pnmpi_level = pnmpi_max_level;
    WRAP_MPI_CALL_PREFIX
    int ret = PMPI_Win_wait(win);
    WRAP_MPI_CALL_POSTFIX
    pnmpi_level = start_level;
    return ret;
  }
  else if (get_pnmpi_mpi_level() > 0)
  {
    WRAP_MPI_CALL_PREFIX
    return PMPI_Win_wait(win);
    WRAP_MPI_CALL_POSTFIX
  }
  else
    {
    /* Store the return address to the application, so modules may check the
     * origin of this MPI call. */
    pnmpi_return_address_set();

    WRAP_MPI_CALL_PREFIX
    _wrap_py_return_val=Internal_XMPI_Win_wait(win);
    WRAP_MPI_CALL_POSTFIX

    /* Reset the return address to the default one. */
    pnmpi_return_address_reset();
  }
}
    return _wrap_py_return_val;
}

/* =============== Fortran Wrappers for MPI_Win_wait =============== */
static void MPI_Win_wait_fortran_wrapper(MPI_Fint *win, MPI_Fint *ierr) { 
    int _wrap_py_return_val = 0;
    WRAP_MPI_CALL_PREFIX
#if (!defined(MPICH_HAS_C2F) && defined(MPICH_NAME) && (MPICH_NAME == 1)) /* MPICH test */
    _wrap_py_return_val = MPI_Win_wait((MPI_Win)(*win));
#else /* MPI-2 safe call */
    _wrap_py_return_val = MPI_Win_wait(MPI_Win_f2c(*win));
#endif /* MPICH test */
    WRAP_MPI_CALL_POSTFIX
    *ierr = _wrap_py_return_val;
}

_EXTERN_C_ void MPI_WIN_WAIT(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_wait_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_wait(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_wait_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_wait_(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_wait_fortran_wrapper(win, ierr);
}

_EXTERN_C_ void mpi_win_wait__(MPI_Fint *win, MPI_Fint *ierr) { 
    MPI_Win_wait_fortran_wrapper(win, ierr);
}

/* ================= End Wrappers for MPI_Win_wait ================= */






int XMPI_Abort(MPI_Comm comm, int errorcode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Abort(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Add_error_class(int *errorclass)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_class(errorclass);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Add_error_code(int errorclass, int *errorcode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_code(errorclass, errorcode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Add_error_string(int errorcode, const char *string)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_string(errorcode, string);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Address(void *location, MPI_Aint *address)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Address(location, address);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alloc_mem(size, info, baseptr);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Attr_delete(MPI_Comm comm, int keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_delete(comm, keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_get(comm, keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_put(comm, keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Barrier(MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Barrier(comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bcast(buffer, count, datatype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bsend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Buffer_attach(void *buffer, int size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Buffer_attach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Buffer_detach(void *buffer, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Buffer_detach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cancel(MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cancel(request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_coords(comm, rank, maxdims, coords);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_get(comm, maxdims, dims, periods, coords);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_map(comm, ndims, dims, periods, newrank);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_rank(comm, coords, rank);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_sub(comm, remain_dims, new_comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Cartdim_get(MPI_Comm comm, int *ndims)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cartdim_get(comm, ndims);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Close_port(const char *port_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Close_port(port_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_accept(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_call_errhandler(MPI_Comm comm, int errorcode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_call_errhandler(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_compare(comm1, comm2, result);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_connect(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create(comm, group, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_group(comm, group, tag, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_delete_attr(comm, comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_disconnect(MPI_Comm *comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_disconnect(comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_dup(comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_dup_with_info(comm, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_free(MPI_Comm *comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_free(comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_free_keyval(int *comm_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_free_keyval(comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_errhandler(comm, erhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_info(comm, info_used);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_name(comm, comm_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_get_parent(MPI_Comm *parent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_parent(parent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_group(MPI_Comm comm, MPI_Group *group)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_idup(comm, newcomm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_join(int fd, MPI_Comm *intercomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_join(fd, intercomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_rank(MPI_Comm comm, int *rank)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_rank(comm, rank);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_remote_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_remote_size(MPI_Comm comm, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_remote_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_errhandler(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_set_info(MPI_Comm comm, MPI_Info info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_info(comm, info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_set_name(MPI_Comm comm, const char *comm_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_name(comm, comm_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_size(MPI_Comm comm, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_split(comm, color, key, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_split_type(comm, split_type, key, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Comm_test_inter(MPI_Comm comm, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_test_inter(comm, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Dims_create(int nnodes, int ndims, int dims[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dims_create(nnodes, ndims, dims);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_create(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Errhandler_free(MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_free(errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_get(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_set(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Error_class(int errorcode, int *errorclass)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Error_class(errorcode, errorclass);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Error_string(int errorcode, char *string, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Error_string(errorcode, string, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_call_errhandler(MPI_File fh, int errorcode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_call_errhandler(fh, errorcode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_close(MPI_File *fh)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_close(fh);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_delete(const char *filename, MPI_Info info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_delete(filename, info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_amode(MPI_File fh, int *amode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_amode(fh, amode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_atomicity(MPI_File fh, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_byte_offset(fh, offset, disp);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_group(MPI_File fh, MPI_Group *group)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_group(fh, group);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_info(MPI_File fh, MPI_Info *info_used)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_info(fh, info_used);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_position(MPI_File fh, MPI_Offset *offset)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_position(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_position_shared(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_size(MPI_File fh, MPI_Offset *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_type_extent(fh, datatype, extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_view(fh, disp, etype, filetype, datarep);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_open(comm, filename, amode, info, fh);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_preallocate(MPI_File fh, MPI_Offset size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_preallocate(fh, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_seek(MPI_File fh, MPI_Offset offset, int whence)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_seek(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_seek_shared(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_set_atomicity(MPI_File fh, int flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_set_info(MPI_File fh, MPI_Info info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_info(fh, info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_set_size(MPI_File fh, MPI_Offset size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_sync(MPI_File fh)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_sync(fh);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Finalize()
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Finalize();
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Finalized(int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Finalized(flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Free_mem(void *base)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Free_mem(base);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_address(const void *location, MPI_Aint *address)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_address(location, address);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_count(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_library_version(char *version, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_library_version(version, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_processor_name(char *name, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_processor_name(name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Get_version(int *version, int *subversion)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_version(version, subversion);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_get(comm, maxindex, maxedges, index, edges);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_map(comm, nnodes, index, edges, newrank);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_neighbors_count(comm, rank, nneighbors);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graphdims_get(comm, nnodes, nedges);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Grequest_complete(MPI_Request request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Grequest_complete(request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_compare(group1, group2, result);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_difference(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_excl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_free(MPI_Group *group)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_free(group);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_incl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_intersection(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_range_excl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_range_incl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_rank(MPI_Group group, int *rank)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_rank(group, rank);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_size(MPI_Group group, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_size(group, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_union(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ibarrier(MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibarrier(comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibcast(buffer, count, datatype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Improbe(source, tag, comm, flag, message, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Imrecv(buf, count, type, message, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_create(MPI_Info *info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_create(info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_delete(MPI_Info info, const char *key)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_delete(info, key);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_dup(MPI_Info info, MPI_Info *newinfo)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_dup(info, newinfo);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_free(MPI_Info *info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_free(info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get(info, key, valuelen, value, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_get_nkeys(MPI_Info info, int *nkeys)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_nkeys(info, nkeys);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_get_nthkey(MPI_Info info, int n, char *key)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_nthkey(info, n, key);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_valuelen(info, key, valuelen, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Info_set(MPI_Info info, const char *key, const char *value)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_set(info, key, value);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Init(int *argc, char ***argv)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Init(argc, argv);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Init_thread(int *argc, char ***argv, int required, int *provided)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Init_thread(argc, argv, required, provided);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Initialized(int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Initialized(flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Intercomm_merge(intercomm, high, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iprobe(source, tag, comm, flag, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Irecv(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Is_thread_main(int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Is_thread_main(flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Isend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Issend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Keyval_free(int *keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Keyval_free(keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Lookup_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Mprobe(source, tag, comm, message, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Mrecv(buf, count, type, message, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Op_commutative(MPI_Op op, int *commute)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_commutative(op, commute);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_create(function, commute, op);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Op_free(MPI_Op *op)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_free(op);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Open_port(MPI_Info info, char *port_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Open_port(info, port_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_external_size(datarep, incount, datatype, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_size(incount, datatype, comm, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Probe(source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Publish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Query_thread(int *provided)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Query_thread(provided);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Recv(buf, count, datatype, source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Request_free(MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Request_free(request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Request_get_status(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Send(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ssend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Start(MPI_Request *request)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Start(request);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Startall(int count, MPI_Request array_of_requests[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Startall(count, array_of_requests);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Status_set_cancelled(MPI_Status *status, int flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Test(MPI_Request *request, int *flag, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Test(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Test_cancelled(const MPI_Status *status, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Test_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testall(count, array_of_requests, flag, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testany(count, array_of_requests, index, flag, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Topo_test(MPI_Comm comm, int *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Topo_test(comm, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_commit(MPI_Datatype *type)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_commit(type);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_contiguous(count, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_complex(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_f90_integer(int r, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_integer(r, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_real(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_resized(oldtype, lb, extent, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_delete_attr(MPI_Datatype type, int type_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_delete_attr(type, type_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_dup(type, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_extent(MPI_Datatype type, MPI_Aint *extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_extent(type, extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_free(MPI_Datatype *type)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_free(type);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_free_keyval(int *type_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_free_keyval(type_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_extent(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_extent_x(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_name(type, type_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_true_extent(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_lb(MPI_Datatype type, MPI_Aint *lb)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_lb(type, lb);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_match_size(int typeclass, int size, MPI_Datatype *type)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_match_size(typeclass, size, type);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_set_attr(type, type_keyval, attr_val);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_set_name(MPI_Datatype type, const char *type_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_set_name(type, type_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_size(MPI_Datatype type, int *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_size(type, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_size_x(MPI_Datatype type, MPI_Count *size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_size_x(type, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_ub(mtype, ub);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpublish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Wait(MPI_Request *request, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wait(request, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitall(count, array_of_requests, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitany(count, array_of_requests, index, status);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_attach(MPI_Win win, void *base, MPI_Aint size)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_attach(win, base, size);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_call_errhandler(MPI_Win win, int errorcode)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_call_errhandler(win, errorcode);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_complete(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_complete(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create(base, size, disp_unit, info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_dynamic(info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_delete_attr(MPI_Win win, int win_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_delete_attr(win, win_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_detach(MPI_Win win, const void *base)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_detach(win, base);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_fence(int assert, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_fence(assert, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_flush(int rank, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush(rank, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_flush_all(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_all(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_flush_local(int rank, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_local(rank, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_flush_local_all(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_local_all(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_free(MPI_Win *win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_free(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_free_keyval(int *win_keyval)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_free_keyval(win_keyval);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_get_group(MPI_Win win, MPI_Group *group)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_group(win, group);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_get_info(MPI_Win win, MPI_Info *info_used)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_info(win, info_used);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_name(win, win_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_lock(lock_type, rank, assert, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_lock_all(int assert, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_lock_all(assert, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_post(MPI_Group group, int assert, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_post(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_attr(win, win_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_set_info(MPI_Win win, MPI_Info info)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_info(win, info);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_set_name(MPI_Win win, const char *win_name)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_name(win, win_name);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_start(MPI_Group group, int assert, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_start(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_sync(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_sync(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_test(MPI_Win win, int *flag)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_test(win, flag);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_unlock(int rank, MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_unlock(rank, win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_unlock_all(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_unlock_all(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

int XMPI_Win_wait(MPI_Win win)
{
  int returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_wait(win);
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

double XMPI_Wtick()
{
  double returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wtick();
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}

double XMPI_Wtime()
{
  double returnVal;
  ++pnmpi_level;
  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wtime();
  WRAP_MPI_CALL_POSTFIX
  --pnmpi_level;
  return returnVal;
}



int XMPI_Abort_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Abort(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Add_error_class_NewStack(PNMPI_modHandle_t stack, int *errorclass)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_class(errorclass);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Add_error_code_NewStack(PNMPI_modHandle_t stack, int errorclass, int *errorcode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_code(errorclass, errorcode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Add_error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, const char *string)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Add_error_string(errorcode, string);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Address_NewStack(PNMPI_modHandle_t stack, void *location, MPI_Aint *address)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Address(location, address);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Alloc_mem_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, MPI_Info info, void *baseptr)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alloc_mem(size, info, baseptr);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Allreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Attr_delete_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_delete(comm, keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Attr_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_get(comm, keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Attr_put_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int keyval, void *attribute_val)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Attr_put(comm, keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Barrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Barrier(comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Bcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bcast(buffer, count, datatype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Bsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bsend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Bsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Buffer_attach_NewStack(PNMPI_modHandle_t stack, void *buffer, int size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Buffer_attach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Buffer_detach_NewStack(PNMPI_modHandle_t stack, void *buffer, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Buffer_detach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cancel_NewStack(PNMPI_modHandle_t stack, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cancel(request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_coords_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxdims, int coords[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_coords(comm, rank, maxdims, coords);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_get(comm, maxdims, dims, periods, coords);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_map(comm, ndims, dims, periods, newrank);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int coords[], int *rank)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_rank(comm, coords, rank);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_shift_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cart_sub_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cart_sub(comm, remain_dims, new_comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Cartdim_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *ndims)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Cartdim_get(comm, ndims);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Close_port_NewStack(PNMPI_modHandle_t stack, const char *port_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Close_port(port_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_accept_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_accept(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int errorcode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_call_errhandler(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_compare_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm1, MPI_Comm comm2, int *result)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_compare(comm1, comm2, result);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_connect_NewStack(PNMPI_modHandle_t stack, const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_connect(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create(comm, group, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_create_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_group(comm, group, tag, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_delete_attr(comm, comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_disconnect_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_disconnect(comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_dup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_dup(comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_dup_with_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_dup_with_info(comm, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_free_NewStack(PNMPI_modHandle_t stack, MPI_Comm *comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_free(comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_free_keyval_NewStack(PNMPI_modHandle_t stack, int *comm_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_free_keyval(comm_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *erhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_errhandler(comm, erhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info *info_used)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_info(comm, info_used);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, char *comm_name, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_name(comm, comm_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_get_parent_NewStack(PNMPI_modHandle_t stack, MPI_Comm *parent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_get_parent(parent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_idup_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_idup(comm, newcomm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_join_NewStack(PNMPI_modHandle_t stack, int fd, MPI_Comm *intercomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_join(fd, intercomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_rank_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *rank)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_rank(comm, rank);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_remote_group_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Group *group)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_remote_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_remote_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_remote_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int comm_keyval, void *attribute_val)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_errhandler(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Info info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_info(comm, info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *comm_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_set_name(comm, comm_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_size_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_split_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int color, int key, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_split(comm, color, key, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_split_type_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_split_type(comm, split_type, key, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Comm_test_inter_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Comm_test_inter(comm, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Compare_and_swap_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Dims_create_NewStack(PNMPI_modHandle_t stack, int nnodes, int ndims, int dims[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dims_create(nnodes, ndims, dims);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Dist_graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Dist_graph_create_adjacent_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Dist_graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Dist_graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Errhandler_create_NewStack(PNMPI_modHandle_t stack, MPI_Handler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_create(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Errhandler_free_NewStack(PNMPI_modHandle_t stack, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_free(errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Errhandler_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_get(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Errhandler_set_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Errhandler errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Errhandler_set(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Error_class_NewStack(PNMPI_modHandle_t stack, int errorcode, int *errorclass)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Error_class(errorcode, errorclass);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Error_string_NewStack(PNMPI_modHandle_t stack, int errorcode, char *string, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Error_string(errorcode, string, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Exscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Fetch_and_op_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int errorcode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_call_errhandler(fh, errorcode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_close_NewStack(PNMPI_modHandle_t stack, MPI_File *fh)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_close(fh);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_delete_NewStack(PNMPI_modHandle_t stack, const char *filename, MPI_Info info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_delete(filename, info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_amode_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *amode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_amode(fh, amode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_byte_offset_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, MPI_Offset *disp)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_byte_offset(fh, offset, disp);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_group_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Group *group)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_group(fh, group);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info *info_used)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_info(fh, info_used);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_position_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_position(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_position_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *offset)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_position_shared(fh, offset);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_type_extent_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_type_extent(fh, datatype, extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_get_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_get_view(fh, disp, etype, filetype, datarep);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iread_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iread_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iread_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iread_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iwrite_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iwrite_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_iwrite_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_iwrite_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_open_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_open(comm, filename, amode, info, fh);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_preallocate_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_preallocate(fh, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_read_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_read_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_seek_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_seek(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_seek_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, int whence)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_seek_shared(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_set_atomicity_NewStack(PNMPI_modHandle_t stack, MPI_File fh, int flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_File file, MPI_Errhandler errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_set_info_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Info info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_info(fh, info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_set_size_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_set_view_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_sync_NewStack(PNMPI_modHandle_t stack, MPI_File fh)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_sync(fh);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_at_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_at_all_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_at_all_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_at_all_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_ordered_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_ordered_begin_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_ordered_end_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_File_write_shared_NewStack(PNMPI_modHandle_t stack, MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_File_write_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Finalize_NewStack(PNMPI_modHandle_t stack)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Finalize();
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Finalized_NewStack(PNMPI_modHandle_t stack, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Finalized(flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Free_mem_NewStack(PNMPI_modHandle_t stack, void *base)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Free_mem(base);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Gather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Gatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_address_NewStack(PNMPI_modHandle_t stack, const void *location, MPI_Aint *address)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_address(location, address);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_count_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_count(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_elements_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, int *count)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_elements_x_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_library_version_NewStack(PNMPI_modHandle_t stack, char *version, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_library_version(version, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_processor_name_NewStack(PNMPI_modHandle_t stack, char *name, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_processor_name(name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Get_version_NewStack(PNMPI_modHandle_t stack, int *version, int *subversion)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Get_version(version, subversion);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graph_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graph_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_get(comm, maxindex, maxedges, index, edges);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graph_map_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_map(comm, nnodes, index, edges, newrank);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graph_neighbors_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int maxneighbors, int neighbors[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graph_neighbors_count_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int rank, int *nneighbors)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graph_neighbors_count(comm, rank, nneighbors);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Graphdims_get_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *nnodes, int *nedges)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Graphdims_get(comm, nnodes, nedges);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Grequest_complete_NewStack(PNMPI_modHandle_t stack, MPI_Request request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Grequest_complete(request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Grequest_start_NewStack(PNMPI_modHandle_t stack, MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_compare_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, int *result)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_compare(group1, group2, result);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_difference_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_difference(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_excl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_free_NewStack(PNMPI_modHandle_t stack, MPI_Group *group)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_free(group);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, const int ranks[], MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_incl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_intersection_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_intersection(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_range_excl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_range_excl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_range_incl_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_range_incl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_rank_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *rank)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_rank(group, rank);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_size_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_size(group, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_translate_ranks_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Group_union_NewStack(PNMPI_modHandle_t stack, MPI_Group group1, MPI_Group group2, MPI_Group *newgroup)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Group_union(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iallgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iallgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iallreduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ialltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ialltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ialltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ibarrier_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibarrier(comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ibcast_NewStack(PNMPI_modHandle_t stack, void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibcast(buffer, count, datatype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ibsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iexscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Igather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Igatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Improbe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Improbe(source, tag, comm, flag, message, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Imrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Imrecv(buf, count, type, message, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ineighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ineighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ineighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ineighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ineighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_create_NewStack(PNMPI_modHandle_t stack, MPI_Info *info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_create(info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_delete_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_delete(info, key);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_dup_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Info *newinfo)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_dup(info, newinfo);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_free_NewStack(PNMPI_modHandle_t stack, MPI_Info *info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_free(info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_get_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int valuelen, char *value, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get(info, key, valuelen, value, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_get_nkeys_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int *nkeys)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_nkeys(info, nkeys);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_get_nthkey_NewStack(PNMPI_modHandle_t stack, MPI_Info info, int n, char *key)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_nthkey(info, n, key);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_get_valuelen_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, int *valuelen, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_get_valuelen(info, key, valuelen, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Info_set_NewStack(PNMPI_modHandle_t stack, MPI_Info info, const char *key, const char *value)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Info_set(info, key, value);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Init_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Init(argc, argv);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Init_thread_NewStack(PNMPI_modHandle_t stack, int *argc, char ***argv, int required, int *provided)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Init_thread(argc, argv, required, provided);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Initialized_NewStack(PNMPI_modHandle_t stack, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Initialized(flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Intercomm_create_NewStack(PNMPI_modHandle_t stack, MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Intercomm_merge_NewStack(PNMPI_modHandle_t stack, MPI_Comm intercomm, int high, MPI_Comm *newintercomm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Intercomm_merge(intercomm, high, newintercomm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iprobe(source, tag, comm, flag, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Irecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Irecv(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ireduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ireduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ireduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Irsend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Is_thread_main_NewStack(PNMPI_modHandle_t stack, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Is_thread_main(flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iscan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iscatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Iscatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Isend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Isend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Issend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Issend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Keyval_create_NewStack(PNMPI_modHandle_t stack, MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Keyval_free_NewStack(PNMPI_modHandle_t stack, int *keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Keyval_free(keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Lookup_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, char *port_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Lookup_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Mprobe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Mprobe(source, tag, comm, message, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Mrecv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Mrecv(buf, count, type, message, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Neighbor_allgather_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Neighbor_allgatherv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Neighbor_alltoall_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Neighbor_alltoallv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Neighbor_alltoallw_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Op_commutative_NewStack(PNMPI_modHandle_t stack, MPI_Op op, int *commute)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_commutative(op, commute);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Op_create_NewStack(PNMPI_modHandle_t stack, MPI_User_function *function, int commute, MPI_Op *op)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_create(function, commute, op);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Op_free_NewStack(PNMPI_modHandle_t stack, MPI_Op *op)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Op_free(op);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Open_port_NewStack(PNMPI_modHandle_t stack, MPI_Info info, char *port_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Open_port(info, port_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Pack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Pack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Pack_external_size_NewStack(PNMPI_modHandle_t stack, const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_external_size(datarep, incount, datatype, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Pack_size_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Datatype datatype, MPI_Comm comm, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Pack_size(incount, datatype, comm, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Probe_NewStack(PNMPI_modHandle_t stack, int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Probe(source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Publish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Publish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Put_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Query_thread_NewStack(PNMPI_modHandle_t stack, int *provided)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Query_thread(provided);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Raccumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Recv_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Recv(buf, count, datatype, source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Recv_init_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Reduce_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Reduce_local_NewStack(PNMPI_modHandle_t stack, const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Reduce_scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Reduce_scatter_block_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Register_datarep_NewStack(PNMPI_modHandle_t stack, const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Request_free_NewStack(PNMPI_modHandle_t stack, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Request_free(request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Request_get_status_NewStack(PNMPI_modHandle_t stack, MPI_Request request, int *flag, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Request_get_status(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Rget_NewStack(PNMPI_modHandle_t stack, void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Rget_accumulate_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Rput_NewStack(PNMPI_modHandle_t stack, const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Rsend_NewStack(PNMPI_modHandle_t stack, const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Rsend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Scan_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Scatter_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Scatterv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Send_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Send(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Send_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Sendrecv_NewStack(PNMPI_modHandle_t stack, const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Sendrecv_replace_NewStack(PNMPI_modHandle_t stack, void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ssend_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ssend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Ssend_init_NewStack(PNMPI_modHandle_t stack, const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Start_NewStack(PNMPI_modHandle_t stack, MPI_Request *request)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Start(request);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Startall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Startall(count, array_of_requests);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Status_set_cancelled_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, int flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Status_set_elements_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, int count)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Status_set_elements_x_NewStack(PNMPI_modHandle_t stack, MPI_Status *status, MPI_Datatype datatype, MPI_Count count)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Status_set_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Test_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, int *flag, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Test(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Test_cancelled_NewStack(PNMPI_modHandle_t stack, const MPI_Status *status, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Test_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Testall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testall(count, array_of_requests, flag, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Testany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testany(count, array_of_requests, index, flag, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Testsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Topo_test_NewStack(PNMPI_modHandle_t stack, MPI_Comm comm, int *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Topo_test(comm, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_commit_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_commit(type);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_contiguous_NewStack(PNMPI_modHandle_t stack, int count, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_contiguous(count, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_darray_NewStack(PNMPI_modHandle_t stack, int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_f90_complex_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_complex(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_f90_integer_NewStack(PNMPI_modHandle_t stack, int r, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_integer(r, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_f90_real_NewStack(PNMPI_modHandle_t stack, int p, int r, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_f90_real(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_hindexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_hindexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_indexed_block_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_resized_NewStack(PNMPI_modHandle_t stack, MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_resized(oldtype, lb, extent, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_struct_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_create_subarray_NewStack(PNMPI_modHandle_t stack, int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_delete_attr(type, type_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_dup_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_dup(type, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_extent(type, extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_free_NewStack(PNMPI_modHandle_t stack, MPI_Datatype *type)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_free(type);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_free_keyval_NewStack(PNMPI_modHandle_t stack, int *type_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_free_keyval(type_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_contents_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_envelope_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_extent(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *lb, MPI_Count *extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_extent_x(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, char *type_name, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_name(type, type_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_true_extent_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_true_extent(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_get_true_extent_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_hindexed_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_hvector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_indexed_NewStack(PNMPI_modHandle_t stack, int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_lb_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Aint *lb)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_lb(type, lb);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_match_size_NewStack(PNMPI_modHandle_t stack, int typeclass, int size, MPI_Datatype *type)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_match_size(typeclass, size, type);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int type_keyval, void *attr_val)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_set_attr(type, type_keyval, attr_val);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, const char *type_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_set_name(type, type_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_size_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, int *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_size(type, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_size_x_NewStack(PNMPI_modHandle_t stack, MPI_Datatype type, MPI_Count *size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_size_x(type, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_struct_NewStack(PNMPI_modHandle_t stack, int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_ub_NewStack(PNMPI_modHandle_t stack, MPI_Datatype mtype, MPI_Aint *ub)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_ub(mtype, ub);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Type_vector_NewStack(PNMPI_modHandle_t stack, int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Unpack_NewStack(PNMPI_modHandle_t stack, const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Unpack_external_NewStack(PNMPI_modHandle_t stack, const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Unpublish_name_NewStack(PNMPI_modHandle_t stack, const char *service_name, MPI_Info info, const char *port_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Unpublish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Wait_NewStack(PNMPI_modHandle_t stack, MPI_Request *request, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wait(request, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Waitall_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitall(count, array_of_requests, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Waitany_NewStack(PNMPI_modHandle_t stack, int count, MPI_Request array_of_requests[], int *index, MPI_Status *status)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitany(count, array_of_requests, index, status);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Waitsome_NewStack(PNMPI_modHandle_t stack, int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[])
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_allocate_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_allocate_shared_NewStack(PNMPI_modHandle_t stack, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_attach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, void *base, MPI_Aint size)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_attach(win, base, size);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_call_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int errorcode)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_call_errhandler(win, errorcode);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_complete_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_complete(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_create_NewStack(PNMPI_modHandle_t stack, void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create(base, size, disp_unit, info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_create_dynamic_NewStack(PNMPI_modHandle_t stack, MPI_Info info, MPI_Comm comm, MPI_Win *win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_dynamic(info, comm, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_create_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_create_keyval_NewStack(PNMPI_modHandle_t stack, MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_delete_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_delete_attr(win, win_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_detach_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const void *base)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_detach(win, base);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_fence_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_fence(assert, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_flush_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush(rank, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_flush_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_all(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_flush_local_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_local(rank, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_flush_local_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_flush_local_all(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_free_NewStack(PNMPI_modHandle_t stack, MPI_Win *win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_free(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_free_keyval_NewStack(PNMPI_modHandle_t stack, int *win_keyval)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_free_keyval(win_keyval);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_get_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_get_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler *errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_get_group_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Group *group)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_group(win, group);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_get_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info *info_used)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_info(win, info_used);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_get_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, char *win_name, int *resultlen)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_get_name(win, win_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_lock_NewStack(PNMPI_modHandle_t stack, int lock_type, int rank, int assert, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_lock(lock_type, rank, assert, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_lock_all_NewStack(PNMPI_modHandle_t stack, int assert, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_lock_all(assert, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_post_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_post(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_set_attr_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int win_keyval, void *attribute_val)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_attr(win, win_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_set_errhandler_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Errhandler errhandler)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_set_info_NewStack(PNMPI_modHandle_t stack, MPI_Win win, MPI_Info info)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_info(win, info);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_set_name_NewStack(PNMPI_modHandle_t stack, MPI_Win win, const char *win_name)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_set_name(win, win_name);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_shared_query_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_start_NewStack(PNMPI_modHandle_t stack, MPI_Group group, int assert, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_start(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_sync_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_sync(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_test_NewStack(PNMPI_modHandle_t stack, MPI_Win win, int *flag)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_test(win, flag);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_unlock_NewStack(PNMPI_modHandle_t stack, int rank, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_unlock(rank, win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_unlock_all_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_unlock_all(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

int XMPI_Win_wait_NewStack(PNMPI_modHandle_t stack, MPI_Win win)
{
  int returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Win_wait(win);
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

double XMPI_Wtick_NewStack(PNMPI_modHandle_t stack)
{
  double returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wtick();
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

double XMPI_Wtime_NewStack(PNMPI_modHandle_t stack)
{
  double returnVal;
  int current = pnmpi_level;
  if (stack>=0)
    pnmpi_level = stack;
  else
    pnmpi_level++;

  WRAP_MPI_CALL_PREFIX
  returnVal=Internal_XMPI_Wtime();
  WRAP_MPI_CALL_POSTFIX
  pnmpi_level = current;
  return returnVal;
}

