
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */




#if (defined(PIC) || defined(__PIC__))
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#pragma weak pmpi_init_thread
#pragma weak PMPI_INIT_THREAD
#pragma weak pmpi_init_thread_
#pragma weak pmpi_init_thread__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT_THREAD(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread_(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread__(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#ifndef WRAP_MPI_CALL_PREFIX
#if defined(__clang__)
#define WRAP_MPI_CALL_PREFIX        \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#elif defined(__INTEL_COMPILER)
#define WRAP_MPI_CALL_PREFIX \
  _Pragma("warning push");   \
  _Pragma("warning(disable:1786)");
#define WRAP_MPI_CALL_POSTFIX _Pragma("warning pop")
#elif (defined(__GNUC__) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4))
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif
#endif

/* This file is part of P^nMPI.
 *
 * Copyright (c)
 *  2008-2018 Lawrence Livermore National Laboratories, United States of America
 *  2011-2016 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2013-2018 RWTH Aachen University, Federal Republic of Germany
 *
 *
 * P^nMPI is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation version 2.1 dated February 1999.
 *
 * P^nMPI is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with P^nMPI; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   51 Franklin St, Fifth Floor
 *   Boston, MA 02110, USA
 *
 *
 * Written by Martin Schulz, schulzm@llnl.gov.
 *
 * LLNL-CODE-402774
 */

/* \brief Count MPI calls.
 *
 * \details This module counts how often each MPI call was invoked. Before MPI
 *  finalizes, statistics will be printed for each rank and a sum of the made
 *  calls by all ranks.
 */


#include <stdio.h>

#include <mpi.h>
#include <pnmpi/debug_io.h>
#include <pnmpi/hooks.h>
#include <pnmpi/private/pmpi_assert.h>
#include <pnmpi/xmpi.h>

#include "atomic.h"


/* If there is no atomic support, we'll limit the threading level of this module
 * to MPI_THREAD_SERIALIZED, so it is safe to use with threaded applications
 * (but they may become slower!). */
#if defined(METRIC_NO_ATOMIC)
const int PNMPI_SupportedThreadingLevel = MPI_THREAD_SERIALIZED;
#endif


/** \brief Struct of counters.
 *
 * \details This struct stores the counters for all MPI calls.
 */
static struct counter
{
  
  metric_atomic_keyword size_t MPI_Abort;
  
  metric_atomic_keyword size_t MPI_Accumulate;
  
  metric_atomic_keyword size_t MPI_Add_error_class;
  
  metric_atomic_keyword size_t MPI_Add_error_code;
  
  metric_atomic_keyword size_t MPI_Add_error_string;
  
  metric_atomic_keyword size_t MPI_Address;
  
  metric_atomic_keyword size_t MPI_Allgather;
  
  metric_atomic_keyword size_t MPI_Allgatherv;
  
  metric_atomic_keyword size_t MPI_Alloc_mem;
  
  metric_atomic_keyword size_t MPI_Allreduce;
  
  metric_atomic_keyword size_t MPI_Alltoall;
  
  metric_atomic_keyword size_t MPI_Alltoallv;
  
  metric_atomic_keyword size_t MPI_Alltoallw;
  
  metric_atomic_keyword size_t MPI_Attr_delete;
  
  metric_atomic_keyword size_t MPI_Attr_get;
  
  metric_atomic_keyword size_t MPI_Attr_put;
  
  metric_atomic_keyword size_t MPI_Barrier;
  
  metric_atomic_keyword size_t MPI_Bcast;
  
  metric_atomic_keyword size_t MPI_Bsend;
  
  metric_atomic_keyword size_t MPI_Bsend_init;
  
  metric_atomic_keyword size_t MPI_Buffer_attach;
  
  metric_atomic_keyword size_t MPI_Buffer_detach;
  
  metric_atomic_keyword size_t MPI_Cancel;
  
  metric_atomic_keyword size_t MPI_Cart_coords;
  
  metric_atomic_keyword size_t MPI_Cart_create;
  
  metric_atomic_keyword size_t MPI_Cart_get;
  
  metric_atomic_keyword size_t MPI_Cart_map;
  
  metric_atomic_keyword size_t MPI_Cart_rank;
  
  metric_atomic_keyword size_t MPI_Cart_shift;
  
  metric_atomic_keyword size_t MPI_Cart_sub;
  
  metric_atomic_keyword size_t MPI_Cartdim_get;
  
  metric_atomic_keyword size_t MPI_Close_port;
  
  metric_atomic_keyword size_t MPI_Comm_accept;
  
  metric_atomic_keyword size_t MPI_Comm_call_errhandler;
  
  metric_atomic_keyword size_t MPI_Comm_compare;
  
  metric_atomic_keyword size_t MPI_Comm_connect;
  
  metric_atomic_keyword size_t MPI_Comm_create;
  
  metric_atomic_keyword size_t MPI_Comm_create_errhandler;
  
  metric_atomic_keyword size_t MPI_Comm_create_group;
  
  metric_atomic_keyword size_t MPI_Comm_create_keyval;
  
  metric_atomic_keyword size_t MPI_Comm_delete_attr;
  
  metric_atomic_keyword size_t MPI_Comm_disconnect;
  
  metric_atomic_keyword size_t MPI_Comm_dup;
  
  metric_atomic_keyword size_t MPI_Comm_dup_with_info;
  
  metric_atomic_keyword size_t MPI_Comm_free;
  
  metric_atomic_keyword size_t MPI_Comm_free_keyval;
  
  metric_atomic_keyword size_t MPI_Comm_get_attr;
  
  metric_atomic_keyword size_t MPI_Comm_get_errhandler;
  
  metric_atomic_keyword size_t MPI_Comm_get_info;
  
  metric_atomic_keyword size_t MPI_Comm_get_name;
  
  metric_atomic_keyword size_t MPI_Comm_get_parent;
  
  metric_atomic_keyword size_t MPI_Comm_group;
  
  metric_atomic_keyword size_t MPI_Comm_idup;
  
  metric_atomic_keyword size_t MPI_Comm_join;
  
  metric_atomic_keyword size_t MPI_Comm_rank;
  
  metric_atomic_keyword size_t MPI_Comm_remote_group;
  
  metric_atomic_keyword size_t MPI_Comm_remote_size;
  
  metric_atomic_keyword size_t MPI_Comm_set_attr;
  
  metric_atomic_keyword size_t MPI_Comm_set_errhandler;
  
  metric_atomic_keyword size_t MPI_Comm_set_info;
  
  metric_atomic_keyword size_t MPI_Comm_set_name;
  
  metric_atomic_keyword size_t MPI_Comm_size;
  
  metric_atomic_keyword size_t MPI_Comm_split;
  
  metric_atomic_keyword size_t MPI_Comm_split_type;
  
  metric_atomic_keyword size_t MPI_Comm_test_inter;
  
  metric_atomic_keyword size_t MPI_Compare_and_swap;
  
  metric_atomic_keyword size_t MPI_Dims_create;
  
  metric_atomic_keyword size_t MPI_Dist_graph_create;
  
  metric_atomic_keyword size_t MPI_Dist_graph_create_adjacent;
  
  metric_atomic_keyword size_t MPI_Dist_graph_neighbors;
  
  metric_atomic_keyword size_t MPI_Dist_graph_neighbors_count;
  
  metric_atomic_keyword size_t MPI_Errhandler_create;
  
  metric_atomic_keyword size_t MPI_Errhandler_free;
  
  metric_atomic_keyword size_t MPI_Errhandler_get;
  
  metric_atomic_keyword size_t MPI_Errhandler_set;
  
  metric_atomic_keyword size_t MPI_Error_class;
  
  metric_atomic_keyword size_t MPI_Error_string;
  
  metric_atomic_keyword size_t MPI_Exscan;
  
  metric_atomic_keyword size_t MPI_Fetch_and_op;
  
  metric_atomic_keyword size_t MPI_File_call_errhandler;
  
  metric_atomic_keyword size_t MPI_File_close;
  
  metric_atomic_keyword size_t MPI_File_create_errhandler;
  
  metric_atomic_keyword size_t MPI_File_delete;
  
  metric_atomic_keyword size_t MPI_File_get_amode;
  
  metric_atomic_keyword size_t MPI_File_get_atomicity;
  
  metric_atomic_keyword size_t MPI_File_get_byte_offset;
  
  metric_atomic_keyword size_t MPI_File_get_errhandler;
  
  metric_atomic_keyword size_t MPI_File_get_group;
  
  metric_atomic_keyword size_t MPI_File_get_info;
  
  metric_atomic_keyword size_t MPI_File_get_position;
  
  metric_atomic_keyword size_t MPI_File_get_position_shared;
  
  metric_atomic_keyword size_t MPI_File_get_size;
  
  metric_atomic_keyword size_t MPI_File_get_type_extent;
  
  metric_atomic_keyword size_t MPI_File_get_view;
  
  metric_atomic_keyword size_t MPI_File_iread;
  
  metric_atomic_keyword size_t MPI_File_iread_all;
  
  metric_atomic_keyword size_t MPI_File_iread_at;
  
  metric_atomic_keyword size_t MPI_File_iread_at_all;
  
  metric_atomic_keyword size_t MPI_File_iread_shared;
  
  metric_atomic_keyword size_t MPI_File_iwrite;
  
  metric_atomic_keyword size_t MPI_File_iwrite_all;
  
  metric_atomic_keyword size_t MPI_File_iwrite_at;
  
  metric_atomic_keyword size_t MPI_File_iwrite_at_all;
  
  metric_atomic_keyword size_t MPI_File_iwrite_shared;
  
  metric_atomic_keyword size_t MPI_File_open;
  
  metric_atomic_keyword size_t MPI_File_preallocate;
  
  metric_atomic_keyword size_t MPI_File_read;
  
  metric_atomic_keyword size_t MPI_File_read_all;
  
  metric_atomic_keyword size_t MPI_File_read_all_begin;
  
  metric_atomic_keyword size_t MPI_File_read_all_end;
  
  metric_atomic_keyword size_t MPI_File_read_at;
  
  metric_atomic_keyword size_t MPI_File_read_at_all;
  
  metric_atomic_keyword size_t MPI_File_read_at_all_begin;
  
  metric_atomic_keyword size_t MPI_File_read_at_all_end;
  
  metric_atomic_keyword size_t MPI_File_read_ordered;
  
  metric_atomic_keyword size_t MPI_File_read_ordered_begin;
  
  metric_atomic_keyword size_t MPI_File_read_ordered_end;
  
  metric_atomic_keyword size_t MPI_File_read_shared;
  
  metric_atomic_keyword size_t MPI_File_seek;
  
  metric_atomic_keyword size_t MPI_File_seek_shared;
  
  metric_atomic_keyword size_t MPI_File_set_atomicity;
  
  metric_atomic_keyword size_t MPI_File_set_errhandler;
  
  metric_atomic_keyword size_t MPI_File_set_info;
  
  metric_atomic_keyword size_t MPI_File_set_size;
  
  metric_atomic_keyword size_t MPI_File_set_view;
  
  metric_atomic_keyword size_t MPI_File_sync;
  
  metric_atomic_keyword size_t MPI_File_write;
  
  metric_atomic_keyword size_t MPI_File_write_all;
  
  metric_atomic_keyword size_t MPI_File_write_all_begin;
  
  metric_atomic_keyword size_t MPI_File_write_all_end;
  
  metric_atomic_keyword size_t MPI_File_write_at;
  
  metric_atomic_keyword size_t MPI_File_write_at_all;
  
  metric_atomic_keyword size_t MPI_File_write_at_all_begin;
  
  metric_atomic_keyword size_t MPI_File_write_at_all_end;
  
  metric_atomic_keyword size_t MPI_File_write_ordered;
  
  metric_atomic_keyword size_t MPI_File_write_ordered_begin;
  
  metric_atomic_keyword size_t MPI_File_write_ordered_end;
  
  metric_atomic_keyword size_t MPI_File_write_shared;
  
  metric_atomic_keyword size_t MPI_Finalized;
  
  metric_atomic_keyword size_t MPI_Free_mem;
  
  metric_atomic_keyword size_t MPI_Gather;
  
  metric_atomic_keyword size_t MPI_Gatherv;
  
  metric_atomic_keyword size_t MPI_Get;
  
  metric_atomic_keyword size_t MPI_Get_accumulate;
  
  metric_atomic_keyword size_t MPI_Get_address;
  
  metric_atomic_keyword size_t MPI_Get_count;
  
  metric_atomic_keyword size_t MPI_Get_elements;
  
  metric_atomic_keyword size_t MPI_Get_elements_x;
  
  metric_atomic_keyword size_t MPI_Get_library_version;
  
  metric_atomic_keyword size_t MPI_Get_processor_name;
  
  metric_atomic_keyword size_t MPI_Get_version;
  
  metric_atomic_keyword size_t MPI_Graph_create;
  
  metric_atomic_keyword size_t MPI_Graph_get;
  
  metric_atomic_keyword size_t MPI_Graph_map;
  
  metric_atomic_keyword size_t MPI_Graph_neighbors;
  
  metric_atomic_keyword size_t MPI_Graph_neighbors_count;
  
  metric_atomic_keyword size_t MPI_Graphdims_get;
  
  metric_atomic_keyword size_t MPI_Grequest_complete;
  
  metric_atomic_keyword size_t MPI_Grequest_start;
  
  metric_atomic_keyword size_t MPI_Group_compare;
  
  metric_atomic_keyword size_t MPI_Group_difference;
  
  metric_atomic_keyword size_t MPI_Group_excl;
  
  metric_atomic_keyword size_t MPI_Group_free;
  
  metric_atomic_keyword size_t MPI_Group_incl;
  
  metric_atomic_keyword size_t MPI_Group_intersection;
  
  metric_atomic_keyword size_t MPI_Group_range_excl;
  
  metric_atomic_keyword size_t MPI_Group_range_incl;
  
  metric_atomic_keyword size_t MPI_Group_rank;
  
  metric_atomic_keyword size_t MPI_Group_size;
  
  metric_atomic_keyword size_t MPI_Group_translate_ranks;
  
  metric_atomic_keyword size_t MPI_Group_union;
  
  metric_atomic_keyword size_t MPI_Iallgather;
  
  metric_atomic_keyword size_t MPI_Iallgatherv;
  
  metric_atomic_keyword size_t MPI_Iallreduce;
  
  metric_atomic_keyword size_t MPI_Ialltoall;
  
  metric_atomic_keyword size_t MPI_Ialltoallv;
  
  metric_atomic_keyword size_t MPI_Ialltoallw;
  
  metric_atomic_keyword size_t MPI_Ibarrier;
  
  metric_atomic_keyword size_t MPI_Ibcast;
  
  metric_atomic_keyword size_t MPI_Ibsend;
  
  metric_atomic_keyword size_t MPI_Iexscan;
  
  metric_atomic_keyword size_t MPI_Igather;
  
  metric_atomic_keyword size_t MPI_Igatherv;
  
  metric_atomic_keyword size_t MPI_Improbe;
  
  metric_atomic_keyword size_t MPI_Imrecv;
  
  metric_atomic_keyword size_t MPI_Ineighbor_allgather;
  
  metric_atomic_keyword size_t MPI_Ineighbor_allgatherv;
  
  metric_atomic_keyword size_t MPI_Ineighbor_alltoall;
  
  metric_atomic_keyword size_t MPI_Ineighbor_alltoallv;
  
  metric_atomic_keyword size_t MPI_Ineighbor_alltoallw;
  
  metric_atomic_keyword size_t MPI_Info_create;
  
  metric_atomic_keyword size_t MPI_Info_delete;
  
  metric_atomic_keyword size_t MPI_Info_dup;
  
  metric_atomic_keyword size_t MPI_Info_free;
  
  metric_atomic_keyword size_t MPI_Info_get;
  
  metric_atomic_keyword size_t MPI_Info_get_nkeys;
  
  metric_atomic_keyword size_t MPI_Info_get_nthkey;
  
  metric_atomic_keyword size_t MPI_Info_get_valuelen;
  
  metric_atomic_keyword size_t MPI_Info_set;
  
  metric_atomic_keyword size_t MPI_Init;
  
  metric_atomic_keyword size_t MPI_Init_thread;
  
  metric_atomic_keyword size_t MPI_Initialized;
  
  metric_atomic_keyword size_t MPI_Intercomm_create;
  
  metric_atomic_keyword size_t MPI_Intercomm_merge;
  
  metric_atomic_keyword size_t MPI_Iprobe;
  
  metric_atomic_keyword size_t MPI_Irecv;
  
  metric_atomic_keyword size_t MPI_Ireduce;
  
  metric_atomic_keyword size_t MPI_Ireduce_scatter;
  
  metric_atomic_keyword size_t MPI_Ireduce_scatter_block;
  
  metric_atomic_keyword size_t MPI_Irsend;
  
  metric_atomic_keyword size_t MPI_Is_thread_main;
  
  metric_atomic_keyword size_t MPI_Iscan;
  
  metric_atomic_keyword size_t MPI_Iscatter;
  
  metric_atomic_keyword size_t MPI_Iscatterv;
  
  metric_atomic_keyword size_t MPI_Isend;
  
  metric_atomic_keyword size_t MPI_Issend;
  
  metric_atomic_keyword size_t MPI_Keyval_create;
  
  metric_atomic_keyword size_t MPI_Keyval_free;
  
  metric_atomic_keyword size_t MPI_Lookup_name;
  
  metric_atomic_keyword size_t MPI_Mprobe;
  
  metric_atomic_keyword size_t MPI_Mrecv;
  
  metric_atomic_keyword size_t MPI_Neighbor_allgather;
  
  metric_atomic_keyword size_t MPI_Neighbor_allgatherv;
  
  metric_atomic_keyword size_t MPI_Neighbor_alltoall;
  
  metric_atomic_keyword size_t MPI_Neighbor_alltoallv;
  
  metric_atomic_keyword size_t MPI_Neighbor_alltoallw;
  
  metric_atomic_keyword size_t MPI_Op_commutative;
  
  metric_atomic_keyword size_t MPI_Op_create;
  
  metric_atomic_keyword size_t MPI_Op_free;
  
  metric_atomic_keyword size_t MPI_Open_port;
  
  metric_atomic_keyword size_t MPI_Pack;
  
  metric_atomic_keyword size_t MPI_Pack_external;
  
  metric_atomic_keyword size_t MPI_Pack_external_size;
  
  metric_atomic_keyword size_t MPI_Pack_size;
  
  metric_atomic_keyword size_t MPI_Pcontrol;
  
  metric_atomic_keyword size_t MPI_Probe;
  
  metric_atomic_keyword size_t MPI_Publish_name;
  
  metric_atomic_keyword size_t MPI_Put;
  
  metric_atomic_keyword size_t MPI_Query_thread;
  
  metric_atomic_keyword size_t MPI_Raccumulate;
  
  metric_atomic_keyword size_t MPI_Recv;
  
  metric_atomic_keyword size_t MPI_Recv_init;
  
  metric_atomic_keyword size_t MPI_Reduce;
  
  metric_atomic_keyword size_t MPI_Reduce_local;
  
  metric_atomic_keyword size_t MPI_Reduce_scatter;
  
  metric_atomic_keyword size_t MPI_Reduce_scatter_block;
  
  metric_atomic_keyword size_t MPI_Register_datarep;
  
  metric_atomic_keyword size_t MPI_Request_free;
  
  metric_atomic_keyword size_t MPI_Request_get_status;
  
  metric_atomic_keyword size_t MPI_Rget;
  
  metric_atomic_keyword size_t MPI_Rget_accumulate;
  
  metric_atomic_keyword size_t MPI_Rput;
  
  metric_atomic_keyword size_t MPI_Rsend;
  
  metric_atomic_keyword size_t MPI_Rsend_init;
  
  metric_atomic_keyword size_t MPI_Scan;
  
  metric_atomic_keyword size_t MPI_Scatter;
  
  metric_atomic_keyword size_t MPI_Scatterv;
  
  metric_atomic_keyword size_t MPI_Send;
  
  metric_atomic_keyword size_t MPI_Send_init;
  
  metric_atomic_keyword size_t MPI_Sendrecv;
  
  metric_atomic_keyword size_t MPI_Sendrecv_replace;
  
  metric_atomic_keyword size_t MPI_Ssend;
  
  metric_atomic_keyword size_t MPI_Ssend_init;
  
  metric_atomic_keyword size_t MPI_Start;
  
  metric_atomic_keyword size_t MPI_Startall;
  
  metric_atomic_keyword size_t MPI_Status_set_cancelled;
  
  metric_atomic_keyword size_t MPI_Status_set_elements;
  
  metric_atomic_keyword size_t MPI_Status_set_elements_x;
  
  metric_atomic_keyword size_t MPI_Test;
  
  metric_atomic_keyword size_t MPI_Test_cancelled;
  
  metric_atomic_keyword size_t MPI_Testall;
  
  metric_atomic_keyword size_t MPI_Testany;
  
  metric_atomic_keyword size_t MPI_Testsome;
  
  metric_atomic_keyword size_t MPI_Topo_test;
  
  metric_atomic_keyword size_t MPI_Type_commit;
  
  metric_atomic_keyword size_t MPI_Type_contiguous;
  
  metric_atomic_keyword size_t MPI_Type_create_darray;
  
  metric_atomic_keyword size_t MPI_Type_create_f90_complex;
  
  metric_atomic_keyword size_t MPI_Type_create_f90_integer;
  
  metric_atomic_keyword size_t MPI_Type_create_f90_real;
  
  metric_atomic_keyword size_t MPI_Type_create_hindexed;
  
  metric_atomic_keyword size_t MPI_Type_create_hindexed_block;
  
  metric_atomic_keyword size_t MPI_Type_create_hvector;
  
  metric_atomic_keyword size_t MPI_Type_create_indexed_block;
  
  metric_atomic_keyword size_t MPI_Type_create_keyval;
  
  metric_atomic_keyword size_t MPI_Type_create_resized;
  
  metric_atomic_keyword size_t MPI_Type_create_struct;
  
  metric_atomic_keyword size_t MPI_Type_create_subarray;
  
  metric_atomic_keyword size_t MPI_Type_delete_attr;
  
  metric_atomic_keyword size_t MPI_Type_dup;
  
  metric_atomic_keyword size_t MPI_Type_extent;
  
  metric_atomic_keyword size_t MPI_Type_free;
  
  metric_atomic_keyword size_t MPI_Type_free_keyval;
  
  metric_atomic_keyword size_t MPI_Type_get_attr;
  
  metric_atomic_keyword size_t MPI_Type_get_contents;
  
  metric_atomic_keyword size_t MPI_Type_get_envelope;
  
  metric_atomic_keyword size_t MPI_Type_get_extent;
  
  metric_atomic_keyword size_t MPI_Type_get_extent_x;
  
  metric_atomic_keyword size_t MPI_Type_get_name;
  
  metric_atomic_keyword size_t MPI_Type_get_true_extent;
  
  metric_atomic_keyword size_t MPI_Type_get_true_extent_x;
  
  metric_atomic_keyword size_t MPI_Type_hindexed;
  
  metric_atomic_keyword size_t MPI_Type_hvector;
  
  metric_atomic_keyword size_t MPI_Type_indexed;
  
  metric_atomic_keyword size_t MPI_Type_lb;
  
  metric_atomic_keyword size_t MPI_Type_match_size;
  
  metric_atomic_keyword size_t MPI_Type_set_attr;
  
  metric_atomic_keyword size_t MPI_Type_set_name;
  
  metric_atomic_keyword size_t MPI_Type_size;
  
  metric_atomic_keyword size_t MPI_Type_size_x;
  
  metric_atomic_keyword size_t MPI_Type_struct;
  
  metric_atomic_keyword size_t MPI_Type_ub;
  
  metric_atomic_keyword size_t MPI_Type_vector;
  
  metric_atomic_keyword size_t MPI_Unpack;
  
  metric_atomic_keyword size_t MPI_Unpack_external;
  
  metric_atomic_keyword size_t MPI_Unpublish_name;
  
  metric_atomic_keyword size_t MPI_Wait;
  
  metric_atomic_keyword size_t MPI_Waitall;
  
  metric_atomic_keyword size_t MPI_Waitany;
  
  metric_atomic_keyword size_t MPI_Waitsome;
  
  metric_atomic_keyword size_t MPI_Win_allocate;
  
  metric_atomic_keyword size_t MPI_Win_allocate_shared;
  
  metric_atomic_keyword size_t MPI_Win_attach;
  
  metric_atomic_keyword size_t MPI_Win_call_errhandler;
  
  metric_atomic_keyword size_t MPI_Win_complete;
  
  metric_atomic_keyword size_t MPI_Win_create;
  
  metric_atomic_keyword size_t MPI_Win_create_dynamic;
  
  metric_atomic_keyword size_t MPI_Win_create_errhandler;
  
  metric_atomic_keyword size_t MPI_Win_create_keyval;
  
  metric_atomic_keyword size_t MPI_Win_delete_attr;
  
  metric_atomic_keyword size_t MPI_Win_detach;
  
  metric_atomic_keyword size_t MPI_Win_fence;
  
  metric_atomic_keyword size_t MPI_Win_flush;
  
  metric_atomic_keyword size_t MPI_Win_flush_all;
  
  metric_atomic_keyword size_t MPI_Win_flush_local;
  
  metric_atomic_keyword size_t MPI_Win_flush_local_all;
  
  metric_atomic_keyword size_t MPI_Win_free;
  
  metric_atomic_keyword size_t MPI_Win_free_keyval;
  
  metric_atomic_keyword size_t MPI_Win_get_attr;
  
  metric_atomic_keyword size_t MPI_Win_get_errhandler;
  
  metric_atomic_keyword size_t MPI_Win_get_group;
  
  metric_atomic_keyword size_t MPI_Win_get_info;
  
  metric_atomic_keyword size_t MPI_Win_get_name;
  
  metric_atomic_keyword size_t MPI_Win_lock;
  
  metric_atomic_keyword size_t MPI_Win_lock_all;
  
  metric_atomic_keyword size_t MPI_Win_post;
  
  metric_atomic_keyword size_t MPI_Win_set_attr;
  
  metric_atomic_keyword size_t MPI_Win_set_errhandler;
  
  metric_atomic_keyword size_t MPI_Win_set_info;
  
  metric_atomic_keyword size_t MPI_Win_set_name;
  
  metric_atomic_keyword size_t MPI_Win_shared_query;
  
  metric_atomic_keyword size_t MPI_Win_start;
  
  metric_atomic_keyword size_t MPI_Win_sync;
  
  metric_atomic_keyword size_t MPI_Win_test;
  
  metric_atomic_keyword size_t MPI_Win_unlock;
  
  metric_atomic_keyword size_t MPI_Win_unlock_all;
  
  metric_atomic_keyword size_t MPI_Win_wait;
  
  metric_atomic_keyword size_t MPI_Wtick;
  
  metric_atomic_keyword size_t MPI_Wtime;
  
} counters;


/** \brief Helper function to print counter struct.
 *
 * \details This function will print all functions called at least once with
 *  their counter.
 *
 *
 * \param c \ref counter struct to be printed.
 */
static void print_counters(struct counter *c)
{
  
  if (c->MPI_Abort > 0)
    printf("  %8zu %s\n", c->MPI_Abort, "MPI_Abort");
  
  if (c->MPI_Accumulate > 0)
    printf("  %8zu %s\n", c->MPI_Accumulate, "MPI_Accumulate");
  
  if (c->MPI_Add_error_class > 0)
    printf("  %8zu %s\n", c->MPI_Add_error_class, "MPI_Add_error_class");
  
  if (c->MPI_Add_error_code > 0)
    printf("  %8zu %s\n", c->MPI_Add_error_code, "MPI_Add_error_code");
  
  if (c->MPI_Add_error_string > 0)
    printf("  %8zu %s\n", c->MPI_Add_error_string, "MPI_Add_error_string");
  
  if (c->MPI_Address > 0)
    printf("  %8zu %s\n", c->MPI_Address, "MPI_Address");
  
  if (c->MPI_Allgather > 0)
    printf("  %8zu %s\n", c->MPI_Allgather, "MPI_Allgather");
  
  if (c->MPI_Allgatherv > 0)
    printf("  %8zu %s\n", c->MPI_Allgatherv, "MPI_Allgatherv");
  
  if (c->MPI_Alloc_mem > 0)
    printf("  %8zu %s\n", c->MPI_Alloc_mem, "MPI_Alloc_mem");
  
  if (c->MPI_Allreduce > 0)
    printf("  %8zu %s\n", c->MPI_Allreduce, "MPI_Allreduce");
  
  if (c->MPI_Alltoall > 0)
    printf("  %8zu %s\n", c->MPI_Alltoall, "MPI_Alltoall");
  
  if (c->MPI_Alltoallv > 0)
    printf("  %8zu %s\n", c->MPI_Alltoallv, "MPI_Alltoallv");
  
  if (c->MPI_Alltoallw > 0)
    printf("  %8zu %s\n", c->MPI_Alltoallw, "MPI_Alltoallw");
  
  if (c->MPI_Attr_delete > 0)
    printf("  %8zu %s\n", c->MPI_Attr_delete, "MPI_Attr_delete");
  
  if (c->MPI_Attr_get > 0)
    printf("  %8zu %s\n", c->MPI_Attr_get, "MPI_Attr_get");
  
  if (c->MPI_Attr_put > 0)
    printf("  %8zu %s\n", c->MPI_Attr_put, "MPI_Attr_put");
  
  if (c->MPI_Barrier > 0)
    printf("  %8zu %s\n", c->MPI_Barrier, "MPI_Barrier");
  
  if (c->MPI_Bcast > 0)
    printf("  %8zu %s\n", c->MPI_Bcast, "MPI_Bcast");
  
  if (c->MPI_Bsend > 0)
    printf("  %8zu %s\n", c->MPI_Bsend, "MPI_Bsend");
  
  if (c->MPI_Bsend_init > 0)
    printf("  %8zu %s\n", c->MPI_Bsend_init, "MPI_Bsend_init");
  
  if (c->MPI_Buffer_attach > 0)
    printf("  %8zu %s\n", c->MPI_Buffer_attach, "MPI_Buffer_attach");
  
  if (c->MPI_Buffer_detach > 0)
    printf("  %8zu %s\n", c->MPI_Buffer_detach, "MPI_Buffer_detach");
  
  if (c->MPI_Cancel > 0)
    printf("  %8zu %s\n", c->MPI_Cancel, "MPI_Cancel");
  
  if (c->MPI_Cart_coords > 0)
    printf("  %8zu %s\n", c->MPI_Cart_coords, "MPI_Cart_coords");
  
  if (c->MPI_Cart_create > 0)
    printf("  %8zu %s\n", c->MPI_Cart_create, "MPI_Cart_create");
  
  if (c->MPI_Cart_get > 0)
    printf("  %8zu %s\n", c->MPI_Cart_get, "MPI_Cart_get");
  
  if (c->MPI_Cart_map > 0)
    printf("  %8zu %s\n", c->MPI_Cart_map, "MPI_Cart_map");
  
  if (c->MPI_Cart_rank > 0)
    printf("  %8zu %s\n", c->MPI_Cart_rank, "MPI_Cart_rank");
  
  if (c->MPI_Cart_shift > 0)
    printf("  %8zu %s\n", c->MPI_Cart_shift, "MPI_Cart_shift");
  
  if (c->MPI_Cart_sub > 0)
    printf("  %8zu %s\n", c->MPI_Cart_sub, "MPI_Cart_sub");
  
  if (c->MPI_Cartdim_get > 0)
    printf("  %8zu %s\n", c->MPI_Cartdim_get, "MPI_Cartdim_get");
  
  if (c->MPI_Close_port > 0)
    printf("  %8zu %s\n", c->MPI_Close_port, "MPI_Close_port");
  
  if (c->MPI_Comm_accept > 0)
    printf("  %8zu %s\n", c->MPI_Comm_accept, "MPI_Comm_accept");
  
  if (c->MPI_Comm_call_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Comm_call_errhandler, "MPI_Comm_call_errhandler");
  
  if (c->MPI_Comm_compare > 0)
    printf("  %8zu %s\n", c->MPI_Comm_compare, "MPI_Comm_compare");
  
  if (c->MPI_Comm_connect > 0)
    printf("  %8zu %s\n", c->MPI_Comm_connect, "MPI_Comm_connect");
  
  if (c->MPI_Comm_create > 0)
    printf("  %8zu %s\n", c->MPI_Comm_create, "MPI_Comm_create");
  
  if (c->MPI_Comm_create_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Comm_create_errhandler, "MPI_Comm_create_errhandler");
  
  if (c->MPI_Comm_create_group > 0)
    printf("  %8zu %s\n", c->MPI_Comm_create_group, "MPI_Comm_create_group");
  
  if (c->MPI_Comm_create_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Comm_create_keyval, "MPI_Comm_create_keyval");
  
  if (c->MPI_Comm_delete_attr > 0)
    printf("  %8zu %s\n", c->MPI_Comm_delete_attr, "MPI_Comm_delete_attr");
  
  if (c->MPI_Comm_disconnect > 0)
    printf("  %8zu %s\n", c->MPI_Comm_disconnect, "MPI_Comm_disconnect");
  
  if (c->MPI_Comm_dup > 0)
    printf("  %8zu %s\n", c->MPI_Comm_dup, "MPI_Comm_dup");
  
  if (c->MPI_Comm_dup_with_info > 0)
    printf("  %8zu %s\n", c->MPI_Comm_dup_with_info, "MPI_Comm_dup_with_info");
  
  if (c->MPI_Comm_free > 0)
    printf("  %8zu %s\n", c->MPI_Comm_free, "MPI_Comm_free");
  
  if (c->MPI_Comm_free_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Comm_free_keyval, "MPI_Comm_free_keyval");
  
  if (c->MPI_Comm_get_attr > 0)
    printf("  %8zu %s\n", c->MPI_Comm_get_attr, "MPI_Comm_get_attr");
  
  if (c->MPI_Comm_get_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Comm_get_errhandler, "MPI_Comm_get_errhandler");
  
  if (c->MPI_Comm_get_info > 0)
    printf("  %8zu %s\n", c->MPI_Comm_get_info, "MPI_Comm_get_info");
  
  if (c->MPI_Comm_get_name > 0)
    printf("  %8zu %s\n", c->MPI_Comm_get_name, "MPI_Comm_get_name");
  
  if (c->MPI_Comm_get_parent > 0)
    printf("  %8zu %s\n", c->MPI_Comm_get_parent, "MPI_Comm_get_parent");
  
  if (c->MPI_Comm_group > 0)
    printf("  %8zu %s\n", c->MPI_Comm_group, "MPI_Comm_group");
  
  if (c->MPI_Comm_idup > 0)
    printf("  %8zu %s\n", c->MPI_Comm_idup, "MPI_Comm_idup");
  
  if (c->MPI_Comm_join > 0)
    printf("  %8zu %s\n", c->MPI_Comm_join, "MPI_Comm_join");
  
  if (c->MPI_Comm_rank > 0)
    printf("  %8zu %s\n", c->MPI_Comm_rank, "MPI_Comm_rank");
  
  if (c->MPI_Comm_remote_group > 0)
    printf("  %8zu %s\n", c->MPI_Comm_remote_group, "MPI_Comm_remote_group");
  
  if (c->MPI_Comm_remote_size > 0)
    printf("  %8zu %s\n", c->MPI_Comm_remote_size, "MPI_Comm_remote_size");
  
  if (c->MPI_Comm_set_attr > 0)
    printf("  %8zu %s\n", c->MPI_Comm_set_attr, "MPI_Comm_set_attr");
  
  if (c->MPI_Comm_set_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Comm_set_errhandler, "MPI_Comm_set_errhandler");
  
  if (c->MPI_Comm_set_info > 0)
    printf("  %8zu %s\n", c->MPI_Comm_set_info, "MPI_Comm_set_info");
  
  if (c->MPI_Comm_set_name > 0)
    printf("  %8zu %s\n", c->MPI_Comm_set_name, "MPI_Comm_set_name");
  
  if (c->MPI_Comm_size > 0)
    printf("  %8zu %s\n", c->MPI_Comm_size, "MPI_Comm_size");
  
  if (c->MPI_Comm_split > 0)
    printf("  %8zu %s\n", c->MPI_Comm_split, "MPI_Comm_split");
  
  if (c->MPI_Comm_split_type > 0)
    printf("  %8zu %s\n", c->MPI_Comm_split_type, "MPI_Comm_split_type");
  
  if (c->MPI_Comm_test_inter > 0)
    printf("  %8zu %s\n", c->MPI_Comm_test_inter, "MPI_Comm_test_inter");
  
  if (c->MPI_Compare_and_swap > 0)
    printf("  %8zu %s\n", c->MPI_Compare_and_swap, "MPI_Compare_and_swap");
  
  if (c->MPI_Dims_create > 0)
    printf("  %8zu %s\n", c->MPI_Dims_create, "MPI_Dims_create");
  
  if (c->MPI_Dist_graph_create > 0)
    printf("  %8zu %s\n", c->MPI_Dist_graph_create, "MPI_Dist_graph_create");
  
  if (c->MPI_Dist_graph_create_adjacent > 0)
    printf("  %8zu %s\n", c->MPI_Dist_graph_create_adjacent, "MPI_Dist_graph_create_adjacent");
  
  if (c->MPI_Dist_graph_neighbors > 0)
    printf("  %8zu %s\n", c->MPI_Dist_graph_neighbors, "MPI_Dist_graph_neighbors");
  
  if (c->MPI_Dist_graph_neighbors_count > 0)
    printf("  %8zu %s\n", c->MPI_Dist_graph_neighbors_count, "MPI_Dist_graph_neighbors_count");
  
  if (c->MPI_Errhandler_create > 0)
    printf("  %8zu %s\n", c->MPI_Errhandler_create, "MPI_Errhandler_create");
  
  if (c->MPI_Errhandler_free > 0)
    printf("  %8zu %s\n", c->MPI_Errhandler_free, "MPI_Errhandler_free");
  
  if (c->MPI_Errhandler_get > 0)
    printf("  %8zu %s\n", c->MPI_Errhandler_get, "MPI_Errhandler_get");
  
  if (c->MPI_Errhandler_set > 0)
    printf("  %8zu %s\n", c->MPI_Errhandler_set, "MPI_Errhandler_set");
  
  if (c->MPI_Error_class > 0)
    printf("  %8zu %s\n", c->MPI_Error_class, "MPI_Error_class");
  
  if (c->MPI_Error_string > 0)
    printf("  %8zu %s\n", c->MPI_Error_string, "MPI_Error_string");
  
  if (c->MPI_Exscan > 0)
    printf("  %8zu %s\n", c->MPI_Exscan, "MPI_Exscan");
  
  if (c->MPI_Fetch_and_op > 0)
    printf("  %8zu %s\n", c->MPI_Fetch_and_op, "MPI_Fetch_and_op");
  
  if (c->MPI_File_call_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_File_call_errhandler, "MPI_File_call_errhandler");
  
  if (c->MPI_File_close > 0)
    printf("  %8zu %s\n", c->MPI_File_close, "MPI_File_close");
  
  if (c->MPI_File_create_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_File_create_errhandler, "MPI_File_create_errhandler");
  
  if (c->MPI_File_delete > 0)
    printf("  %8zu %s\n", c->MPI_File_delete, "MPI_File_delete");
  
  if (c->MPI_File_get_amode > 0)
    printf("  %8zu %s\n", c->MPI_File_get_amode, "MPI_File_get_amode");
  
  if (c->MPI_File_get_atomicity > 0)
    printf("  %8zu %s\n", c->MPI_File_get_atomicity, "MPI_File_get_atomicity");
  
  if (c->MPI_File_get_byte_offset > 0)
    printf("  %8zu %s\n", c->MPI_File_get_byte_offset, "MPI_File_get_byte_offset");
  
  if (c->MPI_File_get_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_File_get_errhandler, "MPI_File_get_errhandler");
  
  if (c->MPI_File_get_group > 0)
    printf("  %8zu %s\n", c->MPI_File_get_group, "MPI_File_get_group");
  
  if (c->MPI_File_get_info > 0)
    printf("  %8zu %s\n", c->MPI_File_get_info, "MPI_File_get_info");
  
  if (c->MPI_File_get_position > 0)
    printf("  %8zu %s\n", c->MPI_File_get_position, "MPI_File_get_position");
  
  if (c->MPI_File_get_position_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_get_position_shared, "MPI_File_get_position_shared");
  
  if (c->MPI_File_get_size > 0)
    printf("  %8zu %s\n", c->MPI_File_get_size, "MPI_File_get_size");
  
  if (c->MPI_File_get_type_extent > 0)
    printf("  %8zu %s\n", c->MPI_File_get_type_extent, "MPI_File_get_type_extent");
  
  if (c->MPI_File_get_view > 0)
    printf("  %8zu %s\n", c->MPI_File_get_view, "MPI_File_get_view");
  
  if (c->MPI_File_iread > 0)
    printf("  %8zu %s\n", c->MPI_File_iread, "MPI_File_iread");
  
  if (c->MPI_File_iread_all > 0)
    printf("  %8zu %s\n", c->MPI_File_iread_all, "MPI_File_iread_all");
  
  if (c->MPI_File_iread_at > 0)
    printf("  %8zu %s\n", c->MPI_File_iread_at, "MPI_File_iread_at");
  
  if (c->MPI_File_iread_at_all > 0)
    printf("  %8zu %s\n", c->MPI_File_iread_at_all, "MPI_File_iread_at_all");
  
  if (c->MPI_File_iread_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_iread_shared, "MPI_File_iread_shared");
  
  if (c->MPI_File_iwrite > 0)
    printf("  %8zu %s\n", c->MPI_File_iwrite, "MPI_File_iwrite");
  
  if (c->MPI_File_iwrite_all > 0)
    printf("  %8zu %s\n", c->MPI_File_iwrite_all, "MPI_File_iwrite_all");
  
  if (c->MPI_File_iwrite_at > 0)
    printf("  %8zu %s\n", c->MPI_File_iwrite_at, "MPI_File_iwrite_at");
  
  if (c->MPI_File_iwrite_at_all > 0)
    printf("  %8zu %s\n", c->MPI_File_iwrite_at_all, "MPI_File_iwrite_at_all");
  
  if (c->MPI_File_iwrite_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_iwrite_shared, "MPI_File_iwrite_shared");
  
  if (c->MPI_File_open > 0)
    printf("  %8zu %s\n", c->MPI_File_open, "MPI_File_open");
  
  if (c->MPI_File_preallocate > 0)
    printf("  %8zu %s\n", c->MPI_File_preallocate, "MPI_File_preallocate");
  
  if (c->MPI_File_read > 0)
    printf("  %8zu %s\n", c->MPI_File_read, "MPI_File_read");
  
  if (c->MPI_File_read_all > 0)
    printf("  %8zu %s\n", c->MPI_File_read_all, "MPI_File_read_all");
  
  if (c->MPI_File_read_all_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_read_all_begin, "MPI_File_read_all_begin");
  
  if (c->MPI_File_read_all_end > 0)
    printf("  %8zu %s\n", c->MPI_File_read_all_end, "MPI_File_read_all_end");
  
  if (c->MPI_File_read_at > 0)
    printf("  %8zu %s\n", c->MPI_File_read_at, "MPI_File_read_at");
  
  if (c->MPI_File_read_at_all > 0)
    printf("  %8zu %s\n", c->MPI_File_read_at_all, "MPI_File_read_at_all");
  
  if (c->MPI_File_read_at_all_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_read_at_all_begin, "MPI_File_read_at_all_begin");
  
  if (c->MPI_File_read_at_all_end > 0)
    printf("  %8zu %s\n", c->MPI_File_read_at_all_end, "MPI_File_read_at_all_end");
  
  if (c->MPI_File_read_ordered > 0)
    printf("  %8zu %s\n", c->MPI_File_read_ordered, "MPI_File_read_ordered");
  
  if (c->MPI_File_read_ordered_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_read_ordered_begin, "MPI_File_read_ordered_begin");
  
  if (c->MPI_File_read_ordered_end > 0)
    printf("  %8zu %s\n", c->MPI_File_read_ordered_end, "MPI_File_read_ordered_end");
  
  if (c->MPI_File_read_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_read_shared, "MPI_File_read_shared");
  
  if (c->MPI_File_seek > 0)
    printf("  %8zu %s\n", c->MPI_File_seek, "MPI_File_seek");
  
  if (c->MPI_File_seek_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_seek_shared, "MPI_File_seek_shared");
  
  if (c->MPI_File_set_atomicity > 0)
    printf("  %8zu %s\n", c->MPI_File_set_atomicity, "MPI_File_set_atomicity");
  
  if (c->MPI_File_set_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_File_set_errhandler, "MPI_File_set_errhandler");
  
  if (c->MPI_File_set_info > 0)
    printf("  %8zu %s\n", c->MPI_File_set_info, "MPI_File_set_info");
  
  if (c->MPI_File_set_size > 0)
    printf("  %8zu %s\n", c->MPI_File_set_size, "MPI_File_set_size");
  
  if (c->MPI_File_set_view > 0)
    printf("  %8zu %s\n", c->MPI_File_set_view, "MPI_File_set_view");
  
  if (c->MPI_File_sync > 0)
    printf("  %8zu %s\n", c->MPI_File_sync, "MPI_File_sync");
  
  if (c->MPI_File_write > 0)
    printf("  %8zu %s\n", c->MPI_File_write, "MPI_File_write");
  
  if (c->MPI_File_write_all > 0)
    printf("  %8zu %s\n", c->MPI_File_write_all, "MPI_File_write_all");
  
  if (c->MPI_File_write_all_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_write_all_begin, "MPI_File_write_all_begin");
  
  if (c->MPI_File_write_all_end > 0)
    printf("  %8zu %s\n", c->MPI_File_write_all_end, "MPI_File_write_all_end");
  
  if (c->MPI_File_write_at > 0)
    printf("  %8zu %s\n", c->MPI_File_write_at, "MPI_File_write_at");
  
  if (c->MPI_File_write_at_all > 0)
    printf("  %8zu %s\n", c->MPI_File_write_at_all, "MPI_File_write_at_all");
  
  if (c->MPI_File_write_at_all_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_write_at_all_begin, "MPI_File_write_at_all_begin");
  
  if (c->MPI_File_write_at_all_end > 0)
    printf("  %8zu %s\n", c->MPI_File_write_at_all_end, "MPI_File_write_at_all_end");
  
  if (c->MPI_File_write_ordered > 0)
    printf("  %8zu %s\n", c->MPI_File_write_ordered, "MPI_File_write_ordered");
  
  if (c->MPI_File_write_ordered_begin > 0)
    printf("  %8zu %s\n", c->MPI_File_write_ordered_begin, "MPI_File_write_ordered_begin");
  
  if (c->MPI_File_write_ordered_end > 0)
    printf("  %8zu %s\n", c->MPI_File_write_ordered_end, "MPI_File_write_ordered_end");
  
  if (c->MPI_File_write_shared > 0)
    printf("  %8zu %s\n", c->MPI_File_write_shared, "MPI_File_write_shared");
  
  if (c->MPI_Finalized > 0)
    printf("  %8zu %s\n", c->MPI_Finalized, "MPI_Finalized");
  
  if (c->MPI_Free_mem > 0)
    printf("  %8zu %s\n", c->MPI_Free_mem, "MPI_Free_mem");
  
  if (c->MPI_Gather > 0)
    printf("  %8zu %s\n", c->MPI_Gather, "MPI_Gather");
  
  if (c->MPI_Gatherv > 0)
    printf("  %8zu %s\n", c->MPI_Gatherv, "MPI_Gatherv");
  
  if (c->MPI_Get > 0)
    printf("  %8zu %s\n", c->MPI_Get, "MPI_Get");
  
  if (c->MPI_Get_accumulate > 0)
    printf("  %8zu %s\n", c->MPI_Get_accumulate, "MPI_Get_accumulate");
  
  if (c->MPI_Get_address > 0)
    printf("  %8zu %s\n", c->MPI_Get_address, "MPI_Get_address");
  
  if (c->MPI_Get_count > 0)
    printf("  %8zu %s\n", c->MPI_Get_count, "MPI_Get_count");
  
  if (c->MPI_Get_elements > 0)
    printf("  %8zu %s\n", c->MPI_Get_elements, "MPI_Get_elements");
  
  if (c->MPI_Get_elements_x > 0)
    printf("  %8zu %s\n", c->MPI_Get_elements_x, "MPI_Get_elements_x");
  
  if (c->MPI_Get_library_version > 0)
    printf("  %8zu %s\n", c->MPI_Get_library_version, "MPI_Get_library_version");
  
  if (c->MPI_Get_processor_name > 0)
    printf("  %8zu %s\n", c->MPI_Get_processor_name, "MPI_Get_processor_name");
  
  if (c->MPI_Get_version > 0)
    printf("  %8zu %s\n", c->MPI_Get_version, "MPI_Get_version");
  
  if (c->MPI_Graph_create > 0)
    printf("  %8zu %s\n", c->MPI_Graph_create, "MPI_Graph_create");
  
  if (c->MPI_Graph_get > 0)
    printf("  %8zu %s\n", c->MPI_Graph_get, "MPI_Graph_get");
  
  if (c->MPI_Graph_map > 0)
    printf("  %8zu %s\n", c->MPI_Graph_map, "MPI_Graph_map");
  
  if (c->MPI_Graph_neighbors > 0)
    printf("  %8zu %s\n", c->MPI_Graph_neighbors, "MPI_Graph_neighbors");
  
  if (c->MPI_Graph_neighbors_count > 0)
    printf("  %8zu %s\n", c->MPI_Graph_neighbors_count, "MPI_Graph_neighbors_count");
  
  if (c->MPI_Graphdims_get > 0)
    printf("  %8zu %s\n", c->MPI_Graphdims_get, "MPI_Graphdims_get");
  
  if (c->MPI_Grequest_complete > 0)
    printf("  %8zu %s\n", c->MPI_Grequest_complete, "MPI_Grequest_complete");
  
  if (c->MPI_Grequest_start > 0)
    printf("  %8zu %s\n", c->MPI_Grequest_start, "MPI_Grequest_start");
  
  if (c->MPI_Group_compare > 0)
    printf("  %8zu %s\n", c->MPI_Group_compare, "MPI_Group_compare");
  
  if (c->MPI_Group_difference > 0)
    printf("  %8zu %s\n", c->MPI_Group_difference, "MPI_Group_difference");
  
  if (c->MPI_Group_excl > 0)
    printf("  %8zu %s\n", c->MPI_Group_excl, "MPI_Group_excl");
  
  if (c->MPI_Group_free > 0)
    printf("  %8zu %s\n", c->MPI_Group_free, "MPI_Group_free");
  
  if (c->MPI_Group_incl > 0)
    printf("  %8zu %s\n", c->MPI_Group_incl, "MPI_Group_incl");
  
  if (c->MPI_Group_intersection > 0)
    printf("  %8zu %s\n", c->MPI_Group_intersection, "MPI_Group_intersection");
  
  if (c->MPI_Group_range_excl > 0)
    printf("  %8zu %s\n", c->MPI_Group_range_excl, "MPI_Group_range_excl");
  
  if (c->MPI_Group_range_incl > 0)
    printf("  %8zu %s\n", c->MPI_Group_range_incl, "MPI_Group_range_incl");
  
  if (c->MPI_Group_rank > 0)
    printf("  %8zu %s\n", c->MPI_Group_rank, "MPI_Group_rank");
  
  if (c->MPI_Group_size > 0)
    printf("  %8zu %s\n", c->MPI_Group_size, "MPI_Group_size");
  
  if (c->MPI_Group_translate_ranks > 0)
    printf("  %8zu %s\n", c->MPI_Group_translate_ranks, "MPI_Group_translate_ranks");
  
  if (c->MPI_Group_union > 0)
    printf("  %8zu %s\n", c->MPI_Group_union, "MPI_Group_union");
  
  if (c->MPI_Iallgather > 0)
    printf("  %8zu %s\n", c->MPI_Iallgather, "MPI_Iallgather");
  
  if (c->MPI_Iallgatherv > 0)
    printf("  %8zu %s\n", c->MPI_Iallgatherv, "MPI_Iallgatherv");
  
  if (c->MPI_Iallreduce > 0)
    printf("  %8zu %s\n", c->MPI_Iallreduce, "MPI_Iallreduce");
  
  if (c->MPI_Ialltoall > 0)
    printf("  %8zu %s\n", c->MPI_Ialltoall, "MPI_Ialltoall");
  
  if (c->MPI_Ialltoallv > 0)
    printf("  %8zu %s\n", c->MPI_Ialltoallv, "MPI_Ialltoallv");
  
  if (c->MPI_Ialltoallw > 0)
    printf("  %8zu %s\n", c->MPI_Ialltoallw, "MPI_Ialltoallw");
  
  if (c->MPI_Ibarrier > 0)
    printf("  %8zu %s\n", c->MPI_Ibarrier, "MPI_Ibarrier");
  
  if (c->MPI_Ibcast > 0)
    printf("  %8zu %s\n", c->MPI_Ibcast, "MPI_Ibcast");
  
  if (c->MPI_Ibsend > 0)
    printf("  %8zu %s\n", c->MPI_Ibsend, "MPI_Ibsend");
  
  if (c->MPI_Iexscan > 0)
    printf("  %8zu %s\n", c->MPI_Iexscan, "MPI_Iexscan");
  
  if (c->MPI_Igather > 0)
    printf("  %8zu %s\n", c->MPI_Igather, "MPI_Igather");
  
  if (c->MPI_Igatherv > 0)
    printf("  %8zu %s\n", c->MPI_Igatherv, "MPI_Igatherv");
  
  if (c->MPI_Improbe > 0)
    printf("  %8zu %s\n", c->MPI_Improbe, "MPI_Improbe");
  
  if (c->MPI_Imrecv > 0)
    printf("  %8zu %s\n", c->MPI_Imrecv, "MPI_Imrecv");
  
  if (c->MPI_Ineighbor_allgather > 0)
    printf("  %8zu %s\n", c->MPI_Ineighbor_allgather, "MPI_Ineighbor_allgather");
  
  if (c->MPI_Ineighbor_allgatherv > 0)
    printf("  %8zu %s\n", c->MPI_Ineighbor_allgatherv, "MPI_Ineighbor_allgatherv");
  
  if (c->MPI_Ineighbor_alltoall > 0)
    printf("  %8zu %s\n", c->MPI_Ineighbor_alltoall, "MPI_Ineighbor_alltoall");
  
  if (c->MPI_Ineighbor_alltoallv > 0)
    printf("  %8zu %s\n", c->MPI_Ineighbor_alltoallv, "MPI_Ineighbor_alltoallv");
  
  if (c->MPI_Ineighbor_alltoallw > 0)
    printf("  %8zu %s\n", c->MPI_Ineighbor_alltoallw, "MPI_Ineighbor_alltoallw");
  
  if (c->MPI_Info_create > 0)
    printf("  %8zu %s\n", c->MPI_Info_create, "MPI_Info_create");
  
  if (c->MPI_Info_delete > 0)
    printf("  %8zu %s\n", c->MPI_Info_delete, "MPI_Info_delete");
  
  if (c->MPI_Info_dup > 0)
    printf("  %8zu %s\n", c->MPI_Info_dup, "MPI_Info_dup");
  
  if (c->MPI_Info_free > 0)
    printf("  %8zu %s\n", c->MPI_Info_free, "MPI_Info_free");
  
  if (c->MPI_Info_get > 0)
    printf("  %8zu %s\n", c->MPI_Info_get, "MPI_Info_get");
  
  if (c->MPI_Info_get_nkeys > 0)
    printf("  %8zu %s\n", c->MPI_Info_get_nkeys, "MPI_Info_get_nkeys");
  
  if (c->MPI_Info_get_nthkey > 0)
    printf("  %8zu %s\n", c->MPI_Info_get_nthkey, "MPI_Info_get_nthkey");
  
  if (c->MPI_Info_get_valuelen > 0)
    printf("  %8zu %s\n", c->MPI_Info_get_valuelen, "MPI_Info_get_valuelen");
  
  if (c->MPI_Info_set > 0)
    printf("  %8zu %s\n", c->MPI_Info_set, "MPI_Info_set");
  
  if (c->MPI_Init > 0)
    printf("  %8zu %s\n", c->MPI_Init, "MPI_Init");
  
  if (c->MPI_Init_thread > 0)
    printf("  %8zu %s\n", c->MPI_Init_thread, "MPI_Init_thread");
  
  if (c->MPI_Initialized > 0)
    printf("  %8zu %s\n", c->MPI_Initialized, "MPI_Initialized");
  
  if (c->MPI_Intercomm_create > 0)
    printf("  %8zu %s\n", c->MPI_Intercomm_create, "MPI_Intercomm_create");
  
  if (c->MPI_Intercomm_merge > 0)
    printf("  %8zu %s\n", c->MPI_Intercomm_merge, "MPI_Intercomm_merge");
  
  if (c->MPI_Iprobe > 0)
    printf("  %8zu %s\n", c->MPI_Iprobe, "MPI_Iprobe");
  
  if (c->MPI_Irecv > 0)
    printf("  %8zu %s\n", c->MPI_Irecv, "MPI_Irecv");
  
  if (c->MPI_Ireduce > 0)
    printf("  %8zu %s\n", c->MPI_Ireduce, "MPI_Ireduce");
  
  if (c->MPI_Ireduce_scatter > 0)
    printf("  %8zu %s\n", c->MPI_Ireduce_scatter, "MPI_Ireduce_scatter");
  
  if (c->MPI_Ireduce_scatter_block > 0)
    printf("  %8zu %s\n", c->MPI_Ireduce_scatter_block, "MPI_Ireduce_scatter_block");
  
  if (c->MPI_Irsend > 0)
    printf("  %8zu %s\n", c->MPI_Irsend, "MPI_Irsend");
  
  if (c->MPI_Is_thread_main > 0)
    printf("  %8zu %s\n", c->MPI_Is_thread_main, "MPI_Is_thread_main");
  
  if (c->MPI_Iscan > 0)
    printf("  %8zu %s\n", c->MPI_Iscan, "MPI_Iscan");
  
  if (c->MPI_Iscatter > 0)
    printf("  %8zu %s\n", c->MPI_Iscatter, "MPI_Iscatter");
  
  if (c->MPI_Iscatterv > 0)
    printf("  %8zu %s\n", c->MPI_Iscatterv, "MPI_Iscatterv");
  
  if (c->MPI_Isend > 0)
    printf("  %8zu %s\n", c->MPI_Isend, "MPI_Isend");
  
  if (c->MPI_Issend > 0)
    printf("  %8zu %s\n", c->MPI_Issend, "MPI_Issend");
  
  if (c->MPI_Keyval_create > 0)
    printf("  %8zu %s\n", c->MPI_Keyval_create, "MPI_Keyval_create");
  
  if (c->MPI_Keyval_free > 0)
    printf("  %8zu %s\n", c->MPI_Keyval_free, "MPI_Keyval_free");
  
  if (c->MPI_Lookup_name > 0)
    printf("  %8zu %s\n", c->MPI_Lookup_name, "MPI_Lookup_name");
  
  if (c->MPI_Mprobe > 0)
    printf("  %8zu %s\n", c->MPI_Mprobe, "MPI_Mprobe");
  
  if (c->MPI_Mrecv > 0)
    printf("  %8zu %s\n", c->MPI_Mrecv, "MPI_Mrecv");
  
  if (c->MPI_Neighbor_allgather > 0)
    printf("  %8zu %s\n", c->MPI_Neighbor_allgather, "MPI_Neighbor_allgather");
  
  if (c->MPI_Neighbor_allgatherv > 0)
    printf("  %8zu %s\n", c->MPI_Neighbor_allgatherv, "MPI_Neighbor_allgatherv");
  
  if (c->MPI_Neighbor_alltoall > 0)
    printf("  %8zu %s\n", c->MPI_Neighbor_alltoall, "MPI_Neighbor_alltoall");
  
  if (c->MPI_Neighbor_alltoallv > 0)
    printf("  %8zu %s\n", c->MPI_Neighbor_alltoallv, "MPI_Neighbor_alltoallv");
  
  if (c->MPI_Neighbor_alltoallw > 0)
    printf("  %8zu %s\n", c->MPI_Neighbor_alltoallw, "MPI_Neighbor_alltoallw");
  
  if (c->MPI_Op_commutative > 0)
    printf("  %8zu %s\n", c->MPI_Op_commutative, "MPI_Op_commutative");
  
  if (c->MPI_Op_create > 0)
    printf("  %8zu %s\n", c->MPI_Op_create, "MPI_Op_create");
  
  if (c->MPI_Op_free > 0)
    printf("  %8zu %s\n", c->MPI_Op_free, "MPI_Op_free");
  
  if (c->MPI_Open_port > 0)
    printf("  %8zu %s\n", c->MPI_Open_port, "MPI_Open_port");
  
  if (c->MPI_Pack > 0)
    printf("  %8zu %s\n", c->MPI_Pack, "MPI_Pack");
  
  if (c->MPI_Pack_external > 0)
    printf("  %8zu %s\n", c->MPI_Pack_external, "MPI_Pack_external");
  
  if (c->MPI_Pack_external_size > 0)
    printf("  %8zu %s\n", c->MPI_Pack_external_size, "MPI_Pack_external_size");
  
  if (c->MPI_Pack_size > 0)
    printf("  %8zu %s\n", c->MPI_Pack_size, "MPI_Pack_size");
  
  if (c->MPI_Pcontrol > 0)
    printf("  %8zu %s\n", c->MPI_Pcontrol, "MPI_Pcontrol");
  
  if (c->MPI_Probe > 0)
    printf("  %8zu %s\n", c->MPI_Probe, "MPI_Probe");
  
  if (c->MPI_Publish_name > 0)
    printf("  %8zu %s\n", c->MPI_Publish_name, "MPI_Publish_name");
  
  if (c->MPI_Put > 0)
    printf("  %8zu %s\n", c->MPI_Put, "MPI_Put");
  
  if (c->MPI_Query_thread > 0)
    printf("  %8zu %s\n", c->MPI_Query_thread, "MPI_Query_thread");
  
  if (c->MPI_Raccumulate > 0)
    printf("  %8zu %s\n", c->MPI_Raccumulate, "MPI_Raccumulate");
  
  if (c->MPI_Recv > 0)
    printf("  %8zu %s\n", c->MPI_Recv, "MPI_Recv");
  
  if (c->MPI_Recv_init > 0)
    printf("  %8zu %s\n", c->MPI_Recv_init, "MPI_Recv_init");
  
  if (c->MPI_Reduce > 0)
    printf("  %8zu %s\n", c->MPI_Reduce, "MPI_Reduce");
  
  if (c->MPI_Reduce_local > 0)
    printf("  %8zu %s\n", c->MPI_Reduce_local, "MPI_Reduce_local");
  
  if (c->MPI_Reduce_scatter > 0)
    printf("  %8zu %s\n", c->MPI_Reduce_scatter, "MPI_Reduce_scatter");
  
  if (c->MPI_Reduce_scatter_block > 0)
    printf("  %8zu %s\n", c->MPI_Reduce_scatter_block, "MPI_Reduce_scatter_block");
  
  if (c->MPI_Register_datarep > 0)
    printf("  %8zu %s\n", c->MPI_Register_datarep, "MPI_Register_datarep");
  
  if (c->MPI_Request_free > 0)
    printf("  %8zu %s\n", c->MPI_Request_free, "MPI_Request_free");
  
  if (c->MPI_Request_get_status > 0)
    printf("  %8zu %s\n", c->MPI_Request_get_status, "MPI_Request_get_status");
  
  if (c->MPI_Rget > 0)
    printf("  %8zu %s\n", c->MPI_Rget, "MPI_Rget");
  
  if (c->MPI_Rget_accumulate > 0)
    printf("  %8zu %s\n", c->MPI_Rget_accumulate, "MPI_Rget_accumulate");
  
  if (c->MPI_Rput > 0)
    printf("  %8zu %s\n", c->MPI_Rput, "MPI_Rput");
  
  if (c->MPI_Rsend > 0)
    printf("  %8zu %s\n", c->MPI_Rsend, "MPI_Rsend");
  
  if (c->MPI_Rsend_init > 0)
    printf("  %8zu %s\n", c->MPI_Rsend_init, "MPI_Rsend_init");
  
  if (c->MPI_Scan > 0)
    printf("  %8zu %s\n", c->MPI_Scan, "MPI_Scan");
  
  if (c->MPI_Scatter > 0)
    printf("  %8zu %s\n", c->MPI_Scatter, "MPI_Scatter");
  
  if (c->MPI_Scatterv > 0)
    printf("  %8zu %s\n", c->MPI_Scatterv, "MPI_Scatterv");
  
  if (c->MPI_Send > 0)
    printf("  %8zu %s\n", c->MPI_Send, "MPI_Send");
  
  if (c->MPI_Send_init > 0)
    printf("  %8zu %s\n", c->MPI_Send_init, "MPI_Send_init");
  
  if (c->MPI_Sendrecv > 0)
    printf("  %8zu %s\n", c->MPI_Sendrecv, "MPI_Sendrecv");
  
  if (c->MPI_Sendrecv_replace > 0)
    printf("  %8zu %s\n", c->MPI_Sendrecv_replace, "MPI_Sendrecv_replace");
  
  if (c->MPI_Ssend > 0)
    printf("  %8zu %s\n", c->MPI_Ssend, "MPI_Ssend");
  
  if (c->MPI_Ssend_init > 0)
    printf("  %8zu %s\n", c->MPI_Ssend_init, "MPI_Ssend_init");
  
  if (c->MPI_Start > 0)
    printf("  %8zu %s\n", c->MPI_Start, "MPI_Start");
  
  if (c->MPI_Startall > 0)
    printf("  %8zu %s\n", c->MPI_Startall, "MPI_Startall");
  
  if (c->MPI_Status_set_cancelled > 0)
    printf("  %8zu %s\n", c->MPI_Status_set_cancelled, "MPI_Status_set_cancelled");
  
  if (c->MPI_Status_set_elements > 0)
    printf("  %8zu %s\n", c->MPI_Status_set_elements, "MPI_Status_set_elements");
  
  if (c->MPI_Status_set_elements_x > 0)
    printf("  %8zu %s\n", c->MPI_Status_set_elements_x, "MPI_Status_set_elements_x");
  
  if (c->MPI_Test > 0)
    printf("  %8zu %s\n", c->MPI_Test, "MPI_Test");
  
  if (c->MPI_Test_cancelled > 0)
    printf("  %8zu %s\n", c->MPI_Test_cancelled, "MPI_Test_cancelled");
  
  if (c->MPI_Testall > 0)
    printf("  %8zu %s\n", c->MPI_Testall, "MPI_Testall");
  
  if (c->MPI_Testany > 0)
    printf("  %8zu %s\n", c->MPI_Testany, "MPI_Testany");
  
  if (c->MPI_Testsome > 0)
    printf("  %8zu %s\n", c->MPI_Testsome, "MPI_Testsome");
  
  if (c->MPI_Topo_test > 0)
    printf("  %8zu %s\n", c->MPI_Topo_test, "MPI_Topo_test");
  
  if (c->MPI_Type_commit > 0)
    printf("  %8zu %s\n", c->MPI_Type_commit, "MPI_Type_commit");
  
  if (c->MPI_Type_contiguous > 0)
    printf("  %8zu %s\n", c->MPI_Type_contiguous, "MPI_Type_contiguous");
  
  if (c->MPI_Type_create_darray > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_darray, "MPI_Type_create_darray");
  
  if (c->MPI_Type_create_f90_complex > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_f90_complex, "MPI_Type_create_f90_complex");
  
  if (c->MPI_Type_create_f90_integer > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_f90_integer, "MPI_Type_create_f90_integer");
  
  if (c->MPI_Type_create_f90_real > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_f90_real, "MPI_Type_create_f90_real");
  
  if (c->MPI_Type_create_hindexed > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_hindexed, "MPI_Type_create_hindexed");
  
  if (c->MPI_Type_create_hindexed_block > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_hindexed_block, "MPI_Type_create_hindexed_block");
  
  if (c->MPI_Type_create_hvector > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_hvector, "MPI_Type_create_hvector");
  
  if (c->MPI_Type_create_indexed_block > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_indexed_block, "MPI_Type_create_indexed_block");
  
  if (c->MPI_Type_create_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_keyval, "MPI_Type_create_keyval");
  
  if (c->MPI_Type_create_resized > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_resized, "MPI_Type_create_resized");
  
  if (c->MPI_Type_create_struct > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_struct, "MPI_Type_create_struct");
  
  if (c->MPI_Type_create_subarray > 0)
    printf("  %8zu %s\n", c->MPI_Type_create_subarray, "MPI_Type_create_subarray");
  
  if (c->MPI_Type_delete_attr > 0)
    printf("  %8zu %s\n", c->MPI_Type_delete_attr, "MPI_Type_delete_attr");
  
  if (c->MPI_Type_dup > 0)
    printf("  %8zu %s\n", c->MPI_Type_dup, "MPI_Type_dup");
  
  if (c->MPI_Type_extent > 0)
    printf("  %8zu %s\n", c->MPI_Type_extent, "MPI_Type_extent");
  
  if (c->MPI_Type_free > 0)
    printf("  %8zu %s\n", c->MPI_Type_free, "MPI_Type_free");
  
  if (c->MPI_Type_free_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Type_free_keyval, "MPI_Type_free_keyval");
  
  if (c->MPI_Type_get_attr > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_attr, "MPI_Type_get_attr");
  
  if (c->MPI_Type_get_contents > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_contents, "MPI_Type_get_contents");
  
  if (c->MPI_Type_get_envelope > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_envelope, "MPI_Type_get_envelope");
  
  if (c->MPI_Type_get_extent > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_extent, "MPI_Type_get_extent");
  
  if (c->MPI_Type_get_extent_x > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_extent_x, "MPI_Type_get_extent_x");
  
  if (c->MPI_Type_get_name > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_name, "MPI_Type_get_name");
  
  if (c->MPI_Type_get_true_extent > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_true_extent, "MPI_Type_get_true_extent");
  
  if (c->MPI_Type_get_true_extent_x > 0)
    printf("  %8zu %s\n", c->MPI_Type_get_true_extent_x, "MPI_Type_get_true_extent_x");
  
  if (c->MPI_Type_hindexed > 0)
    printf("  %8zu %s\n", c->MPI_Type_hindexed, "MPI_Type_hindexed");
  
  if (c->MPI_Type_hvector > 0)
    printf("  %8zu %s\n", c->MPI_Type_hvector, "MPI_Type_hvector");
  
  if (c->MPI_Type_indexed > 0)
    printf("  %8zu %s\n", c->MPI_Type_indexed, "MPI_Type_indexed");
  
  if (c->MPI_Type_lb > 0)
    printf("  %8zu %s\n", c->MPI_Type_lb, "MPI_Type_lb");
  
  if (c->MPI_Type_match_size > 0)
    printf("  %8zu %s\n", c->MPI_Type_match_size, "MPI_Type_match_size");
  
  if (c->MPI_Type_set_attr > 0)
    printf("  %8zu %s\n", c->MPI_Type_set_attr, "MPI_Type_set_attr");
  
  if (c->MPI_Type_set_name > 0)
    printf("  %8zu %s\n", c->MPI_Type_set_name, "MPI_Type_set_name");
  
  if (c->MPI_Type_size > 0)
    printf("  %8zu %s\n", c->MPI_Type_size, "MPI_Type_size");
  
  if (c->MPI_Type_size_x > 0)
    printf("  %8zu %s\n", c->MPI_Type_size_x, "MPI_Type_size_x");
  
  if (c->MPI_Type_struct > 0)
    printf("  %8zu %s\n", c->MPI_Type_struct, "MPI_Type_struct");
  
  if (c->MPI_Type_ub > 0)
    printf("  %8zu %s\n", c->MPI_Type_ub, "MPI_Type_ub");
  
  if (c->MPI_Type_vector > 0)
    printf("  %8zu %s\n", c->MPI_Type_vector, "MPI_Type_vector");
  
  if (c->MPI_Unpack > 0)
    printf("  %8zu %s\n", c->MPI_Unpack, "MPI_Unpack");
  
  if (c->MPI_Unpack_external > 0)
    printf("  %8zu %s\n", c->MPI_Unpack_external, "MPI_Unpack_external");
  
  if (c->MPI_Unpublish_name > 0)
    printf("  %8zu %s\n", c->MPI_Unpublish_name, "MPI_Unpublish_name");
  
  if (c->MPI_Wait > 0)
    printf("  %8zu %s\n", c->MPI_Wait, "MPI_Wait");
  
  if (c->MPI_Waitall > 0)
    printf("  %8zu %s\n", c->MPI_Waitall, "MPI_Waitall");
  
  if (c->MPI_Waitany > 0)
    printf("  %8zu %s\n", c->MPI_Waitany, "MPI_Waitany");
  
  if (c->MPI_Waitsome > 0)
    printf("  %8zu %s\n", c->MPI_Waitsome, "MPI_Waitsome");
  
  if (c->MPI_Win_allocate > 0)
    printf("  %8zu %s\n", c->MPI_Win_allocate, "MPI_Win_allocate");
  
  if (c->MPI_Win_allocate_shared > 0)
    printf("  %8zu %s\n", c->MPI_Win_allocate_shared, "MPI_Win_allocate_shared");
  
  if (c->MPI_Win_attach > 0)
    printf("  %8zu %s\n", c->MPI_Win_attach, "MPI_Win_attach");
  
  if (c->MPI_Win_call_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Win_call_errhandler, "MPI_Win_call_errhandler");
  
  if (c->MPI_Win_complete > 0)
    printf("  %8zu %s\n", c->MPI_Win_complete, "MPI_Win_complete");
  
  if (c->MPI_Win_create > 0)
    printf("  %8zu %s\n", c->MPI_Win_create, "MPI_Win_create");
  
  if (c->MPI_Win_create_dynamic > 0)
    printf("  %8zu %s\n", c->MPI_Win_create_dynamic, "MPI_Win_create_dynamic");
  
  if (c->MPI_Win_create_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Win_create_errhandler, "MPI_Win_create_errhandler");
  
  if (c->MPI_Win_create_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Win_create_keyval, "MPI_Win_create_keyval");
  
  if (c->MPI_Win_delete_attr > 0)
    printf("  %8zu %s\n", c->MPI_Win_delete_attr, "MPI_Win_delete_attr");
  
  if (c->MPI_Win_detach > 0)
    printf("  %8zu %s\n", c->MPI_Win_detach, "MPI_Win_detach");
  
  if (c->MPI_Win_fence > 0)
    printf("  %8zu %s\n", c->MPI_Win_fence, "MPI_Win_fence");
  
  if (c->MPI_Win_flush > 0)
    printf("  %8zu %s\n", c->MPI_Win_flush, "MPI_Win_flush");
  
  if (c->MPI_Win_flush_all > 0)
    printf("  %8zu %s\n", c->MPI_Win_flush_all, "MPI_Win_flush_all");
  
  if (c->MPI_Win_flush_local > 0)
    printf("  %8zu %s\n", c->MPI_Win_flush_local, "MPI_Win_flush_local");
  
  if (c->MPI_Win_flush_local_all > 0)
    printf("  %8zu %s\n", c->MPI_Win_flush_local_all, "MPI_Win_flush_local_all");
  
  if (c->MPI_Win_free > 0)
    printf("  %8zu %s\n", c->MPI_Win_free, "MPI_Win_free");
  
  if (c->MPI_Win_free_keyval > 0)
    printf("  %8zu %s\n", c->MPI_Win_free_keyval, "MPI_Win_free_keyval");
  
  if (c->MPI_Win_get_attr > 0)
    printf("  %8zu %s\n", c->MPI_Win_get_attr, "MPI_Win_get_attr");
  
  if (c->MPI_Win_get_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Win_get_errhandler, "MPI_Win_get_errhandler");
  
  if (c->MPI_Win_get_group > 0)
    printf("  %8zu %s\n", c->MPI_Win_get_group, "MPI_Win_get_group");
  
  if (c->MPI_Win_get_info > 0)
    printf("  %8zu %s\n", c->MPI_Win_get_info, "MPI_Win_get_info");
  
  if (c->MPI_Win_get_name > 0)
    printf("  %8zu %s\n", c->MPI_Win_get_name, "MPI_Win_get_name");
  
  if (c->MPI_Win_lock > 0)
    printf("  %8zu %s\n", c->MPI_Win_lock, "MPI_Win_lock");
  
  if (c->MPI_Win_lock_all > 0)
    printf("  %8zu %s\n", c->MPI_Win_lock_all, "MPI_Win_lock_all");
  
  if (c->MPI_Win_post > 0)
    printf("  %8zu %s\n", c->MPI_Win_post, "MPI_Win_post");
  
  if (c->MPI_Win_set_attr > 0)
    printf("  %8zu %s\n", c->MPI_Win_set_attr, "MPI_Win_set_attr");
  
  if (c->MPI_Win_set_errhandler > 0)
    printf("  %8zu %s\n", c->MPI_Win_set_errhandler, "MPI_Win_set_errhandler");
  
  if (c->MPI_Win_set_info > 0)
    printf("  %8zu %s\n", c->MPI_Win_set_info, "MPI_Win_set_info");
  
  if (c->MPI_Win_set_name > 0)
    printf("  %8zu %s\n", c->MPI_Win_set_name, "MPI_Win_set_name");
  
  if (c->MPI_Win_shared_query > 0)
    printf("  %8zu %s\n", c->MPI_Win_shared_query, "MPI_Win_shared_query");
  
  if (c->MPI_Win_start > 0)
    printf("  %8zu %s\n", c->MPI_Win_start, "MPI_Win_start");
  
  if (c->MPI_Win_sync > 0)
    printf("  %8zu %s\n", c->MPI_Win_sync, "MPI_Win_sync");
  
  if (c->MPI_Win_test > 0)
    printf("  %8zu %s\n", c->MPI_Win_test, "MPI_Win_test");
  
  if (c->MPI_Win_unlock > 0)
    printf("  %8zu %s\n", c->MPI_Win_unlock, "MPI_Win_unlock");
  
  if (c->MPI_Win_unlock_all > 0)
    printf("  %8zu %s\n", c->MPI_Win_unlock_all, "MPI_Win_unlock_all");
  
  if (c->MPI_Win_wait > 0)
    printf("  %8zu %s\n", c->MPI_Win_wait, "MPI_Win_wait");
  
  if (c->MPI_Wtick > 0)
    printf("  %8zu %s\n", c->MPI_Wtick, "MPI_Wtick");
  
  if (c->MPI_Wtime > 0)
    printf("  %8zu %s\n", c->MPI_Wtime, "MPI_Wtime");
  
}


/** \brief Helper function to initialize counter struct.
 *
 * \details This function will set all counters in \p counter to zero.
 *
 *
 * \param c \ref counter struct to be initialized.
 */
static void init_counters(struct counter *c)
{
  
  c->MPI_Abort = metric_atomic_init(0);
  
  c->MPI_Accumulate = metric_atomic_init(0);
  
  c->MPI_Add_error_class = metric_atomic_init(0);
  
  c->MPI_Add_error_code = metric_atomic_init(0);
  
  c->MPI_Add_error_string = metric_atomic_init(0);
  
  c->MPI_Address = metric_atomic_init(0);
  
  c->MPI_Allgather = metric_atomic_init(0);
  
  c->MPI_Allgatherv = metric_atomic_init(0);
  
  c->MPI_Alloc_mem = metric_atomic_init(0);
  
  c->MPI_Allreduce = metric_atomic_init(0);
  
  c->MPI_Alltoall = metric_atomic_init(0);
  
  c->MPI_Alltoallv = metric_atomic_init(0);
  
  c->MPI_Alltoallw = metric_atomic_init(0);
  
  c->MPI_Attr_delete = metric_atomic_init(0);
  
  c->MPI_Attr_get = metric_atomic_init(0);
  
  c->MPI_Attr_put = metric_atomic_init(0);
  
  c->MPI_Barrier = metric_atomic_init(0);
  
  c->MPI_Bcast = metric_atomic_init(0);
  
  c->MPI_Bsend = metric_atomic_init(0);
  
  c->MPI_Bsend_init = metric_atomic_init(0);
  
  c->MPI_Buffer_attach = metric_atomic_init(0);
  
  c->MPI_Buffer_detach = metric_atomic_init(0);
  
  c->MPI_Cancel = metric_atomic_init(0);
  
  c->MPI_Cart_coords = metric_atomic_init(0);
  
  c->MPI_Cart_create = metric_atomic_init(0);
  
  c->MPI_Cart_get = metric_atomic_init(0);
  
  c->MPI_Cart_map = metric_atomic_init(0);
  
  c->MPI_Cart_rank = metric_atomic_init(0);
  
  c->MPI_Cart_shift = metric_atomic_init(0);
  
  c->MPI_Cart_sub = metric_atomic_init(0);
  
  c->MPI_Cartdim_get = metric_atomic_init(0);
  
  c->MPI_Close_port = metric_atomic_init(0);
  
  c->MPI_Comm_accept = metric_atomic_init(0);
  
  c->MPI_Comm_call_errhandler = metric_atomic_init(0);
  
  c->MPI_Comm_compare = metric_atomic_init(0);
  
  c->MPI_Comm_connect = metric_atomic_init(0);
  
  c->MPI_Comm_create = metric_atomic_init(0);
  
  c->MPI_Comm_create_errhandler = metric_atomic_init(0);
  
  c->MPI_Comm_create_group = metric_atomic_init(0);
  
  c->MPI_Comm_create_keyval = metric_atomic_init(0);
  
  c->MPI_Comm_delete_attr = metric_atomic_init(0);
  
  c->MPI_Comm_disconnect = metric_atomic_init(0);
  
  c->MPI_Comm_dup = metric_atomic_init(0);
  
  c->MPI_Comm_dup_with_info = metric_atomic_init(0);
  
  c->MPI_Comm_free = metric_atomic_init(0);
  
  c->MPI_Comm_free_keyval = metric_atomic_init(0);
  
  c->MPI_Comm_get_attr = metric_atomic_init(0);
  
  c->MPI_Comm_get_errhandler = metric_atomic_init(0);
  
  c->MPI_Comm_get_info = metric_atomic_init(0);
  
  c->MPI_Comm_get_name = metric_atomic_init(0);
  
  c->MPI_Comm_get_parent = metric_atomic_init(0);
  
  c->MPI_Comm_group = metric_atomic_init(0);
  
  c->MPI_Comm_idup = metric_atomic_init(0);
  
  c->MPI_Comm_join = metric_atomic_init(0);
  
  c->MPI_Comm_rank = metric_atomic_init(0);
  
  c->MPI_Comm_remote_group = metric_atomic_init(0);
  
  c->MPI_Comm_remote_size = metric_atomic_init(0);
  
  c->MPI_Comm_set_attr = metric_atomic_init(0);
  
  c->MPI_Comm_set_errhandler = metric_atomic_init(0);
  
  c->MPI_Comm_set_info = metric_atomic_init(0);
  
  c->MPI_Comm_set_name = metric_atomic_init(0);
  
  c->MPI_Comm_size = metric_atomic_init(0);
  
  c->MPI_Comm_split = metric_atomic_init(0);
  
  c->MPI_Comm_split_type = metric_atomic_init(0);
  
  c->MPI_Comm_test_inter = metric_atomic_init(0);
  
  c->MPI_Compare_and_swap = metric_atomic_init(0);
  
  c->MPI_Dims_create = metric_atomic_init(0);
  
  c->MPI_Dist_graph_create = metric_atomic_init(0);
  
  c->MPI_Dist_graph_create_adjacent = metric_atomic_init(0);
  
  c->MPI_Dist_graph_neighbors = metric_atomic_init(0);
  
  c->MPI_Dist_graph_neighbors_count = metric_atomic_init(0);
  
  c->MPI_Errhandler_create = metric_atomic_init(0);
  
  c->MPI_Errhandler_free = metric_atomic_init(0);
  
  c->MPI_Errhandler_get = metric_atomic_init(0);
  
  c->MPI_Errhandler_set = metric_atomic_init(0);
  
  c->MPI_Error_class = metric_atomic_init(0);
  
  c->MPI_Error_string = metric_atomic_init(0);
  
  c->MPI_Exscan = metric_atomic_init(0);
  
  c->MPI_Fetch_and_op = metric_atomic_init(0);
  
  c->MPI_File_call_errhandler = metric_atomic_init(0);
  
  c->MPI_File_close = metric_atomic_init(0);
  
  c->MPI_File_create_errhandler = metric_atomic_init(0);
  
  c->MPI_File_delete = metric_atomic_init(0);
  
  c->MPI_File_get_amode = metric_atomic_init(0);
  
  c->MPI_File_get_atomicity = metric_atomic_init(0);
  
  c->MPI_File_get_byte_offset = metric_atomic_init(0);
  
  c->MPI_File_get_errhandler = metric_atomic_init(0);
  
  c->MPI_File_get_group = metric_atomic_init(0);
  
  c->MPI_File_get_info = metric_atomic_init(0);
  
  c->MPI_File_get_position = metric_atomic_init(0);
  
  c->MPI_File_get_position_shared = metric_atomic_init(0);
  
  c->MPI_File_get_size = metric_atomic_init(0);
  
  c->MPI_File_get_type_extent = metric_atomic_init(0);
  
  c->MPI_File_get_view = metric_atomic_init(0);
  
  c->MPI_File_iread = metric_atomic_init(0);
  
  c->MPI_File_iread_all = metric_atomic_init(0);
  
  c->MPI_File_iread_at = metric_atomic_init(0);
  
  c->MPI_File_iread_at_all = metric_atomic_init(0);
  
  c->MPI_File_iread_shared = metric_atomic_init(0);
  
  c->MPI_File_iwrite = metric_atomic_init(0);
  
  c->MPI_File_iwrite_all = metric_atomic_init(0);
  
  c->MPI_File_iwrite_at = metric_atomic_init(0);
  
  c->MPI_File_iwrite_at_all = metric_atomic_init(0);
  
  c->MPI_File_iwrite_shared = metric_atomic_init(0);
  
  c->MPI_File_open = metric_atomic_init(0);
  
  c->MPI_File_preallocate = metric_atomic_init(0);
  
  c->MPI_File_read = metric_atomic_init(0);
  
  c->MPI_File_read_all = metric_atomic_init(0);
  
  c->MPI_File_read_all_begin = metric_atomic_init(0);
  
  c->MPI_File_read_all_end = metric_atomic_init(0);
  
  c->MPI_File_read_at = metric_atomic_init(0);
  
  c->MPI_File_read_at_all = metric_atomic_init(0);
  
  c->MPI_File_read_at_all_begin = metric_atomic_init(0);
  
  c->MPI_File_read_at_all_end = metric_atomic_init(0);
  
  c->MPI_File_read_ordered = metric_atomic_init(0);
  
  c->MPI_File_read_ordered_begin = metric_atomic_init(0);
  
  c->MPI_File_read_ordered_end = metric_atomic_init(0);
  
  c->MPI_File_read_shared = metric_atomic_init(0);
  
  c->MPI_File_seek = metric_atomic_init(0);
  
  c->MPI_File_seek_shared = metric_atomic_init(0);
  
  c->MPI_File_set_atomicity = metric_atomic_init(0);
  
  c->MPI_File_set_errhandler = metric_atomic_init(0);
  
  c->MPI_File_set_info = metric_atomic_init(0);
  
  c->MPI_File_set_size = metric_atomic_init(0);
  
  c->MPI_File_set_view = metric_atomic_init(0);
  
  c->MPI_File_sync = metric_atomic_init(0);
  
  c->MPI_File_write = metric_atomic_init(0);
  
  c->MPI_File_write_all = metric_atomic_init(0);
  
  c->MPI_File_write_all_begin = metric_atomic_init(0);
  
  c->MPI_File_write_all_end = metric_atomic_init(0);
  
  c->MPI_File_write_at = metric_atomic_init(0);
  
  c->MPI_File_write_at_all = metric_atomic_init(0);
  
  c->MPI_File_write_at_all_begin = metric_atomic_init(0);
  
  c->MPI_File_write_at_all_end = metric_atomic_init(0);
  
  c->MPI_File_write_ordered = metric_atomic_init(0);
  
  c->MPI_File_write_ordered_begin = metric_atomic_init(0);
  
  c->MPI_File_write_ordered_end = metric_atomic_init(0);
  
  c->MPI_File_write_shared = metric_atomic_init(0);
  
  c->MPI_Finalized = metric_atomic_init(0);
  
  c->MPI_Free_mem = metric_atomic_init(0);
  
  c->MPI_Gather = metric_atomic_init(0);
  
  c->MPI_Gatherv = metric_atomic_init(0);
  
  c->MPI_Get = metric_atomic_init(0);
  
  c->MPI_Get_accumulate = metric_atomic_init(0);
  
  c->MPI_Get_address = metric_atomic_init(0);
  
  c->MPI_Get_count = metric_atomic_init(0);
  
  c->MPI_Get_elements = metric_atomic_init(0);
  
  c->MPI_Get_elements_x = metric_atomic_init(0);
  
  c->MPI_Get_library_version = metric_atomic_init(0);
  
  c->MPI_Get_processor_name = metric_atomic_init(0);
  
  c->MPI_Get_version = metric_atomic_init(0);
  
  c->MPI_Graph_create = metric_atomic_init(0);
  
  c->MPI_Graph_get = metric_atomic_init(0);
  
  c->MPI_Graph_map = metric_atomic_init(0);
  
  c->MPI_Graph_neighbors = metric_atomic_init(0);
  
  c->MPI_Graph_neighbors_count = metric_atomic_init(0);
  
  c->MPI_Graphdims_get = metric_atomic_init(0);
  
  c->MPI_Grequest_complete = metric_atomic_init(0);
  
  c->MPI_Grequest_start = metric_atomic_init(0);
  
  c->MPI_Group_compare = metric_atomic_init(0);
  
  c->MPI_Group_difference = metric_atomic_init(0);
  
  c->MPI_Group_excl = metric_atomic_init(0);
  
  c->MPI_Group_free = metric_atomic_init(0);
  
  c->MPI_Group_incl = metric_atomic_init(0);
  
  c->MPI_Group_intersection = metric_atomic_init(0);
  
  c->MPI_Group_range_excl = metric_atomic_init(0);
  
  c->MPI_Group_range_incl = metric_atomic_init(0);
  
  c->MPI_Group_rank = metric_atomic_init(0);
  
  c->MPI_Group_size = metric_atomic_init(0);
  
  c->MPI_Group_translate_ranks = metric_atomic_init(0);
  
  c->MPI_Group_union = metric_atomic_init(0);
  
  c->MPI_Iallgather = metric_atomic_init(0);
  
  c->MPI_Iallgatherv = metric_atomic_init(0);
  
  c->MPI_Iallreduce = metric_atomic_init(0);
  
  c->MPI_Ialltoall = metric_atomic_init(0);
  
  c->MPI_Ialltoallv = metric_atomic_init(0);
  
  c->MPI_Ialltoallw = metric_atomic_init(0);
  
  c->MPI_Ibarrier = metric_atomic_init(0);
  
  c->MPI_Ibcast = metric_atomic_init(0);
  
  c->MPI_Ibsend = metric_atomic_init(0);
  
  c->MPI_Iexscan = metric_atomic_init(0);
  
  c->MPI_Igather = metric_atomic_init(0);
  
  c->MPI_Igatherv = metric_atomic_init(0);
  
  c->MPI_Improbe = metric_atomic_init(0);
  
  c->MPI_Imrecv = metric_atomic_init(0);
  
  c->MPI_Ineighbor_allgather = metric_atomic_init(0);
  
  c->MPI_Ineighbor_allgatherv = metric_atomic_init(0);
  
  c->MPI_Ineighbor_alltoall = metric_atomic_init(0);
  
  c->MPI_Ineighbor_alltoallv = metric_atomic_init(0);
  
  c->MPI_Ineighbor_alltoallw = metric_atomic_init(0);
  
  c->MPI_Info_create = metric_atomic_init(0);
  
  c->MPI_Info_delete = metric_atomic_init(0);
  
  c->MPI_Info_dup = metric_atomic_init(0);
  
  c->MPI_Info_free = metric_atomic_init(0);
  
  c->MPI_Info_get = metric_atomic_init(0);
  
  c->MPI_Info_get_nkeys = metric_atomic_init(0);
  
  c->MPI_Info_get_nthkey = metric_atomic_init(0);
  
  c->MPI_Info_get_valuelen = metric_atomic_init(0);
  
  c->MPI_Info_set = metric_atomic_init(0);
  
  c->MPI_Init = metric_atomic_init(0);
  
  c->MPI_Init_thread = metric_atomic_init(0);
  
  c->MPI_Initialized = metric_atomic_init(0);
  
  c->MPI_Intercomm_create = metric_atomic_init(0);
  
  c->MPI_Intercomm_merge = metric_atomic_init(0);
  
  c->MPI_Iprobe = metric_atomic_init(0);
  
  c->MPI_Irecv = metric_atomic_init(0);
  
  c->MPI_Ireduce = metric_atomic_init(0);
  
  c->MPI_Ireduce_scatter = metric_atomic_init(0);
  
  c->MPI_Ireduce_scatter_block = metric_atomic_init(0);
  
  c->MPI_Irsend = metric_atomic_init(0);
  
  c->MPI_Is_thread_main = metric_atomic_init(0);
  
  c->MPI_Iscan = metric_atomic_init(0);
  
  c->MPI_Iscatter = metric_atomic_init(0);
  
  c->MPI_Iscatterv = metric_atomic_init(0);
  
  c->MPI_Isend = metric_atomic_init(0);
  
  c->MPI_Issend = metric_atomic_init(0);
  
  c->MPI_Keyval_create = metric_atomic_init(0);
  
  c->MPI_Keyval_free = metric_atomic_init(0);
  
  c->MPI_Lookup_name = metric_atomic_init(0);
  
  c->MPI_Mprobe = metric_atomic_init(0);
  
  c->MPI_Mrecv = metric_atomic_init(0);
  
  c->MPI_Neighbor_allgather = metric_atomic_init(0);
  
  c->MPI_Neighbor_allgatherv = metric_atomic_init(0);
  
  c->MPI_Neighbor_alltoall = metric_atomic_init(0);
  
  c->MPI_Neighbor_alltoallv = metric_atomic_init(0);
  
  c->MPI_Neighbor_alltoallw = metric_atomic_init(0);
  
  c->MPI_Op_commutative = metric_atomic_init(0);
  
  c->MPI_Op_create = metric_atomic_init(0);
  
  c->MPI_Op_free = metric_atomic_init(0);
  
  c->MPI_Open_port = metric_atomic_init(0);
  
  c->MPI_Pack = metric_atomic_init(0);
  
  c->MPI_Pack_external = metric_atomic_init(0);
  
  c->MPI_Pack_external_size = metric_atomic_init(0);
  
  c->MPI_Pack_size = metric_atomic_init(0);
  
  c->MPI_Pcontrol = metric_atomic_init(0);
  
  c->MPI_Probe = metric_atomic_init(0);
  
  c->MPI_Publish_name = metric_atomic_init(0);
  
  c->MPI_Put = metric_atomic_init(0);
  
  c->MPI_Query_thread = metric_atomic_init(0);
  
  c->MPI_Raccumulate = metric_atomic_init(0);
  
  c->MPI_Recv = metric_atomic_init(0);
  
  c->MPI_Recv_init = metric_atomic_init(0);
  
  c->MPI_Reduce = metric_atomic_init(0);
  
  c->MPI_Reduce_local = metric_atomic_init(0);
  
  c->MPI_Reduce_scatter = metric_atomic_init(0);
  
  c->MPI_Reduce_scatter_block = metric_atomic_init(0);
  
  c->MPI_Register_datarep = metric_atomic_init(0);
  
  c->MPI_Request_free = metric_atomic_init(0);
  
  c->MPI_Request_get_status = metric_atomic_init(0);
  
  c->MPI_Rget = metric_atomic_init(0);
  
  c->MPI_Rget_accumulate = metric_atomic_init(0);
  
  c->MPI_Rput = metric_atomic_init(0);
  
  c->MPI_Rsend = metric_atomic_init(0);
  
  c->MPI_Rsend_init = metric_atomic_init(0);
  
  c->MPI_Scan = metric_atomic_init(0);
  
  c->MPI_Scatter = metric_atomic_init(0);
  
  c->MPI_Scatterv = metric_atomic_init(0);
  
  c->MPI_Send = metric_atomic_init(0);
  
  c->MPI_Send_init = metric_atomic_init(0);
  
  c->MPI_Sendrecv = metric_atomic_init(0);
  
  c->MPI_Sendrecv_replace = metric_atomic_init(0);
  
  c->MPI_Ssend = metric_atomic_init(0);
  
  c->MPI_Ssend_init = metric_atomic_init(0);
  
  c->MPI_Start = metric_atomic_init(0);
  
  c->MPI_Startall = metric_atomic_init(0);
  
  c->MPI_Status_set_cancelled = metric_atomic_init(0);
  
  c->MPI_Status_set_elements = metric_atomic_init(0);
  
  c->MPI_Status_set_elements_x = metric_atomic_init(0);
  
  c->MPI_Test = metric_atomic_init(0);
  
  c->MPI_Test_cancelled = metric_atomic_init(0);
  
  c->MPI_Testall = metric_atomic_init(0);
  
  c->MPI_Testany = metric_atomic_init(0);
  
  c->MPI_Testsome = metric_atomic_init(0);
  
  c->MPI_Topo_test = metric_atomic_init(0);
  
  c->MPI_Type_commit = metric_atomic_init(0);
  
  c->MPI_Type_contiguous = metric_atomic_init(0);
  
  c->MPI_Type_create_darray = metric_atomic_init(0);
  
  c->MPI_Type_create_f90_complex = metric_atomic_init(0);
  
  c->MPI_Type_create_f90_integer = metric_atomic_init(0);
  
  c->MPI_Type_create_f90_real = metric_atomic_init(0);
  
  c->MPI_Type_create_hindexed = metric_atomic_init(0);
  
  c->MPI_Type_create_hindexed_block = metric_atomic_init(0);
  
  c->MPI_Type_create_hvector = metric_atomic_init(0);
  
  c->MPI_Type_create_indexed_block = metric_atomic_init(0);
  
  c->MPI_Type_create_keyval = metric_atomic_init(0);
  
  c->MPI_Type_create_resized = metric_atomic_init(0);
  
  c->MPI_Type_create_struct = metric_atomic_init(0);
  
  c->MPI_Type_create_subarray = metric_atomic_init(0);
  
  c->MPI_Type_delete_attr = metric_atomic_init(0);
  
  c->MPI_Type_dup = metric_atomic_init(0);
  
  c->MPI_Type_extent = metric_atomic_init(0);
  
  c->MPI_Type_free = metric_atomic_init(0);
  
  c->MPI_Type_free_keyval = metric_atomic_init(0);
  
  c->MPI_Type_get_attr = metric_atomic_init(0);
  
  c->MPI_Type_get_contents = metric_atomic_init(0);
  
  c->MPI_Type_get_envelope = metric_atomic_init(0);
  
  c->MPI_Type_get_extent = metric_atomic_init(0);
  
  c->MPI_Type_get_extent_x = metric_atomic_init(0);
  
  c->MPI_Type_get_name = metric_atomic_init(0);
  
  c->MPI_Type_get_true_extent = metric_atomic_init(0);
  
  c->MPI_Type_get_true_extent_x = metric_atomic_init(0);
  
  c->MPI_Type_hindexed = metric_atomic_init(0);
  
  c->MPI_Type_hvector = metric_atomic_init(0);
  
  c->MPI_Type_indexed = metric_atomic_init(0);
  
  c->MPI_Type_lb = metric_atomic_init(0);
  
  c->MPI_Type_match_size = metric_atomic_init(0);
  
  c->MPI_Type_set_attr = metric_atomic_init(0);
  
  c->MPI_Type_set_name = metric_atomic_init(0);
  
  c->MPI_Type_size = metric_atomic_init(0);
  
  c->MPI_Type_size_x = metric_atomic_init(0);
  
  c->MPI_Type_struct = metric_atomic_init(0);
  
  c->MPI_Type_ub = metric_atomic_init(0);
  
  c->MPI_Type_vector = metric_atomic_init(0);
  
  c->MPI_Unpack = metric_atomic_init(0);
  
  c->MPI_Unpack_external = metric_atomic_init(0);
  
  c->MPI_Unpublish_name = metric_atomic_init(0);
  
  c->MPI_Wait = metric_atomic_init(0);
  
  c->MPI_Waitall = metric_atomic_init(0);
  
  c->MPI_Waitany = metric_atomic_init(0);
  
  c->MPI_Waitsome = metric_atomic_init(0);
  
  c->MPI_Win_allocate = metric_atomic_init(0);
  
  c->MPI_Win_allocate_shared = metric_atomic_init(0);
  
  c->MPI_Win_attach = metric_atomic_init(0);
  
  c->MPI_Win_call_errhandler = metric_atomic_init(0);
  
  c->MPI_Win_complete = metric_atomic_init(0);
  
  c->MPI_Win_create = metric_atomic_init(0);
  
  c->MPI_Win_create_dynamic = metric_atomic_init(0);
  
  c->MPI_Win_create_errhandler = metric_atomic_init(0);
  
  c->MPI_Win_create_keyval = metric_atomic_init(0);
  
  c->MPI_Win_delete_attr = metric_atomic_init(0);
  
  c->MPI_Win_detach = metric_atomic_init(0);
  
  c->MPI_Win_fence = metric_atomic_init(0);
  
  c->MPI_Win_flush = metric_atomic_init(0);
  
  c->MPI_Win_flush_all = metric_atomic_init(0);
  
  c->MPI_Win_flush_local = metric_atomic_init(0);
  
  c->MPI_Win_flush_local_all = metric_atomic_init(0);
  
  c->MPI_Win_free = metric_atomic_init(0);
  
  c->MPI_Win_free_keyval = metric_atomic_init(0);
  
  c->MPI_Win_get_attr = metric_atomic_init(0);
  
  c->MPI_Win_get_errhandler = metric_atomic_init(0);
  
  c->MPI_Win_get_group = metric_atomic_init(0);
  
  c->MPI_Win_get_info = metric_atomic_init(0);
  
  c->MPI_Win_get_name = metric_atomic_init(0);
  
  c->MPI_Win_lock = metric_atomic_init(0);
  
  c->MPI_Win_lock_all = metric_atomic_init(0);
  
  c->MPI_Win_post = metric_atomic_init(0);
  
  c->MPI_Win_set_attr = metric_atomic_init(0);
  
  c->MPI_Win_set_errhandler = metric_atomic_init(0);
  
  c->MPI_Win_set_info = metric_atomic_init(0);
  
  c->MPI_Win_set_name = metric_atomic_init(0);
  
  c->MPI_Win_shared_query = metric_atomic_init(0);
  
  c->MPI_Win_start = metric_atomic_init(0);
  
  c->MPI_Win_sync = metric_atomic_init(0);
  
  c->MPI_Win_test = metric_atomic_init(0);
  
  c->MPI_Win_unlock = metric_atomic_init(0);
  
  c->MPI_Win_unlock_all = metric_atomic_init(0);
  
  c->MPI_Win_wait = metric_atomic_init(0);
  
  c->MPI_Wtick = metric_atomic_init(0);
  
  c->MPI_Wtime = metric_atomic_init(0);
  
}


/** \brief PnMPI module initialization hook.
 *
 * \details This function sets all counters to zero.
 */
void PNMPI_Init()
{
  init_counters(&counters);
}


/* Generate wrapper functions for all MPI calls to increment the counter on
 * every call. MPI_Finalize will be ignored, because it will be used to print
 * the statistics below. */

/* ================== C Wrappers for MPI_Abort ================== */
_EXTERN_C_ int PMPI_Abort(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Abort(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Abort);

  WRAP_MPI_CALL_PREFIX
  XMPI_Abort(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Accumulate ================== */
_EXTERN_C_ int PMPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Accumulate);

  WRAP_MPI_CALL_PREFIX
  XMPI_Accumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_class ================== */
_EXTERN_C_ int PMPI_Add_error_class(int *errorclass);
_EXTERN_C_ int MPI_Add_error_class(int *errorclass) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Add_error_class);

  WRAP_MPI_CALL_PREFIX
  XMPI_Add_error_class(errorclass);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_code ================== */
_EXTERN_C_ int PMPI_Add_error_code(int errorclass, int *errorcode);
_EXTERN_C_ int MPI_Add_error_code(int errorclass, int *errorcode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Add_error_code);

  WRAP_MPI_CALL_PREFIX
  XMPI_Add_error_code(errorclass, errorcode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Add_error_string ================== */
_EXTERN_C_ int PMPI_Add_error_string(int errorcode, const char *string);
_EXTERN_C_ int MPI_Add_error_string(int errorcode, const char *string) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Add_error_string);

  WRAP_MPI_CALL_PREFIX
  XMPI_Add_error_string(errorcode, string);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Address ================== */
_EXTERN_C_ int PMPI_Address(void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Address(void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Address);

  WRAP_MPI_CALL_PREFIX
  XMPI_Address(location, address);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Allgather ================== */
_EXTERN_C_ int PMPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Allgather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Allgatherv ================== */
_EXTERN_C_ int PMPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Allgatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alloc_mem ================== */
_EXTERN_C_ int PMPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr);
_EXTERN_C_ int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Alloc_mem);

  WRAP_MPI_CALL_PREFIX
  XMPI_Alloc_mem(size, info, baseptr);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Allreduce ================== */
_EXTERN_C_ int PMPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Allreduce);

  WRAP_MPI_CALL_PREFIX
  XMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alltoall ================== */
_EXTERN_C_ int PMPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Alltoall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alltoallv ================== */
_EXTERN_C_ int PMPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Alltoallv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Alltoallw ================== */
_EXTERN_C_ int PMPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Alltoallw);

  WRAP_MPI_CALL_PREFIX
  XMPI_Alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_delete ================== */
_EXTERN_C_ int PMPI_Attr_delete(MPI_Comm comm, int keyval);
_EXTERN_C_ int MPI_Attr_delete(MPI_Comm comm, int keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Attr_delete);

  WRAP_MPI_CALL_PREFIX
  XMPI_Attr_delete(comm, keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_get ================== */
_EXTERN_C_ int PMPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Attr_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Attr_get(comm, keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Attr_put ================== */
_EXTERN_C_ int PMPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);
_EXTERN_C_ int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Attr_put);

  WRAP_MPI_CALL_PREFIX
  XMPI_Attr_put(comm, keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Barrier ================== */
_EXTERN_C_ int PMPI_Barrier(MPI_Comm comm);
_EXTERN_C_ int MPI_Barrier(MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Barrier);

  WRAP_MPI_CALL_PREFIX
  XMPI_Barrier(comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Bcast ================== */
_EXTERN_C_ int PMPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Bcast);

  WRAP_MPI_CALL_PREFIX
  XMPI_Bcast(buffer, count, datatype, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Bsend ================== */
_EXTERN_C_ int PMPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Bsend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Bsend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Bsend_init ================== */
_EXTERN_C_ int PMPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Bsend_init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Buffer_attach ================== */
_EXTERN_C_ int PMPI_Buffer_attach(void *buffer, int size);
_EXTERN_C_ int MPI_Buffer_attach(void *buffer, int size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Buffer_attach);

  WRAP_MPI_CALL_PREFIX
  XMPI_Buffer_attach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Buffer_detach ================== */
_EXTERN_C_ int PMPI_Buffer_detach(void *buffer, int *size);
_EXTERN_C_ int MPI_Buffer_detach(void *buffer, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Buffer_detach);

  WRAP_MPI_CALL_PREFIX
  XMPI_Buffer_detach(buffer, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cancel ================== */
_EXTERN_C_ int PMPI_Cancel(MPI_Request *request);
_EXTERN_C_ int MPI_Cancel(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cancel);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cancel(request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_coords ================== */
_EXTERN_C_ int PMPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]);
_EXTERN_C_ int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_coords);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_coords(comm, rank, maxdims, coords);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_create ================== */
_EXTERN_C_ int PMPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);
_EXTERN_C_ int MPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_create(old_comm, ndims, dims, periods, reorder, comm_cart);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_get ================== */
_EXTERN_C_ int PMPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);
_EXTERN_C_ int MPI_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_get(comm, maxdims, dims, periods, coords);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_map ================== */
_EXTERN_C_ int PMPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);
_EXTERN_C_ int MPI_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_map);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_map(comm, ndims, dims, periods, newrank);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_rank ================== */
_EXTERN_C_ int PMPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank);
_EXTERN_C_ int MPI_Cart_rank(MPI_Comm comm, const int coords[], int *rank) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_rank);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_rank(comm, coords, rank);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_shift ================== */
_EXTERN_C_ int PMPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);
_EXTERN_C_ int MPI_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_shift);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_shift(comm, direction, disp, rank_source, rank_dest);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cart_sub ================== */
_EXTERN_C_ int PMPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);
_EXTERN_C_ int MPI_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cart_sub);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cart_sub(comm, remain_dims, new_comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Cartdim_get ================== */
_EXTERN_C_ int PMPI_Cartdim_get(MPI_Comm comm, int *ndims);
_EXTERN_C_ int MPI_Cartdim_get(MPI_Comm comm, int *ndims) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Cartdim_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Cartdim_get(comm, ndims);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Close_port ================== */
_EXTERN_C_ int PMPI_Close_port(const char *port_name);
_EXTERN_C_ int MPI_Close_port(const char *port_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Close_port);

  WRAP_MPI_CALL_PREFIX
  XMPI_Close_port(port_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_accept ================== */
_EXTERN_C_ int PMPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_accept);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_accept(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_call_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_call_errhandler(MPI_Comm comm, int errorcode);
_EXTERN_C_ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_call_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_call_errhandler(comm, errorcode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_compare ================== */
_EXTERN_C_ int PMPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);
_EXTERN_C_ int MPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_compare);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_compare(comm1, comm2, result);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_connect ================== */
_EXTERN_C_ int PMPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_connect);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_connect(port_name, info, root, comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create ================== */
_EXTERN_C_ int PMPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_create(comm, group, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_create_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_group ================== */
_EXTERN_C_ int PMPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_create_group);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_create_group(comm, group, tag, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_create_keyval ================== */
_EXTERN_C_ int PMPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);
_EXTERN_C_ int MPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_create_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_delete_attr ================== */
_EXTERN_C_ int PMPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval);
_EXTERN_C_ int MPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_delete_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_delete_attr(comm, comm_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_disconnect ================== */
_EXTERN_C_ int PMPI_Comm_disconnect(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_disconnect(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_disconnect);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_disconnect(comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_dup ================== */
_EXTERN_C_ int PMPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_dup);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_dup(comm, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_dup_with_info ================== */
_EXTERN_C_ int PMPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_dup_with_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_dup_with_info(comm, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_free ================== */
_EXTERN_C_ int PMPI_Comm_free(MPI_Comm *comm);
_EXTERN_C_ int MPI_Comm_free(MPI_Comm *comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_free(comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_free_keyval ================== */
_EXTERN_C_ int PMPI_Comm_free_keyval(int *comm_keyval);
_EXTERN_C_ int MPI_Comm_free_keyval(int *comm_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_free_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_free_keyval(comm_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_attr ================== */
_EXTERN_C_ int PMPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_get_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_get_attr(comm, comm_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);
_EXTERN_C_ int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_get_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_get_errhandler(comm, erhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_info ================== */
_EXTERN_C_ int PMPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used);
_EXTERN_C_ int MPI_Comm_get_info(MPI_Comm comm, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_get_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_get_info(comm, info_used);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_name ================== */
_EXTERN_C_ int PMPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);
_EXTERN_C_ int MPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_get_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_get_name(comm, comm_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_get_parent ================== */
_EXTERN_C_ int PMPI_Comm_get_parent(MPI_Comm *parent);
_EXTERN_C_ int MPI_Comm_get_parent(MPI_Comm *parent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_get_parent);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_get_parent(parent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_group ================== */
_EXTERN_C_ int PMPI_Comm_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_group);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_idup ================== */
_EXTERN_C_ int PMPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);
_EXTERN_C_ int MPI_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_idup);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_idup(comm, newcomm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_join ================== */
_EXTERN_C_ int PMPI_Comm_join(int fd, MPI_Comm *intercomm);
_EXTERN_C_ int MPI_Comm_join(int fd, MPI_Comm *intercomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_join);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_join(fd, intercomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_rank ================== */
_EXTERN_C_ int PMPI_Comm_rank(MPI_Comm comm, int *rank);
_EXTERN_C_ int MPI_Comm_rank(MPI_Comm comm, int *rank) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_rank);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_rank(comm, rank);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_remote_group ================== */
_EXTERN_C_ int PMPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group);
_EXTERN_C_ int MPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_remote_group);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_remote_group(comm, group);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_remote_size ================== */
_EXTERN_C_ int PMPI_Comm_remote_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_remote_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_remote_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_remote_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_attr ================== */
_EXTERN_C_ int PMPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_set_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_set_attr(comm, comm_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_errhandler ================== */
_EXTERN_C_ int PMPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_set_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_set_errhandler(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_info ================== */
_EXTERN_C_ int PMPI_Comm_set_info(MPI_Comm comm, MPI_Info info);
_EXTERN_C_ int MPI_Comm_set_info(MPI_Comm comm, MPI_Info info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_set_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_set_info(comm, info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_set_name ================== */
_EXTERN_C_ int PMPI_Comm_set_name(MPI_Comm comm, const char *comm_name);
_EXTERN_C_ int MPI_Comm_set_name(MPI_Comm comm, const char *comm_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_set_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_set_name(comm, comm_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_size ================== */
_EXTERN_C_ int PMPI_Comm_size(MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Comm_size(MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_size(comm, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_split ================== */
_EXTERN_C_ int PMPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_split);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_split(comm, color, key, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_split_type ================== */
_EXTERN_C_ int PMPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_split_type);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_split_type(comm, split_type, key, info, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Comm_test_inter ================== */
_EXTERN_C_ int PMPI_Comm_test_inter(MPI_Comm comm, int *flag);
_EXTERN_C_ int MPI_Comm_test_inter(MPI_Comm comm, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Comm_test_inter);

  WRAP_MPI_CALL_PREFIX
  XMPI_Comm_test_inter(comm, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Compare_and_swap ================== */
_EXTERN_C_ int PMPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);
_EXTERN_C_ int MPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Compare_and_swap);

  WRAP_MPI_CALL_PREFIX
  XMPI_Compare_and_swap(origin_addr, compare_addr, result_addr, datatype, target_rank, target_disp, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dims_create ================== */
_EXTERN_C_ int PMPI_Dims_create(int nnodes, int ndims, int dims[]);
_EXTERN_C_ int MPI_Dims_create(int nnodes, int ndims, int dims[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Dims_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Dims_create(nnodes, ndims, dims);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_create ================== */
_EXTERN_C_ int PMPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);
_EXTERN_C_ int MPI_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Dist_graph_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Dist_graph_create(comm_old, n, nodes, degrees, targets, weights, info, reorder, newcomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_create_adjacent ================== */
_EXTERN_C_ int PMPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);
_EXTERN_C_ int MPI_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Dist_graph_create_adjacent);

  WRAP_MPI_CALL_PREFIX
  XMPI_Dist_graph_create_adjacent(comm_old, indegree, sources, sourceweights, outdegree, destinations, destweights, info, reorder, comm_dist_graph);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_neighbors ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);
_EXTERN_C_ int MPI_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Dist_graph_neighbors);

  WRAP_MPI_CALL_PREFIX
  XMPI_Dist_graph_neighbors(comm, maxindegree, sources, sourceweights, maxoutdegree, destinations, destweights);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Dist_graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);
_EXTERN_C_ int MPI_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Dist_graph_neighbors_count);

  WRAP_MPI_CALL_PREFIX
  XMPI_Dist_graph_neighbors_count(comm, inneighbors, outneighbors, weighted);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_create ================== */
_EXTERN_C_ int PMPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Errhandler_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Errhandler_create(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_free ================== */
_EXTERN_C_ int PMPI_Errhandler_free(MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_free(MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Errhandler_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Errhandler_free(errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_get ================== */
_EXTERN_C_ int PMPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Errhandler_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Errhandler_get(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Errhandler_set ================== */
_EXTERN_C_ int PMPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Errhandler_set);

  WRAP_MPI_CALL_PREFIX
  XMPI_Errhandler_set(comm, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Error_class ================== */
_EXTERN_C_ int PMPI_Error_class(int errorcode, int *errorclass);
_EXTERN_C_ int MPI_Error_class(int errorcode, int *errorclass) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Error_class);

  WRAP_MPI_CALL_PREFIX
  XMPI_Error_class(errorcode, errorclass);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Error_string ================== */
_EXTERN_C_ int PMPI_Error_string(int errorcode, char *string, int *resultlen);
_EXTERN_C_ int MPI_Error_string(int errorcode, char *string, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Error_string);

  WRAP_MPI_CALL_PREFIX
  XMPI_Error_string(errorcode, string, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Exscan ================== */
_EXTERN_C_ int PMPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Exscan);

  WRAP_MPI_CALL_PREFIX
  XMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Fetch_and_op ================== */
_EXTERN_C_ int PMPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Fetch_and_op);

  WRAP_MPI_CALL_PREFIX
  XMPI_Fetch_and_op(origin_addr, result_addr, datatype, target_rank, target_disp, op, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_call_errhandler ================== */
_EXTERN_C_ int PMPI_File_call_errhandler(MPI_File fh, int errorcode);
_EXTERN_C_ int MPI_File_call_errhandler(MPI_File fh, int errorcode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_call_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_call_errhandler(fh, errorcode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_close ================== */
_EXTERN_C_ int PMPI_File_close(MPI_File *fh);
_EXTERN_C_ int MPI_File_close(MPI_File *fh) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_close);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_close(fh);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_create_errhandler ================== */
_EXTERN_C_ int PMPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_create_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_delete ================== */
_EXTERN_C_ int PMPI_File_delete(const char *filename, MPI_Info info);
_EXTERN_C_ int MPI_File_delete(const char *filename, MPI_Info info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_delete);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_delete(filename, info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_amode ================== */
_EXTERN_C_ int PMPI_File_get_amode(MPI_File fh, int *amode);
_EXTERN_C_ int MPI_File_get_amode(MPI_File fh, int *amode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_amode);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_amode(fh, amode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_atomicity ================== */
_EXTERN_C_ int PMPI_File_get_atomicity(MPI_File fh, int *flag);
_EXTERN_C_ int MPI_File_get_atomicity(MPI_File fh, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_atomicity);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_byte_offset ================== */
_EXTERN_C_ int PMPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);
_EXTERN_C_ int MPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_byte_offset);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_byte_offset(fh, offset, disp);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_errhandler ================== */
_EXTERN_C_ int PMPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_group ================== */
_EXTERN_C_ int PMPI_File_get_group(MPI_File fh, MPI_Group *group);
_EXTERN_C_ int MPI_File_get_group(MPI_File fh, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_group);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_group(fh, group);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_info ================== */
_EXTERN_C_ int PMPI_File_get_info(MPI_File fh, MPI_Info *info_used);
_EXTERN_C_ int MPI_File_get_info(MPI_File fh, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_info(fh, info_used);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_position ================== */
_EXTERN_C_ int PMPI_File_get_position(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_position);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_position(fh, offset);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_position_shared ================== */
_EXTERN_C_ int PMPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset);
_EXTERN_C_ int MPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_position_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_position_shared(fh, offset);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_size ================== */
_EXTERN_C_ int PMPI_File_get_size(MPI_File fh, MPI_Offset *size);
_EXTERN_C_ int MPI_File_get_size(MPI_File fh, MPI_Offset *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_type_extent ================== */
_EXTERN_C_ int PMPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);
_EXTERN_C_ int MPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_type_extent);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_type_extent(fh, datatype, extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_get_view ================== */
_EXTERN_C_ int PMPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);
_EXTERN_C_ int MPI_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_get_view);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_get_view(fh, disp, etype, filetype, datarep);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread ================== */
_EXTERN_C_ int PMPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iread);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iread(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_all ================== */
_EXTERN_C_ int PMPI_File_iread_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iread_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iread_all(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_at ================== */
_EXTERN_C_ int PMPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iread_at);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iread_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_at_all ================== */
_EXTERN_C_ int PMPI_File_iread_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iread_at_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iread_at_all(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iread_shared ================== */
_EXTERN_C_ int PMPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iread_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iread_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite ================== */
_EXTERN_C_ int PMPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iwrite);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iwrite(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_all ================== */
_EXTERN_C_ int PMPI_File_iwrite_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iwrite_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iwrite_all(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_at ================== */
_EXTERN_C_ int PMPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iwrite_at);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iwrite_at(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_at_all ================== */
_EXTERN_C_ int PMPI_File_iwrite_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iwrite_at_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iwrite_at_all(fh, offset, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_iwrite_shared ================== */
_EXTERN_C_ int PMPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
_EXTERN_C_ int MPI_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_iwrite_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_iwrite_shared(fh, buf, count, datatype, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_open ================== */
_EXTERN_C_ int PMPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);
_EXTERN_C_ int MPI_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_open);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_open(comm, filename, amode, info, fh);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_preallocate ================== */
_EXTERN_C_ int PMPI_File_preallocate(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_preallocate(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_preallocate);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_preallocate(fh, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read ================== */
_EXTERN_C_ int PMPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all ================== */
_EXTERN_C_ int PMPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_all_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_all_end ================== */
_EXTERN_C_ int PMPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_all_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at ================== */
_EXTERN_C_ int PMPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_at);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all ================== */
_EXTERN_C_ int PMPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_at_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_at_all_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_at_all_end ================== */
_EXTERN_C_ int PMPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_at_all_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered ================== */
_EXTERN_C_ int PMPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_ordered);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_ordered_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_ordered_end ================== */
_EXTERN_C_ int PMPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_ordered_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_read_shared ================== */
_EXTERN_C_ int PMPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_read_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_read_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_seek ================== */
_EXTERN_C_ int PMPI_File_seek(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_seek);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_seek(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_seek_shared ================== */
_EXTERN_C_ int PMPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);
_EXTERN_C_ int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_seek_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_seek_shared(fh, offset, whence);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_atomicity ================== */
_EXTERN_C_ int PMPI_File_set_atomicity(MPI_File fh, int flag);
_EXTERN_C_ int MPI_File_set_atomicity(MPI_File fh, int flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_set_atomicity);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_set_atomicity(fh, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_errhandler ================== */
_EXTERN_C_ int PMPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_set_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_set_errhandler(file, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_info ================== */
_EXTERN_C_ int PMPI_File_set_info(MPI_File fh, MPI_Info info);
_EXTERN_C_ int MPI_File_set_info(MPI_File fh, MPI_Info info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_set_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_set_info(fh, info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_size ================== */
_EXTERN_C_ int PMPI_File_set_size(MPI_File fh, MPI_Offset size);
_EXTERN_C_ int MPI_File_set_size(MPI_File fh, MPI_Offset size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_set_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_set_size(fh, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_set_view ================== */
_EXTERN_C_ int PMPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);
_EXTERN_C_ int MPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_set_view);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_set_view(fh, disp, etype, filetype, datarep, info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_sync ================== */
_EXTERN_C_ int PMPI_File_sync(MPI_File fh);
_EXTERN_C_ int MPI_File_sync(MPI_File fh) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_sync);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_sync(fh);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write ================== */
_EXTERN_C_ int PMPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all ================== */
_EXTERN_C_ int PMPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_all(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_all_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_all_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_all_end ================== */
_EXTERN_C_ int PMPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_all_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at ================== */
_EXTERN_C_ int PMPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_at);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_at(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all ================== */
_EXTERN_C_ int PMPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_at_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_at_all(fh, offset, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all_begin ================== */
_EXTERN_C_ int PMPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_at_all_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_at_all_begin(fh, offset, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_at_all_end ================== */
_EXTERN_C_ int PMPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_at_all_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_at_all_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered ================== */
_EXTERN_C_ int PMPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_ordered);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_ordered(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered_begin ================== */
_EXTERN_C_ int PMPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
_EXTERN_C_ int MPI_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_ordered_begin);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_ordered_begin(fh, buf, count, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_ordered_end ================== */
_EXTERN_C_ int PMPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_ordered_end);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_ordered_end(fh, buf, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_File_write_shared ================== */
_EXTERN_C_ int PMPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
_EXTERN_C_ int MPI_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_File_write_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_File_write_shared(fh, buf, count, datatype, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Finalized ================== */
_EXTERN_C_ int PMPI_Finalized(int *flag);
_EXTERN_C_ int MPI_Finalized(int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Finalized);

  WRAP_MPI_CALL_PREFIX
  XMPI_Finalized(flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Free_mem ================== */
_EXTERN_C_ int PMPI_Free_mem(void *base);
_EXTERN_C_ int MPI_Free_mem(void *base) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Free_mem);

  WRAP_MPI_CALL_PREFIX
  XMPI_Free_mem(base);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Gather ================== */
_EXTERN_C_ int PMPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Gather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Gather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Gatherv ================== */
_EXTERN_C_ int PMPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Gatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get ================== */
_EXTERN_C_ int PMPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_accumulate ================== */
_EXTERN_C_ int PMPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
_EXTERN_C_ int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_accumulate);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_address ================== */
_EXTERN_C_ int PMPI_Get_address(const void *location, MPI_Aint *address);
_EXTERN_C_ int MPI_Get_address(const void *location, MPI_Aint *address) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_address);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_address(location, address);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_count ================== */
_EXTERN_C_ int PMPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_count);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_count(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_elements ================== */
_EXTERN_C_ int PMPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count);
_EXTERN_C_ int MPI_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_elements);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_elements_x ================== */
_EXTERN_C_ int PMPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);
_EXTERN_C_ int MPI_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_elements_x);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_library_version ================== */
_EXTERN_C_ int PMPI_Get_library_version(char *version, int *resultlen);
_EXTERN_C_ int MPI_Get_library_version(char *version, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_library_version);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_library_version(version, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_processor_name ================== */
_EXTERN_C_ int PMPI_Get_processor_name(char *name, int *resultlen);
_EXTERN_C_ int MPI_Get_processor_name(char *name, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_processor_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_processor_name(name, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Get_version ================== */
_EXTERN_C_ int PMPI_Get_version(int *version, int *subversion);
_EXTERN_C_ int MPI_Get_version(int *version, int *subversion) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Get_version);

  WRAP_MPI_CALL_PREFIX
  XMPI_Get_version(version, subversion);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_create ================== */
_EXTERN_C_ int PMPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);
_EXTERN_C_ int MPI_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graph_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graph_create(comm_old, nnodes, index, edges, reorder, comm_graph);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_get ================== */
_EXTERN_C_ int PMPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);
_EXTERN_C_ int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graph_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graph_get(comm, maxindex, maxedges, index, edges);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_map ================== */
_EXTERN_C_ int PMPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);
_EXTERN_C_ int MPI_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graph_map);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graph_map(comm, nnodes, index, edges, newrank);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_neighbors ================== */
_EXTERN_C_ int PMPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);
_EXTERN_C_ int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graph_neighbors);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graph_neighbors_count ================== */
_EXTERN_C_ int PMPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);
_EXTERN_C_ int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graph_neighbors_count);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graph_neighbors_count(comm, rank, nneighbors);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Graphdims_get ================== */
_EXTERN_C_ int PMPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);
_EXTERN_C_ int MPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Graphdims_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Graphdims_get(comm, nnodes, nedges);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Grequest_complete ================== */
_EXTERN_C_ int PMPI_Grequest_complete(MPI_Request request);
_EXTERN_C_ int MPI_Grequest_complete(MPI_Request request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Grequest_complete);

  WRAP_MPI_CALL_PREFIX
  XMPI_Grequest_complete(request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Grequest_start ================== */
_EXTERN_C_ int PMPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);
_EXTERN_C_ int MPI_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Grequest_start);

  WRAP_MPI_CALL_PREFIX
  XMPI_Grequest_start(query_fn, free_fn, cancel_fn, extra_state, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_compare ================== */
_EXTERN_C_ int PMPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result);
_EXTERN_C_ int MPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_compare);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_compare(group1, group2, result);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_difference ================== */
_EXTERN_C_ int PMPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_difference);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_difference(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_excl ================== */
_EXTERN_C_ int PMPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_excl);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_excl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_free ================== */
_EXTERN_C_ int PMPI_Group_free(MPI_Group *group);
_EXTERN_C_ int MPI_Group_free(MPI_Group *group) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_free(group);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_incl ================== */
_EXTERN_C_ int PMPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_incl);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_incl(group, n, ranks, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_intersection ================== */
_EXTERN_C_ int PMPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_intersection);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_intersection(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_range_excl ================== */
_EXTERN_C_ int PMPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_range_excl);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_range_excl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_range_incl ================== */
_EXTERN_C_ int PMPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_range_incl);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_range_incl(group, n, ranges, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_rank ================== */
_EXTERN_C_ int PMPI_Group_rank(MPI_Group group, int *rank);
_EXTERN_C_ int MPI_Group_rank(MPI_Group group, int *rank) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_rank);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_rank(group, rank);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_size ================== */
_EXTERN_C_ int PMPI_Group_size(MPI_Group group, int *size);
_EXTERN_C_ int MPI_Group_size(MPI_Group group, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_size(group, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_translate_ranks ================== */
_EXTERN_C_ int PMPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);
_EXTERN_C_ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_translate_ranks);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_translate_ranks(group1, n, ranks1, group2, ranks2);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Group_union ================== */
_EXTERN_C_ int PMPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
_EXTERN_C_ int MPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Group_union);

  WRAP_MPI_CALL_PREFIX
  XMPI_Group_union(group1, group2, newgroup);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallgather ================== */
_EXTERN_C_ int PMPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iallgather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iallgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallgatherv ================== */
_EXTERN_C_ int PMPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iallgatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iallgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iallreduce ================== */
_EXTERN_C_ int PMPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iallreduce);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iallreduce(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoall ================== */
_EXTERN_C_ int PMPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ialltoall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ialltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoallv ================== */
_EXTERN_C_ int PMPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ialltoallv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ialltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ialltoallw ================== */
_EXTERN_C_ int PMPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ialltoallw);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ialltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibarrier ================== */
_EXTERN_C_ int PMPI_Ibarrier(MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibarrier(MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ibarrier);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ibarrier(comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibcast ================== */
_EXTERN_C_ int PMPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ibcast);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ibcast(buffer, count, datatype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ibsend ================== */
_EXTERN_C_ int PMPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ibsend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iexscan ================== */
_EXTERN_C_ int PMPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iexscan);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iexscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Igather ================== */
_EXTERN_C_ int PMPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Igather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Igather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Igatherv ================== */
_EXTERN_C_ int PMPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Igatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Igatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Improbe ================== */
_EXTERN_C_ int PMPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Improbe);

  WRAP_MPI_CALL_PREFIX
  XMPI_Improbe(source, tag, comm, flag, message, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Imrecv ================== */
_EXTERN_C_ int PMPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);
_EXTERN_C_ int MPI_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Imrecv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Imrecv(buf, count, type, message, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_allgather ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ineighbor_allgather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ineighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ineighbor_allgatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ineighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ineighbor_alltoall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ineighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ineighbor_alltoallv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ineighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ineighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ineighbor_alltoallw);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ineighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_create ================== */
_EXTERN_C_ int PMPI_Info_create(MPI_Info *info);
_EXTERN_C_ int MPI_Info_create(MPI_Info *info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_create(info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_delete ================== */
_EXTERN_C_ int PMPI_Info_delete(MPI_Info info, const char *key);
_EXTERN_C_ int MPI_Info_delete(MPI_Info info, const char *key) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_delete);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_delete(info, key);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_dup ================== */
_EXTERN_C_ int PMPI_Info_dup(MPI_Info info, MPI_Info *newinfo);
_EXTERN_C_ int MPI_Info_dup(MPI_Info info, MPI_Info *newinfo) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_dup);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_dup(info, newinfo);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_free ================== */
_EXTERN_C_ int PMPI_Info_free(MPI_Info *info);
_EXTERN_C_ int MPI_Info_free(MPI_Info *info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_free(info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get ================== */
_EXTERN_C_ int PMPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag);
_EXTERN_C_ int MPI_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_get);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_get(info, key, valuelen, value, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_nkeys ================== */
_EXTERN_C_ int PMPI_Info_get_nkeys(MPI_Info info, int *nkeys);
_EXTERN_C_ int MPI_Info_get_nkeys(MPI_Info info, int *nkeys) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_get_nkeys);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_get_nkeys(info, nkeys);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_nthkey ================== */
_EXTERN_C_ int PMPI_Info_get_nthkey(MPI_Info info, int n, char *key);
_EXTERN_C_ int MPI_Info_get_nthkey(MPI_Info info, int n, char *key) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_get_nthkey);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_get_nthkey(info, n, key);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_get_valuelen ================== */
_EXTERN_C_ int PMPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag);
_EXTERN_C_ int MPI_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_get_valuelen);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_get_valuelen(info, key, valuelen, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Info_set ================== */
_EXTERN_C_ int PMPI_Info_set(MPI_Info info, const char *key, const char *value);
_EXTERN_C_ int MPI_Info_set(MPI_Info info, const char *key, const char *value) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Info_set);

  WRAP_MPI_CALL_PREFIX
  XMPI_Info_set(info, key, value);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Init ================== */
_EXTERN_C_ int PMPI_Init(int *argc, char ***argv);
_EXTERN_C_ int MPI_Init(int *argc, char ***argv) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Init(argc, argv);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Init_thread ================== */
_EXTERN_C_ int PMPI_Init_thread(int *argc, char ***argv, int required, int *provided);
_EXTERN_C_ int MPI_Init_thread(int *argc, char ***argv, int required, int *provided) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Init_thread);

  WRAP_MPI_CALL_PREFIX
  XMPI_Init_thread(argc, argv, required, provided);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Initialized ================== */
_EXTERN_C_ int PMPI_Initialized(int *flag);
_EXTERN_C_ int MPI_Initialized(int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Initialized);

  WRAP_MPI_CALL_PREFIX
  XMPI_Initialized(flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Intercomm_create ================== */
_EXTERN_C_ int PMPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Intercomm_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Intercomm_create(local_comm, local_leader, bridge_comm, remote_leader, tag, newintercomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Intercomm_merge ================== */
_EXTERN_C_ int PMPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);
_EXTERN_C_ int MPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Intercomm_merge);

  WRAP_MPI_CALL_PREFIX
  XMPI_Intercomm_merge(intercomm, high, newintercomm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iprobe ================== */
_EXTERN_C_ int PMPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iprobe);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iprobe(source, tag, comm, flag, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Irecv ================== */
_EXTERN_C_ int PMPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Irecv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Irecv(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce ================== */
_EXTERN_C_ int PMPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ireduce);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ireduce(sendbuf, recvbuf, count, datatype, op, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce_scatter ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ireduce_scatter);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ireduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ireduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ireduce_scatter_block);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ireduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Irsend ================== */
_EXTERN_C_ int PMPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Irsend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Is_thread_main ================== */
_EXTERN_C_ int PMPI_Is_thread_main(int *flag);
_EXTERN_C_ int MPI_Is_thread_main(int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Is_thread_main);

  WRAP_MPI_CALL_PREFIX
  XMPI_Is_thread_main(flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscan ================== */
_EXTERN_C_ int PMPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iscan);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iscan(sendbuf, recvbuf, count, datatype, op, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscatter ================== */
_EXTERN_C_ int PMPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iscatter);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iscatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Iscatterv ================== */
_EXTERN_C_ int PMPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Iscatterv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Isend ================== */
_EXTERN_C_ int PMPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Isend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Isend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Issend ================== */
_EXTERN_C_ int PMPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Issend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Issend(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Keyval_create ================== */
_EXTERN_C_ int PMPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);
_EXTERN_C_ int MPI_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Keyval_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Keyval_free ================== */
_EXTERN_C_ int PMPI_Keyval_free(int *keyval);
_EXTERN_C_ int MPI_Keyval_free(int *keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Keyval_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Keyval_free(keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Lookup_name ================== */
_EXTERN_C_ int PMPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Lookup_name(const char *service_name, MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Lookup_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Lookup_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Mprobe ================== */
_EXTERN_C_ int PMPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Mprobe);

  WRAP_MPI_CALL_PREFIX
  XMPI_Mprobe(source, tag, comm, message, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Mrecv ================== */
_EXTERN_C_ int PMPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);
_EXTERN_C_ int MPI_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Mrecv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Mrecv(buf, count, type, message, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_allgather ================== */
_EXTERN_C_ int PMPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Neighbor_allgather);

  WRAP_MPI_CALL_PREFIX
  XMPI_Neighbor_allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_allgatherv ================== */
_EXTERN_C_ int PMPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Neighbor_allgatherv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Neighbor_allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoall ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Neighbor_alltoall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Neighbor_alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoallv ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Neighbor_alltoallv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Neighbor_alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Neighbor_alltoallw ================== */
_EXTERN_C_ int PMPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
_EXTERN_C_ int MPI_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Neighbor_alltoallw);

  WRAP_MPI_CALL_PREFIX
  XMPI_Neighbor_alltoallw(sendbuf, sendcounts, sdispls, sendtypes, recvbuf, recvcounts, rdispls, recvtypes, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_commutative ================== */
_EXTERN_C_ int PMPI_Op_commutative(MPI_Op op, int *commute);
_EXTERN_C_ int MPI_Op_commutative(MPI_Op op, int *commute) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Op_commutative);

  WRAP_MPI_CALL_PREFIX
  XMPI_Op_commutative(op, commute);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_create ================== */
_EXTERN_C_ int PMPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op);
_EXTERN_C_ int MPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Op_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Op_create(function, commute, op);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Op_free ================== */
_EXTERN_C_ int PMPI_Op_free(MPI_Op *op);
_EXTERN_C_ int MPI_Op_free(MPI_Op *op) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Op_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Op_free(op);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Open_port ================== */
_EXTERN_C_ int PMPI_Open_port(MPI_Info info, char *port_name);
_EXTERN_C_ int MPI_Open_port(MPI_Info info, char *port_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Open_port);

  WRAP_MPI_CALL_PREFIX
  XMPI_Open_port(info, port_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack ================== */
_EXTERN_C_ int PMPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);
_EXTERN_C_ int MPI_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Pack);

  WRAP_MPI_CALL_PREFIX
  XMPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_external ================== */
_EXTERN_C_ int PMPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);
_EXTERN_C_ int MPI_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Pack_external);

  WRAP_MPI_CALL_PREFIX
  XMPI_Pack_external(datarep, inbuf, incount, datatype, outbuf, outsize, position);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_external_size ================== */
_EXTERN_C_ int PMPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);
_EXTERN_C_ int MPI_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Pack_external_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Pack_external_size(datarep, incount, datatype, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Pack_size ================== */
_EXTERN_C_ int PMPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);
_EXTERN_C_ int MPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Pack_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Pack_size(incount, datatype, comm, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Probe ================== */
_EXTERN_C_ int PMPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Probe);

  WRAP_MPI_CALL_PREFIX
  XMPI_Probe(source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Publish_name ================== */
_EXTERN_C_ int PMPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Publish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Publish_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Publish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Put ================== */
_EXTERN_C_ int PMPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
_EXTERN_C_ int MPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Put);

  WRAP_MPI_CALL_PREFIX
  XMPI_Put(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Query_thread ================== */
_EXTERN_C_ int PMPI_Query_thread(int *provided);
_EXTERN_C_ int MPI_Query_thread(int *provided) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Query_thread);

  WRAP_MPI_CALL_PREFIX
  XMPI_Query_thread(provided);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Raccumulate ================== */
_EXTERN_C_ int PMPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Raccumulate);

  WRAP_MPI_CALL_PREFIX
  XMPI_Raccumulate(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Recv ================== */
_EXTERN_C_ int PMPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Recv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Recv(buf, count, datatype, source, tag, comm, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Recv_init ================== */
_EXTERN_C_ int PMPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Recv_init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce ================== */
_EXTERN_C_ int PMPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Reduce);

  WRAP_MPI_CALL_PREFIX
  XMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce_local ================== */
_EXTERN_C_ int PMPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);
_EXTERN_C_ int MPI_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Reduce_local);

  WRAP_MPI_CALL_PREFIX
  XMPI_Reduce_local(inbuf, inoutbuf, count, datatype, op);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce_scatter ================== */
_EXTERN_C_ int PMPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Reduce_scatter);

  WRAP_MPI_CALL_PREFIX
  XMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Reduce_scatter_block ================== */
_EXTERN_C_ int PMPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Reduce_scatter_block);

  WRAP_MPI_CALL_PREFIX
  XMPI_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Register_datarep ================== */
_EXTERN_C_ int PMPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);
_EXTERN_C_ int MPI_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Register_datarep);

  WRAP_MPI_CALL_PREFIX
  XMPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Request_free ================== */
_EXTERN_C_ int PMPI_Request_free(MPI_Request *request);
_EXTERN_C_ int MPI_Request_free(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Request_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Request_free(request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Request_get_status ================== */
_EXTERN_C_ int PMPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Request_get_status(MPI_Request request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Request_get_status);

  WRAP_MPI_CALL_PREFIX
  XMPI_Request_get_status(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rget ================== */
_EXTERN_C_ int PMPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Rget);

  WRAP_MPI_CALL_PREFIX
  XMPI_Rget(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rget_accumulate ================== */
_EXTERN_C_ int PMPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Rget_accumulate);

  WRAP_MPI_CALL_PREFIX
  XMPI_Rget_accumulate(origin_addr, origin_count, origin_datatype, result_addr, result_count, result_datatype, target_rank, target_disp, target_count, target_datatype, op, win, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rput ================== */
_EXTERN_C_ int PMPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
_EXTERN_C_ int MPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Rput);

  WRAP_MPI_CALL_PREFIX
  XMPI_Rput(origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_cout, target_datatype, win, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rsend ================== */
_EXTERN_C_ int PMPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Rsend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Rsend(ibuf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Rsend_init ================== */
_EXTERN_C_ int PMPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Rsend_init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Scan ================== */
_EXTERN_C_ int PMPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
_EXTERN_C_ int MPI_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Scan);

  WRAP_MPI_CALL_PREFIX
  XMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Scatter ================== */
_EXTERN_C_ int PMPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Scatter);

  WRAP_MPI_CALL_PREFIX
  XMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Scatterv ================== */
_EXTERN_C_ int PMPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
_EXTERN_C_ int MPI_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Scatterv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Send ================== */
_EXTERN_C_ int PMPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Send);

  WRAP_MPI_CALL_PREFIX
  XMPI_Send(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Send_init ================== */
_EXTERN_C_ int PMPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Send_init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Sendrecv ================== */
_EXTERN_C_ int PMPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Sendrecv);

  WRAP_MPI_CALL_PREFIX
  XMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Sendrecv_replace ================== */
_EXTERN_C_ int PMPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
_EXTERN_C_ int MPI_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Sendrecv_replace);

  WRAP_MPI_CALL_PREFIX
  XMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ssend ================== */
_EXTERN_C_ int PMPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
_EXTERN_C_ int MPI_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ssend);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ssend(buf, count, datatype, dest, tag, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Ssend_init ================== */
_EXTERN_C_ int PMPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
_EXTERN_C_ int MPI_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Ssend_init);

  WRAP_MPI_CALL_PREFIX
  XMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Start ================== */
_EXTERN_C_ int PMPI_Start(MPI_Request *request);
_EXTERN_C_ int MPI_Start(MPI_Request *request) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Start);

  WRAP_MPI_CALL_PREFIX
  XMPI_Start(request);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Startall ================== */
_EXTERN_C_ int PMPI_Startall(int count, MPI_Request array_of_requests[]);
_EXTERN_C_ int MPI_Startall(int count, MPI_Request array_of_requests[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Startall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Startall(count, array_of_requests);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_cancelled ================== */
_EXTERN_C_ int PMPI_Status_set_cancelled(MPI_Status *status, int flag);
_EXTERN_C_ int MPI_Status_set_cancelled(MPI_Status *status, int flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Status_set_cancelled);

  WRAP_MPI_CALL_PREFIX
  XMPI_Status_set_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_elements ================== */
_EXTERN_C_ int PMPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count);
_EXTERN_C_ int MPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Status_set_elements);

  WRAP_MPI_CALL_PREFIX
  XMPI_Status_set_elements(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Status_set_elements_x ================== */
_EXTERN_C_ int PMPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);
_EXTERN_C_ int MPI_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Status_set_elements_x);

  WRAP_MPI_CALL_PREFIX
  XMPI_Status_set_elements_x(status, datatype, count);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Test ================== */
_EXTERN_C_ int PMPI_Test(MPI_Request *request, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Test(MPI_Request *request, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Test);

  WRAP_MPI_CALL_PREFIX
  XMPI_Test(request, flag, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Test_cancelled ================== */
_EXTERN_C_ int PMPI_Test_cancelled(const MPI_Status *status, int *flag);
_EXTERN_C_ int MPI_Test_cancelled(const MPI_Status *status, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Test_cancelled);

  WRAP_MPI_CALL_PREFIX
  XMPI_Test_cancelled(status, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Testall ================== */
_EXTERN_C_ int PMPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Testall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Testall(count, array_of_requests, flag, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Testany ================== */
_EXTERN_C_ int PMPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);
_EXTERN_C_ int MPI_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Testany);

  WRAP_MPI_CALL_PREFIX
  XMPI_Testany(count, array_of_requests, index, flag, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Testsome ================== */
_EXTERN_C_ int PMPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Testsome);

  WRAP_MPI_CALL_PREFIX
  XMPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Topo_test ================== */
_EXTERN_C_ int PMPI_Topo_test(MPI_Comm comm, int *status);
_EXTERN_C_ int MPI_Topo_test(MPI_Comm comm, int *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Topo_test);

  WRAP_MPI_CALL_PREFIX
  XMPI_Topo_test(comm, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_commit ================== */
_EXTERN_C_ int PMPI_Type_commit(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_commit(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_commit);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_commit(type);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_contiguous ================== */
_EXTERN_C_ int PMPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_contiguous);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_contiguous(count, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_darray ================== */
_EXTERN_C_ int PMPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_darray);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_darray(size, rank, ndims, gsize_array, distrib_array, darg_array, psize_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_complex ================== */
_EXTERN_C_ int PMPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_f90_complex);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_f90_complex(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_integer ================== */
_EXTERN_C_ int PMPI_Type_create_f90_integer(int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_f90_integer);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_f90_integer(r, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_f90_real ================== */
_EXTERN_C_ int PMPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_f90_real);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_f90_real(p, r, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hindexed ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_hindexed);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hindexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_hindexed_block);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_hindexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_hvector ================== */
_EXTERN_C_ int PMPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_hvector);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_indexed_block ================== */
_EXTERN_C_ int PMPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_indexed_block);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_indexed_block(count, blocklength, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_keyval ================== */
_EXTERN_C_ int PMPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);
_EXTERN_C_ int MPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_keyval(type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_resized ================== */
_EXTERN_C_ int PMPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_resized);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_resized(oldtype, lb, extent, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_struct ================== */
_EXTERN_C_ int PMPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_struct);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_struct(count, array_of_block_lengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_create_subarray ================== */
_EXTERN_C_ int PMPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_create_subarray);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_create_subarray(ndims, size_array, subsize_array, start_array, order, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_delete_attr ================== */
_EXTERN_C_ int PMPI_Type_delete_attr(MPI_Datatype type, int type_keyval);
_EXTERN_C_ int MPI_Type_delete_attr(MPI_Datatype type, int type_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_delete_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_delete_attr(type, type_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_dup ================== */
_EXTERN_C_ int PMPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_dup);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_dup(type, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_extent ================== */
_EXTERN_C_ int PMPI_Type_extent(MPI_Datatype type, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_extent(MPI_Datatype type, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_extent);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_extent(type, extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_free ================== */
_EXTERN_C_ int PMPI_Type_free(MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_free(MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_free(type);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_free_keyval ================== */
_EXTERN_C_ int PMPI_Type_free_keyval(int *type_keyval);
_EXTERN_C_ int MPI_Type_free_keyval(int *type_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_free_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_free_keyval(type_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_attr ================== */
_EXTERN_C_ int PMPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_attr(type, type_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_contents ================== */
_EXTERN_C_ int PMPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);
_EXTERN_C_ int MPI_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_contents);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses, array_of_datatypes);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_envelope ================== */
_EXTERN_C_ int PMPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);
_EXTERN_C_ int MPI_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_envelope);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_extent ================== */
_EXTERN_C_ int PMPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);
_EXTERN_C_ int MPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_extent);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_extent(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);
_EXTERN_C_ int MPI_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_extent_x);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_extent_x(type, lb, extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_name ================== */
_EXTERN_C_ int PMPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen);
_EXTERN_C_ int MPI_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_name(type, type_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_true_extent ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_true_extent);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_true_extent(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_get_true_extent_x ================== */
_EXTERN_C_ int PMPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);
_EXTERN_C_ int MPI_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_get_true_extent_x);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_get_true_extent_x(datatype, true_lb, true_extent);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_hindexed ================== */
_EXTERN_C_ int PMPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_hindexed);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_hvector ================== */
_EXTERN_C_ int PMPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_hvector);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_indexed ================== */
_EXTERN_C_ int PMPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_indexed);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_lb ================== */
_EXTERN_C_ int PMPI_Type_lb(MPI_Datatype type, MPI_Aint *lb);
_EXTERN_C_ int MPI_Type_lb(MPI_Datatype type, MPI_Aint *lb) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_lb);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_lb(type, lb);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_match_size ================== */
_EXTERN_C_ int PMPI_Type_match_size(int typeclass, int size, MPI_Datatype *type);
_EXTERN_C_ int MPI_Type_match_size(int typeclass, int size, MPI_Datatype *type) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_match_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_match_size(typeclass, size, type);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_set_attr ================== */
_EXTERN_C_ int PMPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val);
_EXTERN_C_ int MPI_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_set_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_set_attr(type, type_keyval, attr_val);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_set_name ================== */
_EXTERN_C_ int PMPI_Type_set_name(MPI_Datatype type, const char *type_name);
_EXTERN_C_ int MPI_Type_set_name(MPI_Datatype type, const char *type_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_set_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_set_name(type, type_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_size ================== */
_EXTERN_C_ int PMPI_Type_size(MPI_Datatype type, int *size);
_EXTERN_C_ int MPI_Type_size(MPI_Datatype type, int *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_size);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_size(type, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_size_x ================== */
_EXTERN_C_ int PMPI_Type_size_x(MPI_Datatype type, MPI_Count *size);
_EXTERN_C_ int MPI_Type_size_x(MPI_Datatype type, MPI_Count *size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_size_x);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_size_x(type, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_struct ================== */
_EXTERN_C_ int PMPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_struct);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_struct(count, array_of_blocklengths, array_of_displacements, array_of_types, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_ub ================== */
_EXTERN_C_ int PMPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);
_EXTERN_C_ int MPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_ub);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_ub(mtype, ub);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Type_vector ================== */
_EXTERN_C_ int PMPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
_EXTERN_C_ int MPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Type_vector);

  WRAP_MPI_CALL_PREFIX
  XMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpack ================== */
_EXTERN_C_ int PMPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);
_EXTERN_C_ int MPI_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Unpack);

  WRAP_MPI_CALL_PREFIX
  XMPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype, comm);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpack_external ================== */
_EXTERN_C_ int PMPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);
_EXTERN_C_ int MPI_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Unpack_external);

  WRAP_MPI_CALL_PREFIX
  XMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Unpublish_name ================== */
_EXTERN_C_ int PMPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name);
_EXTERN_C_ int MPI_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Unpublish_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Unpublish_name(service_name, info, port_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Wait ================== */
_EXTERN_C_ int PMPI_Wait(MPI_Request *request, MPI_Status *status);
_EXTERN_C_ int MPI_Wait(MPI_Request *request, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Wait);

  WRAP_MPI_CALL_PREFIX
  XMPI_Wait(request, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Waitall ================== */
_EXTERN_C_ int PMPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);
_EXTERN_C_ int MPI_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Waitall);

  WRAP_MPI_CALL_PREFIX
  XMPI_Waitall(count, array_of_requests, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Waitany ================== */
_EXTERN_C_ int PMPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);
_EXTERN_C_ int MPI_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Waitany);

  WRAP_MPI_CALL_PREFIX
  XMPI_Waitany(count, array_of_requests, index, status);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Waitsome ================== */
_EXTERN_C_ int PMPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
_EXTERN_C_ int MPI_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Waitsome);

  WRAP_MPI_CALL_PREFIX
  XMPI_Waitsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_allocate ================== */
_EXTERN_C_ int PMPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_allocate);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_allocate(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_allocate_shared ================== */
_EXTERN_C_ int PMPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
_EXTERN_C_ int MPI_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_allocate_shared);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_allocate_shared(size, disp_unit, info, comm, baseptr, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_attach ================== */
_EXTERN_C_ int PMPI_Win_attach(MPI_Win win, void *base, MPI_Aint size);
_EXTERN_C_ int MPI_Win_attach(MPI_Win win, void *base, MPI_Aint size) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_attach);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_attach(win, base, size);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_call_errhandler ================== */
_EXTERN_C_ int PMPI_Win_call_errhandler(MPI_Win win, int errorcode);
_EXTERN_C_ int MPI_Win_call_errhandler(MPI_Win win, int errorcode) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_call_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_call_errhandler(win, errorcode);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_complete ================== */
_EXTERN_C_ int PMPI_Win_complete(MPI_Win win);
_EXTERN_C_ int MPI_Win_complete(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_complete);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_complete(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create ================== */
_EXTERN_C_ int PMPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_create);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_create(base, size, disp_unit, info, comm, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_dynamic ================== */
_EXTERN_C_ int PMPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win);
_EXTERN_C_ int MPI_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_create_dynamic);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_create_dynamic(info, comm, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_errhandler ================== */
_EXTERN_C_ int PMPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_create_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_create_errhandler(function, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_create_keyval ================== */
_EXTERN_C_ int PMPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);
_EXTERN_C_ int MPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_create_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_create_keyval(win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_delete_attr ================== */
_EXTERN_C_ int PMPI_Win_delete_attr(MPI_Win win, int win_keyval);
_EXTERN_C_ int MPI_Win_delete_attr(MPI_Win win, int win_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_delete_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_delete_attr(win, win_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_detach ================== */
_EXTERN_C_ int PMPI_Win_detach(MPI_Win win, const void *base);
_EXTERN_C_ int MPI_Win_detach(MPI_Win win, const void *base) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_detach);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_detach(win, base);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_fence ================== */
_EXTERN_C_ int PMPI_Win_fence(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_fence(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_fence);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_fence(assert, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush ================== */
_EXTERN_C_ int PMPI_Win_flush(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_flush);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_flush(rank, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_all ================== */
_EXTERN_C_ int PMPI_Win_flush_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_flush_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_flush_all(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_local ================== */
_EXTERN_C_ int PMPI_Win_flush_local(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_flush_local);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_flush_local(rank, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_flush_local_all ================== */
_EXTERN_C_ int PMPI_Win_flush_local_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_flush_local_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_flush_local_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_flush_local_all(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_free ================== */
_EXTERN_C_ int PMPI_Win_free(MPI_Win *win);
_EXTERN_C_ int MPI_Win_free(MPI_Win *win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_free);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_free(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_free_keyval ================== */
_EXTERN_C_ int PMPI_Win_free_keyval(int *win_keyval);
_EXTERN_C_ int MPI_Win_free_keyval(int *win_keyval) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_free_keyval);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_free_keyval(win_keyval);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_attr ================== */
_EXTERN_C_ int PMPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag);
_EXTERN_C_ int MPI_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_get_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_get_attr(win, win_keyval, attribute_val, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_errhandler ================== */
_EXTERN_C_ int PMPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);
_EXTERN_C_ int MPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_get_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_get_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_group ================== */
_EXTERN_C_ int PMPI_Win_get_group(MPI_Win win, MPI_Group *group);
_EXTERN_C_ int MPI_Win_get_group(MPI_Win win, MPI_Group *group) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_get_group);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_get_group(win, group);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_info ================== */
_EXTERN_C_ int PMPI_Win_get_info(MPI_Win win, MPI_Info *info_used);
_EXTERN_C_ int MPI_Win_get_info(MPI_Win win, MPI_Info *info_used) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_get_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_get_info(win, info_used);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_get_name ================== */
_EXTERN_C_ int PMPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen);
_EXTERN_C_ int MPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_get_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_get_name(win, win_name, resultlen);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_lock ================== */
_EXTERN_C_ int PMPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_lock);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_lock(lock_type, rank, assert, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_lock_all ================== */
_EXTERN_C_ int PMPI_Win_lock_all(int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_lock_all(int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_lock_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_lock_all(assert, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_post ================== */
_EXTERN_C_ int PMPI_Win_post(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_post(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_post);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_post(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_attr ================== */
_EXTERN_C_ int PMPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);
_EXTERN_C_ int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_set_attr);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_set_attr(win, win_keyval, attribute_val);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_errhandler ================== */
_EXTERN_C_ int PMPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);
_EXTERN_C_ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_set_errhandler);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_set_errhandler(win, errhandler);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_info ================== */
_EXTERN_C_ int PMPI_Win_set_info(MPI_Win win, MPI_Info info);
_EXTERN_C_ int MPI_Win_set_info(MPI_Win win, MPI_Info info) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_set_info);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_set_info(win, info);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_set_name ================== */
_EXTERN_C_ int PMPI_Win_set_name(MPI_Win win, const char *win_name);
_EXTERN_C_ int MPI_Win_set_name(MPI_Win win, const char *win_name) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_set_name);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_set_name(win, win_name);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_shared_query ================== */
_EXTERN_C_ int PMPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);
_EXTERN_C_ int MPI_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_shared_query);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_shared_query(win, rank, size, disp_unit, baseptr);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_start ================== */
_EXTERN_C_ int PMPI_Win_start(MPI_Group group, int assert, MPI_Win win);
_EXTERN_C_ int MPI_Win_start(MPI_Group group, int assert, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_start);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_start(group, assert, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_sync ================== */
_EXTERN_C_ int PMPI_Win_sync(MPI_Win win);
_EXTERN_C_ int MPI_Win_sync(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_sync);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_sync(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_test ================== */
_EXTERN_C_ int PMPI_Win_test(MPI_Win win, int *flag);
_EXTERN_C_ int MPI_Win_test(MPI_Win win, int *flag) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_test);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_test(win, flag);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_unlock ================== */
_EXTERN_C_ int PMPI_Win_unlock(int rank, MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock(int rank, MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_unlock);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_unlock(rank, win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_unlock_all ================== */
_EXTERN_C_ int PMPI_Win_unlock_all(MPI_Win win);
_EXTERN_C_ int MPI_Win_unlock_all(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_unlock_all);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_unlock_all(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Win_wait ================== */
_EXTERN_C_ int PMPI_Win_wait(MPI_Win win);
_EXTERN_C_ int MPI_Win_wait(MPI_Win win) { 
    int _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Win_wait);

  WRAP_MPI_CALL_PREFIX
  XMPI_Win_wait(win);
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Wtick ================== */
_EXTERN_C_ double PMPI_Wtick();
_EXTERN_C_ double MPI_Wtick() { 
    double _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Wtick);

  WRAP_MPI_CALL_PREFIX
  XMPI_Wtick();
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}

/* ================== C Wrappers for MPI_Wtime ================== */
_EXTERN_C_ double PMPI_Wtime();
_EXTERN_C_ double MPI_Wtime() { 
    double _wrap_py_return_val = 0;

  metric_atomic_inc(counters.MPI_Wtime);

  WRAP_MPI_CALL_PREFIX
  XMPI_Wtime();
  WRAP_MPI_CALL_POSTFIX
    return _wrap_py_return_val;
}




/* MPI_Pcontrol needs special handling, as it doesn't call a PMPI function and
 * PnMPI does not implement the required XMPI call. Instead PnMPI will act as a
 * multiplexer for MPI_Pcontrol, so all we have to do is increment the counter
 * and return. */

int MPI_Pcontrol(const int level, ...)
{
  metric_atomic_inc(counters.MPI_Pcontrol);

  return MPI_SUCCESS;
}


/** \brief Print the statistics.
 *
 * \details This function will print the statistics to stdout for each rank and
 *  a sum of all ranks.
 *
 * \todo In feature releases this should be done by the PNMPI_AppShutdown hook
 *  to track duplicated MPI_Finalize calls.
 *
 *
 * \return The return value of PMPI_Finalize will be pass through.
 */
int MPI_Finalize()
{
  /* Flush the buffers to avoid fragments in the output.
   *
   * NOTE: This can't fully prevent buffered output being displayed between
   *       fresh one, as there's no guaranty the MPI sends the buffers back
   *       to the host immediately. */
  fflush(stdout);
  fflush(stderr);


  int rank, size;

  PMPI_Comm_rank_assert(MPI_COMM_WORLD, &rank);
  PMPI_Comm_size_assert(MPI_COMM_WORLD, &size);

  /* The entire output will be handled by rank 0 to avoid output collisions, as
   * the individual ranks can't sync their output being displayed at the
   * frontend. Therefore, rank 0 prints a header and its counters first. */
  if (rank == 0)
    {
      printf("\n\n################################\n\n"
             "MPI call stats:\n\n"
             " Rank 0:\n");
      print_counters(&counters);
    }

  /* Rank 0 should receive the counters from other ranks now, to display those.
   * These will be summed up for the total counters printed below, to cache the
   * counters instead of using a reduction for receiving the counters of all
   * ranks a second time. */
  struct counter tmp;
  init_counters(&tmp);
  int n;
  for (n = 1; n < size; n++)
    {
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Abort), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Abort), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Abort += tmp.MPI_Abort;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Accumulate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Accumulate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Accumulate += tmp.MPI_Accumulate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Add_error_class), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Add_error_class), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Add_error_class += tmp.MPI_Add_error_class;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Add_error_code), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Add_error_code), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Add_error_code += tmp.MPI_Add_error_code;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Add_error_string), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Add_error_string), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Add_error_string += tmp.MPI_Add_error_string;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Address), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Address), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Address += tmp.MPI_Address;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Allgather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Allgather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Allgather += tmp.MPI_Allgather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Allgatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Allgatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Allgatherv += tmp.MPI_Allgatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Alloc_mem), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Alloc_mem), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Alloc_mem += tmp.MPI_Alloc_mem;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Allreduce), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Allreduce), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Allreduce += tmp.MPI_Allreduce;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Alltoall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Alltoall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Alltoall += tmp.MPI_Alltoall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Alltoallv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Alltoallv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Alltoallv += tmp.MPI_Alltoallv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Alltoallw), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Alltoallw), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Alltoallw += tmp.MPI_Alltoallw;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Attr_delete), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Attr_delete), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Attr_delete += tmp.MPI_Attr_delete;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Attr_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Attr_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Attr_get += tmp.MPI_Attr_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Attr_put), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Attr_put), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Attr_put += tmp.MPI_Attr_put;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Barrier), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Barrier), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Barrier += tmp.MPI_Barrier;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Bcast), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Bcast), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Bcast += tmp.MPI_Bcast;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Bsend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Bsend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Bsend += tmp.MPI_Bsend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Bsend_init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Bsend_init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Bsend_init += tmp.MPI_Bsend_init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Buffer_attach), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Buffer_attach), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Buffer_attach += tmp.MPI_Buffer_attach;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Buffer_detach), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Buffer_detach), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Buffer_detach += tmp.MPI_Buffer_detach;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cancel), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cancel), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cancel += tmp.MPI_Cancel;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_coords), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_coords), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_coords += tmp.MPI_Cart_coords;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_create += tmp.MPI_Cart_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_get += tmp.MPI_Cart_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_map), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_map), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_map += tmp.MPI_Cart_map;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_rank), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_rank), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_rank += tmp.MPI_Cart_rank;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_shift), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_shift), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_shift += tmp.MPI_Cart_shift;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cart_sub), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cart_sub), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cart_sub += tmp.MPI_Cart_sub;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Cartdim_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Cartdim_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Cartdim_get += tmp.MPI_Cartdim_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Close_port), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Close_port), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Close_port += tmp.MPI_Close_port;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_accept), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_accept), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_accept += tmp.MPI_Comm_accept;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_call_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_call_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_call_errhandler += tmp.MPI_Comm_call_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_compare), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_compare), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_compare += tmp.MPI_Comm_compare;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_connect), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_connect), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_connect += tmp.MPI_Comm_connect;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_create += tmp.MPI_Comm_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_create_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_create_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_create_errhandler += tmp.MPI_Comm_create_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_create_group), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_create_group), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_create_group += tmp.MPI_Comm_create_group;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_create_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_create_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_create_keyval += tmp.MPI_Comm_create_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_delete_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_delete_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_delete_attr += tmp.MPI_Comm_delete_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_disconnect), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_disconnect), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_disconnect += tmp.MPI_Comm_disconnect;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_dup), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_dup), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_dup += tmp.MPI_Comm_dup;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_dup_with_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_dup_with_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_dup_with_info += tmp.MPI_Comm_dup_with_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_free += tmp.MPI_Comm_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_free_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_free_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_free_keyval += tmp.MPI_Comm_free_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_get_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_get_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_get_attr += tmp.MPI_Comm_get_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_get_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_get_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_get_errhandler += tmp.MPI_Comm_get_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_get_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_get_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_get_info += tmp.MPI_Comm_get_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_get_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_get_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_get_name += tmp.MPI_Comm_get_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_get_parent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_get_parent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_get_parent += tmp.MPI_Comm_get_parent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_group), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_group), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_group += tmp.MPI_Comm_group;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_idup), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_idup), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_idup += tmp.MPI_Comm_idup;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_join), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_join), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_join += tmp.MPI_Comm_join;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_rank), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_rank), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_rank += tmp.MPI_Comm_rank;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_remote_group), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_remote_group), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_remote_group += tmp.MPI_Comm_remote_group;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_remote_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_remote_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_remote_size += tmp.MPI_Comm_remote_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_set_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_set_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_set_attr += tmp.MPI_Comm_set_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_set_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_set_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_set_errhandler += tmp.MPI_Comm_set_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_set_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_set_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_set_info += tmp.MPI_Comm_set_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_set_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_set_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_set_name += tmp.MPI_Comm_set_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_size += tmp.MPI_Comm_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_split), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_split), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_split += tmp.MPI_Comm_split;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_split_type), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_split_type), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_split_type += tmp.MPI_Comm_split_type;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Comm_test_inter), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Comm_test_inter), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Comm_test_inter += tmp.MPI_Comm_test_inter;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Compare_and_swap), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Compare_and_swap), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Compare_and_swap += tmp.MPI_Compare_and_swap;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Dims_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Dims_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Dims_create += tmp.MPI_Dims_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Dist_graph_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Dist_graph_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Dist_graph_create += tmp.MPI_Dist_graph_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Dist_graph_create_adjacent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Dist_graph_create_adjacent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Dist_graph_create_adjacent += tmp.MPI_Dist_graph_create_adjacent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Dist_graph_neighbors), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Dist_graph_neighbors), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Dist_graph_neighbors += tmp.MPI_Dist_graph_neighbors;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Dist_graph_neighbors_count), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Dist_graph_neighbors_count), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Dist_graph_neighbors_count += tmp.MPI_Dist_graph_neighbors_count;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Errhandler_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Errhandler_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Errhandler_create += tmp.MPI_Errhandler_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Errhandler_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Errhandler_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Errhandler_free += tmp.MPI_Errhandler_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Errhandler_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Errhandler_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Errhandler_get += tmp.MPI_Errhandler_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Errhandler_set), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Errhandler_set), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Errhandler_set += tmp.MPI_Errhandler_set;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Error_class), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Error_class), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Error_class += tmp.MPI_Error_class;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Error_string), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Error_string), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Error_string += tmp.MPI_Error_string;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Exscan), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Exscan), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Exscan += tmp.MPI_Exscan;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Fetch_and_op), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Fetch_and_op), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Fetch_and_op += tmp.MPI_Fetch_and_op;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_call_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_call_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_call_errhandler += tmp.MPI_File_call_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_close), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_close), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_close += tmp.MPI_File_close;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_create_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_create_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_create_errhandler += tmp.MPI_File_create_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_delete), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_delete), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_delete += tmp.MPI_File_delete;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_amode), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_amode), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_amode += tmp.MPI_File_get_amode;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_atomicity), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_atomicity), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_atomicity += tmp.MPI_File_get_atomicity;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_byte_offset), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_byte_offset), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_byte_offset += tmp.MPI_File_get_byte_offset;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_errhandler += tmp.MPI_File_get_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_group), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_group), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_group += tmp.MPI_File_get_group;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_info += tmp.MPI_File_get_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_position), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_position), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_position += tmp.MPI_File_get_position;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_position_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_position_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_position_shared += tmp.MPI_File_get_position_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_size += tmp.MPI_File_get_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_type_extent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_type_extent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_type_extent += tmp.MPI_File_get_type_extent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_get_view), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_get_view), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_get_view += tmp.MPI_File_get_view;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iread), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iread), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iread += tmp.MPI_File_iread;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iread_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iread_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iread_all += tmp.MPI_File_iread_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iread_at), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iread_at), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iread_at += tmp.MPI_File_iread_at;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iread_at_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iread_at_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iread_at_all += tmp.MPI_File_iread_at_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iread_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iread_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iread_shared += tmp.MPI_File_iread_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iwrite), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iwrite), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iwrite += tmp.MPI_File_iwrite;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iwrite_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iwrite_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iwrite_all += tmp.MPI_File_iwrite_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iwrite_at), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iwrite_at), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iwrite_at += tmp.MPI_File_iwrite_at;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iwrite_at_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iwrite_at_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iwrite_at_all += tmp.MPI_File_iwrite_at_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_iwrite_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_iwrite_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_iwrite_shared += tmp.MPI_File_iwrite_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_open), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_open), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_open += tmp.MPI_File_open;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_preallocate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_preallocate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_preallocate += tmp.MPI_File_preallocate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read += tmp.MPI_File_read;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_all += tmp.MPI_File_read_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_all_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_all_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_all_begin += tmp.MPI_File_read_all_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_all_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_all_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_all_end += tmp.MPI_File_read_all_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_at), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_at), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_at += tmp.MPI_File_read_at;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_at_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_at_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_at_all += tmp.MPI_File_read_at_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_at_all_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_at_all_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_at_all_begin += tmp.MPI_File_read_at_all_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_at_all_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_at_all_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_at_all_end += tmp.MPI_File_read_at_all_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_ordered), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_ordered), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_ordered += tmp.MPI_File_read_ordered;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_ordered_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_ordered_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_ordered_begin += tmp.MPI_File_read_ordered_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_ordered_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_ordered_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_ordered_end += tmp.MPI_File_read_ordered_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_read_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_read_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_read_shared += tmp.MPI_File_read_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_seek), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_seek), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_seek += tmp.MPI_File_seek;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_seek_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_seek_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_seek_shared += tmp.MPI_File_seek_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_set_atomicity), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_set_atomicity), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_set_atomicity += tmp.MPI_File_set_atomicity;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_set_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_set_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_set_errhandler += tmp.MPI_File_set_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_set_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_set_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_set_info += tmp.MPI_File_set_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_set_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_set_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_set_size += tmp.MPI_File_set_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_set_view), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_set_view), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_set_view += tmp.MPI_File_set_view;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_sync), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_sync), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_sync += tmp.MPI_File_sync;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write += tmp.MPI_File_write;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_all += tmp.MPI_File_write_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_all_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_all_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_all_begin += tmp.MPI_File_write_all_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_all_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_all_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_all_end += tmp.MPI_File_write_all_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_at), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_at), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_at += tmp.MPI_File_write_at;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_at_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_at_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_at_all += tmp.MPI_File_write_at_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_at_all_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_at_all_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_at_all_begin += tmp.MPI_File_write_at_all_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_at_all_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_at_all_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_at_all_end += tmp.MPI_File_write_at_all_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_ordered), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_ordered), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_ordered += tmp.MPI_File_write_ordered;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_ordered_begin), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_ordered_begin), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_ordered_begin += tmp.MPI_File_write_ordered_begin;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_ordered_end), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_ordered_end), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_ordered_end += tmp.MPI_File_write_ordered_end;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_File_write_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_File_write_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_File_write_shared += tmp.MPI_File_write_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Finalized), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Finalized), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Finalized += tmp.MPI_Finalized;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Free_mem), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Free_mem), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Free_mem += tmp.MPI_Free_mem;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Gather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Gather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Gather += tmp.MPI_Gather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Gatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Gatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Gatherv += tmp.MPI_Gatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get += tmp.MPI_Get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_accumulate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_accumulate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_accumulate += tmp.MPI_Get_accumulate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_address), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_address), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_address += tmp.MPI_Get_address;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_count), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_count), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_count += tmp.MPI_Get_count;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_elements), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_elements), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_elements += tmp.MPI_Get_elements;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_elements_x), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_elements_x), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_elements_x += tmp.MPI_Get_elements_x;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_library_version), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_library_version), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_library_version += tmp.MPI_Get_library_version;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_processor_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_processor_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_processor_name += tmp.MPI_Get_processor_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Get_version), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Get_version), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Get_version += tmp.MPI_Get_version;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graph_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graph_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graph_create += tmp.MPI_Graph_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graph_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graph_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graph_get += tmp.MPI_Graph_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graph_map), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graph_map), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graph_map += tmp.MPI_Graph_map;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graph_neighbors), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graph_neighbors), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graph_neighbors += tmp.MPI_Graph_neighbors;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graph_neighbors_count), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graph_neighbors_count), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graph_neighbors_count += tmp.MPI_Graph_neighbors_count;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Graphdims_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Graphdims_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Graphdims_get += tmp.MPI_Graphdims_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Grequest_complete), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Grequest_complete), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Grequest_complete += tmp.MPI_Grequest_complete;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Grequest_start), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Grequest_start), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Grequest_start += tmp.MPI_Grequest_start;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_compare), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_compare), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_compare += tmp.MPI_Group_compare;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_difference), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_difference), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_difference += tmp.MPI_Group_difference;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_excl), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_excl), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_excl += tmp.MPI_Group_excl;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_free += tmp.MPI_Group_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_incl), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_incl), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_incl += tmp.MPI_Group_incl;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_intersection), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_intersection), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_intersection += tmp.MPI_Group_intersection;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_range_excl), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_range_excl), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_range_excl += tmp.MPI_Group_range_excl;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_range_incl), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_range_incl), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_range_incl += tmp.MPI_Group_range_incl;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_rank), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_rank), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_rank += tmp.MPI_Group_rank;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_size += tmp.MPI_Group_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_translate_ranks), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_translate_ranks), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_translate_ranks += tmp.MPI_Group_translate_ranks;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Group_union), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Group_union), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Group_union += tmp.MPI_Group_union;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iallgather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iallgather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iallgather += tmp.MPI_Iallgather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iallgatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iallgatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iallgatherv += tmp.MPI_Iallgatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iallreduce), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iallreduce), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iallreduce += tmp.MPI_Iallreduce;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ialltoall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ialltoall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ialltoall += tmp.MPI_Ialltoall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ialltoallv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ialltoallv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ialltoallv += tmp.MPI_Ialltoallv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ialltoallw), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ialltoallw), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ialltoallw += tmp.MPI_Ialltoallw;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ibarrier), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ibarrier), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ibarrier += tmp.MPI_Ibarrier;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ibcast), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ibcast), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ibcast += tmp.MPI_Ibcast;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ibsend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ibsend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ibsend += tmp.MPI_Ibsend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iexscan), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iexscan), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iexscan += tmp.MPI_Iexscan;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Igather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Igather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Igather += tmp.MPI_Igather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Igatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Igatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Igatherv += tmp.MPI_Igatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Improbe), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Improbe), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Improbe += tmp.MPI_Improbe;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Imrecv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Imrecv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Imrecv += tmp.MPI_Imrecv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ineighbor_allgather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ineighbor_allgather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ineighbor_allgather += tmp.MPI_Ineighbor_allgather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ineighbor_allgatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ineighbor_allgatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ineighbor_allgatherv += tmp.MPI_Ineighbor_allgatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ineighbor_alltoall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ineighbor_alltoall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ineighbor_alltoall += tmp.MPI_Ineighbor_alltoall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ineighbor_alltoallv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ineighbor_alltoallv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ineighbor_alltoallv += tmp.MPI_Ineighbor_alltoallv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ineighbor_alltoallw), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ineighbor_alltoallw), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ineighbor_alltoallw += tmp.MPI_Ineighbor_alltoallw;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_create += tmp.MPI_Info_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_delete), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_delete), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_delete += tmp.MPI_Info_delete;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_dup), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_dup), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_dup += tmp.MPI_Info_dup;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_free += tmp.MPI_Info_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_get), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_get), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_get += tmp.MPI_Info_get;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_get_nkeys), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_get_nkeys), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_get_nkeys += tmp.MPI_Info_get_nkeys;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_get_nthkey), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_get_nthkey), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_get_nthkey += tmp.MPI_Info_get_nthkey;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_get_valuelen), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_get_valuelen), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_get_valuelen += tmp.MPI_Info_get_valuelen;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Info_set), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Info_set), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Info_set += tmp.MPI_Info_set;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Init += tmp.MPI_Init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Init_thread), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Init_thread), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Init_thread += tmp.MPI_Init_thread;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Initialized), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Initialized), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Initialized += tmp.MPI_Initialized;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Intercomm_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Intercomm_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Intercomm_create += tmp.MPI_Intercomm_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Intercomm_merge), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Intercomm_merge), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Intercomm_merge += tmp.MPI_Intercomm_merge;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iprobe), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iprobe), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iprobe += tmp.MPI_Iprobe;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Irecv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Irecv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Irecv += tmp.MPI_Irecv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ireduce), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ireduce), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ireduce += tmp.MPI_Ireduce;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ireduce_scatter), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ireduce_scatter), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ireduce_scatter += tmp.MPI_Ireduce_scatter;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ireduce_scatter_block), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ireduce_scatter_block), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ireduce_scatter_block += tmp.MPI_Ireduce_scatter_block;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Irsend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Irsend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Irsend += tmp.MPI_Irsend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Is_thread_main), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Is_thread_main), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Is_thread_main += tmp.MPI_Is_thread_main;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iscan), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iscan), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iscan += tmp.MPI_Iscan;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iscatter), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iscatter), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iscatter += tmp.MPI_Iscatter;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Iscatterv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Iscatterv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Iscatterv += tmp.MPI_Iscatterv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Isend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Isend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Isend += tmp.MPI_Isend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Issend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Issend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Issend += tmp.MPI_Issend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Keyval_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Keyval_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Keyval_create += tmp.MPI_Keyval_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Keyval_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Keyval_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Keyval_free += tmp.MPI_Keyval_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Lookup_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Lookup_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Lookup_name += tmp.MPI_Lookup_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Mprobe), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Mprobe), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Mprobe += tmp.MPI_Mprobe;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Mrecv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Mrecv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Mrecv += tmp.MPI_Mrecv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Neighbor_allgather), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Neighbor_allgather), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Neighbor_allgather += tmp.MPI_Neighbor_allgather;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Neighbor_allgatherv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Neighbor_allgatherv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Neighbor_allgatherv += tmp.MPI_Neighbor_allgatherv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Neighbor_alltoall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Neighbor_alltoall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Neighbor_alltoall += tmp.MPI_Neighbor_alltoall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Neighbor_alltoallv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Neighbor_alltoallv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Neighbor_alltoallv += tmp.MPI_Neighbor_alltoallv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Neighbor_alltoallw), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Neighbor_alltoallw), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Neighbor_alltoallw += tmp.MPI_Neighbor_alltoallw;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Op_commutative), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Op_commutative), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Op_commutative += tmp.MPI_Op_commutative;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Op_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Op_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Op_create += tmp.MPI_Op_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Op_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Op_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Op_free += tmp.MPI_Op_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Open_port), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Open_port), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Open_port += tmp.MPI_Open_port;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Pack), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Pack), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Pack += tmp.MPI_Pack;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Pack_external), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Pack_external), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Pack_external += tmp.MPI_Pack_external;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Pack_external_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Pack_external_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Pack_external_size += tmp.MPI_Pack_external_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Pack_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Pack_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Pack_size += tmp.MPI_Pack_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Pcontrol), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Pcontrol), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Pcontrol += tmp.MPI_Pcontrol;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Probe), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Probe), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Probe += tmp.MPI_Probe;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Publish_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Publish_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Publish_name += tmp.MPI_Publish_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Put), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Put), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Put += tmp.MPI_Put;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Query_thread), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Query_thread), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Query_thread += tmp.MPI_Query_thread;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Raccumulate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Raccumulate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Raccumulate += tmp.MPI_Raccumulate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Recv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Recv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Recv += tmp.MPI_Recv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Recv_init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Recv_init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Recv_init += tmp.MPI_Recv_init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Reduce), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Reduce), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Reduce += tmp.MPI_Reduce;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Reduce_local), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Reduce_local), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Reduce_local += tmp.MPI_Reduce_local;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Reduce_scatter), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Reduce_scatter), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Reduce_scatter += tmp.MPI_Reduce_scatter;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Reduce_scatter_block), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Reduce_scatter_block), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Reduce_scatter_block += tmp.MPI_Reduce_scatter_block;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Register_datarep), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Register_datarep), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Register_datarep += tmp.MPI_Register_datarep;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Request_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Request_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Request_free += tmp.MPI_Request_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Request_get_status), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Request_get_status), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Request_get_status += tmp.MPI_Request_get_status;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Rget), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Rget), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Rget += tmp.MPI_Rget;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Rget_accumulate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Rget_accumulate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Rget_accumulate += tmp.MPI_Rget_accumulate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Rput), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Rput), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Rput += tmp.MPI_Rput;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Rsend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Rsend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Rsend += tmp.MPI_Rsend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Rsend_init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Rsend_init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Rsend_init += tmp.MPI_Rsend_init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Scan), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Scan), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Scan += tmp.MPI_Scan;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Scatter), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Scatter), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Scatter += tmp.MPI_Scatter;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Scatterv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Scatterv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Scatterv += tmp.MPI_Scatterv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Send), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Send), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Send += tmp.MPI_Send;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Send_init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Send_init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Send_init += tmp.MPI_Send_init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Sendrecv), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Sendrecv), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Sendrecv += tmp.MPI_Sendrecv;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Sendrecv_replace), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Sendrecv_replace), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Sendrecv_replace += tmp.MPI_Sendrecv_replace;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ssend), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ssend), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ssend += tmp.MPI_Ssend;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Ssend_init), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Ssend_init), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Ssend_init += tmp.MPI_Ssend_init;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Start), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Start), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Start += tmp.MPI_Start;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Startall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Startall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Startall += tmp.MPI_Startall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Status_set_cancelled), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Status_set_cancelled), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Status_set_cancelled += tmp.MPI_Status_set_cancelled;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Status_set_elements), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Status_set_elements), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Status_set_elements += tmp.MPI_Status_set_elements;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Status_set_elements_x), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Status_set_elements_x), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Status_set_elements_x += tmp.MPI_Status_set_elements_x;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Test), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Test), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Test += tmp.MPI_Test;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Test_cancelled), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Test_cancelled), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Test_cancelled += tmp.MPI_Test_cancelled;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Testall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Testall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Testall += tmp.MPI_Testall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Testany), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Testany), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Testany += tmp.MPI_Testany;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Testsome), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Testsome), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Testsome += tmp.MPI_Testsome;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Topo_test), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Topo_test), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Topo_test += tmp.MPI_Topo_test;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_commit), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_commit), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_commit += tmp.MPI_Type_commit;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_contiguous), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_contiguous), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_contiguous += tmp.MPI_Type_contiguous;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_darray), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_darray), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_darray += tmp.MPI_Type_create_darray;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_f90_complex), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_f90_complex), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_f90_complex += tmp.MPI_Type_create_f90_complex;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_f90_integer), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_f90_integer), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_f90_integer += tmp.MPI_Type_create_f90_integer;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_f90_real), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_f90_real), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_f90_real += tmp.MPI_Type_create_f90_real;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_hindexed), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_hindexed), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_hindexed += tmp.MPI_Type_create_hindexed;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_hindexed_block), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_hindexed_block), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_hindexed_block += tmp.MPI_Type_create_hindexed_block;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_hvector), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_hvector), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_hvector += tmp.MPI_Type_create_hvector;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_indexed_block), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_indexed_block), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_indexed_block += tmp.MPI_Type_create_indexed_block;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_keyval += tmp.MPI_Type_create_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_resized), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_resized), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_resized += tmp.MPI_Type_create_resized;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_struct), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_struct), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_struct += tmp.MPI_Type_create_struct;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_create_subarray), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_create_subarray), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_create_subarray += tmp.MPI_Type_create_subarray;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_delete_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_delete_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_delete_attr += tmp.MPI_Type_delete_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_dup), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_dup), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_dup += tmp.MPI_Type_dup;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_extent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_extent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_extent += tmp.MPI_Type_extent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_free += tmp.MPI_Type_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_free_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_free_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_free_keyval += tmp.MPI_Type_free_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_attr += tmp.MPI_Type_get_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_contents), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_contents), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_contents += tmp.MPI_Type_get_contents;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_envelope), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_envelope), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_envelope += tmp.MPI_Type_get_envelope;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_extent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_extent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_extent += tmp.MPI_Type_get_extent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_extent_x), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_extent_x), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_extent_x += tmp.MPI_Type_get_extent_x;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_name += tmp.MPI_Type_get_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_true_extent), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_true_extent), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_true_extent += tmp.MPI_Type_get_true_extent;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_get_true_extent_x), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_get_true_extent_x), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_get_true_extent_x += tmp.MPI_Type_get_true_extent_x;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_hindexed), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_hindexed), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_hindexed += tmp.MPI_Type_hindexed;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_hvector), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_hvector), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_hvector += tmp.MPI_Type_hvector;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_indexed), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_indexed), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_indexed += tmp.MPI_Type_indexed;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_lb), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_lb), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_lb += tmp.MPI_Type_lb;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_match_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_match_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_match_size += tmp.MPI_Type_match_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_set_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_set_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_set_attr += tmp.MPI_Type_set_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_set_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_set_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_set_name += tmp.MPI_Type_set_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_size), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_size), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_size += tmp.MPI_Type_size;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_size_x), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_size_x), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_size_x += tmp.MPI_Type_size_x;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_struct), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_struct), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_struct += tmp.MPI_Type_struct;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_ub), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_ub), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_ub += tmp.MPI_Type_ub;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Type_vector), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Type_vector), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Type_vector += tmp.MPI_Type_vector;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Unpack), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Unpack), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Unpack += tmp.MPI_Unpack;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Unpack_external), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Unpack_external), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Unpack_external += tmp.MPI_Unpack_external;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Unpublish_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Unpublish_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Unpublish_name += tmp.MPI_Unpublish_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Wait), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Wait), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Wait += tmp.MPI_Wait;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Waitall), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Waitall), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Waitall += tmp.MPI_Waitall;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Waitany), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Waitany), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Waitany += tmp.MPI_Waitany;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Waitsome), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Waitsome), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Waitsome += tmp.MPI_Waitsome;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_allocate), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_allocate), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_allocate += tmp.MPI_Win_allocate;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_allocate_shared), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_allocate_shared), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_allocate_shared += tmp.MPI_Win_allocate_shared;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_attach), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_attach), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_attach += tmp.MPI_Win_attach;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_call_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_call_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_call_errhandler += tmp.MPI_Win_call_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_complete), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_complete), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_complete += tmp.MPI_Win_complete;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_create), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_create), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_create += tmp.MPI_Win_create;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_create_dynamic), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_create_dynamic), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_create_dynamic += tmp.MPI_Win_create_dynamic;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_create_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_create_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_create_errhandler += tmp.MPI_Win_create_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_create_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_create_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_create_keyval += tmp.MPI_Win_create_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_delete_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_delete_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_delete_attr += tmp.MPI_Win_delete_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_detach), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_detach), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_detach += tmp.MPI_Win_detach;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_fence), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_fence), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_fence += tmp.MPI_Win_fence;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_flush), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_flush), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_flush += tmp.MPI_Win_flush;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_flush_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_flush_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_flush_all += tmp.MPI_Win_flush_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_flush_local), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_flush_local), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_flush_local += tmp.MPI_Win_flush_local;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_flush_local_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_flush_local_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_flush_local_all += tmp.MPI_Win_flush_local_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_free), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_free), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_free += tmp.MPI_Win_free;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_free_keyval), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_free_keyval), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_free_keyval += tmp.MPI_Win_free_keyval;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_get_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_get_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_get_attr += tmp.MPI_Win_get_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_get_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_get_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_get_errhandler += tmp.MPI_Win_get_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_get_group), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_get_group), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_get_group += tmp.MPI_Win_get_group;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_get_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_get_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_get_info += tmp.MPI_Win_get_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_get_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_get_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_get_name += tmp.MPI_Win_get_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_lock), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_lock), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_lock += tmp.MPI_Win_lock;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_lock_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_lock_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_lock_all += tmp.MPI_Win_lock_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_post), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_post), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_post += tmp.MPI_Win_post;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_set_attr), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_set_attr), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_set_attr += tmp.MPI_Win_set_attr;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_set_errhandler), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_set_errhandler), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_set_errhandler += tmp.MPI_Win_set_errhandler;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_set_info), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_set_info), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_set_info += tmp.MPI_Win_set_info;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_set_name), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_set_name), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_set_name += tmp.MPI_Win_set_name;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_shared_query), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_shared_query), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_shared_query += tmp.MPI_Win_shared_query;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_start), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_start), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_start += tmp.MPI_Win_start;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_sync), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_sync), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_sync += tmp.MPI_Win_sync;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_test), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_test), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_test += tmp.MPI_Win_test;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_unlock), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_unlock), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_unlock += tmp.MPI_Win_unlock;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_unlock_all), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_unlock_all), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_unlock_all += tmp.MPI_Win_unlock_all;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Win_wait), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Win_wait), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Win_wait += tmp.MPI_Win_wait;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Wtick), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Wtick), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Wtick += tmp.MPI_Wtick;
          }
      
        if (rank > 0)
          PMPI_Send(&(counters.MPI_Wtime), 1, MPI_UNSIGNED_LONG, 0, 0,
                    MPI_COMM_WORLD);
        else
          {
            PMPI_Recv(&(tmp.MPI_Wtime), 1, MPI_UNSIGNED_LONG, n, 0,
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            counters.MPI_Wtime += tmp.MPI_Wtime;
          }
      

      if (rank == 0)
        {
          printf("\n Rank %d:\n", n);
          print_counters(&tmp);
        }
    }

  /* Print the total counters. These have been summed up in the counter struct
   * of rank 0, so these have not to be received a second time. */
  if (rank == 0) {
    printf("\n Total:\n");
    print_counters(&counters);
    fflush(stdout);
  }


  /* Call original MPI_Finalize. */
  return XMPI_Finalize();
}
